


/*******************************************************************************************
 *                                                                                         *
 * Examples1 : GLCD_Touch Screen(ET-TFT240320TP-3.2 Rev.B) For "ET-BASE AVR MEGA128"       *
 *                                                                                         *
 *                                  DISPLAY PORTRAIT (Ver.)                                *
 *                                                                                         *
 *      MODE : Interface GLCD Parallel data 8 bit                                          *
 *                                                                                         *
 *******************************************************************************************
 *                                                                                         *
 * Target MCU     : ATMEL AVR-MEGA128                                                      *
 *       	      : X-TAL : 16 MHz                                                         * 
 *                                                                                         *
 * Editor         : AVR Studio 4.15 Build 623                                              *
 * Compiler       : WinAVR-20081205                                                        *
 * Create By      : Mr. Sittiphol Yooyod (WWW.ETT.CO.TH)                                   *
 * Last Update    : 27/August/2009                                                         *
 *																				           *
 * Port Interface :  -- MCU-Mega128--	  	  	       --LCD-ET-TFT240320TP-3.2 Rev.B--    *
 *                                                                                         *
 *     DAta GLCD  :     PF[0..7]          Connect          DB8..DB15                       *
 *                      				                                                   *
 *     Contrl GLCD:     PE2               Connect          CS-H						       *
 *                      PE3               Connect          RS-H(0=cmm,1=data)              *
 *                      PE4               Connect          WR-H(Write)                     *
 *                      PE5               Connect          RD-H(Read)                      *
 *                      PE6               Connect          RES-H(Reset)                    *
 *     Back Light :     PE7               Connect          BL-H(LCD-Back Light(Active=1))  *
 *                                                                                         *
 *																					       *
 *   																				       *
 *     Touch Sceen:    PD2 (MISO-in)      Connect 		    MISO					       *
 *                     PD3 (MOSI-out)     Connect 		    MOSI					       *
 *                     PD4 (SCLK-out)	  Connect 		    SCLK					       *
 *                     PD5 (BUSY-in)	  Connect 		    BUSY(No_USE)				   *
 *                     PD6 (PEN-in)	      Connect 		    PEN					           *
 *                     PD7 (TC_CS-out)	  Connect 		    TC_CS					       *
 *                                                                                         *
 *                                                                                         *
 *    UART0(BaudRate=9600)   :    PE0 =  RXD0                                              *
 *                                PE1 =  TXD0                                              *
 *                                                                                         *
 *******************************************************************************************/




#include <stdio.h>
#include <avr/io.h>                  //define port
#include <compat/deprecated.h>       //call function sbi(port,bit),cbi(port,bit) = Set,Clear bit
#include <avr/pgmspace.h>            //for progrm data to Area Flash

#define F_CPU 16000000UL             //CPU Clock Frequency (Hz)
#include <util/delay.h>              //call function delay



/******************************************************
 *			 Font 7x11(English)				          *
 *											          *
 * -ASCII fonts from 0x20 ~ 0x7F(DEC 32 ~ 126)     	  *
 *  composed of following characters 			      *
 *                                                    * 
 * -XSize : 7  pixcels							      *
 * -YSize : 11 pixcels						       	  *
 *											       	  *
 *   ....... <- b0, Blank, Not used				      *
 *   ....... <- b1, Blank, Not used				      *
 *   ....... <- b2, Blank, Not used				      *
 *   ....... <- b3, Blank, Not used				      *
 *   ....... <- b4, Blank, Not used				      *
 *   ..End.. <- b5(2nd byte)					      *
 *   ..OO... <- b6(2nd byte)					      *
 *   ..OO... <- b7(2nd byte)					      *
 *   ..OO... <- b0(1st byte)					      *
 *   ..OO... <- b1(1st byte)					      *
 *   ..OO... <- b2(1st byte)					      *
 *   ..OO... <- b3(1st byte)					      *
 *   ..OO... <- b4(1st byte)				 	      *
 *   ..OO... <- b5(1st byte)					      *
 *   ..OO... <- b6(1st byte)					      *
 *   .Start..<- b7(1st byte)					      *
 *												      *
 ******************************************************/

//7x11 ascii font, each character takes 14 bytes, total 1330 bytes, draw from left bottom to left top

  prog_char ascii_7x11[95][14] = {                                                       
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x20 space
  {0x00, 0x00, 0x07, 0xc0, 0xdf, 0xe0, 0xdf, 0xe0, 0x07, 0xc0, 0x00, 0x00, 0x00, 0x1f}, //0x21 !
  {0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x1f}, //0x22 "
  {0x00, 0x00, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x00, 0x1f}, //0x23 #
  {0x13, 0x00, 0x27, 0x80, 0x24, 0x80, 0x7f, 0xc0, 0x24, 0x80, 0x38, 0x80, 0x19, 0x1f}, //0x24 $
  {0x30, 0x80, 0x19, 0x40, 0x2d, 0x40, 0x56, 0x80, 0x53, 0x00, 0x21, 0x80, 0x00, 0x1f}, //0x25 %
  {0x38, 0x00, 0x7d, 0x80, 0x47, 0xc0, 0x5e, 0x40, 0x7b, 0xc0, 0x21, 0x80, 0x58, 0x1f}, //0x26 &
  {0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x27 '
  {0x00, 0x00, 0x00, 0x00, 0x3f, 0x80, 0x7f, 0xc0, 0xc0, 0x60, 0x80, 0x20, 0x00, 0x1f}, //0x28 (
  {0x00, 0x00, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x29 )
  {0x00, 0x00, 0x24, 0x80, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0x24, 0x80, 0x00, 0x1f}, //0x2a *
  {0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2b +
  {0x00, 0x00, 0x80, 0x00, 0xe0, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2c ,
  {0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2d -
  {0x00, 0x00, 0x00, 0x00, 0xc0, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2e .
  {0xc0, 0x00, 0xf0, 0x00, 0x3c, 0x00, 0x0f, 0x00, 0x03, 0xc0, 0x00, 0xe0, 0x00, 0x3f}, //0x2f /
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x30 0
  {0x00, 0x00, 0x80, 0x80, 0x80, 0xc0, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x00, 0x1f}, //0x31 1
  {0xe0, 0xc0, 0xf0, 0xe0, 0x98, 0x20, 0x8c, 0x20, 0x84, 0x20, 0x87, 0xe0, 0x83, 0xdf}, //0x32 2
  {0x60, 0xc0, 0xe0, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x33 3
  {0x38, 0x00, 0x3e, 0x00, 0x27, 0x80, 0x21, 0xe0, 0xff, 0xe0, 0xff, 0xe0, 0x20, 0x1f}, //0x34 4
  {0x83, 0xe0, 0x83, 0xe0, 0x82, 0x20, 0x82, 0x20, 0xc2, 0x20, 0x7e, 0x20, 0x3c, 0x3f}, //0x35 5
  {0x7f, 0x00, 0xff, 0xc0, 0x82, 0x60, 0x82, 0x20, 0x82, 0x20, 0xfe, 0x20, 0x7c, 0x3f}, //0x36 6
  {0x00, 0x20, 0x00, 0x20, 0xe0, 0x20, 0xfc, 0x20, 0x1f, 0x20, 0x03, 0xe0, 0x00, 0xff}, //0x37 7
  {0x7b, 0xc0, 0xff, 0xe0, 0x86, 0x20, 0x86, 0x20, 0x8c, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x38 8
  {0x03, 0xc0, 0x87, 0xe0, 0x84, 0x20, 0xc4, 0x20, 0x64, 0x20, 0x3f, 0xe0, 0x1f, 0xdf}, //0x39 9
  {0x00, 0x00, 0x00, 0x00, 0x31, 0x80, 0x31, 0x80, 0x31, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3a :
  {0x00, 0x00, 0x00, 0x00, 0xb1, 0x80, 0xf1, 0x80, 0x71, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3b ;
  {0x04, 0x00, 0x0e, 0x00, 0x1b, 0x00, 0x31, 0x80, 0x60, 0xc0, 0xc0, 0x60, 0x80, 0x3f}, //0x3c <
  {0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x1f}, //0x3d =
  {0x80, 0x20, 0xc0, 0x60, 0x60, 0xc0, 0x31, 0x80, 0x1b, 0x00, 0x0e, 0x00, 0x04, 0x1f}, //0x3e >
  {0x01, 0xc0, 0x01, 0xe0, 0x00, 0x20, 0xd8, 0x20, 0xdc, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x3f ?
  {0x7f, 0x40, 0x83, 0x20, 0x83, 0x20, 0xff, 0x20, 0x80, 0x60, 0xff, 0xe0, 0x7f, 0xdf}, //0x40 @
  {0xff, 0x00, 0xff, 0x80, 0x08, 0xc0, 0x08, 0x60, 0x08, 0xc0, 0xff, 0x80, 0xff, 0x1f}, //0x41 A
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x42 B
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xe0, 0xe0, 0x60, 0xdf}, //0x43 C
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x9f}, //0x44 D
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x80, 0x3f}, //0x45 E
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x3f}, //0x46 F
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x88, 0x20, 0xf8, 0xe0, 0xf8, 0xdf}, //0x47 G
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x48 H
  {0x00, 0x00, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x00, 0x00, 0x00, 0x1f}, //0x49 I
  {0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x4a J
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x1f, 0x00, 0x7b, 0xc0, 0xe0, 0xe0, 0x80, 0x3f}, //0x4b K
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x4c L
  {0xff, 0xe0, 0xff, 0xe0, 0x00, 0x80, 0x07, 0x00, 0x00, 0x80, 0xff, 0xe0, 0xff, 0xff}, //0x4d M
  {0xff, 0xe0, 0xff, 0xe0, 0x03, 0x80, 0x0e, 0x00, 0x38, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x4e N
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x4f O
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x50 P
  {0x7f, 0xc0, 0xff, 0xe0, 0x90, 0x20, 0xb0, 0x20, 0xe0, 0x20, 0x7f, 0xe0, 0xff, 0xdf}, //0x51 Q
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x0c, 0x20, 0xff, 0xe0, 0xf3, 0xdf}, //0x52 R
  {0x43, 0xc0, 0xc7, 0xe0, 0x86, 0x20, 0x84, 0x20, 0x8c, 0x20, 0xfc, 0x60, 0x78, 0x5f}, //0x53 S
  {0x00, 0x20, 0x00, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x20, 0x00, 0x20, 0x00, 0x1f}, //0x54 T
  {0x7f, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x55 U
  {0x1f, 0xe0, 0x3f, 0xe0, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0xe0, 0x1f, 0xff}, //0x56 V
  {0x1f, 0xe0, 0xff, 0xe0, 0xe0, 0x00, 0x1c, 0x00, 0xe0, 0x00, 0xff, 0xe0, 0x1f, 0xff}, //0x57 W
  {0xc0, 0x60, 0xf1, 0xe0, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0xf1, 0xe0, 0xc0, 0x7f}, //0x58 X
  {0x03, 0xe0, 0x07, 0xe0, 0xfc, 0x00, 0xfc, 0x00, 0x07, 0xe0, 0x03, 0xe0, 0x00, 0x1f}, //0x59 Y
  {0xc0, 0x20, 0xf0, 0x20, 0xbc, 0x20, 0x8f, 0x20, 0x83, 0xa0, 0x80, 0xe0, 0x80, 0x7f}, //0x5a Z
  {0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0x00, 0x1f}, //0x5b [
  {0x00, 0x20, 0x00, 0xe0, 0x03, 0xc0, 0x0f, 0x00, 0x3c, 0x00, 0xf0, 0x00, 0xc0, 0x1f}, /*0x5c \*/
  {0x00, 0x00, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x1f}, //0x5d ]
  {0x00, 0x80, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0x1f}, //0x5e ^
  {0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x5f _
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x1f}, //0x60 `
  {0x70, 0x00, 0xf9, 0x00, 0x89, 0x00, 0x89, 0x00, 0x89, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x61 a
  {0xff, 0xe0, 0xff, 0xe0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x62 b
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xc3, 0x00, 0x42, 0x1f}, //0x63 c
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x64 d
  {0x7e, 0x00, 0xff, 0x00, 0x91, 0x00, 0x91, 0x00, 0x91, 0x00, 0x9f, 0x00, 0x5e, 0x1f}, //0x65 e
  {0x04, 0x00, 0xff, 0xc0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x1f}, //0x66 f
  {0x9e, 0x00, 0xbf, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x67 g
  {0xff, 0xe0, 0xff, 0xe0, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x68 h
  {0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0xff, 0x60, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x69 i
  {0x80, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0x7f, 0x60, 0x00, 0x1f}, //0x6a j
  {0xff, 0xe0, 0xff, 0xe0, 0x18, 0x00, 0x3c, 0x00, 0x66, 0x00, 0xc3, 0x00, 0xc1, 0x1f}, //0x6b k
  {0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x6c l
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x3f, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6d m
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6e n
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x6f o
  {0xff, 0x00, 0xff, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0x3f, 0x00, 0x1e, 0x1f}, //0x70 p
  {0x1e, 0x00, 0x3f, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x71 q
  {0x01, 0x00, 0xff, 0x00, 0xff, 0x00, 0x04, 0x00, 0x02, 0x00, 0x03, 0x00, 0x03, 0x1f}, //0x72 r
  {0x8e, 0x00, 0x9f, 0x00, 0x99, 0x00, 0x99, 0x00, 0x99, 0x00, 0xf9, 0x00, 0x71, 0x1f}, //0x73 s
  {0x01, 0x00, 0x01, 0x00, 0x7f, 0xc0, 0xff, 0xc0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x1f}, //0x74 t
  {0x7f, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0x00, 0xff, 0x1f}, //0x75 u
  {0x1f, 0x00, 0x3f, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0x00, 0x1f, 0x1f}, //0x76 v
  {0x3f, 0x00, 0xff, 0x00, 0xc0, 0x00, 0x3c, 0x00, 0xc0, 0x00, 0xff, 0x00, 0x3f, 0x1f}, //0x77 w
  {0xe3, 0x00, 0xf7, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0xf7, 0x00, 0xe3, 0x1f}, //0x78 x
  {0x8f, 0x00, 0x9f, 0x00, 0x90, 0x00, 0x90, 0x00, 0x90, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x79 y
  {0xc1, 0x00, 0xe1, 0x00, 0xb1, 0x00, 0x99, 0x00, 0x8d, 0x00, 0x87, 0x00, 0x83, 0x1f}, //0x7a z
  {0x00, 0x00, 0x04, 0x00, 0x7f, 0xc0, 0xe0, 0xe0, 0x80, 0x20, 0x80, 0x00, 0x00, 0x1f}, //0x7b {
  {0x00, 0x00, 0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x7c |
  {0x00, 0x00, 0x80, 0x20, 0xe0, 0xe0, 0x7f, 0xc0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x5f}, //0x7d }
  {0x00, 0x60, 0x00, 0x20, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0xc0, 0x00, 0x5f}  //0x7e ~
};





/*########***** Define Color *****########
  #####  Color = RRRRR GGGGGG BBBBB ######
  ########************************######## */

#define no_bg               0x0001			//No Color Back Ground
#define BLACK               0x0000
#define WHITE               0xFFFF
#define RED                 0x8000
#define GREEN               0x0400
#define BLUE                0x0010
#define YELLOW           	0xFFF0
#define CYAN                0x0410
#define MAGENTA             0x8010
#define BROWN               0xFC00
#define OLIVE	            0x8400

#define BRIGHT_RED          0xF800
#define BRIGHT_GREEN        0x07E0
#define BRIGHT_BLUE         0x001F
#define BRIGHT_YELLOW       0xFFE0
#define BRIGHT_CYAN         0x07FF
#define BRIGHT_MAGENTA    	0xF81F

#define LIGHT_GRAY          0x8410
#define DARK_GRAY         	0x4208
#define LIGHT_BLUE          0x841F
#define LIGHT_GREEN         0x87F0
#define LIGHT_CYAN          0x87FF
#define LIGHT_RED         	0xFC10
#define LIGHT_MAGENTA      	0xFC1F

#define GRAY0       	    0xE71C
#define GRAY1         	    0xC618
#define GRAY2             	0xA514
#define GRAY3             	0x630C
#define GRAY4            	0x4208
#define GRAY5	            0x2104
#define	GRAY6	          	0x3186	

#define	BLUE0	            0x1086	
#define	BLUE1       		0x3188 
#define	BLUE2          	    0x4314
#define BLUE3         	    0x861C	

#define	CYAN0	            0x3D34
#define	CYAN1               0x1DF7		

#define	GREEN0              0x0200	
#define	GREEN1              0x0208	


//********** Valiable for Touch Screen Function GLCD ************

long dif_adc_X,dif_adc_Y              ;
long buf_adc_X[100],buf_adc_Y[100]    ;
long ave_adc_X,ave_adc_Y              ;


long tcs_ave_X[3]				       ;	// Keep Value adc Touch Point X
long tcs_ave_Y[3]                      ;    // Keep Value adc Touch Point Y
long dis_XD[3] = {24,215,120}          ;    // Value refer Point X at 10% of Display(X=240:0-239) 3 Position
long dis_YD[3] = {32,160,287}          ;    // Value refer Point Y at 10% of Display(Y=320:0-319) 3 Position
long divider,An,Bn,Cn,Dn,En,Fn,X,Y     ;	// Valiable for keep coefficient Calibrat and position touch Screen
unsigned char num1,num2,num3           ;	// Variable for keep data Dec to Ascii




//************ 1.Function Control Graphic LCD ET-TFT240320TP-3.2 REV.B (Driver SPFD5408A) ***********

void gp_cs_hi(void)                      ;
void gp_cs_lo(void)                      ;
void gp_rs_hi(void)					     ;
void gp_rs_lo(void)                      ;
void gp_wr_hi(void)					     ;
void gp_wr_lo(void)                      ;
void gp_rd_hi(void)					     ;
void gp_rd_lo(void)					     ;
void gp_res_hi(void)				     ;
void gp_res_lo(void)				     ;
void gp_bl_on(void)					     ;
void gp_bl_off(void)				     ;
void gp_wr_cmm(unsigned char cmm)         ;
void gp_wr_data(unsigned int dw)         ;
void init_glcd(void)                     ;


//************** 2.Function Control SPI Touch-Screen (ADS7846) ****************


void tcs_sck_hi(void)                             ;
void tcs_sck_lo(void)                             ;
void tcs_cs_hi(void)					          ;
void tcs_cs_lo(void)					          ;
void tcs_sdo_hi(void)					          ;
void tcs_sdo_lo(void)				  	          ;
void tcs_busy(void)                               ;
char status_touch(void)                           ;
unsigned char tcs_wr(unsigned char DataByte)      ;
void tcs_read(void)                               ;
void ave_x_y(unsigned char num)	                  ;

void touch_calibrate(void)                        ;
void set_matrix(void)                             ;
void get_point(unsigned char num)                 ;


//********************** 3.Function Application of GLCD **********************


void bg_color(unsigned int bg_color)                                                                            ;
void plot_dot(unsigned int Xadd,unsigned int Yadd,unsigned int color)                                           ;
void plot_mark(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2,unsigned int color)              ;

void text_7x11(unsigned char row,int adx,int ady,unsigned int fg_clr,unsigned int bg_clr)                       ;
void lcd_printStr(char *str,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)                    ;
void lcd_print3Cha(char ch1,char ch2,char ch3,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)  ;




/***************************************************************
 **                 Function use For Printf(UART0)            **
 ***************************************************************/

/*
//--------------------- Prototype function ---------------------

static int uart_putchar(char c,FILE *stream) ;
static FILE uart_str = FDEV_SETUP_STREAM(uart_putchar,NULL,_FDEV_SETUP_WRITE);
void Init_Serial(unsigned int baudrate);


//----------------------- UART0 Putchar -------------------------

static int uart_putchar(char c,FILE *stream)
 {
   if(c=='\a')
     {
	  fputs("*ring*\n",stderr) ;
	  return 0 ;
     }
   
   if(c=='\n')
     uart_putchar('\r',stream) ;

   loop_until_bit_is_set(UCSR0A,UDRE);
   UDR0 = c ;

   return 0 ;
 }

//------------------- Initial UART0 -----------------------

static void UART0_Init(void)
 {

   UCSR0B = 0x00      ;     //disable while setting baud rate
   UBRR0H = 0         ;
   UBRR0L = 103       ;     //Set Baud Rate 9600
  
   UCSR0C = 0x06      ;     //Set Frame format : 8data,NoneParity,1stop bit
   UCSR0B = 0x98      ;     //Uart0 Control Enable Reciver and transmitter
   
   stdout = &uart_str ;     //Set address uart_str to stdout

 }
*/

/*****************************************
 **         Function Delay ms           **
 *****************************************/

 void delay_ms(unsigned int cnt)
  {
    for(;cnt>0;cnt--)
       _delay_ms(1);
  }


/*****************************************
 **        Function Initial Port        **
 *****************************************/

 void init_port(void)
  {
   
   DDRF  = 0xFF     ;        //Set PortF(0..7) = Output for data GLCD Byte Hig

   DDRE |= 0xFC     ;        //Set PortE(2..7) = Output for Control GLCD(CS,RS,WR,RD,Reset,BL)  
   
   DDRD  = 0x98     ;        //Set PortD = PD7,PD4,PD3:Output ;PD6,PD5,PD2:Input for Touch Screen
   
   //------ Set Value Default Port ------ 

   PORTF = 0        ;
   PORTE = 0x7C     ;
   PORTD = 0x80     ;

  }


//*******************************************************************
//*********** Convert Dec to Ascii  3 digit(000-999) ****************
//*******************************************************************        

void dec_to_ascii(unsigned int num)
 {
	 num1  = (num/100) + 0x30  ;//colum 100
	 num   =  num%100          ;
	 num2  = (num/10)  + 0x30  ;//colum 10
	 num3  =  (num%10) + 0x30  ;//colum 1	 

 }



/*#########################################################################
  ##                               MAIN PROGRAM                          ##
  #########################################################################*/

int main(void)
 {
   	int x1=113,y1=160,x2=153,y2= 120 ;


 //------------------- Start GLCD and Cribrate Touch Screen ---------------
  
   init_port()			 ;
   init_glcd()           ;
   touch_calibrate()	 ;
 //  UART0_Init()          ;

   lcd_printStr("Touch X: ",70,290,WHITE,BLACK) ;  //Plot Text	Color White
   lcd_printStr("Touch Y: ",70,310,WHITE,BLACK) ;

   //----------- Start Main Program -------------	
    while(1)
	 {	  		
	  
		if(status_touch())               	  //return (1) if tap touch screen is detected.
		{
		 
			    get_point(20)                 ; //Calculate Position Address Display Keep result in valiable X,Y
   

			    plot_mark(x1,y1,x2,y2,BLACK) ; //Del Cursot '+' in Position Touch old


			    x1 = X-7 ;
			    y1 = Y   ;
			    x2 = X   ;
			    y2 = Y-7 ;
			    plot_mark(x1,y1,x2,y2,BRIGHT_RED)			              ; //Plot Cursot '+' in Position Touch Screen

			    lcd_printStr("Touch X: ",70,290,WHITE,BLACK)              ; //Plot String Color White
			    lcd_printStr("Touch Y: ",70,310,WHITE,BLACK)              ;

		  	    dec_to_ascii(X)	                                          ; //Convert Position X-Addres to ASCII
			    lcd_print3Cha(num1,num2,num3,145,290,BRIGHT_YELLOW,BLACK) ; //Plot Position X-Address on GLCD
			    dec_to_ascii(Y)	                                          ; //Convert Position Y-Addres to ASCII
		   	    lcd_print3Cha(num1,num2,num3,145,310,BRIGHT_YELLOW,BLACK) ; //Plot Position Y-Address on GLCD	
		}   
     }

   return 0 ;

}


/* ###################################################################################################
   ##                                                                                               ##
   ##                                        SUB-PROGRAM                                            ##
   ##                                                                                               ##
   ################################################################################################### */



/*##########**********************************************************************************#########
  ##########**                                                                              **#########
  ##########** 1.Function Control of Graphic LCD EL-TFT240320TP-3.2 REV.B(Driver SPFD5408S) **#########
  ##########**                                                                              **#########                 
  ##########**********************************************************************************######### */



//******** Control Pin cs(PE2=CS) ********

void gp_cs_hi(void)
 {
   sbi(PORTE,2) ;	   //PE2 = Hi
 }

void gp_cs_lo(void)
{
  cbi(PORTE,2) ;	   //PE2 = Lo
}

//******* Control Pin rs (PE3=RS) *******

void gp_rs_hi(void)
 {
   sbi(PORTE,3);	   //PE3 = Hi
 }


void gp_rs_lo(void)
{
  cbi(PORTE,3) ;	   //PE3 = Lo
}

//******* Control Pin wr(PE4=WR) *********

void gp_wr_hi(void)
 {
   sbi(PORTE,4);	   //PE4 = Hi
 }

void gp_wr_lo(void)
{
   cbi(PORTE,4) ;      //PE4 = Lo
}

//****** Control Pin rd(PE5=RD) ******
void gp_rd_hi(void)
 {
    sbi(PORTE,5) ;	   //PE5 = Hi
 }

void gp_rd_lo(void)
{
    cbi(PORTE,5) ;	   //PE5 = Lo
}
 
//******** Control Pin res(PE6=RES) *********
void gp_res_hi(void)
 {
   sbi(PORTE,6)  ;	   //PE6 = Hi
 }


void gp_res_lo(void)
{
   cbi(PORTE,6) ;	   //PE6 = Lo
}

//********* Control Back Light(PE7=Back Light)**********

void gp_bl_on(void)
{
  sbi(PORTE,7)    ;	   //PE7 = Hi(ON)
}

void gp_bl_off(void)
{
  cbi(PORTE,7)    ;	   //PE7 = Lo(OFF)
}



//*********************************************************
//************* Write Address Command to LCD **************
//*********************************************************

void gp_wr_cmm(unsigned char cmm)
 {

    //**********-- Sent Command 8 bit High --**********

     gp_cs_lo()                        ;

   //-----Write the command to the data bus.----

	 PORTF  = 0x00                     ;

   //----Set the RS signal low, indicating a command.-----
     gp_rs_lo()                        ;  //RS=0
    
   //-----Assert the write enable signal.-------
	 gp_wr_lo()                        ;  //wr=0 

   //------Deassert the write enable signal.-------
   	 gp_wr_hi()	                       ;  //wr=1


    //**********-- Sent Command 8 bit Low --*********

	 
   //-----Write the command to the data bus.----

	 PORTF  = cmm                     ;

    
   //-----Assert the write enable signal.-------
	 gp_wr_lo()                        ;  //wr=0 

   //------Deassert the write enable signal.-------
   	 gp_wr_hi()	                       ;  //wr=1

  
   //------Set the RS signal high, indicating that following writes are data.------
     gp_rs_hi()                        ;  //RS=1

 }
 



//***********************************************
//************* Write data to LCD ***************
//***********************************************

void gp_wr_data(unsigned int dw)
 {	

  //**********-- Sent Command 8 bit High --*********

      gp_cs_lo()                           ;

   //----- Write data to data bus. 8 bit PF[8..15] ------
	  

      PORTF    = dw>>8                     ;
	  	  
   //----- Assert the write enable signal.--------

	  gp_wr_lo()                           ;  //wr = 0
    
   //------ Deassert the write enable signal.-------- 

 	  gp_wr_hi()                           ;  //wr = 1
	 

 //**********-- Sent Command 8 bit Low --*********
    

   //----- Write data to data bus. 16 bit PF[8..15] ------
	  
	
      PORTF    = dw                        ;
	  	  
   //----- Assert the write enable signal.--------

	  gp_wr_lo()                           ;  //wr = 0
    
   //------ Deassert the write enable signal.-------- 

 	  gp_wr_hi()                           ;  //wr = 1
	 
	  gp_cs_hi()                           ;

 }


 
//***************************************************** 
//***************** Initial GLCD **********************
//*****************************************************

void init_glcd(void)
 {
    long cnt ;
    
    //-------------- Start GLCD --------------

	  gp_rd_hi()            ; 

  	  gp_res_lo()           ;    // Reset GLCD 
      delay_ms(1)           ;    // Delay for 1ms.	 
      gp_res_hi()           ;    // Stop Signal Reset GLCD    
      delay_ms(1)           ;	 // Delay for 1ms while the LCD comes out of reset.
   
  
      gp_wr_cmm(0x00)       ;	 // Command Start Oscillater
      gp_wr_data(0x0001)    ;	 // Enable the oscillator.
   
      delay_ms(10)          ;    // Delay for 10ms while the oscillator stabilizes.
   

	 //----- Display Setting -----
	 	
	  gp_wr_cmm(0x01)       ;	 // Configure the output drivers.
      gp_wr_data(0x0100)    ;
      gp_wr_cmm(0x02)       ;	 // Configure the LCD A/C drive waveform.
      gp_wr_data(0x0700)    ;	 // Line inversion 0x0700
      gp_wr_cmm(0x03)       ;    // Configure Entry Mode
      gp_wr_data(0x1230)    ; 	 // Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
   

   //------ Configure and enable the LCD power supply.-------

	 //---- Power Control -----

	  gp_wr_cmm(0x07)       ;    // Enable internal operation of the LCD controller.
      gp_wr_data(0x0101)    ;	 // 0011
	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x0000)    ;  //0x00B0
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0007)    ;   //0x0037
	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x0000)    ;  //0x011E
	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x0000)    ;  //0x1A00
	  delay_ms(20)          ;

	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x16B0)    ;
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0037)    ;
	  delay_ms(20)          ; //50

	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x013E)    ;
	  delay_ms(20)          ;  //50

	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x1A00)    ;
      gp_wr_cmm(0x29)       ;
      gp_wr_data(0x000F)    ;      
      delay_ms(20)          ;    //50- Delay for 40ms while the power supply stabilizes.
    
   
	 
	  //------- Clear the contents of the display buffer. --------
   
      gp_wr_cmm(0x22);


      for(cnt = 0;cnt<((long)240*320);cnt++)
         gp_wr_data(BLACK)   ;    // Back-Ground Color Black
      
   

	  gp_wr_cmm(0x20)       ;    //Command Horizontal GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 
	  gp_wr_cmm(0x21)       ;    //Command Vertical GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 

	  gp_wr_cmm(0x50)       ;  	 //Command Horizontal Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x51)       ;  	 //Command Horizontal Address End Position(239)
	  gp_wr_data(0x00EF)    ;  

	  
	  gp_wr_cmm(0x52)       ;  	 //Command Vertical Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x53)       ;  	 //Command Verticall Address End Position(239)
	  gp_wr_data(0x013F)    ;  
  

      gp_wr_cmm(0x60)       ;    // Set the number of lines to scan.
      gp_wr_data(0x2700)    ;

      gp_wr_cmm(0x61)       ;    // Enable grayscale inversion of the source outputs.
      gp_wr_data(0x0001)    ;

    
   //------- GAMMA Control -------

	  gp_wr_cmm(0x30)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x31)       ; 
	  gp_wr_data(0x0403)    ;
  	  gp_wr_cmm(0x32)       ; 
	  gp_wr_data(0x0404)    ;
      gp_wr_cmm(0x35)       ; 
	  gp_wr_data(0x0002)    ;
      gp_wr_cmm(0x36)       ; 
	  gp_wr_data(0x0707)    ;
	  gp_wr_cmm(0x37)       ; 
	  gp_wr_data(0x0606)    ;
      gp_wr_cmm(0x38)       ; 
	  gp_wr_data(0x0106)    ; 
	  gp_wr_cmm(0x39)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x3C)       ; 
	  gp_wr_data(0x0700)    ;
  	  gp_wr_cmm(0x3D)       ; 
	  gp_wr_data(0x0707)    ;

    
      gp_wr_cmm(0x07)       ;	 //Enable the image display.
      gp_wr_data(0x0173)    ;	 
	
      gp_bl_on()            ;	 //Back Light ON
	  
	  delay_ms(20)          ;	 //Delay for 20ms, which is equivalent to two frames.

 }





/* #########*********************************************************#############
   #########**                                                     **############# 
   #########**   2.Function Touch Screen (ADS7846:SPI-Interface)   **#############
   #########**                                                     **#############            
   #########*********************************************************#############	*/


//*********** CS [PD7] *************
 void tcs_cs_hi(void)
 {
   sbi(PORTD,7)  ;   //PD7 =1
 }

void tcs_cs_lo(void)
 {
   cbi(PORTD,7) ;   //PD7 =0
 }

//*********** SCK [PD4] **************

void tcs_sck_hi(void)
 {
   sbi(PORTD,4) ;   //PD4 =1
 }

void tcs_sck_lo(void) 
{
   cbi(PORTD,4) ;  //PD4 =0
}


 //*********** Data MOSI [PD3]************

void tcs_sdo_hi(void)
 {
   sbi(PORTD,3) ;   //PD3 =1
 }

void tcs_sdo_lo(void)
 {
  cbi(PORTD,3) ;	//PD3 =0
 }



///************************************************************************
//*******          Function Check Status Press Touch Screen       ********
//************************************************************************
//******** Return Value : 											   ***
//********               Function return value '0'= Not Touched screen ***
//********               Function return value '1'= Touched screen     ***
//************************************************************************


char status_touch(void)
 {
  
  if((PIND & 0x40)== 0x40)	//Read PD6
	return 0 ;			    //No Touch
  else
    return 1 ;			    //Press Touch screen
 }


 //****************************************************************************************
 //******      Function SPI Write and Read Data 1 Byte from Touch Screen ADS7846     ******
 //****************************************************************************************
 //******       Parameter : 												         ******
 //******                  DataByte = data or command control ADS7846 (1Byte)        ****** 
 //******    Return Value : 														 ******
 //******                  Return value adc from touched times 1 byte Pass Function  ******         
 //****************************************************************************************

unsigned char tcs_wr(unsigned char DataByte)			  	
{
  unsigned char Bit,result=0 ;				 	   

     for(Bit=0;Bit<8;Bit++)                  // 8 Bit Write
      {		
	   	tcs_sck_hi()                     ;	 // Start Clock  High

		//--------- write Data(PD3) ---------

		if((DataByte & 0x80)== 0x80)
		   tcs_sdo_hi()                  ;   // Set bit data(PD3) = 1
		else
		   tcs_sdo_lo()                  ;	 // Clear bit data(PD3) = 0
		   	
		tcs_sck_lo()                     ;   // Clock low

		DataByte <<= 1                   ;	 // Shift Next Bit Data
      
        
	  	//---------- Read Data(PD2) -------

	    result <<= 1                   ;	
		 										  
	     if ((PIND & 0x04)== 0x04)                // If SDI = "1"						
	      {
	       result |= 0x01                ;       // Keep Result of Read-data  									
	      }
			  
	  }

	  return (result)                    ;
}



//******************************************************************************** 
//******    Function Read X-Y-Position ADC Touch Screen-12 bit adc          ******
//********************************************************************************
//******        Parameter :	No                                              ******
//******    Return Value  :													******
//******                    dif_adc_X = Keep Result ADC X-Position(12 bit)  ******
//******                    dif_adc_Y = Keep result ADC Y-Position(12 bit)  ******                      
//********************************************************************************

void tcs_read(void)
 {
    unsigned int tcs_adc_X ,tcs_adc_Y       ;
	unsigned char buf_data[4]               ;

	if(status_touch())
	{
	//--- Set default Pin control Touch Screen ----
	  tcs_sdo_lo() ;
	  tcs_sck_lo() ;
	  tcs_cs_lo () ;


      buf_data[0] = tcs_wr(0xD0)            ;	//Write Command Measure X-Position 
	  buf_data[0] = tcs_wr(0x00)            ;	//Read ADC data X-Position (7-bit byte High) data: 0ddddddd	(bit) 
	  buf_data[1] = tcs_wr(0x90)            ;	//Write Command Measure Y-Position ; Read ADC data X-Position (5-bit byte Low)  data:ddddd000(bit)
	  buf_data[2] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(7-bit byte High) data: 0ddddddd (bit)
	  buf_data[3] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(5-bit byte Low)  data: ddddd000 (bit)

	  tcs_adc_X  = buf_data[0]              ;	// Mark ADC Data X-Position 12 bit
	  tcs_adc_X  = tcs_adc_X << 5           ;
	  tcs_adc_X |= buf_data[1] >> 3         ;
	  tcs_adc_X  = tcs_adc_X & 0x0FFF       ;
	 
	  tcs_adc_Y  = buf_data[2]              ;   // Mark ADC Data Y-Position 12 bit
	  tcs_adc_Y  = tcs_adc_Y << 5           ;   // Shift 7 bit High
	  tcs_adc_Y |= buf_data[3] >> 3         ;   // Shift 5 bit low
	  tcs_adc_Y  = tcs_adc_Y & 0x0FFF       ;   // total ADC data 12 bit
	

	  tcs_cs_hi()                           ;
	 
	 
	  dif_adc_X = tcs_adc_X                 ;	//ADC 12 bit :LCD Start Portrait
	  dif_adc_Y = 4095-tcs_adc_Y	        ;

   }
 }


//******************************************************************
//**              Function Calibrate Touch Green                  **
//**  Touch at Point + on Screen	3 Point(3 step)	for Caribrate **
//******************************************************************
//**      Parameter : No                                          **
//**   Return value : No                                          **
//**              												  **
//******************************************************************

void touch_calibrate(void)	
{									
	unsigned char n,touch_mark_point=0;
	

	plot_mark(17,32,24,25,BRIGHT_RED)                                    ; //Plot Signe + for Position Calibrate Touch Point1
	lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ; //Plot Text	Color Bright blue
	lcd_printStr("        STEP 1 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
    lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK)       ; //Plot Text	Color Black
	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK)       ;


	while(touch_mark_point != 3)	//Number Mark Point Calibrate = 3 Point  
	{
		if(status_touch())        	//return (1) if touch screen is detected.
		{		
			for(n=0;n<50;n++)
			 {
				tcs_read()                ;	//Read portrait(ADC X,Y) by touch value from ADS7846 and update dif_adc_X & dif_adc_Y
				buf_adc_X[n] = dif_adc_X  ;	//General buffer for averaging and filter touch screen adc read outs.
				buf_adc_Y[n] = dif_adc_Y  ;	//General buffer for averaging and filter touch screen adc read outs.
			 }
		
			ave_x_y(50)                   ;	//Take average out of 50 touch measurements, excluding max and min measurement.

			tcs_ave_X[touch_mark_point]=  ave_adc_X ;
			tcs_ave_Y[touch_mark_point]=  ave_adc_Y ;
		
			 
			touch_mark_point++            ;	//touch_cal_status;	//0=xy1 cal, 1=xy2 cal, 2=xy3 cal, 3=calibration success. 0xff=calibration not performed.
		
			while(status_touch()){;}        //Waite until not Touch screen ,

			switch(touch_mark_point)		//Select Caribrate Next Step 2 and 3 
			 {
					case 1:
					
						bg_color(BLACK)              ;	// Clear Screen
						plot_mark(208,160,215,153,BRIGHT_RED);	// Plot Signe + for Position Calibrate Touch Point2

                       	lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ; //Plot Text	Color Bright blue
                      	lcd_printStr("        STEP 2 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
                        lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK) ;	   //Plot Text	Color white
                     	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK) ;
 


						break;
					case 2:
					
						bg_color(BLACK)              ; // // Clear Screen
						plot_mark(113,287,120,280,BRIGHT_RED);	// Plot Signe + for Position Calibrate Touch Point3
						
						lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ;	//Plot Text	Color Bright blue
                      	lcd_printStr("        STEP 3 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
                        lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK) ;	   //Plot Text	Color white
                     	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK) ;
                          

						break;
					case 3:
					
						bg_color(BLACK)              ; //Clear Screen

						lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK)  ;	//Plot Text	Color Bright blue
                      	lcd_printStr("          Succeded!         ",8,150,BRIGHT_GREEN,BLACK) ;
                        lcd_printStr("   -Tap Screen for exit-    ",8,190,WHITE,BLACK)        ;

                       
					    while(!status_touch()){;}  
						delay_ms(100);
						bg_color(BLACK)              ; // Clear Screen

						break;
			}
			 
		}
	 delay_ms(100) ;	
	}
																			 
	set_matrix()   ;  // Calculate Coefficient for use find Position Address real on Display
	

	//You can Keep Value data in valiabie devider,An,Bn,Cn,Dn,En,Fn to E2Prom  so that not 
	//come back	 Caribrate.


//  printf("\n\r divider = %d",divider) ;
//	printf("\n\r      An = %d",An) ;
//	printf("\n\r      Bn = %d",Bn) ;
//	printf("\n\r      Cn = %d",Cn) ;
//  printf("\n\r      Dn = %d",Dn) ;
//	printf("\n\r      En = %d",En) ;
//	printf("\n\r      Fn = %d",Fn) ;
  
 
  }




//*********************************************************************************************
//******  Function Calculate average of measurements,by cut value max and min measurement. ****
//*********************************************************************************************
//******      Parameter :					                                          *********
//******			     num = number times of measurement adc from touch screen      *********
//******    Return value:														      *********
//******                 ave_adc_X = Keep result average adc X-Position			      *********
//******                 ave_adc_Y = Keep result average adc Y-Position               *********     
//*********************************************************************************************

void ave_x_y(unsigned char num)				
{
	unsigned char i;
	long sum_x, sum_y;
	long min_x, min_y, max_x, max_y;

	sum_x=0;sum_y=0;min_x=0xffff;min_y=0xffff,max_x=0;max_y=0;

	for(i=0;i<num;i++)
	{
		sum_x += buf_adc_X[i]         ;
		sum_y += buf_adc_Y[i]         ;

		if(buf_adc_X[i]< min_x)		                         //search for minimum X value
		{
			min_x = buf_adc_X[i]      ;
		}
		else if (buf_adc_X[i]> max_x)	                    //search for maximum X value
		{
			max_x = buf_adc_X[i]      ;
		}

		if(buf_adc_Y[i]< min_y)	  	                        //search for minimum Y value
		{
			min_y = buf_adc_Y[i]      ;
		}
		else if(buf_adc_Y[i]> max_y)	  	                //search for maximum Y value
		{
			max_y = buf_adc_Y[i]      ;
		}
	}
	ave_adc_X = (sum_x - min_x - max_x ) / (num-2) ;  	   //update x average value
	ave_adc_Y = (sum_y - min_y - max_y ) / (num-2) ;	   //update y average value
}




//****************************************************************************
//********* Function Calculate Matrix Find value Coeffcient            *******
//****************************************************************************
//*********     Parameter  : No                                        *******
//*********   Return Value : Keep value Coeffcient for use calculate   *******                                      ***********
//*********                  X-Y-Position in variable divider,An,Bn,Cn *******                   
//*********                  Dn,En,Fn                                  *******
//****************************************************************************

 void set_matrix()
  {
	divider =  	((tcs_ave_X[0]-tcs_ave_X[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
	            ((tcs_ave_X[1]-tcs_ave_X[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))      ;


         An = 	((dis_XD[0]-dis_XD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_XD[1]-dis_XD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

	 	 Bn = 	((tcs_ave_X[0]-tcs_ave_X[2])*(dis_XD[1]-dis_XD[2]))-
				((dis_XD[0]-dis_XD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Cn =  	(tcs_ave_X[2]*dis_XD[1]-tcs_ave_X[1]*dis_XD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_XD[2]-tcs_ave_X[2]*dis_XD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_XD[0]-tcs_ave_X[0]*dis_XD[1])*tcs_ave_Y[2]   ;

		 Dn =   ((dis_YD[0]-dis_YD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_YD[1]-dis_YD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

		 En =   ((tcs_ave_X[0]-tcs_ave_X[2])*(dis_YD[1]-dis_YD[2]))-
				((dis_YD[0]-dis_YD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Fn =  	(tcs_ave_X[2]*dis_YD[1]-tcs_ave_X[1]*dis_YD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_YD[2]-tcs_ave_X[2]*dis_YD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_YD[0]-tcs_ave_X[0]*dis_YD[1])*tcs_ave_Y[2]   ;

  }


//****************************************************************************
//*********** Function Get Position Address real of Display ******************
//****************************************************************************
//***         Parameter  :   										      ****
//***                     num = Number times for Measure adc from Touched **** 
//***       Return Value :											      ****
//***                       X = Keep value address X-Position		      ****
//***                       Y = Keep value address Y-Position             **** 
//****************************************************************************

void get_point(unsigned char num)
 {
    unsigned char nm  ; 
	
		  nm=0                              ;
		  while(status_touch() && nm<num)	  //Measure touch x,y 10 times if the Touch kept pressed,
		   {
			  tcs_read()                    ; //Read value ADC Touch X-Y 
			  buf_adc_X[nm]=	dif_adc_X   ; //keep value ADC Touch-X
		      buf_adc_Y[nm]=	dif_adc_Y   ; //keep value ADC Touch-Y

 			  nm++                          ; 
		   }

		  if(nm==num)	                      //if the touch xy successfuly collected,
		   {	
				ave_x_y(num)                ; //take average out of 10 touch measurements, excluding max and min measurement.
			    
				dif_adc_X = ave_adc_X       ; //keep average ADC Touch-X
				dif_adc_Y = ave_adc_Y       ; //keep average ADC Touch-Y
               
			    X = ((An*dif_adc_X)+(Bn*dif_adc_Y)+Cn)/divider  ; 
	            Y = ((Dn*dif_adc_X)+(En*dif_adc_Y)+Fn)/divider  ;
		   }

 }




/* #########*********************************************************#############
   #########**                                                     **############# 
   #########**   3.Function Application of GLCD                    **#############
   #########**                                                     **#############            
   #########*********************************************************#############	*/


//***********************************************************************
//**         Function Set  Background color or Clear Screen            **
//***********************************************************************
//**                 Parameter :							           **
//**                            bg_color =  BackGround color of Screen **
//***********************************************************************

void bg_color(unsigned int bg_color)
 {	
    long  cnt  ;
 
    gp_wr_cmm(0x20)           ;    //Command Horizontal GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000 
    gp_wr_cmm(0x21)           ;    //Command Vertical GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000  
    
	gp_wr_cmm(0x22)                        ;
	
    for(cnt = 0;cnt<((long)240*320);cnt++)
       gp_wr_data(bg_color)                ; // Back-Ground Color Black
   
 }

 

//**********************************************************
//*****             Function Plot Signe+               *****
//**********************************************************
//***** Paramiter:                                     *****
//*****           x1,y1 = Position start Line '-'(Hor) *****
//*****                   plot from Left to Right      *****
//*****           x2,y2 = Position start Line '|'(Ver) *****
//*****                   plot from botton to top      *****
//**********************************************************

void plot_mark(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2,unsigned int color)
 {
    unsigned char i          ;

  	//---- Line- ----
     gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(x1)          ;  //Sent X_address  CGRAM
	 		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(y1)          ;  //Sent Y_address  CGRAM
	 
	 gp_wr_cmm(0x22)         ;  //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;

	//------Line | --------
      gp_wr_cmm(0x03)        ;   //Command Configure Entry Mode
      gp_wr_data(0x1238)     ; 	 //Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.

	 gp_wr_cmm(0x20)         ;   //Command Set Adddress Hor(X)
     gp_wr_data(x2)          ;   //Sent X_Address CGRAM 
	 		
     gp_wr_cmm(0x21)         ;   //Command Set Address Ver(Y)
     gp_wr_data(y2)          ;	 //Sent Y_Address CGRAM 
	 
	 gp_wr_cmm(0x22)         ;   //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;
 
	 gp_wr_cmm(0x03)         ;   //Command Configure Entry Mode
     gp_wr_data(0x1230)      ; 	 //Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.

 	
 }


//*************************************************************
//**                Function Plot Dot 1 Pixel                **
//*************************************************************
//**      Parameter :										 **
//**                 Xadd,Yadd = Position X,Y for Plot Dot   **
//**                     color = Color of dot                **
//*************************************************************

 void plot_dot(unsigned int Xadd,unsigned int Yadd,unsigned int color)
  {
	
	 gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(Xadd)        ;  //Sent X_address for CGRAM 
		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(Yadd)        ;  //Sent Y_address for CGRAM 
	  			
     gp_wr_cmm(0x22)         ;  //Command Write Data RGB

     gp_wr_data(color)       ;  //write data
 }



//********************************************************************************
//**                Function Print Text 1 Charecter size 7x11                   **
//********************************************************************************
//**    Parameter :  															**
//**               row      = Ascii Code (Position buffer keep text)		    **
//**               adx,ady  = Position X,Y for begin plot text by will 			**
//**                          begin plot from bottom left to top left   		**
//**               fg_clr   = Color of text										**
//**               bg_clr   = Color background of text(if bg_clr = no_bg or 1=	**
//**                          non color background)								**
//********************************************************************************


 void text_7x11(unsigned char row,int adx,int ady,unsigned int fg_clr,unsigned int bg_clr)
  {
   int ax,ay                  ;
   unsigned char m,n,tx        ;
     
	 ax = adx         ;
	 ay = ady         ; 
	 row = row-0x20   ;

  //------- Print Text 1 Charecter(data 14 Byte) -------------
	  
	  for(m=0;m<14;m++)
	   {
	    //---Sent data byte1=8 bit----

        tx = pgm_read_byte(&ascii_7x11[row][m])     ;  //Read data Ascii
		 
	 
		for(n=0;n<8;n++)		       //Loop Sent data  1 byte(8bit)
		 {
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ax)        ;  //Sent X_Address CGRAM	 		
              gp_wr_cmm(0x21)       ;  //Command Set Address Ver(Y)
              gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data 
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ax)        ;  //Sent X_Address CGRAM	
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;  //Sent Data
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay   = ay-1              ;  //Decrement Y-address
		 } 

		 m++                        ;  //Increment Next pointter byte Data 


		  //------- Sent data byte2=3 bit -----

		 tx =  pgm_read_byte(&ascii_7x11[row][m])     ;  //Read data byte2
	
		for(n=0;n<3;n++)			   //Loop sent data byte2 = 3 bit
		 {						    ;
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ax)        ;  //Sent X_Address CGRAM		
              gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
              gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ax)        ;  //Sent X_Address CGRAM	 		
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay = ay-1                ;  //Decrement Y-address
		 } 

		ax = ax+1                   ; //Complet sent data 2 byte(11bit) Increment X-Address
		ay = ady                    ; //Set Position Y-address old
 
	  }	


	 //----Fill Back ground Color Position space between Charecter 1 Colum -------
 
     if(bg_clr != 1)     
     {
	  for(n=0;n<11;n++)
	  {
	    gp_wr_cmm(0x20)            ;  //Command Set Adddress Hor(X)
        gp_wr_data(ax)             ;  //Sent X_Address CGRAM	 		
        gp_wr_cmm(0x21)            ;  //Command Set Adddress Ver(Y)
        gp_wr_data(ay)             ;  //Sent Y_Address CGRAM
	    gp_wr_cmm(0x22)            ;  //Command Write data
        gp_wr_data(bg_clr)         ;
		ay = ay-1                  ;  //Incre ment Y-Address
	  }
     }
  }


 //************************************************************************
 //**                         Function Print String                      **
 //************************************************************************
 //**      Parameter :													 **
 //**                 *str = Charecter ASCII (String)					 **
 //**          cur_x,cur_y = Position X,Y for begin plot text by will 	 **
 //**                        begin plot from bottom left to top left     **
 //**             fg_color = color of Text								 **
 //**             bg_color = color Background of text                    **											
 //************************************************************************

void lcd_printStr(char *str,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)
{
  unsigned char i;

	for (i=0; str[i] != '\0'; i++)
	{					 
	 text_7x11(str[i],cur_x,cur_y,fg_color,bg_color);
	 cur_x += 8 ; 
		
	}
 }
 
//*********************************************************
//************ Function Print Text 3 Charecter ************
//*********************************************************

void lcd_print3Cha(char ch1,char ch2,char ch3,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)
{	 
 			 
  text_7x11(ch1,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ; 
  text_7x11(ch2,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ; 
  text_7x11(ch3,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ;  	
 }
 










