

/*******************************************************************************************
 *                                                                                         *
 * Examples3 : GLCD_Touch Screen(ET-TFT240320TP-3.2 Rev.B) For "ET-AVR STAMP MEGA128"      *
 *                                                                                         *
 *                              DISPLAY PORTRAIT(Ver.)                                     *
 *                                                                                         *
 *      MODE : Interface GLCD Parallel data 8 bit                                          *
 *                                                                                         *
 *******************************************************************************************
 *                                                                                         *
 * Target MCU     : ATMEL AVR-MEGA128                                                      *
 *       	      : X-TAL : 16 MHz                                                         * 
 *                                                                                         *
 * Editor         : AVR Studio 4.15 Build 623                                              *
 * Compiler       : WinAVR-20081205                                                        *
 * Create By      : Mr. Sittiphol Yooyod (WWW.ETT.CO.TH)                                   *
 * Last Update    : 24/August/2009                                                         *
 *																				           *
 * Port Interface :  -- MCU-Mega128--	  	  	       --LCD-ET-TFT240320TP-3.2 Rev.B--    *
 *                                                                                         *
 *                      PF[0..7]          Connect          DB8..DB15                       *
 *                      				                                                   *
 *     Contrl GLCD:     PE2               Connect          CS-H						       *
 *                      PE3               Connect          RS-H(0=cmm,1=data)              *
 *                      PE4               Connect          WR-H(Write)                     *
 *                      PE5               Connect          RD-H(Read)                      *
 *                      PE6               Connect          RES-H(Reset)                    *
 *     Back Light :     PE7               Connect          BL-H(LCD-Back Light(Active=1))  *
 *                                                                                         *
 *																					       *
 *   																				       *
 *     Touch Sceen:    PD2 (MISO-in)	  Connect 		    MISO					       *
 *                     PD3 (MOSI-out)	  Connect 		    MOSI					       *
 *                     PD4 (SCLK-out)     Connect 		    SCLK					       *
 *                     PD5 (BUSY-in)	  Connect 		    BUSY(No_USE)				   *
 *                     PD6 (PEN-in)	      Connect 		    PEN					           *
 *                     PD7 (TC_CS-out)	  Connect 		    TC_CS					       *
 *                                                                                         *
 *                                                                                         *
 *    UART0(BaudRate=9600)   :    PE0 =  RXD0                                              *
 *                                PE1 =  TXD0                                              *
 *                                                                                         *
 *******************************************************************************************/




#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>                  //define port
#include <compat/deprecated.h>       //call function sbi(port,bit),cbi(port,bit) = Set,Clear bit
#include <avr/pgmspace.h>            //for progrm data to Area Flash

#define F_CPU 16000000UL             //CPU Clock Frequency (Hz)
#include <util/delay.h>              //call function delay


//----------------------- Picture Engine 1 ------------------

prog_char engine1[8030] = {
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XE1,0X61,0X80,0X51,
0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE1,0X61,0X61,0X7A,0XC2,0X92,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE1,0X61,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X21,0X59,0XE0,0X30,0XE0,0X30,
0X80,0X51,0X80,0X51,0XE1,0X61,0XA6,0XA3,0X61,0X7A,0XC2,0X92,0X61,0X7A,0XE1,0X61,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X51,0X80,0X51,0X61,0X7A,0XA6,0XA3,0XC2,0X92,
0XA6,0XA3,0X61,0X7A,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0X80,0X51,
0X80,0X51,0X61,0X7A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,
0X61,0X7A,0X61,0X7A,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X80,0X18,
0X21,0X40,0X80,0X18,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X0B,0X9C,0XA6,0XA3,0XE1,0X61,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XA6,0XA3,
0X0B,0X9C,0X0B,0X9C,0XC2,0X92,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,
0XE1,0X61,0X80,0X51,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0X92,0X41,0XAB,0X21,0X59,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0X21,0X59,0X21,0X40,0X21,0X40,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X59,
0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X41,0XAB,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,
0X61,0X7A,0X80,0X51,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0XC2,0X92,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X80,0X18,0X21,0X59,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X61,0X7A,0XE1,0X61,0XC2,0X92,0X0B,0X9C,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,
0X41,0XAB,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X80,0X51,0X61,0X7A,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X61,0X7A,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,
0XC2,0X92,0X61,0X7A,0XE1,0X61,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X80,0X51,0X61,0X7A,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XC2,0X92,0XC2,0X92,0X61,0X7A,0XE1,0X61,0XC2,0X92,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0X41,0XAB,0XC9,0XD4,0XC2,0XC3,
0X41,0XAB,0X61,0X7A,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0XC2,0X92,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0X92,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X41,0XAB,0XC9,0XD4,
0XC2,0XC3,0XC9,0XD4,0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,0XC2,0X92,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XC2,0X92,
0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0X61,0X7A,0X21,0X59,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,
0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X18,0X80,0X18,0X80,0X51,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,
0XC2,0X92,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XA6,0XA3,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0X61,0X7A,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,
0XC2,0X92,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0X80,0X18,
0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XA6,0XA3,0X21,0X59,
0X21,0X40,0X21,0X40,0XE1,0X61,0XC9,0XD4,0XA6,0XA3,0X21,0X40,0X21,0X40,0X21,0X40,
0XA6,0XA3,0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0XE1,0X61,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,
0XE1,0X61,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X21,0X40,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC9,0XD4,
0X41,0XAB,0X21,0X40,0X21,0X40,0X61,0X7A,0XC9,0XD4,0X41,0XAB,0X21,0X40,0X21,0X40,
0XE1,0X61,0XC9,0XD4,0XA6,0XA3,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0X41,0XAB,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XA6,0XA3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X51,
0XE0,0X30,0XE0,0X30,0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC2,0XC3,0XC2,0XC3,0XE1,0X61,0X21,0X40,0XA6,0XA3,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,
0X21,0X40,0XC9,0XD4,0XC9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X21,0X40,0X80,0X18,0X21,0X59,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0XA6,0XA3,0XC9,0XD4,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,
0XC9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X59,0XC9,0XD4,0XC9,0XD4,0X21,0X40,0X21,0X40,
0X21,0X59,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,
0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0X61,0X7A,0X21,0X40,0X21,0X40,0XA6,0XA3,
0XB1,0XD5,0XC2,0X92,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XA6,0XA3,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0XC9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE1,0X61,
0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XE1,0X61,0X41,0XAB,0XC9,0XD4,0XA6,0XA3,
0X61,0X7A,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC2,0XC3,0XC9,0XD4,0XA6,0XA3,0XC2,0X92,
0XB1,0XD5,0X41,0XAB,0X21,0X59,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X61,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0XA6,0XA3,0XC9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0XC2,0X92,0XC9,0XD4,0X41,0XAB,0XC2,0X92,
0X41,0XAB,0XE1,0X61,0XE0,0X30,0X80,0X51,0XC2,0X92,0X21,0X59,0XE1,0X61,0X41,0XAB,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XA6,0XA3,0XE1,0X61,0X21,0X40,0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X59,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC9,0XD4,0XA6,0XA3,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XE1,0X61,0X80,0X51,0XC2,0X92,0XC9,0XD4,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0X92,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X59,
0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,
0XC9,0XD4,0XA6,0XA3,0X41,0XAB,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,
0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0XC2,0X92,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X41,0XAB,0XC2,0X92,0X41,0XAB,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XC2,0X92,0X41,0XAB,0XC2,0X92,0X80,0X51,0X61,0X7A,0XC9,0XD4,0XC9,0XD4,
0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC2,0X92,0XC9,0XD4,0X41,0XAB,0XE1,0X61,
0XC2,0X92,0XC2,0XC3,0XC9,0XD4,0X21,0X59,0XA6,0XA3,0XB1,0XD5,0XC9,0XD4,0X61,0X7A,
0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0X92,0XE1,0X61,
0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X51,0X80,0X51,0XE1,0X61,0X61,0X7A,0XC2,0XC3,
0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0X92,0X61,0X7A,0XA6,0XA3,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XC2,0X92,0XC9,0XD4,0X41,0XAB,0X80,0X51,0X80,0X51,0X80,0X51,0X61,0X7A,
0XC9,0XD4,0XC9,0XD4,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0X92,0XC2,0X92,0X61,0X7A,0X61,0X7A,0XA6,0XA3,0XC9,0XD4,0XA6,0XA3,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC9,0XD4,0XC9,0XD4,0X61,0X7A,0X61,0X7A,0XC9,0XD4,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XA6,0XA3,0XE1,0X61,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XC2,0X92,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0X0B,0X9C,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X61,0X7A,0XC9,0XD4,0XC2,0X92,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X0B,0X9C,0XB1,0XD5,0XC9,0XD4,0XA6,0XA3,0XA6,0XA3,0X41,0XAB,
0XC2,0X92,0X41,0XAB,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X80,0X51,0X80,0X51,0XC9,0XD4,0XA6,0XA3,0XE1,0X61,
0XC2,0X92,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC2,0X92,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X51,0XC2,0X92,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0XE1,0X61,0X21,0X59,0X21,0X59,0X21,0X59,0XE1,0X61,
0X61,0X7A,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0X0B,0X9C,0XC9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0X0B,0X9C,0XC2,0X92,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X41,0XAB,0XC2,0X92,0XE1,0X61,0X21,0X40,0X21,0X59,0X80,0X51,0X21,0X59,
0X21,0X59,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X41,0XAB,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XC2,0X92,0XC9,0XD4,0XC9,0XD4,
0X41,0XAB,0XC2,0X92,0XC2,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X41,0XAB,0XC2,0XC3,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X80,0X51,0XC2,0X92,0XC9,0XD4,0X21,0X59,0X21,0X59,0XA6,0XA3,0X0B,0X9C,
0XA6,0XA3,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XA6,0XA3,0XE1,0X61,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0XC9,0XD4,0XC2,0X92,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X61,0X7A,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XC9,0XD4,0XA6,0XA3,0XE1,0X61,0XA6,0XA3,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0X61,0X7A,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X41,0XAB,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XC2,0X92,0X41,0XAB,0XE1,0X61,0X21,0X59,0X21,0X40,0X61,0X7A,0X41,0XAB,
0XE1,0X61,0X80,0X51,0XE0,0X30,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,
0XC2,0X92,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X59,0X61,0X7A,0XA6,0XA3,0XE1,0X61,
0X21,0X59,0XE1,0X61,0X61,0X7A,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XE1,0X61,
0X80,0X18,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XC2,0X92,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X21,0X40,0X41,0XAB,
0XC2,0XC3,0XC2,0X92,0XE0,0X30,0X21,0X59,0XC2,0X92,0X41,0XAB,0X61,0X7A,0X80,0X51,
0XE1,0X61,0XC2,0X92,0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X59,0XE1,0X61,
0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0X41,0XAB,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XC2,0X92,0XC2,0XC3,0X61,0X7A,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,0XC2,0X92,0X61,0X7A,0X41,0XAB,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X61,0X7A,0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XB1,0XD5,0XC2,0X92,
0X41,0XAB,0X41,0XAB,0X61,0X7A,0XA6,0XA3,0XE1,0X61,0XC2,0X92,0XC2,0XC3,0X61,0X7A,
0X21,0X40,0X21,0X40,0X61,0X7A,0XC2,0XC3,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,
0XC2,0X92,0X80,0X51,0XE0,0X30,0X80,0X51,0XE0,0X30,0XE1,0X61,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X61,0X7A,0XC2,0XC3,0X61,0X7A,0XC2,0X92,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XC2,0X92,0XB1,0XD5,0XE1,0X61,0X21,0X40,0X21,0X40,0X61,0X7A,0XC9,0XD4,
0XA6,0XA3,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC9,0XD4,0XA6,0XA3,0X61,0X7A,0XC2,0XC3,
0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC2,0X92,0X0B,0X9C,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0X61,0X7A,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XC2,0X92,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,0XC2,0XC3,0XC2,0X92,
0XC2,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0XC9,0XD4,0XC9,0XD4,0X21,0X40,0X21,0X59,0XC2,0X92,
0XC9,0XD4,0XA6,0XA3,0X21,0X40,0XE1,0X61,0X41,0XAB,0XC9,0XD4,0XC2,0X92,0X21,0X59,
0X61,0X7A,0XE1,0X61,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0XC9,0XD4,0XC9,0XD4,0X0B,0X9C,0XA6,0XA3,0XC9,0XD4,
0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XE0,0X30,0X80,0X18,0X80,0X18,0XC2,0X92,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0XC3,
0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X40,0X61,0X7A,0XC9,0XD4,
0XA6,0XA3,0X21,0X40,0X21,0X40,0XE1,0X61,0XA6,0XA3,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,
0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0X21,0X59,
0X21,0X40,0X21,0X40,0X41,0XAB,0XC9,0XD4,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X80,0X51,0X80,0X51,0X61,0X7A,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0XC2,0XC3,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,
0XA6,0XA3,0XB1,0XD5,0XA6,0XA3,0X61,0X7A,0XC2,0XC3,0X41,0XAB,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,
0XC9,0XD4,0X61,0X7A,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0XC9,0XD4,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X21,0X59,0X80,0X18,0X21,0X40,
0X21,0X40,0XE1,0X61,0XA6,0XA3,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,
0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XA6,0XA3,0XC2,0XC3,0XE1,0X61,0XC2,0X92,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,0X21,0X40,
0X21,0X40,0X21,0X40,0X61,0X7A,0XC2,0X92,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XE1,0X61,0X21,0X40,
0X21,0X40,0X61,0X7A,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X61,0X7A,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X41,0XAB,
0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0X92,0XC9,0XD4,0XC2,0X92,0X80,0X51,0X61,0X7A,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X41,0XAB,
0X0B,0X9C,0X0B,0X9C,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,
0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,
0X61,0X7A,0XA6,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0X61,0X7A,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0XC9,0XD4,0X0B,0X9C,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XA6,0XA3,0XC9,0XD4,0XE1,0X61,0X80,0X51,0X61,0X7A,0XE1,0X61,
0X80,0X51,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC2,0X92,
0X61,0X7A,0X61,0X7A,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0X92,0X80,0X51,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0XE0,0X30,0X80,0X51,0XE1,0X61,0XA6,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0X61,0X7A,0X21,0X59,0X21,0X59,0X61,0X7A,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XB1,0XD5,0XC2,0X92,0XE1,0X61,0X80,0X51,0X61,0X7A,
0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0X80,0X51,0X61,0X7A,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X51,0X41,0XAB,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XB1,0XD5,0X41,0XAB,0XE1,0X61,0XC2,0X92,0XE1,0X61,
0X61,0X7A,0XE1,0X61,0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0X61,0X7A,0X21,0X59,0XE1,0X61,0X41,0XAB,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XE1,0X61,0XE0,0X30,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,
0XC2,0X92,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE1,0X61,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,
0XB1,0XD5,0XA6,0XA3,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC9,0XD4,0XC9,0XD4,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XA6,0XA3,0XB1,0XD5,0XA6,0XA3,0X80,0X51,0X80,0X51,0XE1,0X61,
0XC2,0X92,0X41,0XAB,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0XC2,0X92,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0X61,0X7A,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0XA6,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0X61,0X7A,
0XA6,0XA3,0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,
0XB1,0XD5,0XA6,0XA3,0X80,0X51,0X61,0X7A,0XC2,0X92,0X61,0X7A,0XC2,0X92,0XC9,0XD4,
0XC9,0XD4,0XA6,0XA3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,0XA6,0XA3,0X80,0X51,0XE1,0X61,0XE1,0X61,
0X80,0X51,0X80,0X51,0XC2,0X92,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XA6,0XA3,0XA6,0XA3,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0XE0,0X30,0X61,0X7A,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XA6,0XA3,0XC2,0X92,0X80,0X51,0X21,0X59,0X61,0X7A,0X61,0X7A,0X41,0XAB,0X61,0X7A,
0X80,0X51,0XC2,0X92,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XA6,0XA3,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE0,0X30,0X61,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0XA6,0XA3,0XB1,0XD5,0XB1,0XD5,
0XC9,0XD4,0X41,0XAB,0XC2,0X92,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X59,0XC2,0XC3,0XC2,0XC3,
0X80,0X51,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XA6,0XA3,0XA6,0XA3,0XC9,0XD4,
0X0B,0X9C,0XA6,0XA3,0X61,0X7A,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,
0X41,0XAB,0X61,0X7A,0X21,0X40,0X21,0X40,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,0XC2,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0X61,0X7A,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC2,0X92,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XA6,0XA3,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC9,0XD4,
0X61,0X7A,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,
0XE1,0X61,0XC2,0XC3,0XC2,0XC3,0X21,0X59,0X80,0X18,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0XC3,0XA6,0XA3,0X61,0X7A,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X61,0X7A,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XC9,0XD4,
0XA6,0XA3,0X80,0X51,0X21,0X59,0XE1,0X61,0XC2,0X92,0XC2,0X92,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,
0X21,0X59,0XE1,0X61,0XC2,0X92,0XC2,0XC3,0XC2,0X92,0X21,0X40,0X21,0X59,0XE1,0X61,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0XC2,0X92,0X61,0X7A,0XC2,0X92,0XC9,0XD4,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0XE1,0X61,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE1,0X61,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0X0B,0X9C,0XC2,0X92,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0XC2,0X92,0XE1,0X61,0X21,0X59,0X21,0X40,0X21,0X59,0XC9,0XD4,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X41,0XAB,0XE1,0X61,0X80,0X51,0X80,0X51,0XC2,0X92,0XC2,0XC3,
0XC2,0X92,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X21,0X40,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0XA6,0XA3,0XC2,0X92,0X61,0X7A,0X61,0X7A,0XC9,0XD4,0XA6,0XA3,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X18,0X80,0X18,
0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE1,0X61,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,0XC2,0XC3,0X61,0X7A,
0X21,0X59,0X21,0X40,0XC2,0X92,0XC2,0XC3,0XC2,0X92,0X21,0X59,0X21,0X40,0X61,0X7A,
0XC2,0XC3,0X41,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0X80,0X18,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0XA6,0XA3,0X0B,0X9C,0XA6,0XA3,0XE1,0X61,0X80,0X51,0XA6,0XA3,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X80,0X18,
0X80,0X18,0X80,0X51,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE1,0X61,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X51,
0X80,0X51,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,0XE1,0X61,0XC2,0X92,
0X80,0X51,0X21,0X40,0X80,0X18,0X21,0X59,0X41,0XAB,0X61,0X7A,0X21,0X40,0X21,0X40,
0X21,0X40,0X61,0X7A,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,
0X21,0X40,0XA6,0XA3,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XC9,0XD4,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,
0XA6,0XA3,0XC9,0XD4,0XA6,0XA3,0XA6,0XA3,0XC2,0XC3,0XC9,0XD4,0XA6,0XA3,0XC9,0XD4,
0XC2,0XC3,0XA6,0XA3,0XA6,0XA3,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XA6,0XA3,0XA6,0XA3,
0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,
0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,
0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,
0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,0XA6,0XA3,
0XA6,0XA3,0XA6,0XA3,0X0B,0X9C,0XA6,0XA3,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0XE1,0X61,0XE1,0X61,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0XE1,0X61,0X80,0X51,0X80,0X51,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0XE0,0X30,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18};



//----------------------- Picture Engine 2 ------------------

prog_char engine2[8030] = {
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,
0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XE1,0X61,0X80,0X51,
0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,
0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,
0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,
0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,
0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE1,0X61,0X61,0X7A,0XC2,0X92,
0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X61,0X7A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE1,0X61,
0XC2,0X92,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X21,0X40,0XE0,0X30,
0X21,0X40,0X80,0X51,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XE1,0X61,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X51,0X80,0X51,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XA6,0XAB,0XC2,0X92,0X61,0X7A,0XE1,0X61,0X80,0X51,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE1,0X61,0X80,0X51,
0X80,0X51,0X61,0X7A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,
0X61,0X7A,0X61,0X7A,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,
0X21,0X59,0X21,0X40,0XE1,0X61,0XA6,0XAB,0XC2,0X92,0XC2,0X92,0XA6,0XAB,0XE1,0X61,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XA6,0XAB,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X61,0X7A,0X80,0X51,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,
0XE1,0X61,0X80,0X51,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0X92,0X41,0XAB,0X21,0X59,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0XA6,0XAB,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,0XC2,0X92,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0XA6,0XAB,0X61,0X7A,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0X80,0X51,0XE1,0X61,0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,
0X61,0X7A,0X80,0X51,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XA6,0XAB,0X21,0X40,
0X21,0X40,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X61,0X7A,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X80,0X51,0X61,0X7A,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X61,0X7A,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,
0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0X92,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X61,0X7A,0XE1,0X61,0XC2,0X92,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,
0XE9,0XD4,0XA6,0XAB,0XA6,0XAB,0XC2,0X92,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0X61,0X7A,0XC2,0X92,0XA6,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0XC2,0X92,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0X92,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC2,0X92,0XE9,0XD4,
0XC2,0XC3,0X41,0XAB,0X61,0X7A,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XC2,0X92,
0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,0XC2,0XC3,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X59,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X80,0X51,0X80,0X51,0X80,0X51,
0X61,0X7A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0X80,0X18,0X80,0X51,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0X61,0X7A,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC2,0X92,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X61,0X7A,0X41,0XAB,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0X80,0X18,
0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XA6,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0XA6,0XAB,0XE9,0XD4,0XE1,0X61,0X21,0X40,0X21,0X40,0XE1,0X61,0XE9,0XD4,
0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XE1,0X61,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,
0XE1,0X61,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0XA6,0XAB,0XE9,0XD4,0X21,0X59,
0X21,0X40,0X21,0X40,0X41,0XAB,0XC2,0XC3,0XC2,0X92,0X21,0X40,0X21,0X40,0XA6,0XAB,
0XE9,0XD4,0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0XC2,0X92,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XA6,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X51,
0XE0,0X30,0XE0,0X30,0X80,0X51,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X61,0X7A,
0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC2,0XC3,
0XC2,0XC3,0X21,0X40,0X21,0X40,0XA6,0XAB,0XC2,0XC3,0XC2,0XC3,0X21,0X59,0X21,0X59,
0XE9,0XD4,0XC2,0XC3,0XE1,0X61,0X21,0X40,0X21,0X40,0X61,0X7A,0XE9,0XD4,0X61,0X7A,
0X21,0X40,0X21,0X40,0X21,0X59,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0XE9,0XD4,0XE9,0XD4,0X0B,0X9C,0XC2,0X92,0XE1,0X61,0XE1,0X61,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XE1,0X61,0X21,0X40,0X21,0X40,0X61,0X7A,0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XC2,0X92,0X21,0X59,0X21,0X59,0XB1,0XD5,0XA6,0XAB,
0XE1,0X61,0X21,0X40,0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0X61,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XC2,0X92,0XE9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE0,0X30,0X80,0X51,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,
0XE9,0XD4,0XA6,0XAB,0X21,0X40,0X21,0X59,0XE9,0XD4,0XE9,0XD4,0X21,0X59,0X21,0X40,
0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,
0X61,0X7A,0X21,0X59,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XA6,0XAB,0XE9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0XC2,0X92,0XC2,0XC3,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0XA6,0XAB,0XB1,0XD5,0XB1,0XD5,0XC2,0X92,0XC2,0X92,0XC2,0XC3,0XE9,0XD4,
0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,
0X41,0XAB,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0X92,0X41,0XAB,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0XC2,0XC3,0X41,0XAB,0X61,0X7A,0XE1,0X61,0XE9,0XD4,0XA6,0XAB,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XE1,0X61,0X80,0X51,0X61,0X7A,0XE9,0XD4,0XC2,0XC3,0XC2,0X92,
0XC2,0X92,0X61,0X7A,0X80,0X51,0X80,0X51,0XC2,0X92,0XC2,0X92,0X21,0X59,0X41,0XAB,
0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,
0XE9,0XD4,0XC2,0XC3,0XE9,0XD4,0XC2,0X92,0X61,0X7A,0X0B,0X9C,0XA6,0XAB,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0X92,0X41,0XAB,0XC2,0X92,0X41,0XAB,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,
0XE9,0XD4,0XE9,0XD4,0XE1,0X61,0X80,0X51,0XE1,0X61,0X61,0X7A,0XE9,0XD4,0XE9,0XD4,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC2,0X92,0XE9,0XD4,0X41,0XAB,0XC2,0X92,
0X41,0XAB,0XC2,0X92,0X61,0X7A,0X21,0X59,0XC2,0X92,0X21,0X59,0X21,0X59,0X21,0X59,
0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XC2,0X92,0XE1,0X61,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0XC2,0X92,0XA6,0XAB,
0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC2,0X92,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0XA6,0XAB,0XA6,0XAB,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XC2,0X92,
0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0X92,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XA6,0XAB,0XE9,0XD4,0X41,0XAB,0XE1,0X61,
0XE1,0X61,0X41,0XAB,0XE9,0XD4,0X61,0X7A,0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XC2,0X92,
0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XA6,0XAB,0XE1,0X61,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XA6,0XAB,0X80,0X51,
0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0X61,0X7A,0X80,0X51,0X21,0X59,0X21,0X59,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X80,0X51,0X80,0X51,0XA6,0XAB,0XB1,0XD5,0XE9,0XD4,0XA6,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0X92,0XC2,0X92,0XA6,0XAB,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XC2,0X92,0XE1,0X61,
0X61,0X7A,0XE1,0X61,0X80,0X51,0XA6,0XAB,0XE9,0XD4,0XC2,0X92,0X61,0X7A,0XA6,0XAB,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0X61,0X7A,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X51,0X61,0X7A,0XC2,0XC3,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,
0XC2,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XC2,0XC3,
0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X61,0X7A,0X0B,0X9C,0XE9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0X0B,0X9C,0XC2,0X92,0X80,0X51,0XE1,0X61,
0XC2,0X92,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X80,0X51,0XA6,0XAB,0XE9,0XD4,0XE1,0X61,
0X61,0X7A,0XC2,0XC3,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XA6,0XAB,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XC2,0X92,0XC2,0XC3,0XE9,0XD4,
0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XE9,0XD4,
0X41,0XAB,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,
0X80,0X51,0X61,0X7A,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X59,0X80,0X51,0XE1,0X61,
0X21,0X40,0XA6,0XAB,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XA6,0XAB,0XE1,0X61,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XA6,0XAB,0X41,0XAB,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,
0X41,0XAB,0X61,0X7A,0X21,0X59,0X80,0X51,0XE1,0X61,0X61,0X7A,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X80,0X51,0X61,0X7A,0XE9,0XD4,0X61,0X7A,0X21,0X40,0X21,0X59,0XA6,0XAB,
0XC2,0X92,0X61,0X7A,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0X61,0X7A,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X61,0X7A,0XC2,0XC3,0XE9,0XD4,0XC2,0X92,0XC2,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,0X41,0XAB,0X21,0X59,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0XE1,0X61,0X21,0X59,0X21,0X40,0X21,0X40,0X41,0XAB,0XC2,0XC3,0XE1,0X61,
0X80,0X51,0XE0,0X30,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0XE0,0X30,0XC2,0X92,
0XC2,0X92,0XE1,0X61,0X21,0X59,0XE1,0X61,0XA6,0XAB,0XA6,0XAB,0X21,0X40,0XC2,0X92,
0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0XE1,0X61,
0X80,0X18,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XC2,0XC3,0X41,0XAB,0X61,0X7A,0XC2,0X92,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,0X21,0X59,0X21,0X59,0XC2,0XC3,0X41,0XAB,
0X61,0X7A,0X21,0X59,0XE1,0X61,0XC2,0X92,0X41,0XAB,0XE1,0X61,0X80,0X51,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X59,0X61,0X7A,0X41,0XAB,0X21,0X59,
0X21,0X59,0XE1,0X61,0XE1,0X61,0XC2,0X92,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0X41,0XAB,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XC2,0X92,0XC2,0XC3,0XC2,0X92,0XC2,0X92,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,0XC2,0X92,0X61,0X7A,0XC2,0X92,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0XA6,0XAB,0XE9,0XD4,0X61,0X7A,0X41,0XAB,
0XC2,0X92,0XC2,0X92,0X0B,0X9C,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,
0X21,0X59,0XC2,0X92,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0X92,0XE0,0X30,0XE0,0X30,0X80,0X51,0XE0,0X30,0XE1,0X61,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X61,0X7A,0XC2,0XC3,0XC2,0X92,0XC2,0X92,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X61,0X7A,
0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0XA6,0XAB,0XE9,0XD4,0X21,0X40,0X21,0X40,0X21,0X59,0XC2,0X92,0XE9,0XD4,0XE1,0X61,
0X21,0X59,0XE1,0X61,0X41,0XAB,0XE9,0XD4,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X21,0X59,
0X21,0X40,0X21,0X40,0X61,0X7A,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,
0XC2,0XC3,0XC2,0X92,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XC2,0X92,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,0XC2,0XC3,0X61,0X7A,
0XC2,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0XC2,0X92,0XB1,0XD5,0X61,0X7A,0X21,0X40,0X21,0X59,0XC2,0X92,0XE9,0XD4,
0XA6,0XAB,0X21,0X59,0XE1,0X61,0XC2,0XC3,0XE9,0XD4,0XE1,0X61,0X21,0X59,0X61,0X7A,
0X41,0XAB,0X0B,0X9C,0X21,0X59,0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XC2,0X92,0X0B,0X9C,0XE9,0XD4,0XE9,0XD4,
0XE9,0XD4,0XC2,0XC3,0XC2,0X92,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XE0,0X30,0X80,0X18,0X80,0X18,0X61,0X7A,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0XC3,
0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0XC2,0X92,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X40,0X61,0X7A,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0XA6,0XAB,0XB1,0XD5,0XA6,0XAB,0XA6,0XAB,0XC2,0XC3,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XE9,0XD4,0X61,0X7A,0X21,0X40,
0X61,0X7A,0XC2,0XC3,0XE9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,
0XA6,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0XC2,0XC3,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X61,0X7A,0XE9,0XD4,
0X0B,0X9C,0X21,0X40,0X21,0X40,0XC2,0X92,0X41,0XAB,0XB1,0XD5,0XE9,0XD4,0XB1,0XD5,
0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X80,0X51,0X80,0X51,
0XE1,0X61,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,0X61,0X7A,
0XC2,0X92,0XE9,0XD4,0X61,0X7A,0XC2,0X92,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,0X21,0X59,
0XC2,0X92,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XE9,0XD4,0XC2,0XC3,0XE9,0XD4,0XB1,0XD5,
0XE9,0XD4,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XE1,0X61,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X59,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XA6,0XAB,0X41,0XAB,0X80,0X51,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X21,0X59,0X21,0X40,
0X21,0X40,0XE1,0X61,0XC2,0X92,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,
0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0XA6,0XAB,0XC2,0XC3,0XE9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XE1,0X61,0XC2,0X92,
0XE9,0XD4,0XE9,0XD4,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,
0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0X92,0XE9,0XD4,0XC2,0X92,0X80,0X51,0X61,0X7A,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC2,0X92,0XE1,0X61,
0X80,0X18,0X21,0X40,0X61,0X7A,0XC2,0X92,0XA6,0XAB,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,
0XC2,0XC3,0X41,0XAB,0XC2,0X92,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0XA6,0XAB,0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC2,0X92,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,0XC2,0X92,0X41,0XAB,
0XC2,0XC3,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XA6,0XAB,0XE9,0XD4,0X61,0X7A,0X80,0X51,0X61,0X7A,0XE1,0X61,
0X80,0X51,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,
0XA6,0XAB,0X0B,0X9C,0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0X92,0X80,0X51,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0XE0,0X30,0X80,0X51,0XE1,0X61,0XA6,0XAB,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC2,0XC3,0X41,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XA6,0XAB,0XB1,0XD5,0XA6,0XAB,0XE1,0X61,0X61,0X7A,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XE9,0XD4,0XC2,0X92,0X61,0X7A,0XE1,0X61,0X61,0X7A,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC2,0X92,0X0B,0X9C,0XB1,0XD5,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X51,0X41,0XAB,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XA6,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0X92,0XE1,0X61,0X80,0X51,0X21,0X59,0X41,0XAB,
0XC2,0X92,0X41,0XAB,0X41,0XAB,0XE9,0XD4,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XB1,0XD5,0XA6,0XAB,0X80,0X51,0X80,0X51,0X61,0X7A,
0XC2,0X92,0X80,0X51,0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0XC2,0X92,0XE1,0X61,0X80,0X51,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XE1,0X61,0XE0,0X30,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,
0XC2,0X92,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XA6,0XAB,0XA6,0XAB,0XE9,0XD4,
0XA6,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X61,0X7A,0X41,0XAB,0XC2,0XC3,0XC2,0X92,0X61,0X7A,0XE9,0XD4,0XE9,0XD4,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0X41,0XAB,0XA6,0XAB,0XB1,0XD5,0XA6,0XAB,0X80,0X51,0XE1,0X61,0XE1,0X61,
0XE0,0X30,0X61,0X7A,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0X61,0X7A,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0XA6,0XAB,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,
0XC2,0XC3,0XA6,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XA6,0XAB,0XE9,0XD4,0XC2,0X92,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0XC3,
0XE9,0XD4,0XA6,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XE9,0XD4,0XB1,0XD5,0XC2,0X92,0XE1,0X61,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0X80,0X51,0X61,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC2,0X92,0XC2,0X92,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0XE0,0X30,0X61,0X7A,0XE9,0XD4,0XB1,0XD5,0XE9,0XD4,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XA6,0XAB,0XB1,0XD5,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0X61,0X7A,0XC2,0X92,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XA6,0XAB,0XE9,0XD4,
0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XA6,0XAB,0XE1,0X61,0X80,0X51,0X80,0X51,0X61,0X7A,
0XC2,0X92,0XE1,0X61,0X21,0X40,0X21,0X59,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X41,0XAB,0XC2,0X92,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0XA6,0XAB,0XB1,0XD5,0XE9,0XD4,
0XB1,0XD5,0XC2,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0XA6,0XAB,0X61,0X7A,0X80,0X51,0X21,0X59,0X61,0X7A,0XC2,0X92,
0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XA6,0XAB,0XA6,0XAB,0X0B,0X9C,
0XE9,0XD4,0XA6,0XAB,0X61,0X7A,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,
0XE1,0X61,0XC2,0XC3,0X41,0XAB,0X80,0X18,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,0XC2,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0X61,0X7A,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC2,0X92,0XE9,0XD4,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X59,
0XE9,0XD4,0XC2,0X92,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,
0XE1,0X61,0XE1,0X61,0XC2,0X92,0XC2,0XC3,0XE1,0X61,0X21,0X59,0XE1,0X61,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,0X61,0X7A,
0XE1,0X61,0X61,0X7A,0XC2,0XC3,0XA6,0XAB,0X61,0X7A,0XC2,0X92,0X61,0X7A,0X61,0X7A,
0XC2,0X92,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X61,0X7A,
0XE9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0X21,0X59,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XA6,0XAB,0XE9,0XD4,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC2,0X92,0X80,0X51,
0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0X41,0XAB,
0XC2,0X92,0X21,0X40,0X21,0X40,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC2,0X92,
0X61,0X7A,0X61,0X7A,0XC2,0X92,0XC2,0XC3,0XA6,0XAB,0XC2,0X92,0XC2,0X92,0XC2,0X92,
0X61,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0XE1,0X61,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE1,0X61,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XE9,0XD4,0XC2,0X92,0XE1,0X61,0X21,0X40,0X21,0X59,0XC2,0XC3,0X41,0XAB,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XC2,0X92,0X61,0X7A,0X80,0X51,0X80,0X51,0XC2,0X92,
0X41,0XAB,0X41,0XAB,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X59,0X21,0X40,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0XA6,0XAB,0XC2,0X92,0XE1,0X61,0XC2,0X92,0XE9,0XD4,0X41,0XAB,0XC2,0X92,0XC2,0X92,
0XC2,0X92,0XC2,0X92,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0X80,0X18,0X80,0X18,
0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0X92,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE1,0X61,0XC2,0XC3,0XE9,0XD4,0XE9,0XD4,0XC2,0XC3,0X61,0X7A,0XE1,0X61,
0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X59,0XE1,0X61,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X59,0XC2,0XC3,
0XC2,0X92,0XE1,0X61,0X21,0X40,0X61,0X7A,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0X21,0X40,
0X21,0X59,0X41,0XAB,0XC2,0XC3,0X21,0X59,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0XA6,0XAB,0X0B,0X9C,0XA6,0XAB,0X61,0X7A,0XE1,0X61,0XA6,0XAB,0XC2,0X92,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0X92,0X80,0X18,
0X80,0X18,0X80,0X51,0X41,0XAB,0XC2,0X92,0X41,0XAB,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE1,0X61,0XA6,0XAB,0XE9,0XD4,0XC2,0XC3,0XA6,0XAB,0X21,0X40,
0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X41,0XAB,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,0XC2,0X92,0X21,0X40,
0X80,0X18,0X21,0X40,0XE1,0X61,0XE1,0X61,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,
0X21,0X40,0XA6,0XAB,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XE9,0XD4,0XA6,0XAB,0XE9,0XD4,0XA6,0XAB,
0XA6,0XAB,0XE9,0XD4,0XA6,0XAB,0XA6,0XAB,0XC2,0XC3,0XA6,0XAB,0XC2,0XC3,0XE9,0XD4,
0XC2,0XC3,0XA6,0XAB,0XA6,0XAB,0XC2,0XC3,0XE9,0XD4,0XC2,0XC3,0XA6,0XAB,0XA6,0XAB,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,0XE9,0XD4,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,
0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,0XA6,0XAB,
0XC2,0X92,0X0B,0X9C,0X0B,0X9C,0XC2,0X92,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,};



//----------------------- Picture Engine 3 ------------------ 
prog_char engine3[8030] = { 
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC1,0X8A,0X41,0X7A,0X41,0X7A,
0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0XE1,0X61,0X80,0X51,
0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE1,0X61,0X41,0X7A,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0X7A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE1,0X61,
0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XE0,0X30,0XE1,0X61,0XC1,0X8A,
0XE1,0X61,0XC1,0X50,0X41,0X7A,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XE1,0X61,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X51,0X80,0X51,0X41,0X7A,0X86,0XA3,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0X80,0X51,0X80,0X51,0X41,0X7A,0X41,0X7A,0XC1,0X50,0XC1,0X50,
0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,
0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,
0XC1,0X50,0XC1,0X50,0XC1,0X50,0XC1,0X50,0XE0,0X30,0XC1,0X50,0X41,0X7A,0X80,0X51,
0X80,0X51,0X41,0X7A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC1,0X8A,
0XC1,0X8A,0X41,0X7A,0X80,0X18,0X80,0X18,0X21,0X40,0X80,0X18,0X21,0X40,0XE1,0X61,
0X86,0XA3,0X0B,0X9C,0X0B,0X9C,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0XE1,0X61,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XE0,0X30,0XC1,0X8A,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X41,0X7A,0XC1,0X50,
0X80,0X18,0X21,0X40,0X80,0X18,0X80,0X18,0X21,0X40,0X80,0X18,0X80,0X18,0X21,0X40,
0X80,0X18,0X80,0X18,0X21,0X40,0X80,0X18,0X80,0X18,0X21,0X40,0X80,0X18,0X80,0X18,
0X21,0X40,0X80,0X18,0X80,0X18,0X21,0X40,0X80,0X18,0X21,0X40,0X80,0X18,0X80,0X18,
0XE1,0X61,0X80,0X51,0X41,0X7A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XC1,0X50,0X80,0X51,0XE1,0X61,0X0B,0X9C,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC1,0X8A,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC1,0X8A,0X80,0X51,0X80,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0XE1,0X61,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC1,0X50,0X41,0X7A,0X41,0XAB,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,
0X41,0X7A,0X80,0X51,0X41,0X7A,0X41,0X7A,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE0,0X30,
0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0XC1,0X8A,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0XC1,0X50,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XE1,0X61,0X41,0X7A,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0X7A,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X41,0X7A,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0XE1,0X61,0X86,0XA3,0X0B,0X9C,0X0B,0X9C,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0X41,0XAB,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,0XE1,0X61,0XC1,0X8A,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0X0B,0X9C,0XC9,0XD4,0XC1,0X8A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X41,0X7A,0XC1,0X8A,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X80,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X41,0X7A,0X86,0XA3,0X0B,0X9C,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC1,0X8A,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X41,0XAB,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X0B,0X9C,0X86,0XA3,0X86,0XA3,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0XC1,0X8A,0X41,0X7A,
0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X80,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0X7A,
0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0XE1,0X61,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XC1,0X8A,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X80,0X51,0X80,0X51,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC1,0X8A,
0X41,0X7A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0X7A,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC1,0X8A,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X80,0X18,0X80,0X18,0X80,0X51,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,
0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X50,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,
0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XE1,0X61,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X80,0X18,0X80,0X18,
0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,
0XC9,0XD4,0XC9,0XD4,0XC1,0X50,0X21,0X40,0X21,0X40,0X86,0XA3,0XC9,0XD4,0X41,0X7A,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X8A,
0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,
0XE1,0X61,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,
0X0B,0X9C,0XE1,0X61,0X21,0X40,0X21,0X40,0X41,0X7A,0X86,0XA3,0X21,0X40,0X21,0X40,
0XC1,0X50,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X21,0X40,0X21,0X40,0XC9,0XD4,0XC2,0XC3,
0XE1,0X61,0X21,0X40,0X21,0X40,0XE1,0X61,0X86,0XA3,0XC1,0X50,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0X7A,
0XC9,0XD4,0X0B,0X9C,0XC1,0X8A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC2,0XC3,0X86,0XA3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X51,
0XE0,0X30,0XE0,0X30,0X80,0X51,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0X41,0X7A,
0X86,0XA3,0X86,0XA3,0XC2,0XC3,0X21,0X40,0XC1,0X50,0XC9,0XD4,0XC9,0XD4,0X41,0X7A,
0X21,0X40,0X21,0X40,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X80,0X51,0XC1,0X8A,0XC9,0XD4,
0XC2,0XC3,0XE1,0X61,0X21,0X40,0XC1,0X50,0XC9,0XD4,0XC9,0XD4,0X41,0X7A,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0XC1,0X50,0X80,0X51,0XC1,0X50,0X80,0X51,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,
0XC9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0XC1,0X8A,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X21,0X40,0X21,0X40,0XC1,0X8A,0XC2,0XC3,
0XC2,0XC3,0X41,0X7A,0X0B,0X9C,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,
0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0X41,0X7A,0X86,0XA3,0XC9,0XD4,0XC1,0X8A,0XC1,0X50,
0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X21,0X40,0X41,0X7A,0XC1,0X8A,0X41,0X7A,
0X41,0XAB,0XC9,0XD4,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0XE1,0X61,
0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,
0X41,0XAB,0XC1,0X50,0X21,0X40,0XC1,0X8A,0XE1,0X61,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0X86,0XA3,0X41,0X7A,
0X21,0X40,0X21,0X40,0XC1,0X50,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,
0X41,0XAB,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X86,0XA3,0XC2,0XC3,
0X41,0XAB,0X41,0X7A,0X86,0XA3,0XC9,0XD4,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0X7A,0X41,0X7A,
0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X86,0XA3,0XC2,0XC3,0XC1,0X8A,0XC2,0XC3,
0XC2,0XC3,0XC9,0XD4,0XE1,0X61,0X80,0X51,0XC9,0XD4,0XC9,0XD4,0XC1,0X8A,0XC1,0X50,
0X0B,0X9C,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC2,0XC3,
0XC9,0XD4,0X41,0X7A,0XC1,0X50,0XC9,0XD4,0X0B,0X9C,0XC1,0X8A,0X41,0X7A,0X41,0X7A,
0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0X41,0XAB,0XE1,0X61,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC9,0XD4,0XC9,0XD4,
0X86,0XA3,0X80,0X51,0X80,0X51,0X41,0X7A,0XC9,0XD4,0X86,0XA3,0X41,0XAB,0XC2,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0X41,0X7A,0XE1,0X61,0X80,0X51,0X41,0X7A,0XC9,0XD4,0X41,0XAB,0XC1,0X8A,
0XE1,0X61,0XC1,0X8A,0XC9,0XD4,0XB1,0XD5,0X86,0XA3,0XC1,0X8A,0XC2,0XC3,0XC9,0XD4,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,
0X86,0XA3,0X86,0XA3,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC1,0X50,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0XC1,0X8A,0X41,0XAB,
0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X86,0XA3,0X86,0XA3,
0X41,0X7A,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XC1,0X8A,0XB1,0XD5,0XC9,0XD4,
0X41,0XAB,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0X41,0X7A,0X41,0X7A,0XE1,0X61,0XC1,0X8A,0XC9,0XD4,0XC2,0XC3,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0XE1,0X61,0XC1,0X50,0X41,0X7A,0XC9,0XD4,0X41,0X7A,0XC1,0X8A,
0XC2,0XC3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC1,0X8A,0XE1,0X61,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0XC1,0X8A,0XC2,0XC3,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0X41,0X7A,0XE0,0X30,0XE1,0X61,
0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0XC1,0X8A,0X41,0XAB,0XC1,0X8A,
0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X80,0X51,0XC1,0X50,0XC1,0X50,0XE1,0X61,0XE1,0X61,0X41,0X7A,0XE1,0X61,0X41,0X7A,
0XC9,0XD4,0XC9,0XD4,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC1,0X8A,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X86,0XA3,0XC9,0XD4,0X86,0XA3,0XE1,0X61,
0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0X80,0X51,0X41,0X7A,0XE1,0X61,0XC1,0X50,0XC1,0X50,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0X86,0XA3,0XE1,0X61,0X80,0X51,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XC1,0X8A,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,
0X80,0X51,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0XC1,0X8A,
0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0X7A,0XC2,0XC3,0XC1,0X8A,0X80,0X51,
0X80,0X51,0X80,0X51,0X0B,0X9C,0XB1,0XD5,0XC9,0XD4,0X86,0XA3,0X41,0XAB,0X41,0XAB,
0XC1,0X8A,0X41,0XAB,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC1,0X8A,0XE1,0X61,
0XE1,0X61,0X41,0X7A,0XC2,0XC3,0X86,0XA3,0X80,0X51,0XC9,0XD4,0XB1,0XD5,0X86,0XA3,
0X86,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC1,0X8A,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0XC1,0X50,0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,
0X86,0XA3,0XC1,0X8A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC9,0XD4,0XC2,0XC3,0XE1,0X61,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0X7A,0X0B,0X9C,0XC9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0X0B,0X9C,0XC1,0X8A,0XE1,0X61,0X41,0X7A,
0XE1,0X61,0X41,0X7A,0X80,0X51,0XC1,0X8A,0XB1,0XD5,0X86,0XA3,0X41,0X7A,0X41,0XAB,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0X41,0XAB,0X80,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XC1,0X8A,0XC2,0XC3,0XC9,0XD4,
0XC2,0XC3,0X86,0XA3,0X86,0XA3,0XE1,0X61,0XE1,0X61,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,0XC2,0XC3,0X41,0X7A,
0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X51,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,
0XC1,0X8A,0X41,0X7A,0XE1,0X61,0XE1,0X61,0X80,0X51,0XC1,0X8A,0XC9,0XD4,0XE1,0X61,
0XE1,0X61,0XC1,0X8A,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XC2,0XC3,0XE1,0X61,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC9,0XD4,0X86,0XA3,0X41,0X7A,0X41,0X7A,0XE1,0X61,0X41,0X7A,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X41,0X7A,
0X80,0X51,0X21,0X40,0XC1,0X50,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0XC1,0X8A,0XC2,0XC3,0XE1,0X61,0X21,0X40,0XC1,0X50,0X80,0X51,0XE1,0X61,
0X21,0X40,0XC1,0X50,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X41,0X7A,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XE1,0X61,0X41,0X7A,
0XE1,0X61,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X41,0X7A,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0XC1,0X8A,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,0X80,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XC1,0X50,0X21,0X40,0X21,0X40,0XC1,0X50,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0X80,0X51,
0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE0,0X30,0X80,0X51,0XC1,0X8A,0XE1,0X61,
0X80,0X51,0X80,0X51,0X41,0X7A,0XC9,0XD4,0XC1,0X8A,0X21,0X40,0X21,0X40,0X80,0X51,
0X41,0X7A,0XC1,0X50,0XC1,0X8A,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XE1,0X61,
0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X41,0X7A,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XC1,0X50,0X41,0XAB,0XC2,0XC3,0XC1,0X8A,0XC1,0X8A,0XE1,0X61,0XE1,0X61,
0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X80,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X50,0X41,0X7A,0XC2,0XC3,0XC1,0X8A,0X41,0X7A,
0XC1,0X50,0X41,0X7A,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X80,0X51,0XE1,0X61,0X41,0XAB,
0XC2,0XC3,0XC1,0X50,0XC1,0X50,0XE1,0X61,0X41,0XAB,0X86,0XA3,0X21,0X40,0X21,0X40,
0X86,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0X41,0XAB,
0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X41,0X7A,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0XE1,0X61,
0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,0X41,0XAB,
0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0XC9,0XD4,0X86,0XA3,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0X86,0XA3,0XC9,0XD4,0X41,0X7A,0XC2,0XC3,0X41,0XAB,0XC1,0X50,0X21,0X40,0X41,0X7A,
0X41,0XAB,0XC2,0XC3,0X21,0X40,0X21,0X40,0XC1,0X50,0X41,0X7A,0X41,0X7A,0XC1,0X50,
0XC1,0X50,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X86,0XA3,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,
0XC1,0X8A,0XC1,0X50,0XE0,0X30,0X80,0X51,0XE0,0X30,0XE1,0X61,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0X7A,0XC2,0XC3,0XC1,0X8A,0XC1,0X8A,
0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X8A,
0XE1,0X61,0X21,0X40,0X21,0X40,0X80,0X51,0X86,0XA3,0XC9,0XD4,0XC1,0X50,0X21,0X40,
0XE1,0X61,0XC2,0XC3,0XC9,0XD4,0XC1,0X8A,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,
0XE1,0X61,0XC1,0X8A,0X41,0XAB,0XC1,0X50,0X21,0X40,0X21,0X40,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X80,0X51,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC1,0X8A,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XC1,0X8A,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,0XC2,0XC3,0XE1,0X61,
0XC1,0X8A,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,
0X41,0X7A,0XC1,0X8A,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0XC9,0XD4,0XB1,0XD5,0XC1,0X50,0X21,0X40,0XC1,0X50,0X41,0XAB,0XB1,0XD5,0XC1,0X8A,
0XC1,0X50,0XC1,0X8A,0XC2,0XC3,0XC9,0XD4,0X80,0X51,0XC1,0X50,0XC1,0X8A,0XC2,0XC3,
0XC9,0XD4,0XC1,0X50,0XE1,0X61,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC2,0XC3,0XC9,0XD4,
0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XE0,0X30,0X80,0X18,0X80,0X18,0XC1,0X8A,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XE1,0X61,0X41,0X7A,0XC2,0XC3,
0X41,0X7A,0XC1,0X8A,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XC1,0X50,0X41,0X7A,0XB1,0XD5,0X86,0XA3,0X41,0X7A,0X86,0XA3,0X41,0XAB,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0X86,0XA3,0X41,0X7A,0X41,0XAB,
0XC2,0XC3,0X86,0XA3,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X8A,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X80,0X18,0XE0,0X30,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,
0XC2,0XC3,0XC1,0X8A,0XC1,0X8A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X80,0X51,0X21,0X40,0X41,0X7A,0XC1,0X50,
0X21,0X40,0X21,0X40,0XE1,0X61,0X86,0XA3,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XC2,0XC3,0XC9,0XD4,0X41,0X7A,0X21,0X40,0X21,0X40,0X80,0X51,0XC1,0X8A,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X86,0XA3,0XB1,0XD5,0XB1,0XD5,
0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0XC9,0XD4,0XE1,0X61,0X41,0X7A,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X80,0X51,0XE1,0X61,0XC9,0XD4,
0XB1,0XD5,0XC1,0X50,0X80,0X51,0XC2,0XC3,0X41,0XAB,0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XB1,0XD5,0X0B,0X9C,0XC1,0X50,0X41,0X7A,0XC9,0XD4,0XC9,0XD4,
0X41,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,0XC1,0X8A,
0XE1,0X61,0XE1,0X61,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0XC1,0X8A,
0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,
0XC1,0X8A,0X86,0XA3,0X41,0XAB,0X80,0X51,0X41,0X7A,0XE1,0X61,0X80,0X51,0XE1,0X61,
0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X80,0X51,0XC1,0X50,
0X41,0X7A,0XC9,0XD4,0XB1,0XD5,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC2,0XC3,0XC2,0XC3,
0X41,0XAB,0XC1,0X50,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,
0XC1,0X50,0XC1,0X50,0X41,0X7A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,
0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC1,0X8A,0XC9,0XD4,0XC1,0X8A,0XE1,0X61,0X41,0X7A,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X80,0X51,
0X21,0X40,0X80,0X51,0X41,0X7A,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X41,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,
0XC1,0X8A,0X86,0XA3,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,
0XC2,0XC3,0XC1,0X50,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X86,0XA3,0XC9,0XD4,0XC1,0X8A,0XE1,0X61,0XC1,0X8A,0XE1,0X61,
0X80,0X51,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,
0X80,0X51,0X80,0X18,0XE1,0X61,0X41,0XAB,0X41,0XAB,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,
0XC2,0XC3,0XC1,0X8A,0X80,0X51,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X51,0X41,0X7A,0X86,0XA3,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XC9,0XD4,0XC1,0X50,0X21,0X40,0XC1,0X50,0XE1,0X61,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0X0B,0X9C,0XB1,0XD5,0XC2,0XC3,0XC1,0X8A,
0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,
0XC2,0XC3,0X41,0XAB,0X41,0XAB,0XC9,0XD4,0XC1,0X8A,0X80,0X51,0XE1,0X61,0XC1,0X8A,
0XE1,0X61,0X80,0X51,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XE1,0X61,
0XC1,0X8A,0X86,0XA3,0X41,0X7A,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE1,0X61,0XC1,0X8A,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0X41,0X7A,
0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X51,0XC9,0XD4,0XC9,0XD4,0XC1,0X8A,0XE1,0X61,
0XC1,0X8A,0X41,0XAB,0XC1,0X8A,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XB1,0XD5,0X86,0XA3,0XE1,0X61,0XE1,0X61,0XE0,0X30,
0XE1,0X61,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0X41,0X7A,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XE1,0X61,0XE0,0X30,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,
0X41,0XAB,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0X86,0XA3,
0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X51,0X86,0XA3,0XE1,0X61,0X80,0X51,
0X41,0X7A,0X41,0XAB,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0XC9,0XD4,0XC9,0XD4,0X41,0XAB,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X86,0XA3,0XB1,0XD5,0X86,0XA3,0X41,0X7A,0XE1,0X61,0XE1,0X61,
0X80,0X51,0X41,0X7A,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0XE1,0X61,0XC1,0X8A,0XE1,0X61,0XE1,0X61,0X86,0XA3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0X41,0X7A,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X51,0X86,0XA3,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0XC1,0X8A,
0XC1,0X50,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X50,
0X21,0X40,0XC1,0X50,0XC1,0X8A,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0X41,0X7A,0XC2,0XC3,
0XC9,0XD4,0X86,0XA3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0XC9,0XD4,0XB1,0XD5,0XC1,0X8A,0X80,0X51,0XE1,0X61,0XC1,0X8A,
0X41,0X7A,0XC1,0X50,0XE1,0X61,0X80,0X51,0X80,0X51,0XE1,0X61,0X41,0X7A,0XE1,0X61,
0X41,0X7A,0X41,0X7A,0XE1,0X61,0X41,0X7A,0XC1,0X8A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,
0XC2,0XC3,0X41,0XAB,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,
0X80,0X51,0X41,0X7A,0X41,0X7A,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0XE0,0X30,0X41,0X7A,0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,
0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC1,0X50,0XC9,0XD4,0XC9,0XD4,0XE1,0X61,0X41,0X7A,0X41,0X7A,
0X41,0X7A,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XC9,0XD4,0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X41,0X7A,0X80,0X51,0X80,0X51,0X80,0X51,
0X41,0X7A,0XC2,0XC3,0X41,0X7A,0X80,0X51,0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,
0X41,0X7A,0XE1,0X61,0X41,0X7A,0XE1,0X61,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0X41,0XAB,
0X41,0XAB,0XC2,0XC3,0X41,0X7A,0X41,0X7A,0X41,0X7A,0X41,0X7A,0XE1,0X61,0XE1,0X61,
0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE1,0X61,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X51,0XC2,0XC3,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0X86,0XA3,0XC1,0X50,0X80,0X51,0X80,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X0B,0X9C,0XC9,0XD4,0XE1,0X61,0XE1,0X61,0X41,0X7A,
0X41,0X7A,0XC1,0X8A,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X86,0XA3,0X86,0XA3,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X41,0X7A,0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,
0XE1,0X61,0X41,0X7A,0XC1,0X8A,0X86,0XA3,0XC1,0X8A,0X80,0X51,0X80,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XE1,0X61,0X41,0X7A,0X41,0XAB,0XC1,0X8A,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XE1,0X61,0XC1,0X8A,0X41,0X7A,0X41,0X7A,0X41,0X7A,
0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,
0X41,0X7A,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC1,0X8A,0XC9,0XD4,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XC9,0XD4,0XC9,0XD4,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC2,0XC3,0X41,0X7A,0X80,0X51,0XC1,0X50,
0X41,0X7A,0X41,0XAB,0X41,0XAB,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XC1,0X8A,
0XE1,0X61,0X80,0X51,0XC1,0X50,0X41,0X7A,0XE1,0X61,0X41,0X7A,0X80,0X51,0X80,0X51,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0XC1,0X8A,0XC2,0XC3,0X86,0XA3,0X41,0X7A,0XC1,0X8A,0XC1,0X8A,0X41,0X7A,
0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,
0XC2,0XC3,0XC2,0XC3,0X80,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X41,0X7A,
0XC9,0XD4,0XB1,0XD5,0XC9,0XD4,0XC2,0XC3,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0XC1,0X50,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X51,0X80,0X51,0X21,0X40,
0X21,0X40,0XE1,0X61,0XC9,0XD4,0XC1,0X8A,0X80,0X51,0XE1,0X61,0XE1,0X61,0X41,0X7A,
0X80,0X51,0X80,0X51,0X80,0X51,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X41,0X7A,
0X41,0XAB,0XC2,0XC3,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XE1,0X61,0XE1,0X61,
0X80,0X51,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X41,0X7A,0X86,0XA3,
0X41,0X7A,0XE1,0X61,0X41,0X7A,0XC9,0XD4,0X41,0XAB,0XC1,0X8A,0XC1,0X8A,0XC1,0X8A,
0XC1,0X8A,0X41,0XAB,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X80,0X51,0X80,0X18,0X41,0X7A,
0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0X7A,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE1,0X61,0XC2,0XC3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XE1,0X61,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X86,0XA3,0XC9,0XD4,0XE1,0X61,0XE1,0X61,0X80,0X51,0X41,0XAB,
0X41,0XAB,0XE1,0X61,0X80,0X51,0XE1,0X61,0XC1,0X8A,0XC1,0X8A,0XE1,0X61,0X80,0X51,
0X41,0X7A,0X41,0XAB,0XC2,0XC3,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X41,0X7A,0X86,0XA3,
0X86,0XA3,0XC1,0X8A,0XE1,0X61,0X41,0X7A,0XC2,0XC3,0X86,0XA3,0XC1,0X8A,0X41,0XAB,
0XC1,0X8A,0XC1,0X8A,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0XC1,0X8A,0X80,0X18,0X80,0X18,
0XE0,0X30,0XC2,0XC3,0XC2,0XC3,0XC2,0XC3,0X41,0XAB,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE1,0X61,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC1,0X50,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XC1,0X50,0X86,0XA3,0XC1,0X8A,0X80,0X51,0X21,0X40,0X21,0X40,
0XC2,0XC3,0X41,0XAB,0X41,0X7A,0XC1,0X50,0XE1,0X61,0XC2,0XC3,0X41,0XAB,0X41,0X7A,
0X21,0X40,0X21,0X40,0XC1,0X8A,0X41,0XAB,0X41,0X7A,0X21,0X40,0X21,0X40,0X80,0X18,
0XE1,0X61,0X41,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X41,0X7A,
0X0B,0X9C,0X0B,0X9C,0X86,0XA3,0X41,0X7A,0XE1,0X61,0X86,0XA3,0XC1,0X8A,0XC1,0X8A,
0X41,0X7A,0X41,0X7A,0X41,0X7A,0XC1,0X8A,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X80,0X18,
0X80,0X18,0XC1,0X50,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0X7A,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE1,0X61,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0X41,0X7A,
0XC1,0X50,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X50,0XC1,0X50,0X21,0X40,0X21,0X40,
0X21,0X40,0X41,0XAB,0XC1,0X8A,0XC1,0X50,0X21,0X40,0X21,0X40,0X41,0X7A,0X41,0XAB,
0X80,0X51,0X80,0X18,0X21,0X40,0X21,0X40,0XC1,0X50,0X21,0X40,0X21,0X40,0X80,0X18,
0XC1,0X50,0X86,0XA3,0X41,0X7A,0X41,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X41,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XC9,0XD4,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,
0X86,0XA3,0XC9,0XD4,0XC2,0XC3,0X86,0XA3,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,
0XC9,0XD4,0X86,0XA3,0XC9,0XD4,0XC2,0XC3,0XC9,0XD4,0XC2,0XC3,0X86,0XA3,0X86,0XA3,
0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,0XC9,0XD4,
0XB1,0XD5,0XC9,0XD4,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,
0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,
0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,
0XC2,0XC3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,0X86,0XA3,
0X86,0XA3,0X0B,0X9C,0X0B,0X9C,0X86,0XA3,0X41,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X41,0X7A,0X0B,0X9C,0X0B,0X9C,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X86,0XA3,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,0XB1,0XD5,
0XB1,0XD5,0XB1,0XD5,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X41,0X7A,0X41,0X7A,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X80,0X51,0XE1,0X61,0XE1,0X61,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0X80,0X51,0XE0,0X30,0X80,0X51,0X80,0X51,
0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0X80,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,};
 
 
 
 //----------------------- Picture Engine 4 ---------------------------
  
prog_char engine4[8030] = { 
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X26,0XC4,
0XA1,0XC3,0X26,0XC4,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,
0X0A,0XD5,0X0A,0XD5,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0XA1,0XC3,
0X26,0XC4,0XA1,0XC3,0X26,0XC4,0X41,0XAB,0X41,0XAB,0XC1,0X92,0XE1,0X61,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,
0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,
0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X80,0X18,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE1,0X61,0X61,0X7A,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XC1,0X92,0X61,0X7A,0X60,0X51,
0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE1,0X61,
0XC1,0X92,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0XC1,0X92,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE1,0X61,
0X61,0X7A,0X66,0X9B,0X66,0X9B,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X61,0X7A,0XE1,0X61,
0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X60,0X51,0X60,0X51,0X61,0X7A,0X66,0X9B,0X66,0X9B,
0X61,0X7A,0X61,0X7A,0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0X60,0X51,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE0,0X30,0XE0,0X30,
0X60,0X51,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE0,0X30,
0XE0,0X30,0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE1,0X61,0X60,0X51,
0X60,0X51,0X61,0X7A,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0XC1,0X92,
0X61,0X7A,0X61,0X7A,0X80,0X18,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE0,0X30,0XE0,0X30,0X60,0X51,0X0B,0X9C,0X0B,0X9C,0X66,0X9B,0X66,0X9B,0XE1,0X61,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X66,0X9B,
0X66,0X9B,0X66,0X9B,0X0B,0X9C,0X66,0X9B,0X21,0X40,0X60,0X51,0X61,0X7A,0X60,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,
0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X80,0X18,
0X21,0X40,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0X80,0X18,
0XE1,0X61,0X60,0X51,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X41,0XAB,
0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X60,0X51,0XC1,0X92,0X41,0XAB,0X26,0XC4,0X0A,0XD5,0XC1,0X92,
0XE1,0X61,0X60,0X51,0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X0B,0X9C,
0X66,0X9B,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC1,0X92,0X41,0XAB,0XA1,0XC3,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X26,0XC4,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0X61,0X7A,0X60,0X51,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X60,0X51,0XE0,0X30,
0X60,0X51,0X60,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X60,0X51,0XC1,0X92,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X66,0X9B,0X61,0X7A,
0XE1,0X61,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X60,0X51,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,
0XA1,0XC3,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X60,0X51,0X26,0XC4,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,
0X41,0XAB,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X60,0X51,0X61,0X7A,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X61,0X7A,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0XA1,0XC3,
0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X60,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X60,0X51,0XC1,0X92,0X61,0X7A,0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,
0XA1,0XC3,0XC1,0X92,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X61,0X7A,0XE1,0X61,0XC1,0X92,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X60,0X51,0XA1,0XC3,0X26,0XC4,0X26,0XC4,
0X0A,0XD5,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XC1,0X92,0XC1,0X92,0XC1,0X92,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X60,0X51,0X61,0X7A,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X61,0X7A,0XC1,0X92,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XC1,0X92,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0X41,0XAB,0X26,0XC4,
0X0A,0XD5,0X26,0XC4,0X0A,0XD5,0X66,0X9B,0X66,0X9B,0X61,0X7A,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X60,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0XA1,0XC3,
0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X61,0X7A,0XC1,0X92,0X41,0XAB,0XC1,0X92,0XC1,0X92,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE1,0X61,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X60,0X51,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC1,0X92,
0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X80,0X18,0XE1,0X61,0XC1,0X92,
0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X92,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X41,0XAB,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X80,0X18,0X80,0X18,0X60,0X51,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X61,0X7A,0X60,0X51,0X60,0X51,0XE1,0X61,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XA1,0XC3,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,0X0A,0XD5,
0XD1,0XD5,0X26,0XC4,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0X80,0X18,0X80,0X18,
0X60,0X51,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X0A,0XD5,
0X66,0X9B,0X21,0X40,0X21,0X40,0X21,0X40,0X0A,0XD5,0X26,0XC4,0X60,0X51,0X21,0X40,
0X21,0X40,0X60,0X51,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0XE1,0X61,0XC1,0X92,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X66,0X9B,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X26,0XC4,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,
0XE1,0X61,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X26,0XC4,0XA1,0XC3,0X60,0X51,0X21,0X40,0XE1,0X61,0X0A,0XD5,0X41,0XAB,0X60,0X51,
0X21,0X40,0X21,0X40,0X26,0XC4,0X0A,0XD5,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X80,0X18,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X26,0XC4,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X60,0X51,0X60,0X51,
0XE0,0X30,0XE0,0X30,0X60,0X51,0XA1,0XC3,0X26,0XC4,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,
0X0B,0X9C,0X61,0X7A,0X21,0X40,0X60,0X51,0X26,0XC4,0X26,0XC4,0X21,0X40,0X21,0X40,
0X60,0X51,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X60,0X51,0X66,0X9B,0X26,0XC4,0XA1,0XC3,
0XE1,0X61,0X21,0X40,0X61,0X7A,0X0A,0XD5,0XC1,0X92,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X41,0XAB,
0X26,0XC4,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X60,0X51,
0X60,0X51,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XA1,0XC3,0X41,0XAB,0XC1,0X92,0X61,0X7A,
0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0XC1,0X92,0X21,0X40,0X26,0XC4,0X26,0XC4,0X66,0X9B,
0X21,0X40,0X61,0X7A,0X26,0XC4,0X26,0XC4,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,
0X26,0XC4,0X0A,0XD5,0X0B,0X9C,0X0A,0XD5,0X26,0XC4,0X61,0X7A,0X21,0X40,0X21,0X40,
0X21,0X40,0X61,0X7A,0X21,0X40,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,
0X41,0XAB,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X26,0XC4,0XA1,0XC3,0XC1,0X92,
0XC1,0X92,0X0A,0XD5,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0XE1,0X61,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X26,0XC4,0XA1,0XC3,0X41,0XAB,0X41,0XAB,
0XC1,0X92,0X60,0X51,0XE0,0X30,0XE1,0X61,0XE1,0X61,0X21,0X40,0X61,0X7A,0XA1,0XC3,
0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X26,0XC4,0X41,0XAB,0X60,0X51,
0X21,0X40,0X0B,0X9C,0X0A,0XD5,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0X41,0XAB,0X41,0XAB,0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X61,0X7A,
0XE1,0X61,0XE1,0X61,0X66,0X9B,0X0A,0XD5,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,0XC1,0X92,0X26,0XC4,0XA1,0XC3,0X41,0XAB,
0X41,0XAB,0XC1,0X92,0X21,0X40,0X66,0X9B,0X61,0X7A,0X21,0X40,0X21,0X40,0XE1,0X61,
0XA1,0XC3,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,
0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0X61,0X7A,0XE0,0X30,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XC1,0X92,0XC1,0X92,0X41,0XAB,0X41,0XAB,0XC1,0X92,0XC1,0X92,0X41,0XAB,0XE1,0X61,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0X66,0X9B,0X61,0X7A,0X60,0X51,
0X60,0X51,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X0A,0XD5,0X26,0XC4,0X41,0XAB,0XA1,0XC3,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC1,0X92,0XC1,0X92,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XE1,0X61,0X60,0X51,0X61,0X7A,0X0A,0XD5,0XC1,0X92,0X61,0X7A,
0XA1,0XC3,0X26,0XC4,0X66,0X9B,0X60,0X51,0X26,0XC4,0X0A,0XD5,0X66,0X9B,0XE1,0X61,
0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,
0X26,0XC4,0X66,0X9B,0X66,0X9B,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,
0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0XC1,0X92,0XE1,0X61,0X21,0X40,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X41,0XAB,0XC1,0X92,0X41,0XAB,
0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X60,0X51,
0X21,0X40,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X0A,0XD5,0X26,0XC4,
0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XC1,0X92,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC1,0X92,0X0A,0XD5,0X41,0XAB,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0X66,0X9B,0XD1,0XD5,0X26,0XC4,0X61,0X7A,0X41,0XAB,0X0A,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X66,0X9B,0XE1,0X61,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XE1,0X61,0X61,0X7A,0X26,0XC4,
0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0X26,0XC4,0XA1,0XC3,0XC1,0X92,0X80,0X18,0XE1,0X61,
0X61,0X7A,0XE1,0X61,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X41,0XAB,0XC1,0X92,0XC1,0X92,
0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X60,0X51,0X41,0XAB,0X41,0XAB,0XE1,0X61,0X60,0X51,0X61,0X7A,
0X0A,0XD5,0X0A,0XD5,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XC1,0X92,0XC1,0X92,0X61,0X7A,0X61,0X7A,0X26,0XC4,0X0A,0XD5,0X41,0XAB,0XC1,0X92,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X60,0X51,0XE1,0X61,0X26,0XC4,0X66,0X9B,0X61,0X7A,
0XA1,0XC3,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X66,0X9B,0XE1,0X61,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,
0XE1,0X61,0XC1,0X92,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X61,0X7A,
0XC1,0X92,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X66,0X9B,0X0A,0XD5,0X61,0X7A,0X60,0X51,0X60,0X51,
0X60,0X51,0XE1,0X61,0X0B,0X9C,0XD1,0XD5,0X0A,0XD5,0X26,0XC4,0X41,0XAB,0X41,0XAB,
0XC1,0X92,0XC1,0X92,0X66,0X9B,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X61,0X7A,0X60,0X51,
0XC1,0X92,0X41,0XAB,0XC1,0X92,0XE1,0X61,0X60,0X51,0X60,0X51,0XE1,0X61,0X60,0X51,
0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0XC1,0X92,0X60,0X51,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X60,0X51,0X61,0X7A,0XA1,0XC3,0X0A,0XD5,0X26,0XC4,0X0A,0XD5,
0X0B,0X9C,0X66,0X9B,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X92,0X26,0XC4,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0X60,0X51,0XE1,0X61,0XE1,0X61,0XE0,0X30,0XE1,0X61,0X66,0X9B,0X0A,0XD5,0XD1,0XD5,
0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X0B,0X9C,0X66,0X9B,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X60,0X51,0X26,0XC4,0X26,0XC4,0X21,0X40,0X66,0X9B,0X0A,0XD5,0X0B,0X9C,
0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0XC1,0X92,0XE1,0X61,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0XC1,0X92,0XA1,0XC3,0X0A,0XD5,
0X0A,0XD5,0XC1,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0XC1,0X92,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,0X60,0X51,0X21,0X40,
0X60,0X51,0X61,0X7A,0XC1,0X92,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,0X60,0X51,
0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,0X60,0X51,0X61,0X7A,
0XE1,0X61,0XE1,0X61,0X60,0X51,0XE1,0X61,0X0A,0XD5,0X66,0X9B,0XC1,0X92,0X66,0X9B,
0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0X26,0XC4,0XE1,0X61,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X60,0X51,0X41,0XAB,
0X26,0XC4,0XA1,0XC3,0XC1,0X92,0XC1,0X92,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0XC1,0X92,0X61,0X7A,0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0XA1,0XC3,0XA1,0XC3,0XE1,0X61,0X60,0X51,0XE0,0X30,0X60,0X51,
0X60,0X51,0X60,0X51,0XE0,0X30,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,
0X61,0X7A,0XC1,0X92,0XE1,0X61,0X60,0X51,0X60,0X51,0X61,0X7A,0X26,0XC4,0XE1,0X61,
0X60,0X51,0XE1,0X61,0X41,0XAB,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X61,0X7A,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0XE0,0X30,0X60,0X51,0X61,0X7A,
0XE1,0X61,0X60,0X51,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X61,0X7A,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0XC1,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X61,0X7A,0X26,0XC4,0XC1,0X92,0XE1,0X61,0X60,0X51,0XE1,0X61,
0XC1,0X92,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X60,0X51,0X61,0X7A,0X60,0X51,0X60,0X51,
0X60,0X51,0XC1,0X92,0X26,0XC4,0XE1,0X61,0X21,0X40,0X21,0X40,0XE1,0X61,0X61,0X7A,
0X21,0X40,0X21,0X40,0XC1,0X92,0X26,0XC4,0X26,0XC4,0X0A,0XD5,0X41,0XAB,0XE1,0X61,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X61,0X7A,0X41,0XAB,0X60,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X92,0XC1,0X92,0X61,0X7A,0XE1,0X61,0X21,0X40,
0XC1,0X92,0XA1,0XC3,0XC1,0X92,0XE1,0X61,0X60,0X51,0X61,0X7A,0X41,0XAB,0XC1,0X92,
0X60,0X51,0X60,0X51,0X61,0X7A,0X26,0XC4,0XC1,0X92,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X41,0XAB,
0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X61,0X7A,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0XE0,0X30,0XC1,0X92,0XA1,0XC3,0XC1,0X92,0XC1,0X92,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X61,0X7A,0XC1,0X92,0X61,0X7A,0XC1,0X92,
0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X61,0X7A,0X0A,0XD5,0X60,0X51,0X21,0X40,0X60,0X51,0X61,0X7A,0X0A,0XD5,
0X66,0X9B,0XC1,0X92,0XA1,0XC3,0XC1,0X92,0X60,0X51,0X21,0X40,0XC1,0X92,0XA1,0XC3,
0X41,0XAB,0X21,0X40,0X21,0X40,0XE1,0X61,0XC1,0X92,0X66,0X9B,0X21,0X40,0X21,0X40,
0X21,0X40,0X26,0XC4,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,
0XC1,0X92,0XE0,0X30,0XE0,0X30,0X60,0X51,0XE0,0X30,0XE1,0X61,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X61,0X7A,0XA1,0XC3,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X61,0X7A,
0X41,0XAB,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,0X0B,0X9C,0X21,0X40,0X21,0X40,0XE1,0X61,
0X26,0XC4,0X26,0XC4,0X61,0X7A,0XC1,0X92,0X61,0X7A,0X0A,0XD5,0X66,0X9B,0X61,0X7A,
0X41,0XAB,0XC1,0X92,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X60,0X51,0X21,0X40,
0X21,0X40,0X60,0X51,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X26,0XC4,0X26,0XC4,0X26,0XC4,
0XA1,0XC3,0XC1,0X92,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XC1,0X92,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,
0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X60,0X51,0XE1,0X61,0XA1,0XC3,0X61,0X7A,
0XC1,0X92,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,
0X61,0X7A,0XC1,0X92,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,
0X61,0X7A,0X21,0X40,0X21,0X40,0XE1,0X61,0X66,0X9B,0X0A,0XD5,0X60,0X51,0X60,0X51,
0XC1,0X92,0X26,0XC4,0X0A,0XD5,0X21,0X40,0X60,0X51,0X41,0XAB,0X26,0XC4,0X66,0X9B,
0X60,0X51,0XE1,0X61,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X60,0X51,0XC1,0X92,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X60,0X51,0XE1,0X61,0X60,0X51,0X60,0X51,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XE0,0X30,0X80,0X18,0X80,0X18,0XC1,0X92,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XA1,0XC3,
0XC1,0X92,0XC1,0X92,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,
0X61,0X7A,0X61,0X7A,0XC1,0X92,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,
0X26,0XC4,0XD1,0XD5,0X60,0X51,0X21,0X40,0XC1,0X92,0X41,0XAB,0XD1,0XD5,0X0A,0XD5,
0X0A,0XD5,0X26,0XC4,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X66,0X9B,0XA1,0XC3,0X26,0XC4,
0XC1,0X92,0X21,0X40,0X21,0X40,0X60,0X51,0X61,0X7A,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0X80,0X18,0XE0,0X30,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X26,0XC4,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0XE1,0X61,0X0A,0XD5,0XD1,0XD5,0X26,0XC4,0X0A,0XD5,0XA1,0XC3,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X26,0XC4,
0X0A,0XD5,0X66,0X9B,0X21,0X40,0X60,0X51,0XA1,0XC3,0X0A,0XD5,0X61,0X7A,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,
0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X26,0XC4,0XA1,0XC3,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0X61,0X7A,0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X61,0X7A,
0XC1,0X92,0X26,0XC4,0X60,0X51,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XE1,0X61,0XE1,0X61,0X21,0X40,
0X21,0X40,0X21,0X40,0X61,0X7A,0XC1,0X92,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,
0X0A,0XD5,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0B,0X9C,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X60,0X51,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,
0XD1,0XD5,0XD1,0XD5,0X26,0XC4,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X61,0X7A,
0X61,0X7A,0X61,0X7A,0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XC1,0X92,0XC1,0X92,0X41,0XAB,
0X61,0X7A,0X26,0XC4,0X41,0XAB,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X61,0X7A,0X66,0X9B,
0XD1,0XD5,0X61,0X7A,0X61,0X7A,0XA1,0XC3,0XA1,0XC3,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,
0X0A,0XD5,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X0A,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X26,0XC4,0XA1,0XC3,0XE1,0X61,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X41,0XAB,
0X0B,0X9C,0XC1,0X92,0XE1,0X61,0X61,0X7A,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X41,0XAB,
0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC1,0X92,0X0A,0XD5,0X41,0XAB,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X60,0X51,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XC1,0X92,0XE1,0X61,
0X60,0X51,0X26,0XC4,0XD1,0XD5,0X0A,0XD5,0XA1,0XC3,0X26,0XC4,0XA1,0XC3,0X0A,0XD5,
0X26,0XC4,0X41,0XAB,0XC1,0X92,0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,0XE1,0X61,
0X61,0X7A,0X41,0XAB,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,
0XE1,0X61,0X21,0X40,0X21,0X40,0X61,0X7A,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,
0XA1,0XC3,0XC1,0X92,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0XC1,0X92,0X26,0XC4,0X26,0XC4,0XE1,0X61,0X60,0X51,0XC1,0X92,0XE1,0X61,
0X60,0X51,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC1,0X92,
0X60,0X51,0XE0,0X30,0XE1,0X61,0X66,0X9B,0X0A,0XD5,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XC1,0X92,0X60,0X51,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0XE0,0X30,0X60,0X51,0XE1,0X61,0X26,0XC4,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XC1,0X92,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X61,0X7A,0XA1,0XC3,0XA1,0XC3,
0X41,0XAB,0X26,0XC4,0X26,0XC4,0XC1,0X92,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,
0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,
0XA1,0XC3,0X41,0XAB,0X41,0XAB,0X0A,0XD5,0XC1,0X92,0XE1,0X61,0X60,0X51,0XE1,0X61,
0XE1,0X61,0X60,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,
0XC1,0X92,0X60,0X51,0X21,0X40,0X41,0XAB,0X41,0XAB,0XA1,0XC3,0X0A,0XD5,0XA1,0XC3,
0XA1,0XC3,0XE1,0X61,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X60,0X51,0XC1,0X92,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X26,0XC4,0XA1,0XC3,0XE1,0X61,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,0X0A,0XD5,0X0A,0XD5,
0XC1,0X92,0X61,0X7A,0XC1,0X92,0X26,0XC4,0X26,0XC4,0X41,0XAB,0XA1,0XC3,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XD1,0XD5,0X26,0XC4,0XE1,0X61,0XE1,0X61,0X60,0X51,
0X61,0X7A,0XE1,0X61,0X60,0X51,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,
0XE1,0X61,0XC1,0X92,0X61,0X7A,0X61,0X7A,0X41,0XAB,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,
0XA1,0XC3,0XE1,0X61,0XE0,0X30,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X60,0X51,
0X41,0XAB,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0X26,0XC4,0X41,0XAB,0X60,0X51,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XD1,0XD5,0X0A,0XD5,
0XE1,0X61,0X61,0X7A,0XC1,0X92,0XC1,0X92,0XC1,0X92,0X0A,0XD5,0X26,0XC4,0X41,0XAB,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X66,0X9B,0XD1,0XD5,0X66,0X9B,0X60,0X51,0X61,0X7A,0XC1,0X92,
0X60,0X51,0XE1,0X61,0XE1,0X61,0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X61,0X7A,0XE1,0X61,0X61,0X7A,0X26,0XC4,0X26,0XC4,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0X61,0X7A,0X60,0X51,0XE1,0X61,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X60,0X51,0X66,0X9B,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XC1,0X92,0X66,0X9B,
0X60,0X51,0X60,0X51,0X61,0X7A,0XC1,0X92,0XC1,0X92,0X41,0XAB,0X61,0X7A,0X26,0XC4,
0X0A,0XD5,0X26,0XC4,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0X26,0XC4,0X0A,0XD5,0XD1,0XD5,0XC1,0X92,0X60,0X51,0XE1,0X61,0X60,0X51,
0XC1,0X92,0X66,0X9B,0XC1,0X92,0X60,0X51,0X60,0X51,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X61,0X7A,0X26,0XC4,0XA1,0XC3,
0XA1,0XC3,0X41,0XAB,0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,
0XE1,0X61,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0XE0,0X30,0X61,0X7A,0X0A,0XD5,0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,
0X61,0X7A,0X61,0X7A,0X66,0X9B,0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X60,0X51,0X21,0X40,0X21,0X40,0X60,0X51,0X41,0XAB,0XA1,0XC3,0X61,0X7A,0XE1,0X61,
0XE1,0X61,0X66,0X9B,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,0X26,0XC4,
0X0A,0XD5,0XD1,0XD5,0X0A,0XD5,0X0B,0X9C,0XE1,0X61,0XE1,0X61,0X60,0X51,0XE1,0X61,
0X60,0X51,0XE1,0X61,0X61,0X7A,0X41,0XAB,0X60,0X51,0X60,0X51,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XC1,0X92,
0X41,0XAB,0X26,0XC4,0X61,0X7A,0XE1,0X61,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,
0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XE1,0X61,0XE0,0X30,
0XE0,0X30,0X80,0X18,0X80,0X18,0X80,0X18,0X60,0X51,0X26,0XC4,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,0X66,0X9B,0X60,0X51,
0X61,0X7A,0X61,0X7A,0X61,0X7A,0XE1,0X61,0XE1,0X61,0X66,0X9B,0X0B,0X9C,0X0A,0XD5,
0X0B,0X9C,0X66,0X9B,0XC1,0X92,0XE1,0X61,0X60,0X51,0X60,0X51,0XE1,0X61,0X61,0X7A,
0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,0X61,0X7A,0X60,0X51,0X60,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XE1,0X61,0X61,0X7A,0XC1,0X92,0X61,0X7A,
0XC1,0X92,0XA1,0XC3,0X26,0XC4,0XE1,0X61,0XC1,0X92,0X61,0X7A,0X61,0X7A,0XE1,0X61,
0X41,0XAB,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,
0X61,0X7A,0X80,0X18,0XE0,0X30,0X80,0X18,0X80,0X18,0XE0,0X30,0XC1,0X92,0X0A,0XD5,
0X0A,0XD5,0X0A,0XD5,0XA1,0XC3,0XC1,0X92,0X61,0X7A,0XE1,0X61,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X0A,0XD5,0X26,0XC4,0XE1,0X61,
0XE1,0X61,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X60,0X51,0XE1,0X61,0X60,0X51,0X60,0X51,
0XE0,0X30,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XE1,0X61,
0X41,0XAB,0X41,0XAB,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XE1,0X61,0X60,0X51,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0XE1,0X61,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0X26,0XC4,0X41,0XAB,0X61,0X7A,0XC1,0X92,0X61,0X7A,0X61,0X7A,
0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X41,0XAB,0XA1,0XC3,0XA1,0XC3,
0XA1,0XC3,0XA1,0XC3,0X60,0X51,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,0XE1,0X61,
0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X26,0XC4,0X60,0X51,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0X66,0X9B,0XE1,0X61,
0X60,0X51,0X21,0X40,0XE1,0X61,0X26,0XC4,0XC1,0X92,0X60,0X51,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X60,0X51,0X60,0X51,0X60,0X51,0XE1,0X61,0XE1,0X61,0X60,0X51,0X60,0X51,
0X61,0X7A,0X41,0XAB,0XA1,0XC3,0XE1,0X61,0X21,0X40,0X80,0X18,0XE1,0X61,0XE1,0X61,
0X60,0X51,0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X61,0X7A,0X66,0X9B,
0XC1,0X92,0X61,0X7A,0XC1,0X92,0X26,0XC4,0X66,0X9B,0XC1,0X92,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X60,0X51,0X80,0X18,0XE1,0X61,
0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0X61,0X7A,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0X61,0X7A,0X26,0XC4,0X0A,0XD5,0X0A,0XD5,0X0A,0XD5,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X60,0X51,
0X60,0X51,0X21,0X40,0X21,0X40,0XE1,0X61,0X0A,0XD5,0X61,0X7A,0XE1,0X61,0X60,0X51,
0X61,0X7A,0X41,0XAB,0XE1,0X61,0X60,0X51,0XE1,0X61,0XC1,0X92,0X41,0XAB,0X61,0X7A,
0X21,0X40,0X60,0X51,0XC1,0X92,0XA1,0XC3,0XC1,0X92,0X21,0X40,0X21,0X40,0X60,0X51,
0XE1,0X61,0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,0X66,0X9B,
0X66,0X9B,0X61,0X7A,0X60,0X51,0X61,0X7A,0X26,0XC4,0X41,0XAB,0XC1,0X92,0XC1,0X92,
0XC1,0X92,0XC1,0X92,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0X80,0X18,0X80,0X18,
0XE0,0X30,0XA1,0XC3,0XA1,0XC3,0XA1,0XC3,0XC1,0X92,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0XE1,0X61,0X26,0XC4,0XD1,0XD5,0X0A,0XD5,0XD1,0XD5,0X0B,0X9C,0X61,0X7A,
0XE1,0X61,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X66,0X9B,0XA1,0XC3,0X61,0X7A,0X21,0X40,
0X21,0X40,0X41,0XAB,0XA1,0XC3,0X61,0X7A,0X60,0X51,0X60,0X51,0X41,0XAB,0XA1,0XC3,
0XC1,0X92,0X21,0X40,0X21,0X40,0X60,0X51,0X61,0X7A,0X60,0X51,0X21,0X40,0X80,0X18,
0XE1,0X61,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X61,0X7A,
0X0B,0X9C,0X0B,0X9C,0X26,0XC4,0XE1,0X61,0XE1,0X61,0X66,0X9B,0XC1,0X92,0X61,0X7A,
0XC1,0X92,0X61,0X7A,0X61,0X7A,0XC1,0X92,0X41,0XAB,0X41,0XAB,0XC1,0X92,0X80,0X18,
0X80,0X18,0X60,0X51,0X41,0XAB,0X41,0XAB,0X41,0XAB,0X61,0X7A,0XE0,0X30,0XE0,0X30,
0X80,0X18,0X80,0X18,0XE1,0X61,0X41,0XAB,0X26,0XC4,0X0A,0XD5,0X26,0XC4,0X0B,0X9C,
0X0A,0XD5,0X0B,0X9C,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,0XE1,0X61,0XE1,0X61,0X21,0X40,
0X80,0X18,0X21,0X40,0X61,0X7A,0X41,0XAB,0X60,0X51,0X21,0X40,0X80,0X18,0X60,0X51,
0XC1,0X92,0XE1,0X61,0X21,0X40,0X80,0X18,0X21,0X40,0X21,0X40,0X21,0X40,0X21,0X40,
0X21,0X40,0X66,0X9B,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X61,0X7A,0X66,0X9B,0X0B,0X9C,0XD1,0XD5,0X0A,0XD5,0X66,0X9B,0X0A,0XD5,0X26,0XC4,
0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,
0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X66,0X9B,
0X66,0X9B,0X66,0X9B,0X66,0X9B,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X26,0XC4,0X66,0X9B,
0X26,0XC4,0X66,0X9B,0X26,0XC4,0X26,0XC4,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,
0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,
0X66,0X9B,0X66,0X9B,0X66,0X9B,0X26,0XC4,0X26,0XC4,0X66,0X9B,0X66,0X9B,0X66,0X9B,
0X66,0X9B,0X26,0XC4,0X26,0XC4,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,0X66,0X9B,
0X66,0X9B,0X26,0XC4,0X0B,0X9C,0X66,0X9B,0X61,0X7A,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X66,0X9B,0X0B,0X9C,0X0B,0X9C,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0X0B,0X9C,0X0B,0X9C,0X66,0X9B,0XE1,0X61,0XE1,0X61,0XE1,0X61,
0X60,0X51,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X66,0X9B,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,0XD1,0XD5,
0XD1,0XD5,0XD1,0XD5,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X66,0X9B,0XE1,0X61,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0XE1,0X61,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,
0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X0B,0X9C,0X66,0X9B,0XE1,0X61,0X60,0X51,0XE1,0X61,
0XE1,0X61,0XE1,0X61,0X60,0X51,0XE1,0X61,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,0X60,0X51,0X60,0X51,0X60,0X51,
0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0X60,0X51,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,
0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,
0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,
0XE0,0X30,0X80,0X18,0XE0,0X30,0XE0,0X30,0XE0,0X30,0XE0,0X30,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,
0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18,0X80,0X18};



 //----------------------- Picture intro --------------------- 

prog_char intro[17380] = {
0XCD,0X51,0XED,0X51,0XED,0X51,0XED,0X51,0XED,0X51,0X0E,0X52,0X0E,0X52,0X0E,0X52,
0X0E,0X52,0X0E,0X52,0X0E,0X52,0X0F,0X52,0X0F,0X52,0X0F,0X52,0X0F,0X52,0X0F,0X52,
0X0F,0X52,0X0F,0X52,0X0F,0X52,0X0F,0X52,0X0F,0X52,0X10,0X52,0X10,0X52,0X10,0X52,
0X10,0X52,0X10,0X52,0X10,0X52,0X10,0X52,0X10,0X52,0X10,0X52,0X30,0X52,0X30,0X5A,
0X30,0X5A,0X30,0X5A,0X50,0X5A,0X50,0X5A,0X50,0X5A,0X50,0X5A,0X50,0X5A,0X50,0X62,
0X70,0X62,0X70,0X62,0X70,0X62,0X70,0X5A,0X70,0X62,0X71,0X62,0X71,0X62,0X91,0X62,
0X91,0X62,0X91,0X62,0XB1,0X62,0XB1,0X6A,0XB1,0X6A,0XB1,0X62,0XB1,0X6A,0XB1,0X6A,
0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,
0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,
0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,
0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,
0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0XB1,0X6A,0X90,0X6A,0XB1,0X6A,0X91,0X6A,
0X90,0X6A,0X90,0X6A,0X90,0X6A,0X90,0X6A,0X90,0X6A,0X90,0X6A,0X70,0X62,0X70,0X62,
0X70,0X62,0X70,0X62,0X70,0X62,0X50,0X62,0X50,0X62,0X50,0X62,0X4F,0X62,0X4F,0X62,
0X4F,0X62,0X2F,0X62,0X2E,0X62,0X2E,0X62,0X0E,0X5A,0X0E,0X5A,0X0E,0X5A,0X0E,0X5A,
0X0E,0X5A,0X0E,0X5A,0X0E,0X5A,0X0E,0X52,0X0E,0X52,0X0E,0X52,0X0E,0X52,0X0E,0X52,
0XED,0X51,0XED,0X51,0X0E,0X52,0X0E,0X52,0X0E,0X52,0X0E,0X52,0XEE,0X51,0XED,0X51,
0XED,0X51,0XED,0X51,0XED,0X51,0XCD,0X51,0XCD,0X51,0XED,0X51,0XCC,0X51,0XAC,0X49,
0XAB,0X49,0XAB,0X49,0X8B,0X49,0X8B,0X49,0X8B,0X49,0X8B,0X49,0X6B,0X49,0X6A,0X49,
0X6A,0X49,0X69,0X49,0X69,0X41,0X68,0X49,0X68,0X41,0X48,0X49,0X0F,0X4A,0X0F,0X42,
0X30,0X42,0X30,0X4A,0X30,0X42,0X30,0X42,0X30,0X4A,0X31,0X4A,0X51,0X4A,0X51,0X4A,
0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X72,0X4A,
0X72,0X4A,0X72,0X4A,0X93,0X52,0X93,0X52,0X93,0X52,0X93,0X52,0XB3,0X52,0XB3,0X52,
0XB3,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XD3,0X5A,0XB3,0X52,0XD3,0X5A,0XB3,0X52,
0XD3,0X5A,0XF3,0X5A,0XD4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X2E,0X4A,
0XC4,0X18,0X06,0X29,0XF3,0X62,0X14,0X63,0X13,0X63,0X14,0X63,0XF3,0X62,0XF3,0X62,
0XF3,0X62,0XF3,0X62,0XF2,0X62,0XF2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X5A,0XD2,0X5A,
0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,0X91,0X5A,0X90,0X5A,0X90,0X5A,
0X70,0X5A,0X91,0X5A,0X70,0X5A,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,0X50,0X52,
0X50,0X52,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X30,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X30,0X4A,
0X2F,0X4A,0X0F,0X4A,0X0E,0X4A,0XEE,0X41,0XEE,0X41,0XED,0X41,0XCD,0X41,0XCD,0X41,
0XCD,0X41,0XCD,0X41,0XCD,0X41,0XAD,0X41,0XAC,0X39,0X8C,0X39,0X8B,0X41,0X8A,0X39,
0X8A,0X39,0X8A,0X39,0X8A,0X39,0X69,0X39,0X0F,0X4A,0X30,0X42,0X30,0X42,0X30,0X4A,
0X51,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,
0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X52,0X4A,0X72,0X4A,0X72,0X4A,0X93,0X52,0X93,0X52,0X93,0X52,
0X93,0X52,0X93,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XD3,0X5A,0XD3,0X5A,
0XD3,0X5A,0XD3,0X5A,0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XF3,0X5A,0XF3,0X5A,
0XF4,0X5A,0XF4,0X5A,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X35,0X63,0X55,0X63,0X34,0X63,
0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X35,0X63,
0X55,0X6B,0X55,0X6B,0X55,0X6B,0X55,0X63,0X55,0X6B,0X34,0X63,0X55,0X63,0X34,0X63,
0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X55,0X63,0X06,0X29,0X62,0X10,0X82,0X10,
0X90,0X52,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X6B,0X13,0X63,0X13,0X63,
0XF3,0X62,0XF3,0X62,0XF3,0X62,0XF3,0X62,0XF3,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,
0XD2,0X62,0XD2,0X62,0XB1,0X62,0XB1,0X62,0XB1,0X62,0X91,0X62,0X91,0X5A,0X91,0X5A,
0X91,0X5A,0X91,0X5A,0X91,0X5A,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,
0X51,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X2F,0X4A,0X0F,0X4A,0X0F,0X4A,
0X0E,0X4A,0XEE,0X41,0XEE,0X41,0XEE,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,
0XCD,0X41,0XAC,0X39,0X8C,0X39,0X8B,0X39,0X8B,0X41,0X8B,0X41,0X8A,0X39,0X8A,0X39,
0X8A,0X39,0X69,0X39,0X2F,0X4A,0X30,0X4A,0X30,0X42,0X50,0X4A,0X50,0X4A,0X51,0X4A,
0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X72,0X4A,0X73,0X4A,
0X72,0X4A,0X73,0X4A,0X93,0X52,0X93,0X52,0XB3,0X52,0X93,0X52,0XB3,0X52,0XB3,0X52,
0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XF4,0X5A,
0XD3,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,0X34,0X63,
0X34,0X63,0X34,0X63,0X34,0X63,0X55,0X63,0X55,0X63,0X55,0X6B,0X55,0X63,0X55,0X6B,
0X55,0X63,0X55,0X6B,0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X55,0X6B,0X75,0X6B,
0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,
0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X55,0X6B,0X75,0X6B,0X75,0X6B,
0X55,0X6B,0X55,0X6B,0X75,0X6B,0XE5,0X20,0X62,0X10,0X62,0X10,0X6F,0X52,0X54,0X6B,
0X34,0X6B,0X34,0X6B,0X34,0X6B,0X14,0X6B,0X14,0X63,0X14,0X6B,0X13,0X6B,0X13,0X63,
0X13,0X6B,0XF3,0X6A,0XF3,0X6A,0XF2,0X6A,0XF2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,
0XD2,0X62,0XD2,0X62,0XD1,0X62,0XB1,0X62,0XB2,0X5A,0XB2,0X62,0XB2,0X5A,0XB1,0X5A,
0X92,0X5A,0X91,0X5A,0X91,0X5A,0X91,0X5A,0X71,0X52,0X71,0X52,0X50,0X52,0X50,0X52,
0X50,0X52,0X50,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X0F,0X4A,0X0F,0X4A,0XEE,0X49,0XEE,0X41,0XEE,0X41,
0XCD,0X41,0XCD,0X41,0XEE,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,0XAC,0X41,0XAC,0X41,
0X8C,0X39,0X8B,0X39,0X8B,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X39,
0X2F,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,
0X51,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X72,0X4A,0X73,0X4A,0X93,0X52,0X93,0X52,0X72,0X4A,0XB3,0X52,
0XB3,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,
0XD3,0X5A,0XF4,0X5A,0XD3,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,
0X34,0X63,0X34,0X63,0X55,0X63,0X55,0X63,0X55,0X63,0X55,0X6B,0X55,0X6B,0X55,0X6B,
0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X95,0X6B,0X75,0X6B,
0X95,0X6B,0X75,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X95,0X73,0X95,0X73,0XB5,0X73,0XB5,0X73,0X95,0X6B,0X95,0X73,0X95,0X6B,
0X95,0X73,0X95,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X6B,
0X95,0X6B,0XE5,0X20,0X62,0X10,0X82,0X10,0X8F,0X52,0X75,0X6B,0X75,0X6B,0X75,0X6B,
0X74,0X73,0X54,0X6B,0X54,0X6B,0X34,0X6B,0X34,0X6B,0X13,0X6B,0X13,0X6B,0X13,0X73,
0X13,0X6B,0XF3,0X6A,0XF3,0X6A,0XF3,0X6A,0XF2,0X6A,0XD2,0X62,0XD2,0X62,0XD2,0X62,
0XD2,0X62,0XD2,0X62,0XD2,0X62,0XB2,0X62,0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,
0XB2,0X5A,0X91,0X5A,0X91,0X5A,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,
0X71,0X52,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X50,0X4A,0X50,0X4A,0X4F,0X4A,
0X2F,0X4A,0X0F,0X4A,0X0F,0X4A,0XEE,0X49,0XEE,0X49,0XEE,0X49,0XEE,0X49,0XCD,0X41,
0XCD,0X41,0XEE,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,0XAC,0X41,0XAC,0X41,0X8B,0X41,
0X8A,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X39,0X8A,0X41,0X2F,0X4A,0X2F,0X42,
0X30,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X52,0X4A,0X72,0X4A,0X93,0X52,
0X93,0X52,0X93,0X52,0X93,0X52,0X93,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,
0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XF4,0X5A,0XD4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X34,0X63,0X34,0X63,0X34,0X63,0X34,0X63,0X55,0X63,0X55,0X63,0X55,0X6B,0X55,0X6B,
0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X95,0X6B,0X95,0X6B,0X95,0X6B,
0X95,0X73,0XB5,0X73,0XB5,0X73,0X95,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,
0XB6,0X73,0XB6,0X73,0XB6,0X73,0XB5,0X73,0XB5,0X73,0XB5,0X73,0XB5,0X73,0XB6,0X73,
0XB6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,
0XB6,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,0XD6,0X73,0X26,0X29,
0X62,0X10,0X62,0X10,0X8F,0X52,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X75,0X73,0X74,0X73,0X54,0X73,0X54,0X73,0X33,0X73,0X13,0X73,0X13,0X73,
0X13,0X73,0X13,0X73,0X13,0X6B,0X13,0X6B,0XF3,0X6A,0XF2,0X6A,0XF3,0X6A,0XF2,0X6A,
0XD2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,0XB2,0X62,0XB2,0X5A,0X91,0X5A,
0X91,0X5A,0X71,0X5A,0X91,0X5A,0X91,0X52,0X91,0X52,0X71,0X52,0X71,0X52,0X71,0X52,
0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X50,0X4A,0X50,0X4A,0X2F,0X52,0X0E,0X52,
0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0E,0X4A,0XEE,0X49,0X0E,0X4A,0X0E,0X4A,
0X0F,0X4A,0XEE,0X41,0XCD,0X41,0XCD,0X41,0XAC,0X41,0X8B,0X41,0X8B,0X41,0X8B,0X41,
0X8B,0X41,0X8B,0X41,0X8B,0X41,0X8A,0X41,0X2F,0X4A,0X2F,0X4A,0X2F,0X4A,0X30,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X52,0X4A,0X52,0X4A,
0X52,0X4A,0X52,0X4A,0X72,0X4A,0X72,0X4A,0X93,0X52,0X93,0X52,0XB3,0X52,0XB3,0X52,
0XB3,0X52,0XD3,0X5A,0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XD3,0X5A,0XD4,0X5A,0XD4,0X5A,
0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,0XF4,0X5A,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,0X55,0X6B,0X55,0X6B,0X55,0X6B,
0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X95,0X6B,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X95,0X73,0XB5,0X73,0XB5,0X73,0XB5,0X73,0XB6,0X73,0XB6,0X73,0XB6,0X73,
0XD6,0X73,0XB6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,0XD6,0X73,
0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,
0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X73,0XD6,0X7B,0XD6,0X73,0XD6,0X73,
0XD6,0X7B,0XD6,0X73,0XD6,0X73,0XD6,0X7B,0XF6,0X83,0X26,0X29,0X82,0X10,0X82,0X10,
0X8F,0X52,0XB5,0X73,0XB5,0X73,0XB5,0X73,0XB5,0X73,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X75,0X73,0X74,0X73,0X74,0X7B,0X54,0X7B,0X54,0X73,0X33,0X73,0X33,0X73,
0X33,0X73,0X13,0X73,0X13,0X73,0X13,0X73,0X13,0X6B,0X13,0X6B,0XF2,0X6A,0XD2,0X62,
0XD2,0X62,0XD1,0X62,0XB1,0X62,0XB1,0X62,0XB1,0X62,0XB1,0X5A,0X91,0X5A,0X91,0X5A,
0X91,0X5A,0X91,0X5A,0X91,0X5A,0X91,0X5A,0X71,0X52,0X71,0X52,0X71,0X52,0X71,0X52,
0X71,0X52,0X71,0X52,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X30,0X4A,0X2F,0X4A,
0X30,0X4A,0X30,0X4A,0X30,0X4A,0X2F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0XEE,0X41,
0XCD,0X41,0XAC,0X41,0XAC,0X41,0X8B,0X41,0X8B,0X41,0X8B,0X41,0X8B,0X41,0X8B,0X41,
0X8B,0X41,0X8A,0X41,0X0F,0X4A,0X2F,0X4A,0X2F,0X4A,0X2F,0X4A,0X50,0X4A,0X50,0X4A,
0X50,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X51,0X4A,0X52,0X4A,0X72,0X4A,0X72,0X4A,
0X72,0X4A,0X93,0X52,0XB3,0X52,0XB3,0X52,0XB3,0X52,0XD3,0X5A,0XB3,0X52,0XD3,0X5A,
0XD3,0X5A,0XD3,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,0X14,0X63,0X14,0X63,0XF4,0X5A,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,0X34,0X63,
0X54,0X6B,0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X6B,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X95,0X73,0XB5,0X73,0XB5,0X73,0XB6,0X73,0XB5,0X73,0XB6,0X73,0XB6,0X73,
0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X7B,
0XD6,0X7B,0XD6,0X7B,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,
0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,
0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XD6,0X83,0XF6,0X83,0XF6,0X83,
0XF6,0X83,0XF6,0X83,0X37,0X8C,0X47,0X31,0X82,0X10,0X62,0X10,0X8F,0X5A,0XD6,0X7B,
0XD6,0X7B,0XB6,0X7B,0XD6,0X7B,0XB6,0X7B,0XB6,0X7B,0XB6,0X7B,0XB5,0X73,0X95,0X73,
0X95,0X7B,0X94,0X7B,0X94,0X7B,0X74,0X7B,0X74,0X73,0X74,0X73,0X54,0X73,0X54,0X73,
0X34,0X73,0X33,0X73,0X13,0X73,0X13,0X73,0X13,0X6B,0XF2,0X6A,0XD1,0X6A,0XD1,0X6A,
0XD1,0X6A,0XB1,0X62,0XB1,0X62,0XB1,0X62,0X91,0X62,0X91,0X5A,0X91,0X5A,0X91,0X5A,
0X91,0X5A,0XB1,0X5A,0X91,0X5A,0X91,0X52,0X71,0X52,0X70,0X52,0X71,0X52,0X71,0X52,
0X71,0X52,0X51,0X52,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X50,0X4A,0X50,0X4A,0X30,0X4A,0X0F,0X4A,0XEE,0X49,0XCD,0X49,0XAC,0X49,0XAB,0X41,
0XAC,0X41,0XAC,0X41,0XAC,0X41,0XAC,0X41,0XAC,0X41,0XAC,0X41,0XAC,0X41,0X8B,0X41,
0X2F,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,
0X71,0X4A,0X72,0X4A,0X72,0X4A,0X72,0X4A,0X72,0X4A,0X72,0X4A,0X72,0X4A,0X93,0X52,
0X93,0X52,0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,0XF4,0X5A,
0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X34,0X63,0X34,0X63,0X55,0X6B,0X55,0X6B,0X54,0X6B,0X75,0X73,0X95,0X73,
0X75,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0XB5,0X73,0XB5,0X73,0XB5,0X73,
0XB6,0X7B,0XB6,0X7B,0XD6,0X7B,0XD6,0X83,0XD6,0X83,0XD6,0X83,0XD6,0X83,0XD6,0X83,
0XD6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0XF6,0X83,0X17,0X84,0X17,0X84,0X17,0X8C,
0X37,0X8C,0X17,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,
0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,
0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X37,0X8C,0X57,0X94,
0X77,0X94,0X26,0X31,0X82,0X10,0X82,0X10,0XD0,0X62,0XD6,0X83,0XF6,0X83,0XF6,0X83,
0XD6,0X83,0XD6,0X83,0XD6,0X83,0XD6,0X83,0XD6,0X83,0XD6,0X7B,0XB5,0X83,0XB5,0X83,
0XB5,0X83,0XB4,0X83,0X94,0X7B,0X95,0X7B,0X95,0X7B,0X94,0X7B,0X74,0X73,0X54,0X73,
0X33,0X73,0X13,0X73,0X13,0X73,0X12,0X73,0XF1,0X72,0XF1,0X6A,0XD1,0X6A,0XD1,0X6A,
0XD1,0X6A,0XB1,0X6A,0XB1,0X62,0XB1,0X62,0XB1,0X62,0XB1,0X62,0XB2,0X5A,0XB2,0X62,
0XB1,0X5A,0X91,0X5A,0X91,0X5A,0X91,0X5A,0X91,0X5A,0X71,0X52,0X71,0X52,0X71,0X52,
0X50,0X52,0X50,0X4A,0X51,0X4A,0X50,0X4A,0X51,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X0F,0X4A,0X0E,0X4A,0XEE,0X49,0XCD,0X49,0XCD,0X49,0XCD,0X49,0XCD,0X49,0XCD,0X49,
0XCD,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,0XCD,0X41,0XAC,0X41,0X2F,0X4A,0X50,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,0X51,0X4A,0X72,0X4A,
0X72,0X4A,0X72,0X52,0X72,0X52,0X92,0X52,0X92,0X52,0X92,0X52,0XB3,0X52,0XB3,0X52,
0XB3,0X52,0XD3,0X5A,0XD3,0X5A,0XF4,0X5A,0XF4,0X5A,0X14,0X63,0X14,0X63,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,0X34,0X63,0X34,0X6B,0X54,0X6B,0X55,0X6B,
0X75,0X6B,0X74,0X73,0X75,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0XB5,0X73,
0XB5,0X73,0XB5,0X7B,0XB5,0X7B,0XD6,0X7B,0XD6,0X7B,0XD6,0X83,0XD6,0X83,0XF6,0X83,
0XF6,0X83,0XF6,0X8B,0XF6,0X8B,0XF6,0X8B,0X16,0X8C,0X16,0X8C,0X17,0X8C,0X37,0X8C,
0X37,0X8C,0X57,0X8C,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,
0X77,0X94,0X77,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X77,0X94,0X77,0X94,
0X77,0X94,0X77,0X94,0X77,0X94,0X57,0X94,0X77,0X94,0X77,0X94,0X77,0X94,0X77,0X94,
0X77,0X94,0X57,0X94,0X77,0X94,0X57,0X94,0X57,0X94,0X77,0X94,0X97,0X9C,0X26,0X29,
0X83,0X10,0X83,0X18,0X31,0X6B,0X16,0X8C,0X37,0X94,0X16,0X8C,0X16,0X8C,0X16,0X8C,
0XF6,0X8B,0XF6,0X8B,0XF6,0X8B,0XF6,0X83,0XD6,0X83,0XD5,0X83,0XD5,0X83,0XB5,0X83,
0XB5,0X83,0XB4,0X83,0XB4,0X83,0XB4,0X7B,0X94,0X7B,0X74,0X7B,0X53,0X7B,0X53,0X7B,
0X33,0X7B,0X32,0X7B,0X11,0X7B,0X11,0X7B,0XF1,0X72,0XF1,0X72,0XD1,0X6A,0XD1,0X6A,
0XD1,0X6A,0XD1,0X6A,0XD1,0X62,0XD2,0X62,0XD2,0X62,0XB1,0X62,0X91,0X62,0XB1,0X5A,
0XB1,0X62,0XB1,0X5A,0X91,0X5A,0X70,0X5A,0X70,0X52,0X70,0X52,0X70,0X52,0X50,0X52,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X0F,0X4A,0XEE,0X51,0XEE,0X51,0XEE,0X49,
0XEE,0X49,0XEE,0X49,0XEE,0X49,0XEE,0X49,0XEE,0X49,0X0E,0X4A,0X0F,0X4A,0XEE,0X49,
0XEE,0X41,0XEE,0X41,0XEE,0X41,0XED,0X49,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X51,0X4A,0X51,0X4A,0X51,0X4A,0X71,0X4A,0X71,0X52,0X71,0X52,0X92,0X52,0X72,0X52,
0X92,0X52,0X92,0X5A,0XB3,0X52,0XB3,0X5A,0XD3,0X5A,0XD3,0X5A,0XF3,0X5A,0XF3,0X5A,
0X14,0X63,0XF4,0X62,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,
0X54,0X6B,0X55,0X6B,0X75,0X6B,0X75,0X6B,0X75,0X73,0X95,0X73,0X95,0X73,0X95,0X73,
0X95,0X73,0X95,0X73,0X95,0X73,0XB5,0X7B,0XB5,0X7B,0XB5,0X7B,0XD6,0X83,0XD6,0X83,
0XF6,0X83,0XF6,0X83,0XF6,0X8B,0XF6,0X8B,0XF6,0X8B,0X16,0X8C,0X16,0X8C,0X16,0X94,
0X36,0X94,0X37,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X77,0X94,0X77,0X94,
0X77,0X94,0X77,0X94,0X77,0X94,0X77,0X94,0X97,0X9C,0X97,0X9C,0X77,0X94,0X97,0X9C,
0X97,0X9C,0X97,0X9C,0X97,0X9C,0X97,0X9C,0XB7,0X9C,0XB7,0X9C,0XB7,0XA4,0XB7,0XA4,
0XD7,0XA4,0XB7,0X9C,0XB7,0X9C,0XB7,0X9C,0XB7,0X9C,0X97,0X9C,0X97,0X9C,0X97,0X9C,
0X97,0X9C,0X97,0X9C,0X97,0X9C,0XD7,0XA4,0XD8,0XA4,0XC9,0X41,0X83,0X10,0XA3,0X18,
0XD5,0X83,0X57,0X94,0X57,0X94,0X57,0X94,0X57,0X94,0X37,0X94,0X36,0X94,0X36,0X94,
0X36,0X94,0X16,0X94,0X16,0X94,0X16,0X8C,0XF5,0X8B,0XF5,0X8B,0XD5,0X8B,0XD5,0X8B,
0XB5,0X8B,0XB4,0X83,0XB4,0X83,0X94,0X83,0X93,0X83,0X73,0X83,0X72,0X83,0X31,0X83,
0X31,0X83,0X31,0X7B,0X11,0X7B,0X10,0X7B,0XF1,0X72,0XF1,0X72,0XF1,0X72,0XD1,0X6A,
0XD1,0X6A,0XD0,0X6A,0XD1,0X6A,0XB0,0X6A,0XB1,0X62,0XB1,0X62,0XB1,0X62,0XB1,0X5A,
0XB1,0X5A,0X91,0X5A,0X70,0X5A,0X70,0X52,0X50,0X52,0X50,0X52,0X50,0X52,0X50,0X4A,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X2F,0X4A,0X0E,0X52,0X2F,0X4A,0X2F,0X4A,0X30,0X4A,
0X2F,0X4A,0X2F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,
0X0F,0X4A,0XEE,0X49,0X2F,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X51,0X4A,
0X51,0X4A,0X71,0X52,0X71,0X52,0X71,0X52,0X91,0X5A,0X91,0X5A,0X91,0X5A,0XB2,0X5A,
0XB2,0X5A,0XD2,0X5A,0XD3,0X5A,0XD3,0X62,0XF3,0X62,0XF3,0X62,0XF3,0X62,0X14,0X63,
0X14,0X63,0X14,0X63,0X14,0X63,0X34,0X63,0X54,0X6B,0X55,0X6B,0X75,0X6B,0X75,0X6B,
0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0XB5,0X7B,0XB5,0X7B,0XB5,0X83,0XB5,0X7B,
0XB5,0X83,0XD5,0X83,0XD6,0X83,0XD6,0X83,0XF6,0X83,0XF6,0X8B,0X16,0X8C,0X16,0X8C,
0X16,0X8C,0X16,0X94,0X36,0X94,0X36,0X94,0X36,0X94,0X37,0X94,0X57,0X94,0X77,0X94,
0X77,0X94,0X77,0X94,0X77,0X9C,0X97,0X9C,0X97,0X9C,0XB7,0X9C,0XB7,0X9C,0XB7,0XA4,
0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,0XD7,0XA4,
0XD7,0XA4,0XF7,0XA4,0XD8,0XA4,0XF7,0XA4,0XF8,0XAC,0XF8,0XAC,0XF8,0XA4,0XF7,0XA4,
0XF7,0XA4,0XF8,0XA4,0XD7,0XA4,0XF7,0XA4,0XF7,0XA4,0XD7,0XA4,0XF7,0XA4,0XD7,0XA4,
0XD7,0XA4,0XF8,0XA4,0X19,0XAD,0X67,0X31,0X83,0X18,0X83,0X18,0X14,0X8C,0X97,0X9C,
0XD7,0XA4,0XB7,0X9C,0X97,0X9C,0X77,0X9C,0X77,0X9C,0X56,0X9C,0X56,0X9C,0X36,0X94,
0X36,0X94,0X36,0X94,0X36,0X94,0X36,0X94,0X16,0X94,0XF5,0X93,0XD4,0X93,0XD4,0X93,
0XB3,0X8B,0X93,0X8B,0X93,0X8B,0X93,0X8B,0X71,0X93,0X71,0X8B,0X51,0X8B,0X30,0X8B,
0X31,0X83,0X31,0X83,0X31,0X7B,0X10,0X7B,0X11,0X7B,0XF0,0X72,0XD0,0X72,0XD0,0X72,
0XD1,0X6A,0XD1,0X6A,0XD1,0X6A,0XB1,0X62,0XB1,0X62,0XB1,0X62,0X91,0X5A,0X90,0X5A,
0X70,0X5A,0X70,0X52,0X70,0X52,0X50,0X52,0X50,0X52,0X70,0X52,0X71,0X52,0X50,0X52,
0X50,0X52,0X50,0X52,0X50,0X52,0X2F,0X52,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,
0X30,0X4A,0X30,0X4A,0X2F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0X0F,0X4A,0XEE,0X49,
0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X71,0X4A,0X71,0X52,0X71,0X52,0X72,0X52,
0X92,0X52,0X92,0X52,0X91,0X5A,0X91,0X5A,0XB1,0X5A,0XB1,0X62,0XD2,0X62,0XD2,0X62,
0XD2,0X62,0XF2,0X62,0XF3,0X62,0X14,0X63,0X14,0X63,0X14,0X6B,0X14,0X6B,0X34,0X6B,
0X34,0X6B,0X75,0X6B,0X75,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,0X95,0X73,
0XB5,0X7B,0XB5,0X7B,0XB5,0X83,0XD5,0X83,0XD5,0X83,0XD5,0X83,0XD6,0X83,0XF6,0X83,
0XF6,0X8B,0X16,0X8C,0X16,0X8C,0X16,0X8C,0X16,0X94,0X36,0X94,0X36,0X94,0X36,0X94,
0X37,0X94,0X77,0X94,0X77,0X9C,0X97,0X9C,0X97,0X9C,0X97,0X9C,0XB7,0X9C,0XB7,0X9C,
0XB7,0XA4,0XD7,0XA4,0XD7,0XA4,0XF7,0XA4,0XF7,0XAC,0XF8,0XAC,0XF8,0XAC,0XF8,0XAC,
0XF8,0XAC,0X18,0XAD,0XF8,0XAC,0XF8,0XAC,0XF8,0XAC,0X18,0XAD,0X19,0XAD,0X19,0XAD,
0X19,0XAD,0X18,0XAD,0X18,0XAD,0X19,0XB5,0X19,0XB5,0X19,0XB5,0X19,0XAD,0X18,0XAD,
0X19,0XAD,0X19,0XB5,0X19,0XB5,0X19,0XB5,0X18,0XAD,0X18,0XAD,0X18,0XAD,0X19,0XAD,
0XD3,0X83,0X83,0X18,0X83,0X18,0XA3,0X18,0X56,0X94,0XF7,0XA4,0XF8,0XAC,0XF8,0XA4,
0XF7,0XA4,0XF7,0XA4,0XD7,0XA4,0XD7,0XA4,0XB7,0X9C,0X97,0X9C,0X56,0XA4,0X56,0XA4,
0X56,0XA4,0X56,0X9C,0X36,0X9C,0X36,0X9C,0X15,0X9C,0XF4,0X9B,0XF4,0X9B,0XB3,0X9B,
0XB2,0X9B,0X92,0X93,0X92,0X9B,0X92,0X93,0X91,0X93,0X71,0X93,0X71,0X93,0X51,0X8B,
0X51,0X8B,0X51,0X83,0X31,0X83,0X10,0X7B,0XF0,0X7A,0XF1,0X72,0XF1,0X6A,0XD1,0X6A,
0XD1,0X6A,0XD1,0X6A,0XD1,0X62,0XB1,0X62,0XB1,0X62,0X70,0X5A,0X4F,0X52,0X50,0X52,
0X70,0X52,0X71,0X52,0X70,0X52,0X71,0X5A,0X91,0X52,0X71,0X52,0X70,0X52,0X71,0X52,
0X70,0X52,0X50,0X52,0X50,0X52,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X50,0X4A,0X2F,0X4A,
0X0F,0X4A,0X0F,0X4A,0XEE,0X41,0XED,0X41,0XCC,0X41,0XAB,0X49,0X50,0X52,0X50,0X4A,
0X71,0X52,0X71,0X52,0X71,0X52,0X92,0X52,0X92,0X52,0X92,0X52,0XB2,0X5A,0XB2,0X5A,
0XD2,0X5A,0XD2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,0XD2,0X62,0XF2,0X6A,0XF3,0X6A,
0X13,0X6B,0X13,0X6B,0X13,0X6B,0X13,0X6B,0X54,0X73,0X74,0X73,0X75,0X73,0X95,0X73,
0X95,0X73,0X95,0X73,0X95,0X7B,0X95,0X7B,0X95,0X7B,0XB5,0X7B,0XB5,0X7B,0XB5,0X83,
0XB5,0X83,0XD5,0X83,0XD5,0X83,0XF6,0X8B,0XF6,0X8B,0XF6,0X8B,0X16,0X8C,0X16,0X94,
0X16,0X94,0X16,0X94,0X36,0X94,0X36,0X94,0X56,0X94,0X77,0X9C,0X97,0X9C,0X97,0X9C,
0XB7,0X9C,0X97,0X9C,0X77,0X9C,0X97,0XA4,0X96,0XA4,0XB7,0XA4,0XF7,0XA4,0X18,0XAD,
0X38,0XAD,0X18,0XAD,0X19,0XAD,0X19,0XAD,0X19,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,
0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,
0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,
0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X6B,0X52,0X83,0X10,
0X83,0X18,0XA4,0X20,0XB6,0XA4,0X18,0XAD,0X38,0XB5,0X17,0XB5,0X18,0XAD,0X18,0XAD,
0X17,0XAD,0X17,0XAD,0XF7,0XAC,0XD7,0XA4,0XD7,0XA4,0XB6,0XAC,0X96,0XA4,0X96,0XA4,
0X96,0XA4,0X76,0XA4,0X56,0X9C,0X55,0X9C,0X14,0XA4,0XD3,0XA3,0XD3,0X9B,0XB2,0X9B,
0XB2,0X9B,0XB2,0X9B,0X91,0X9B,0X91,0X9B,0X91,0X93,0X92,0X93,0X72,0X8B,0X72,0X8B,
0X52,0X83,0X32,0X83,0X32,0X7B,0X12,0X73,0XF1,0X72,0XF1,0X72,0XD1,0X6A,0XD1,0X6A,
0XD1,0X6A,0XB1,0X62,0X70,0X5A,0X4F,0X5A,0X70,0X5A,0X90,0X5A,0X70,0X5A,0X70,0X5A,
0X70,0X5A,0X70,0X5A,0X70,0X5A,0X70,0X52,0X70,0X52,0X70,0X52,0X71,0X52,0X91,0X52,
0X71,0X52,0X50,0X4A,0X50,0X4A,0X2F,0X4A,0X2F,0X4A,0X0F,0X4A,0XED,0X49,0XCD,0X41,
0XED,0X49,0XEE,0X51,0XED,0X49,0XED,0X51,0X51,0X52,0X71,0X52,0X91,0X52,0X92,0X52,
0X92,0X5A,0X92,0X5A,0XB2,0X5A,0XB2,0X5A,0XB2,0X5A,0XD2,0X62,0XD2,0X62,0XD2,0X62,
0XD2,0X62,0XD2,0X62,0XF2,0X62,0XF3,0X6A,0X13,0X6B,0X13,0X73,0X13,0X73,0X13,0X73,
0X34,0X73,0X54,0X73,0X74,0X73,0X94,0X73,0X95,0X7B,0XB5,0X7B,0XB5,0X7B,0XB5,0X83,
0XB5,0X83,0XB5,0X83,0XB4,0X83,0XB4,0X83,0XB4,0X83,0XB4,0X83,0XB5,0X83,0XB5,0X83,
0XB4,0X83,0XD5,0X8B,0XF5,0X8B,0XF5,0X93,0X16,0X94,0X16,0X94,0X16,0X94,0X36,0X94,
0X36,0X94,0X56,0X9C,0X76,0X9C,0X76,0X9C,0X76,0X9C,0X56,0X9C,0X56,0X9C,0X56,0XA4,
0X96,0XA4,0XD7,0XAC,0X18,0XAD,0X18,0XB5,0X38,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,
0X39,0XB5,0X39,0XB5,0X39,0XB5,0X39,0XB5,0X59,0XB5,0X5A,0XBD,0X5A,0XBD,0X5A,0XB5,
0X5A,0XBD,0X5A,0XBD,0X5A,0XB5,0X7A,0XBD,0X5A,0XBD,0X7A,0XBD,0X7A,0XBD,0X7A,0XBD,
0X9A,0XBD,0X9A,0XBD,0X9A,0XBD,0X7A,0XBD,0X7A,0XBD,0X7A,0XBD,0X5A,0XBD,0X5A,0XBD,
0X5A,0XBD,0X5A,0XBD,0X7A,0XBD,0X7A,0XBD,0X6B,0X5A,0X83,0X18,0X83,0X18,0XC3,0X20,
0XD7,0XAC,0X38,0XB5,0X38,0XBD,0X38,0XBD,0XF7,0XB4,0XF2,0X93,0XD6,0XAC,0X18,0XB5,
0X17,0XB5,0X17,0XB5,0X17,0XB5,0XF7,0XAC,0XF7,0XAC,0XD6,0XAC,0XB6,0XA4,0XB7,0XA4,
0X96,0XA4,0X55,0XA4,0X34,0XAC,0X34,0XA4,0XF3,0XA3,0XF3,0XA3,0XD2,0XA3,0XB2,0XA3,
0XB2,0X9B,0XB2,0X9B,0X92,0X93,0X93,0X93,0X93,0X8B,0X92,0X8B,0X72,0X8B,0X52,0X83,
0X31,0X83,0X11,0X7B,0XF1,0X72,0XF1,0X72,0XD0,0X72,0XD0,0X72,0X8F,0X62,0X6F,0X5A,
0X4F,0X5A,0X6F,0X5A,0X4F,0X5A,0X4F,0X52,0X4F,0X52,0X4F,0X52,0X4F,0X52,0X4F,0X52,
0X4F,0X52,0X70,0X5A,0X70,0X5A,0X91,0X5A,0X91,0X5A,0X91,0X52,0X70,0X52,0X50,0X52,
0X50,0X52,0X2F,0X52,0X0E,0X4A,0X0E,0X4A,0X2F,0X4A,0X2F,0X4A,0X4F,0X4A,0X2F,0X4A,
0X2F,0X4A,0X2E,0X4A,0X50,0X52,0X71,0X52,0X91,0X5A,0X91,0X5A,0XB2,0X5A,0XB2,0X5A,
0XB2,0X5A,0XD2,0X62,0XD2,0X62,0XF2,0X62,0XD2,0X62,0XF2,0X62,0XF2,0X6A,0XF3,0X6A,
0X13,0X6B,0X13,0X73,0X13,0X73,0X13,0X73,0X33,0X73,0X53,0X7B,0X74,0X7B,0X74,0X7B,
0X74,0X7B,0X94,0X7B,0X94,0X7B,0XB4,0X83,0XB5,0X83,0XB5,0X83,0XB5,0X83,0XB4,0X8B,
0XB4,0X83,0XB4,0X83,0XB4,0X83,0XB4,0X83,0X94,0X83,0X73,0X83,0X73,0X83,0X73,0X83,
0X93,0X83,0XB4,0X83,0XD4,0X8B,0XF5,0X93,0X16,0X94,0X15,0X94,0XF5,0X93,0XF5,0X93,
0XF5,0X93,0XF5,0X93,0X35,0X9C,0X36,0X9C,0X96,0XA4,0XD7,0XAC,0X18,0XAD,0X38,0XB5,
0X38,0XB5,0X39,0XB5,0X59,0XBD,0X59,0XBD,0X7A,0XBD,0X7A,0XBD,0X9A,0XBD,0X9A,0XBD,
0XBA,0XBD,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,
0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,
0XDA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,0XBA,0XC5,
0XBA,0XC5,0XBA,0XC5,0XED,0X6A,0X82,0X18,0X83,0X18,0XC3,0X20,0X17,0XB5,0X78,0XC5,
0X58,0XC5,0X58,0XC5,0X0A,0X52,0X62,0X18,0XC8,0X41,0X38,0XB5,0X38,0XB5,0X38,0XB5,
0X37,0XB5,0X37,0XB5,0X37,0XB5,0X17,0XB5,0XF6,0XB4,0XD6,0XB4,0XD6,0XB4,0X95,0XB4,
0X54,0XAC,0X34,0XAC,0X33,0XAC,0X13,0XAC,0X13,0XAC,0XF3,0XA3,0XD2,0XA3,0XD3,0X9B,
0X92,0X9B,0X92,0X93,0X92,0X93,0X92,0X93,0X71,0X8B,0X51,0X8B,0X31,0X83,0X11,0X7B,
0XF0,0X72,0XCF,0X72,0X8F,0X6A,0X4F,0X62,0X2F,0X5A,0X2F,0X5A,0X2E,0X5A,0X2F,0X52,
0X2F,0X52,0X2E,0X5A,0X2F,0X52,0X2F,0X52,0X2E,0X52,0X2F,0X52,0X4F,0X52,0X70,0X5A,
0X91,0X5A,0XB1,0X5A,0X91,0X5A,0X91,0X5A,0X90,0X5A,0X70,0X52,0X50,0X52,0X4F,0X52,
0X2F,0X52,0X2E,0X52,0X2F,0X52,0X2F,0X52,0X2F,0X52,0X2F,0X52,0X2F,0X52,0X0E,0X52,
0X4F,0X52,0X6F,0X52,0X70,0X5A,0X70,0X5A,0X70,0X5A,0X91,0X5A,0XB1,0X62,0XD2,0X62,
0XF2,0X6A,0XF2,0X6A,0XF2,0X6A,0XF2,0X6A,0X13,0X6B,0X13,0X73,0X13,0X73,0X33,0X73,
0X33,0X73,0X53,0X7B,0X53,0X7B,0X53,0X7B,0X53,0X83,0X93,0X83,0X93,0X83,0X93,0X83,
0XB4,0X83,0XB4,0X83,0XB4,0X8B,0XB4,0X8B,0XB4,0X8B,0XB4,0X8B,0XB4,0X8B,0XB4,0X8B,
0X94,0X8B,0X93,0X83,0X73,0X83,0X53,0X7B,0X33,0X7B,0X32,0X7B,0X12,0X7B,0X32,0X7B,
0X73,0X83,0X93,0X8B,0XD4,0X93,0XD4,0X8B,0XB4,0X8B,0X93,0X8B,0X93,0X8B,0XD4,0X8B,
0X15,0X9C,0X96,0XA4,0XD7,0XAC,0XF7,0XAC,0X38,0XB5,0X38,0XB5,0X38,0XBD,0X59,0XBD,
0X79,0XBD,0X99,0XBD,0X99,0XC5,0X99,0XC5,0X99,0XBD,0X99,0XC5,0XB9,0XC5,0XBA,0XC5,
0XDA,0XCD,0XDA,0XCD,0XFA,0XCD,0XFA,0XCD,0XDA,0XCD,0XFA,0XCD,0XDA,0XCD,0XDA,0XCD,
0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,
0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XFA,0XCD,0XDA,0XCD,0XFA,0XCD,0XFB,0XCD,
0X0D,0X6B,0X83,0X18,0X83,0X18,0XE4,0X20,0X57,0XC5,0X99,0XCD,0X78,0XC5,0X17,0XBD,
0XC4,0X20,0X83,0X18,0XC4,0X20,0X16,0XBD,0X57,0XC5,0X38,0XBD,0X38,0XBD,0X38,0XBD,
0X37,0XBD,0X17,0XBD,0XF6,0XBC,0XD6,0XBC,0XB5,0XB4,0X74,0XB4,0X34,0XAC,0X33,0XAC,
0X13,0XAC,0X13,0XAC,0XF3,0XAB,0XD2,0XAB,0XD2,0XA3,0XD2,0XA3,0XB2,0X9B,0X92,0X9B,
0X92,0X9B,0X91,0X93,0X50,0X8B,0X10,0X83,0XEF,0X7A,0X8E,0X7A,0X4D,0X72,0X2D,0X62,
0X2D,0X5A,0X0E,0X52,0X0D,0X52,0XED,0X51,0XED,0X51,0XCD,0X51,0XED,0X51,0XED,0X51,
0X0E,0X52,0X2E,0X5A,0X4F,0X5A,0X4F,0X5A,0X90,0X5A,0XB1,0X62,0X91,0X5A,0X91,0X5A,
0X91,0X5A,0X91,0X5A,0X91,0X5A,0X70,0X5A,0X4F,0X52,0X0E,0X52,0XED,0X51,0XED,0X49,
0XED,0X51,0XEE,0X51,0X0D,0X52,0X0E,0X52,0X0E,0X52,0X0D,0X52,0X0E,0X4A,0XEE,0X49,
0X0E,0X4A,0X0E,0X52,0XEE,0X51,0XED,0X49,0XEE,0X51,0X4F,0X5A,0X90,0X62,0XD1,0X6A,
0XD1,0X6A,0XF1,0X72,0X12,0X73,0X12,0X73,0X33,0X73,0X53,0X7B,0X53,0X83,0X73,0X83,
0X93,0X83,0X94,0X83,0X94,0X83,0X93,0X8B,0X93,0X8B,0X93,0X8B,0XB3,0X8B,0XB3,0X8B,
0XB4,0X8B,0XD4,0X8B,0XB3,0X8B,0XB3,0X8B,0XB3,0X8B,0XB4,0X8B,0XB4,0X8B,0XB3,0X8B,
0X93,0X8B,0X93,0X8B,0X73,0X83,0X52,0X83,0X32,0X7B,0X11,0X7B,0X12,0X7B,0X12,0X7B,
0X12,0X7B,0X53,0X83,0X93,0X8B,0X93,0X8B,0X93,0X8B,0XB3,0X8B,0XB3,0X8B,0XF4,0X93,
0X55,0XA4,0XD6,0XAC,0XF7,0XB4,0X17,0XB5,0X38,0XBD,0X38,0XBD,0X38,0XBD,0X38,0XBD,
0X58,0XBD,0X37,0XBD,0X37,0XBD,0X58,0XBD,0X78,0XC5,0X38,0XBD,0X58,0XBD,0X58,0XC5,
0X99,0XC5,0XB9,0XC5,0XDA,0XCD,0XFA,0XCD,0XFA,0XCD,0X1A,0XD6,0X1B,0XD6,0X3B,0XD6,
0X3B,0XD6,0X3B,0XD6,0X5B,0XDE,0X5B,0XD6,0X3B,0XD6,0X3B,0XD6,0X3B,0XD6,0X3B,0XD6,
0X3B,0XD6,0X3B,0XD6,0X3B,0XD6,0X1B,0XD6,0X1B,0XD6,0X1B,0XD6,0X6F,0X83,0X83,0X18,
0X83,0X18,0XC3,0X20,0X57,0XC5,0XB9,0XCD,0XB8,0XCD,0XF5,0XB4,0XA3,0X18,0X82,0X10,
0XC4,0X20,0X37,0XBD,0X78,0XC5,0X58,0XC5,0X58,0XC5,0X58,0XBD,0X37,0XBD,0X16,0XBD,
0XF6,0XBC,0XF5,0XBC,0XB5,0XBC,0X94,0XBC,0X74,0XB4,0X54,0XB4,0X54,0XB4,0X33,0XAC,
0X13,0XAC,0XD2,0XAB,0XB1,0XA3,0X91,0X9B,0X91,0X9B,0X70,0X93,0X2F,0X8B,0X0F,0X8B,
0XCE,0X82,0XAE,0X82,0X8D,0X7A,0X6D,0X72,0X6E,0X6A,0X4E,0X62,0X0D,0X5A,0X0D,0X52,
0XEC,0X51,0X0C,0X52,0X0C,0X52,0X0D,0X5A,0X2D,0X5A,0X4E,0X5A,0X4E,0X5A,0X6F,0X62,
0X6F,0X62,0XB0,0X62,0XD1,0X6A,0XD1,0X6A,0XB1,0X62,0X90,0X5A,0X70,0X5A,0X70,0X5A,
0X4F,0X5A,0X4F,0X5A,0X4F,0X5A,0X4F,0X5A,0X0E,0X52,0X0D,0X52,0X0D,0X52,0X0D,0X52,
0X2E,0X52,0X2E,0X5A,0X2E,0X5A,0X0D,0X5A,0X8A,0X41,0X8B,0X41,0X8B,0X41,0XAC,0X41,
0XCD,0X41,0XCC,0X49,0XED,0X49,0XED,0X51,0X4F,0X5A,0XB0,0X6A,0XF1,0X72,0X12,0X7B,
0X12,0X7B,0X32,0X7B,0X32,0X7B,0X52,0X83,0X73,0X83,0X93,0X8B,0XB4,0X8B,0XB4,0X8B,
0XD4,0X8B,0XD4,0X8B,0XD4,0X93,0XD4,0X93,0XF4,0X93,0XF4,0X9B,0XF4,0X93,0XF4,0X93,
0XD4,0X93,0XB3,0X93,0X93,0X8B,0X93,0X8B,0XB3,0X8B,0X93,0X8B,0X93,0X8B,0X93,0X8B,
0X92,0X8B,0X92,0X8B,0X72,0X8B,0X51,0X83,0X52,0X83,0X32,0X7B,0XF1,0X72,0X11,0X7B,
0X52,0X7B,0X72,0X83,0X93,0X8B,0X93,0X8B,0X93,0X8B,0XB3,0X93,0XD3,0X9B,0X14,0XA4,
0X55,0XA4,0X96,0XAC,0XD6,0XB4,0XF6,0XB4,0X16,0XBD,0XF6,0XB4,0X17,0XBD,0X17,0XBD,
0X16,0XBD,0XF6,0XBC,0X58,0XC5,0X57,0XC5,0X17,0XBD,0X16,0XBD,0X37,0XBD,0X78,0XC5,
0XB8,0XCD,0XDA,0XCD,0X1A,0XD6,0X1A,0XD6,0X1A,0XD6,0X1A,0XD6,0X1B,0XD6,0X3B,0XD6,
0X5B,0XDE,0X7B,0XDE,0X7B,0XDE,0X7B,0XDE,0X7B,0XDE,0X7B,0XDE,0X7B,0XDE,0X7B,0XDE,
0X7B,0XDE,0X1A,0XD6,0XB9,0XCD,0X5B,0XDE,0XF1,0X8B,0X83,0X18,0X83,0X18,0X83,0X18,
0X16,0XB5,0X1A,0XD6,0X1A,0XD6,0XF5,0XB4,0X83,0X18,0X82,0X18,0X04,0X21,0X77,0XCD,
0X98,0XCD,0X78,0XCD,0X78,0XCD,0X77,0XC5,0X57,0XC5,0X36,0XC5,0X36,0XC5,0X15,0XC5,
0XD5,0XBC,0XD5,0XBC,0XB4,0XBC,0X94,0XBC,0X53,0XB4,0X12,0XB4,0XF2,0XB3,0XD1,0XAB,
0X91,0XA3,0X91,0XA3,0X70,0X9B,0X50,0X9B,0X2F,0X93,0XEE,0X92,0XCE,0X8A,0X8D,0X82,
0X6D,0X7A,0X6D,0X6A,0X2D,0X62,0X4D,0X62,0X2D,0X62,0XEC,0X59,0X0C,0X5A,0XEC,0X59,
0X2C,0X62,0X4D,0X6A,0X6E,0X6A,0X8F,0X6A,0XAF,0X6A,0XD0,0X6A,0XF1,0X72,0XF2,0X6A,
0XD1,0X6A,0XD1,0X6A,0XB0,0X6A,0X6F,0X62,0X90,0X62,0X90,0X62,0X90,0X62,0X6F,0X5A,
0X4F,0X5A,0X4F,0X5A,0X2D,0X5A,0X2D,0X5A,0X2E,0X5A,0X4E,0X62,0X4F,0X62,0X6F,0X5A,
0X4F,0X5A,0X4E,0X5A,0X68,0X31,0X69,0X31,0X69,0X39,0X8A,0X39,0X8A,0X39,0XAB,0X41,
0XCC,0X49,0X2E,0X5A,0X6F,0X62,0XB0,0X72,0X11,0X7B,0X32,0X83,0X72,0X83,0X73,0X83,
0X72,0X83,0X72,0X8B,0X93,0X8B,0XB4,0X8B,0XD4,0X93,0XF5,0X93,0XF5,0X93,0XF4,0X9B,
0XF4,0X9B,0X14,0X9C,0X14,0X9C,0X14,0X9C,0X14,0X9C,0X14,0XA4,0X35,0XA4,0X14,0XA4,
0XF3,0X9B,0XD3,0XA3,0XD3,0X9B,0XB3,0X93,0XB3,0X93,0X93,0X8B,0X93,0X93,0X92,0X93,
0X92,0X8B,0X92,0X8B,0X92,0X8B,0X72,0X8B,0X31,0X83,0X31,0X83,0X11,0X7B,0X32,0X83,
0X72,0X8B,0X93,0X8B,0X93,0X8B,0XB3,0X93,0XD3,0X9B,0XF3,0XA3,0X14,0XA4,0XF4,0X9B,
0X14,0XA4,0X54,0XAC,0X34,0XA4,0X14,0XA4,0X54,0XAC,0X95,0XB4,0XB5,0XB4,0X95,0XB4,
0XF6,0XBC,0X37,0XC5,0X16,0XBD,0X57,0XC5,0X78,0XCD,0X98,0XCD,0XB8,0XCD,0XB8,0XCD,
0XB8,0XCD,0XF9,0XD5,0X3A,0XD6,0X5A,0XDE,0X3A,0XD6,0X3A,0XDE,0X5B,0XDE,0XBB,0XE6,
0XBB,0XE6,0XBB,0XE6,0XBB,0XE6,0XBB,0XE6,0XBB,0XE6,0X9B,0XE6,0XD9,0XCD,0X46,0X31,
0X04,0X29,0X76,0XBD,0XB3,0XA4,0X83,0X18,0X83,0X18,0X83,0X18,0X97,0XBD,0X7B,0XDE,
0X5B,0XDE,0XB4,0XAC,0X82,0X10,0X82,0X10,0XE4,0X20,0XB8,0XCD,0XB7,0XD5,0X97,0XD5,
0X97,0XD5,0X76,0XD5,0X35,0XCD,0X35,0XCD,0X35,0XCD,0X35,0XCD,0XF5,0XC4,0XD4,0XC4,
0XD4,0XC4,0X94,0XBC,0X94,0XBC,0X73,0XBC,0X32,0XB4,0XD1,0XB3,0XD1,0XAB,0X90,0XA3,
0X4F,0X9B,0X2F,0X8B,0XCE,0X8A,0XAD,0X82,0X6D,0X82,0X6D,0X7A,0X4D,0X6A,0X2D,0X62,
0X2D,0X62,0X0D,0X62,0X0C,0X62,0X2C,0X62,0X4C,0X6A,0X6E,0X72,0X8E,0X72,0XAF,0X72,
0XF0,0X7A,0XF0,0X7A,0X11,0X7B,0XF1,0X7A,0XF1,0X72,0XF1,0X72,0XD1,0X72,0XD0,0X6A,
0X8F,0X6A,0X6F,0X62,0X8F,0X62,0X6F,0X62,0X6F,0X62,0X6F,0X62,0X6F,0X62,0X6E,0X62,
0X6E,0X62,0X6E,0X62,0X6F,0X62,0X8F,0X6A,0X6F,0X62,0X6F,0X62,0X6F,0X62,0X4E,0X62,
0X27,0X29,0X47,0X29,0X47,0X29,0X48,0X29,0X89,0X39,0XAC,0X41,0XED,0X51,0X0D,0X52,
0X6F,0X62,0XD0,0X72,0X52,0X83,0X93,0X8B,0X93,0X8B,0X93,0X8B,0X93,0X8B,0XB3,0X93,
0XD4,0X93,0XF4,0X9B,0XF4,0X9B,0XF4,0X9B,0X14,0X9C,0X14,0XA4,0X35,0XA4,0X14,0XA4,
0X34,0XA4,0X34,0XAC,0X34,0XAC,0X55,0XAC,0X55,0XAC,0X54,0XAC,0X34,0XAC,0X34,0XAC,
0X34,0XA4,0X13,0XAC,0XF3,0XA3,0XB2,0X9B,0XB2,0X9B,0XD2,0XA3,0XD3,0X9B,0XB2,0X9B,
0X92,0X93,0X92,0X93,0X92,0X93,0X72,0X8B,0X51,0X83,0X31,0X8B,0X31,0X83,0X51,0X8B,
0X72,0X8B,0X72,0X8B,0X92,0X93,0XD3,0X9B,0X13,0XA4,0XF3,0XA3,0XD3,0X9B,0XD3,0X9B,
0XF3,0XA3,0XB2,0X93,0XF3,0XA3,0X54,0XB4,0XD5,0XBC,0XD5,0XBC,0X16,0XBD,0XF5,0XBC,
0X94,0XB4,0X54,0XB4,0X36,0XC5,0X56,0XC5,0XB8,0XCD,0X77,0XCD,0X98,0XCD,0XF9,0XD5,
0X5A,0XDE,0X9B,0XE6,0X9B,0XE6,0X9B,0XDE,0X7A,0XDE,0X5A,0XDE,0X9B,0XDE,0XBB,0XE6,
0XDB,0XE6,0XFB,0XE6,0XFB,0XE6,0XFB,0XE6,0XD3,0XA4,0XA2,0X18,0X82,0X18,0X72,0X94,
0X92,0X9C,0X82,0X18,0X83,0X18,0XA3,0X18,0X97,0XBD,0X7B,0XDE,0X7B,0XDE,0X31,0X94,
0X82,0X10,0X82,0X10,0XE4,0X20,0XB7,0XD5,0XB8,0XD5,0X97,0XD5,0X76,0XD5,0X55,0XD5,
0XF4,0XCC,0X35,0XCD,0X55,0XD5,0X55,0XCD,0X35,0XCD,0X15,0XCD,0XD4,0XC4,0XB3,0XC4,
0X52,0XBC,0X32,0XBC,0XF1,0XB3,0XD1,0XB3,0XB1,0XAB,0X70,0XA3,0X4F,0X9B,0X2F,0X93,
0X0F,0X93,0XCE,0X8A,0X8D,0X7A,0X6D,0X72,0X6D,0X72,0X6D,0X72,0X4C,0X72,0X2C,0X72,
0X6C,0X7A,0X8D,0X7A,0XCF,0X82,0X10,0X83,0X10,0X83,0X30,0X83,0X30,0X83,0X10,0X83,
0XF0,0X7A,0XF0,0X7A,0XD0,0X72,0XD0,0X72,0XD0,0X72,0XAF,0X72,0X8F,0X72,0X6E,0X6A,
0X4D,0X6A,0X4D,0X6A,0X4D,0X6A,0X6E,0X72,0X8E,0X6A,0X8E,0X72,0X8F,0X6A,0X8F,0X6A,
0X8F,0X6A,0X8F,0X6A,0X8F,0X6A,0X8F,0X6A,0X8F,0X6A,0X6E,0X6A,0X68,0X31,0X68,0X31,
0X68,0X31,0X68,0X31,0X89,0X39,0XAA,0X41,0XCC,0X49,0XEC,0X51,0X2D,0X5A,0X11,0X73,
0XB3,0X8B,0XB3,0X93,0XD4,0X93,0XB3,0X93,0XD3,0X9B,0XF4,0X9B,0X14,0XA4,0X34,0XA4,
0X34,0XA4,0X34,0XA4,0X34,0XAC,0X34,0XAC,0X34,0XAC,0X54,0XAC,0X75,0XB4,0X95,0XB4,
0X95,0XB4,0X95,0XB4,0X95,0XB4,0X95,0XB4,0XB5,0XB4,0XB5,0XBC,0X95,0XBC,0X74,0XB4,
0X54,0XB4,0X54,0XB4,0X33,0XAC,0X13,0XAC,0XF3,0XA3,0XD2,0XA3,0XB2,0XA3,0XB2,0X9B,
0X91,0X9B,0X92,0X9B,0X92,0X9B,0X71,0X93,0X51,0X8B,0X71,0X8B,0X71,0X93,0X71,0X93,
0X72,0X8B,0X51,0X8B,0XB2,0X9B,0X13,0XA4,0XF3,0XA3,0X13,0XAC,0X34,0XB4,0X13,0XAC,
0X54,0XB4,0XB5,0XBC,0X95,0XBC,0XF5,0XBC,0X16,0XC5,0X36,0XC5,0XB5,0XBC,0X94,0XB4,
0X74,0XB4,0X94,0XB4,0X16,0XC5,0X77,0XCD,0X97,0XD5,0X39,0XDE,0X9B,0XE6,0XFB,0XE6,
0XFB,0XEE,0X1C,0XEF,0XFC,0XEE,0XFC,0XEE,0XDB,0XE6,0XBB,0XE6,0XDB,0XE6,0XFB,0XE6,
0XFB,0XE6,0XFC,0XE6,0X31,0X94,0X82,0X18,0X82,0X18,0XCF,0X83,0X6E,0X73,0X83,0X18,
0X82,0X18,0X83,0X18,0X35,0XB5,0X7B,0XE6,0X7B,0XDE,0X8F,0X83,0X82,0X10,0X82,0X10,
0X86,0X39,0X97,0XCD,0X97,0XD5,0X35,0XCD,0XD4,0XCC,0XD3,0XCC,0XD3,0XCC,0X14,0XCD,
0XF4,0XCC,0X35,0XD5,0X14,0XCD,0XD4,0XC4,0X93,0XC4,0XF1,0XBB,0XD1,0XB3,0XD0,0XB3,
0XB0,0XB3,0X8F,0XAB,0X6F,0XA3,0X4F,0X9B,0X2F,0X93,0X0E,0X93,0XEE,0X8A,0XCE,0X8A,
0X6D,0X7A,0X4C,0X72,0X6C,0X7A,0X6C,0X82,0X8C,0X8A,0XAD,0X8A,0XEE,0X8A,0X0F,0X8B,
0X2F,0X8B,0X2F,0X8B,0X30,0X8B,0X30,0X83,0X10,0X83,0X0F,0X83,0XEF,0X7A,0XAF,0X7A,
0XAF,0X7A,0XAE,0X7A,0X8D,0X7A,0X8D,0X7A,0X8E,0X7A,0X8E,0X72,0XAE,0X7A,0XAE,0X7A,
0XCF,0X7A,0XCF,0X7A,0XF0,0X7A,0XD0,0X72,0XD0,0X72,0XD0,0X72,0XD0,0X72,0XD0,0X72,
0XD0,0X72,0XD0,0X72,0XD0,0X72,0XAF,0X72,0X0D,0X52,0XCC,0X49,0XAB,0X41,0XAB,0X49,
0XCC,0X49,0X0D,0X52,0X2D,0X52,0X2D,0X5A,0X6E,0X62,0XCF,0X72,0X92,0X8B,0XD4,0X9B,
0XF4,0X9B,0XF3,0XA3,0XF3,0XA3,0X14,0XA4,0X34,0XAC,0X34,0XAC,0X54,0XAC,0X74,0XB4,
0X74,0XB4,0XB5,0XB4,0XD5,0XB4,0XD5,0XBC,0XF6,0XBC,0XF6,0XBC,0XF6,0XBC,0X16,0XBD,
0X16,0XBD,0X16,0XC5,0X37,0XC5,0X16,0XC5,0X16,0XC5,0XF5,0XBC,0XF5,0XBC,0X94,0XBC,
0X74,0XBC,0X53,0XB4,0X33,0XB4,0X12,0XAC,0XF2,0XAB,0XD2,0XAB,0XB1,0XA3,0X91,0XA3,
0X91,0X9B,0X70,0X9B,0X50,0X93,0X50,0X8B,0X2F,0X8B,0X2F,0X8B,0X30,0X8B,0XF0,0X7A,
0XF0,0X7A,0X30,0X8B,0X91,0X9B,0XB2,0XA3,0XD2,0XA3,0XD2,0XA3,0X33,0XAC,0X74,0XB4,
0X94,0XB4,0X94,0XBC,0XB5,0XBC,0XD5,0XBC,0XB5,0XBC,0X74,0XB4,0X74,0XB4,0X73,0XB4,
0XD4,0XC4,0X76,0XCD,0XB7,0XDD,0X5A,0XE6,0XBB,0XE6,0XFB,0XEE,0X1C,0XEF,0X3C,0XF7,
0X3C,0XF7,0X3C,0XF7,0X1C,0XF7,0X3C,0XF7,0X1C,0XF7,0X1C,0XEF,0X1C,0XEF,0X96,0XC5,
0X45,0X31,0X83,0X18,0X82,0X18,0X2D,0X73,0X6A,0X52,0X83,0X18,0X82,0X10,0X82,0X10,
0XD3,0XAC,0X7A,0XE6,0X5A,0XE6,0X2C,0X73,0X62,0X10,0X82,0X10,0XC8,0X49,0X73,0XBC,
0X73,0XBC,0X73,0XBC,0X53,0XBC,0X32,0XBC,0X32,0XBC,0X31,0XC4,0X31,0XC4,0XB3,0XCC,
0X93,0XC4,0X93,0XC4,0X32,0XC4,0XB0,0XB3,0X6F,0XAB,0X2E,0XA3,0X0E,0X9B,0XED,0X92,
0X0E,0X93,0X0E,0X93,0XAD,0X92,0XAD,0X92,0XAD,0X8A,0X6C,0X8A,0X6C,0X82,0X8C,0X8A,
0XAC,0X92,0XCD,0X9A,0X0E,0X9B,0X2F,0X93,0X2F,0X93,0X2F,0X8B,0X50,0X93,0X50,0X93,
0X70,0X93,0X2F,0X8B,0X2F,0X8B,0X0F,0X8B,0XCD,0X8A,0XAD,0X8A,0XAD,0X82,0XAD,0X8A,
0XAE,0X82,0XEE,0X8A,0XEF,0X8A,0X0F,0X8B,0X30,0X8B,0X10,0X8B,0X30,0X8B,0X30,0X83,
0X31,0X83,0X11,0X83,0X31,0X7B,0X11,0X7B,0XF1,0X7A,0XF1,0X72,0XF1,0X72,0XF1,0X72,
0XF0,0X72,0XB0,0X72,0X8F,0X6A,0X8F,0X62,0X6F,0X5A,0X6E,0X5A,0X2D,0X5A,0X4E,0X62,
0X8F,0X6A,0XD0,0X72,0XF0,0X7A,0X51,0X8B,0X92,0X8B,0XF3,0X9B,0X34,0XA4,0X34,0XAC,
0X54,0XAC,0X54,0XB4,0X75,0XB4,0X95,0XB4,0XB5,0XBC,0XD6,0XBC,0XF6,0XBC,0X16,0XBD,
0X16,0XBD,0X36,0XC5,0X57,0XC5,0X37,0XC5,0X57,0XC5,0X57,0XC5,0X77,0XCD,0X77,0XCD,
0X97,0XCD,0X97,0XCD,0X97,0XCD,0X77,0XCD,0X56,0XCD,0X15,0XC5,0XF5,0XC4,0XF5,0XC4,
0XD4,0XC4,0X93,0XC4,0X53,0XBC,0X33,0XB4,0XF2,0XB3,0XF2,0XB3,0XB1,0XAB,0X90,0XAB,
0X6F,0XA3,0X4F,0X9B,0X4F,0X93,0X2F,0X93,0X0F,0X8B,0XCE,0X82,0XEF,0X82,0X0F,0X8B,
0X2F,0X93,0X4F,0X9B,0X70,0X9B,0XB1,0XA3,0XF2,0XAB,0X12,0XAC,0X13,0XAC,0X13,0XAC,
0X33,0XB4,0X13,0XB4,0X94,0XBC,0XD4,0XC4,0XB4,0XC4,0XD3,0XC4,0X34,0XD5,0X96,0XD5,
0XF7,0XE5,0X79,0XE6,0XDA,0XEE,0XFB,0XEE,0X1C,0XEF,0X3C,0XF7,0X3C,0XF7,0X5D,0XFF,
0X5C,0XF7,0X3C,0XF7,0X3C,0XF7,0X1C,0XEF,0X1C,0XEF,0XAE,0X83,0X83,0X18,0XA3,0X18,
0X82,0X18,0XCB,0X62,0XC7,0X41,0X82,0X18,0X62,0X10,0X62,0X10,0X51,0XA4,0X39,0XDE,
0X59,0XE6,0XCB,0X6A,0X62,0X10,0X62,0X10,0X0A,0X5A,0XD1,0XAB,0X90,0XA3,0XB1,0XAB,
0XD1,0XAB,0XB1,0XAB,0XD1,0XB3,0XD1,0XB3,0XB0,0XAB,0X32,0XBC,0X32,0XBC,0X11,0XBC,
0X8F,0XAB,0X2E,0XAB,0XCD,0X92,0XAD,0X92,0XAD,0X8A,0XAC,0X92,0X8C,0X92,0X8C,0X92,
0X8C,0X92,0X8C,0X92,0X6C,0X92,0X8C,0X9A,0XAC,0XA2,0XCC,0XA2,0X2E,0XA3,0X2E,0X9B,
0X4F,0X9B,0X4F,0X9B,0X4F,0X9B,0X4F,0X9B,0X4F,0X9B,0X2F,0X93,0X2F,0X93,0X0E,0X9B,
0XEE,0X9A,0XCD,0X9A,0XCD,0X9A,0XED,0X9A,0X0E,0X9B,0X0E,0X9B,0X2E,0X9B,0X4F,0X9B,
0X50,0X9B,0X70,0X93,0X70,0X93,0X71,0X93,0X71,0X93,0X51,0X8B,0X72,0X8B,0X52,0X83,
0X51,0X83,0X31,0X7B,0X31,0X7B,0X51,0X7B,0X11,0X7B,0XF1,0X7A,0XD0,0X72,0XB0,0X72,
0XD0,0X7A,0XF0,0X7A,0XD0,0X72,0XAF,0X72,0XCF,0X7A,0X0F,0X83,0X30,0X8B,0X51,0X8B,
0X51,0X8B,0X92,0X93,0X13,0XA4,0X54,0XB4,0X75,0XB4,0X74,0XB4,0X74,0XB4,0XB5,0XB4,
0XB5,0XBC,0XD5,0XBC,0XF6,0XBC,0X16,0XBD,0X36,0XC5,0X36,0XC5,0X57,0XC5,0X77,0XCD,
0X77,0XCD,0X97,0XCD,0X97,0XCD,0X97,0XD5,0X97,0XD5,0XB7,0XD5,0X97,0XD5,0X96,0XD5,
0X96,0XD5,0X76,0XD5,0X35,0XCD,0X14,0XCD,0XD4,0XCC,0XD4,0XCC,0XD3,0XCC,0X72,0XC4,
0X52,0XBC,0X12,0XBC,0XD1,0XB3,0X90,0XAB,0X4F,0XA3,0X4E,0XAB,0X8F,0XAB,0XB0,0XB3,
0XB0,0XB3,0X90,0XAB,0X6F,0XAB,0X6F,0XA3,0X6F,0XA3,0X6F,0XA3,0X90,0XAB,0XD1,0XB3,
0XD1,0XB3,0XD1,0XB3,0XD1,0XAB,0XB1,0XAB,0XD1,0XAB,0XD1,0XAB,0XF1,0XAB,0X52,0XBC,
0XB3,0XC4,0X93,0XC4,0XD3,0XCC,0XF3,0XD4,0X54,0XDD,0XB5,0XDD,0X17,0XE6,0X79,0XEE,
0XFB,0XEE,0X1B,0XEF,0X3C,0XF7,0X3C,0XF7,0X3C,0XF7,0X3C,0XF7,0X5C,0XF7,0X5C,0XFF,
0X3C,0XF7,0X1B,0XF7,0X1B,0XF7,0XAE,0X83,0XA3,0X18,0X83,0X18,0X83,0X18,0XAA,0X62,
0X86,0X39,0X82,0X18,0X82,0X10,0X82,0X10,0X2C,0X8B,0X96,0XDD,0XF7,0XDD,0X29,0X52,
0X82,0X10,0X62,0X10,0X2D,0X83,0XB3,0XC4,0X73,0XBC,0X12,0XB4,0XB1,0XA3,0X70,0X9B,
0XB0,0XAB,0X90,0XA3,0X4F,0X9B,0X6F,0XA3,0X4F,0XA3,0XED,0X92,0XCD,0X92,0XCD,0X9A,
0XAC,0X9A,0X8C,0X92,0X8C,0X92,0X8C,0X9A,0X6B,0X9A,0X6B,0X9A,0X6B,0X9A,0X8B,0XA2,
0XAC,0XA2,0XCC,0XA2,0XED,0XA2,0X0E,0X9B,0X0D,0X9B,0X2E,0X9B,0X2E,0X9B,0X4F,0X9B,
0X4E,0XA3,0X2E,0XA3,0X2D,0XA3,0X4F,0XA3,0X8F,0XAB,0XB0,0XB3,0XB0,0XB3,0XB0,0XB3,
0X90,0XAB,0X90,0XAB,0X90,0XAB,0X90,0XA3,0XB0,0XA3,0XB1,0XA3,0XB2,0XA3,0XB1,0X9B,
0X91,0X9B,0X91,0X9B,0X91,0X9B,0X91,0X93,0X71,0X8B,0X71,0X8B,0X51,0X8B,0X51,0X8B,
0X51,0X83,0X31,0X83,0X31,0X83,0XF0,0X7A,0XD0,0X72,0XAF,0X6A,0XCF,0X7A,0XF0,0X7A,
0X0F,0X8B,0X0F,0X8B,0X0F,0X8B,0X2F,0X8B,0X50,0X93,0X91,0X9B,0X91,0X9B,0X13,0XAC,
0X54,0XB4,0X74,0XB4,0X74,0XB4,0X94,0XB4,0X74,0XB4,0X13,0XAC,0X13,0XAC,0X73,0XB4,
0XB4,0XBC,0XF5,0XC4,0XD5,0XC4,0XF5,0XC4,0XF5,0XC4,0X35,0XCD,0X97,0XD5,0XB7,0XD5,
0XD7,0XD5,0XD7,0XDD,0XD7,0XDD,0XD7,0XDD,0XF7,0XDD,0XF7,0XDD,0XD7,0XDD,0XD7,0XDD,
0XB6,0XDD,0X75,0XD5,0X34,0XD5,0XD3,0XCC,0XB2,0XCC,0X72,0XCC,0X10,0XBC,0XD0,0XB3,
0XAF,0XB3,0X6F,0XB3,0X2E,0XA3,0X2D,0XA3,0X6E,0XB3,0X6E,0XB3,0XCF,0XBB,0X10,0XC4,
0X10,0XC4,0XD0,0XB3,0X8F,0XAB,0X6F,0XAB,0XD0,0XBB,0X10,0XBC,0XF1,0XBB,0X72,0XC4,
0X52,0XBC,0X11,0XBC,0XB0,0XAB,0XD1,0XB3,0X32,0XBC,0X92,0XCC,0XD3,0XD4,0XF3,0XD4,
0X13,0XDD,0XF2,0XDC,0X53,0XE5,0X16,0XEE,0X98,0XF6,0X1A,0XF7,0X1B,0XF7,0X3B,0XF7,
0X3C,0XF7,0X3B,0XF7,0X1B,0XF7,0XFB,0XF6,0XDA,0XF6,0XFA,0XF6,0XF9,0XF6,0XFA,0XF6,
0XD9,0XF6,0XCE,0X93,0X83,0X18,0XA3,0X18,0X83,0X18,0X49,0X5A,0X04,0X29,0X82,0X18,
0X82,0X10,0X82,0X10,0X2C,0X9B,0X71,0XCC,0X11,0XBC,0X45,0X31,0X83,0X10,0X62,0X10,
0X31,0XA4,0X14,0XCD,0X93,0XC4,0X12,0XB4,0X12,0XBC,0XB0,0XAB,0X90,0XAB,0X0E,0X93,
0XCD,0X8A,0X8D,0X82,0X6C,0X7A,0X6C,0X82,0X4B,0X82,0X4B,0X8A,0X2A,0X92,0X6B,0X9A,
0X6B,0XA2,0X6B,0XA2,0X4B,0XAA,0X6B,0XAA,0X8B,0XAA,0XAB,0XA2,0XCC,0XA2,0XCC,0X9A,
0XCC,0X92,0XED,0X9A,0X0E,0X9B,0X4E,0XA3,0X4E,0XAB,0X8F,0XB3,0XB0,0XB3,0XD0,0XB3,
0X51,0XC4,0X92,0XC4,0X72,0XC4,0X32,0XBC,0X12,0XBC,0X11,0XBC,0X31,0XBC,0X32,0XBC,
0X12,0XB4,0X11,0XB4,0XF1,0XB3,0XD1,0XAB,0XD2,0XAB,0XD2,0XA3,0XD1,0XA3,0X91,0X9B,
0X91,0X9B,0X70,0X9B,0X50,0X9B,0X50,0X93,0X50,0X93,0X50,0X8B,0X51,0X8B,0X51,0X8B,
0X31,0X83,0X10,0X7B,0XCF,0X72,0X6E,0X6A,0X0F,0X8B,0X0F,0X8B,0X50,0X93,0X50,0X93,
0X50,0X93,0X91,0X9B,0X70,0X9B,0X50,0X9B,0XB1,0XA3,0X13,0XAC,0X13,0XAC,0X53,0XB4,
0X53,0XB4,0XF2,0XAB,0XD1,0XAB,0XF1,0XB3,0X93,0XBC,0X73,0XBC,0X32,0XB4,0XD1,0XAB,
0X91,0XA3,0X70,0X9B,0XB0,0XAB,0X93,0XC4,0X55,0XD5,0XD7,0XDD,0XF7,0XDD,0X17,0XE6,
0X18,0XE6,0X18,0XE6,0X18,0XE6,0X18,0XE6,0X17,0XE6,0X17,0XE6,0X17,0XE6,0X17,0XE6,
0XD6,0XDD,0X95,0XDD,0X33,0XDD,0XD2,0XD4,0X50,0XCC,0XEF,0XBB,0X10,0XC4,0XCF,0XBB,
0X6E,0XB3,0X0D,0XAB,0X0D,0XAB,0XCC,0XAA,0XEC,0XAA,0X4D,0XB3,0X8E,0XBB,0X4D,0XB3,
0X0C,0XB3,0X6D,0XBB,0X10,0XC4,0XB2,0XD4,0XB2,0XD4,0X13,0XD5,0XF3,0XD4,0X92,0XCC,
0XB2,0XCC,0XF3,0XD4,0X13,0XD5,0X13,0XDD,0XF3,0XDC,0XF2,0XDC,0X74,0XE5,0X57,0XEE,
0XB8,0XF6,0XB8,0XF6,0XB9,0XF6,0XB9,0XF6,0XB8,0XF6,0XB8,0XF6,0XB8,0XF6,0XB9,0XF6,
0X78,0XEE,0X77,0XF6,0XF9,0XF6,0X3A,0XFF,0X5B,0XFF,0X7C,0XFF,0X5B,0XFF,0X0F,0X8C,
0XA3,0X18,0X83,0X18,0X82,0X10,0XA7,0X51,0XC3,0X20,0X62,0X10,0X82,0X18,0X82,0X18,
0X8D,0XA3,0X30,0XC4,0XEF,0XBB,0XE4,0X28,0X82,0X18,0X62,0X10,0XAF,0X9B,0X12,0XBC,
0XF1,0XBB,0X52,0XC4,0X72,0XC4,0XD0,0XB3,0X6F,0XA3,0X6C,0X7A,0X0B,0X6A,0X0B,0X6A,
0X0A,0X7A,0XEA,0X79,0X2A,0X92,0X4B,0XA2,0X4A,0XAA,0X6A,0XB2,0X6A,0XB2,0X6A,0XB2,
0X6B,0XAA,0X8B,0XA2,0X8C,0X9A,0X8C,0X92,0X8C,0X92,0X8C,0X92,0XCD,0X92,0X2E,0XA3,
0X6E,0XAB,0XD0,0XB3,0X72,0XCC,0XB2,0XCC,0XD2,0XCC,0XD2,0XD4,0XB2,0XCC,0XB2,0XCC,
0XB2,0XCC,0XD3,0XCC,0XF3,0XCC,0XD3,0XC4,0XD3,0XC4,0XB3,0XC4,0XB3,0XC4,0X73,0XBC,
0X32,0XB4,0X12,0XB4,0XF1,0XAB,0XF1,0XAB,0XB1,0XA3,0XD1,0XA3,0X91,0X9B,0X91,0X9B,
0X91,0X9B,0X91,0X9B,0X71,0X93,0X71,0X93,0X71,0X93,0X50,0X8B,0X30,0X83,0XEF,0X7A,
0X6D,0X6A,0XEA,0X59,0X0F,0X8B,0XEF,0X82,0X50,0X93,0X50,0X93,0X50,0X93,0X70,0X9B,
0X70,0X9B,0X50,0X93,0XB1,0XA3,0XF2,0XAB,0X12,0XB4,0XB0,0XAB,0X2E,0X9B,0XB0,0XAB,
0X93,0XC4,0XF4,0XCC,0XD4,0XCC,0X93,0XC4,0X11,0XBC,0XB0,0XAB,0XED,0X92,0XAD,0X8A,
0X2E,0X9B,0X31,0XBC,0X75,0XDD,0XF7,0XE5,0X18,0XE6,0X38,0XE6,0X38,0XEE,0X17,0XE6,
0X17,0XE6,0X37,0XEE,0X38,0XEE,0X37,0XEE,0X17,0XEE,0X17,0XEE,0X17,0XE6,0XF6,0XE5,
0XB5,0XE5,0X74,0XDD,0X33,0XDD,0XB1,0XD4,0X71,0XCC,0X8F,0XB3,0XED,0XA2,0X6B,0X92,
0X0A,0X8A,0X0A,0X92,0X4A,0XAA,0X8A,0XBA,0XEB,0XC2,0XAD,0XCB,0X2F,0XD4,0X11,0XDD,
0X73,0XE5,0X94,0XE5,0XD4,0XE5,0X73,0XE5,0X33,0XDD,0X53,0XDD,0X93,0XE5,0X32,0XE5,
0XD1,0XDC,0X11,0XDD,0XD4,0XED,0X98,0XF6,0X1B,0XF7,0XFA,0XF6,0X99,0XF6,0X57,0XEE,
0X57,0XEE,0X16,0XEE,0X57,0XEE,0X57,0XEE,0X78,0XF6,0X77,0XF6,0XD8,0XFE,0X1A,0XFF,
0X5B,0XFF,0X5B,0XFF,0X3A,0XFF,0X3A,0XFF,0X7C,0XFF,0X71,0X9C,0X83,0X18,0X83,0X18,
0X82,0X10,0XEB,0X82,0XC3,0X20,0X82,0X18,0X82,0X10,0X62,0X10,0X4F,0XB4,0X13,0XDD,
0X70,0XC4,0XA3,0X20,0X83,0X18,0XA3,0X18,0X10,0XBC,0X51,0XC4,0X10,0XC4,0XF0,0XBB,
0XB0,0XB3,0X6F,0XAB,0X6C,0X82,0X0A,0X7A,0XEA,0X81,0X0A,0X92,0X0A,0XA2,0X4A,0XB2,
0X8A,0XBA,0XAA,0XC2,0XCB,0XC2,0XEB,0XBA,0X4D,0XBB,0X2D,0XB3,0X4E,0XB3,0X4E,0XAB,
0X2E,0XA3,0X4F,0XA3,0X6F,0XAB,0X8F,0XB3,0X8F,0XB3,0XF0,0XBB,0X31,0XC4,0X51,0XCC,
0X91,0XCC,0XF2,0XD4,0XF3,0XD4,0X54,0XD5,0X75,0XD5,0X95,0XDD,0X95,0XD5,0X75,0XD5,
0X55,0XCD,0X34,0XCD,0X14,0XCD,0XF4,0XC4,0XD3,0XC4,0XB3,0XBC,0X73,0XBC,0X52,0XB4,
0X11,0XAC,0XD1,0XAB,0XD1,0XA3,0XB1,0XA3,0X90,0X9B,0X70,0X9B,0X70,0X9B,0X70,0X93,
0X50,0X93,0X30,0X8B,0X0F,0X8B,0X0F,0X83,0XCE,0X7A,0X6C,0X6A,0X0B,0X5A,0XC9,0X51,
0X6D,0X72,0X8D,0X7A,0XCE,0X82,0XCE,0X7A,0XCF,0X82,0X0F,0X8B,0X0F,0X8B,0X4F,0X93,
0XB0,0XA3,0XD1,0XAB,0X90,0XAB,0XED,0X9A,0X4F,0XA3,0XF1,0XB3,0X52,0XC4,0X52,0XC4,
0X31,0XBC,0XF0,0XBB,0X6E,0XAB,0XED,0X9A,0X0D,0XA3,0X71,0XC4,0X74,0XDD,0XF6,0XE5,
0X17,0XEE,0X37,0XEE,0X58,0XEE,0X58,0XEE,0X77,0XEE,0X78,0XEE,0X58,0XEE,0X37,0XEE,
0X37,0XEE,0X57,0XEE,0X16,0XEE,0XF6,0XE5,0X15,0XE6,0X15,0XE6,0X16,0XEE,0X36,0XEE,
0XF5,0XE5,0XD1,0XD4,0XAE,0XBB,0X6B,0X9A,0X2A,0X92,0X2A,0X9A,0XCB,0XB2,0X4C,0XC3,
0X0E,0XD4,0X32,0XE5,0XF5,0XED,0X36,0XEE,0XB4,0XE5,0XB3,0XE5,0X35,0XEE,0XD4,0XED,
0X72,0XE5,0X0E,0XB4,0X11,0XE5,0XF1,0XDC,0XF0,0XE4,0X11,0XE5,0XB3,0XED,0X77,0XF6,
0XD9,0XF6,0XB8,0XF6,0X57,0XEE,0X36,0XEE,0XF5,0XE5,0X73,0XDD,0X53,0XDD,0X73,0XE5,
0XB4,0XE5,0XD3,0XE5,0X55,0XEE,0XD8,0XFE,0X3A,0XFF,0X9B,0XFF,0X7B,0XFF,0X39,0XFF,
0XF9,0XFE,0XD8,0XFE,0X19,0XFF,0XF2,0XAC,0X83,0X18,0X83,0X18,0X62,0X10,0X0B,0X7B,
0XE4,0X28,0X62,0X10,0X82,0X18,0X83,0X18,0X53,0XCD,0X36,0XEE,0X32,0XC5,0X83,0X18,
0X82,0X18,0X24,0X31,0X10,0XBC,0X10,0XC4,0X10,0XC4,0X10,0XC4,0X4E,0XB3,0X2A,0X92,
0XE9,0X99,0X09,0XAA,0X49,0XBA,0X89,0XCA,0XEB,0XD2,0X6C,0XD3,0XCD,0XD3,0XEE,0XD3,
0X2F,0XD4,0X50,0XD4,0X70,0XD4,0X91,0XD4,0X91,0XC4,0X71,0XC4,0XF0,0XB3,0XAF,0XAB,
0X8E,0XAB,0X8E,0XAB,0XCF,0XB3,0X10,0XC4,0X92,0XCC,0XD3,0XCC,0X33,0XD5,0X94,0XDD,
0XB5,0XDD,0XF6,0XE5,0XF6,0XDD,0XB5,0XDD,0XB5,0XDD,0X74,0XD5,0X34,0XD5,0XF4,0XCC,
0XD3,0XCC,0XB3,0XC4,0XB2,0XC4,0X93,0XC4,0X52,0XBC,0X52,0XBC,0X31,0XB4,0X11,0XB4,
0XF1,0XAB,0XD1,0XAB,0XD1,0XA3,0XB0,0XA3,0X90,0XA3,0X90,0XA3,0X6F,0X9B,0X4F,0X93,
0X0F,0X8B,0X0E,0X8B,0XEE,0X82,0XCE,0X7A,0X8D,0X7A,0X6C,0X72,0X88,0X41,0XCA,0X49,
0XEB,0X51,0X2B,0X62,0X2C,0X62,0X4C,0X62,0X8D,0X7A,0X6C,0X72,0X6F,0XA3,0X2E,0XA3,
0X8C,0X8A,0X2B,0X6A,0X8C,0X7A,0XEE,0X8A,0XED,0X82,0XAC,0X82,0XAC,0X82,0X4B,0X72,
0X2B,0X7A,0XAD,0X8A,0X8F,0XAB,0X92,0XCC,0X13,0XD5,0X74,0XDD,0XB4,0XE5,0XF5,0XE5,
0X57,0XEE,0X97,0XF6,0X98,0XF6,0X98,0XF6,0XB8,0XF6,0X97,0XF6,0X97,0XF6,0X56,0XEE,
0X15,0XE6,0XF4,0XED,0XD4,0XED,0X14,0XEE,0X36,0XEE,0XD4,0XED,0XB0,0XDC,0XAD,0XCB,
0X8C,0XCB,0X0D,0XD4,0X2E,0XD4,0X31,0XE5,0XD4,0XED,0X76,0XF6,0XB8,0XF6,0XB7,0XF6,
0XB3,0XED,0X4F,0XDC,0X6F,0XDC,0XF0,0XE4,0X51,0XE5,0X31,0XE5,0X71,0XE5,0XC7,0X59,
0X51,0XE5,0X51,0XE5,0XB3,0XED,0XF4,0XED,0X14,0XEE,0X35,0XEE,0X15,0XEE,0X15,0XEE,
0XD4,0XE5,0X32,0XDD,0X91,0XD4,0X71,0XD4,0XB1,0XDC,0XD0,0XDC,0XD0,0XDC,0X92,0XED,
0XD7,0XF6,0X9B,0XFF,0X9B,0XFF,0XBC,0XFF,0XBC,0XFF,0XDD,0XFF,0XFD,0XFF,0XFD,0XFF,
0XFD,0XFF,0XD5,0XBD,0X82,0X18,0X83,0X18,0X62,0X10,0XEE,0X93,0XE4,0X20,0X82,0X10,
0X82,0X10,0XA3,0X18,0XD1,0XC4,0X73,0XE5,0XED,0XAB,0X83,0X18,0X83,0X18,0X25,0X39,
0XAD,0XC3,0X4C,0XC3,0XEB,0XBA,0X6A,0XBA,0X29,0XBA,0X29,0XCA,0X89,0XD2,0XEA,0XDA,
0X4B,0XD3,0X8C,0XD3,0XAD,0XD3,0XEE,0XD3,0XEE,0XCB,0XAE,0XCB,0X4C,0XAB,0XCA,0X8A,
0XE7,0X59,0X04,0X31,0XA2,0X18,0X61,0X10,0X41,0X08,0X81,0X10,0X61,0X08,0X20,0X08,
0X41,0X10,0XA2,0X18,0X45,0X39,0X89,0X62,0X0F,0X9C,0X94,0XD5,0X57,0XEE,0X16,0XE6,
0XD5,0XDD,0X94,0XDD,0X74,0XDD,0X33,0XD5,0X13,0XD5,0XF2,0XCC,0XD2,0XCC,0X92,0XC4,
0X51,0XBC,0X30,0XBC,0XF0,0XB3,0XF0,0XB3,0XD0,0XB3,0XB0,0XAB,0XCF,0XAB,0XAF,0XA3,
0X6F,0X9B,0X2E,0X9B,0X0E,0X9B,0X0E,0X93,0X2E,0X93,0X2E,0X93,0X0E,0X8B,0X0E,0X8B,
0X0E,0X83,0XCE,0X82,0XAD,0X82,0X4C,0X7A,0X47,0X39,0X88,0X39,0X89,0X41,0XA9,0X41,
0XA8,0X49,0XA8,0X49,0XC9,0X51,0XEA,0X59,0X4C,0X6A,0X2B,0X62,0XAA,0X49,0X88,0X39,
0X89,0X41,0XCA,0X51,0XA9,0X51,0XA8,0X51,0XA8,0X51,0XCA,0X59,0X4B,0X6A,0XAC,0X82,
0X0E,0X9B,0XAF,0XAB,0XD2,0XD4,0X93,0XDD,0XD4,0XE5,0XF5,0XE5,0XD4,0XE5,0XD4,0XE5,
0XD4,0XE5,0XF4,0XE5,0X15,0XEE,0X35,0XEE,0X56,0XF6,0X56,0XF6,0X55,0XEE,0X14,0XEE,
0X92,0XE5,0X51,0XE5,0XD0,0XDC,0XED,0XDB,0X0D,0XDC,0X10,0XE5,0XF4,0XED,0XD7,0XF6,
0X7A,0XFF,0XBC,0XFF,0X7A,0XFF,0XF8,0XFE,0X55,0XF6,0X34,0XF6,0X34,0XF6,0X76,0XF6,
0XF8,0XFE,0XF8,0XFE,0XF7,0XFE,0XD7,0XF6,0X95,0XF6,0X89,0X6A,0XF4,0XDD,0XF8,0XFE,
0X5A,0XFF,0X7A,0XFF,0X39,0XFF,0XF8,0XFE,0X76,0XF6,0X72,0XE5,0X90,0XD4,0XCE,0XCB,
0XAD,0XCB,0XCD,0XCB,0XCD,0XDB,0X4E,0XE4,0XD2,0XED,0X18,0XFF,0XBB,0XFF,0XBB,0XFF,
0XDC,0XFF,0XDD,0XFF,0XDD,0XFF,0XBC,0XFF,0X9B,0XFF,0X5A,0XFF,0X3A,0XFF,0XF1,0XB4,
0X82,0X18,0X82,0X18,0X62,0X10,0X0B,0X93,0XC3,0X20,0X82,0X10,0X82,0X18,0XA3,0X20,
0XAC,0XBB,0X8C,0XD3,0X27,0X82,0XA3,0X18,0X83,0X18,0X65,0X51,0XEA,0XDA,0XCA,0XDA,
0XC9,0XDA,0X0A,0XDB,0X8C,0XDB,0X0D,0XE4,0XED,0XD3,0XED,0XD3,0XEE,0XD3,0X4F,0XD4,
0XB0,0XDC,0X8F,0XCC,0XA9,0X82,0X04,0X31,0X21,0X08,0X20,0X00,0X83,0X21,0XC2,0X18,
0X41,0X00,0X00,0X00,0X20,0X00,0XE6,0X5A,0XE4,0X5A,0X04,0X5B,0XE1,0X20,0X40,0X08,
0X00,0X00,0X00,0X00,0X00,0X00,0X61,0X10,0X85,0X31,0X8D,0X83,0X94,0XD5,0X36,0XEE,
0XF5,0XE5,0XD4,0XE5,0X93,0XDD,0X33,0XDD,0XF2,0XD4,0XD1,0XCC,0X50,0XC4,0X10,0XBC,
0X10,0XBC,0XF0,0XB3,0XCF,0XB3,0X8F,0XAB,0X4E,0X9B,0XED,0X9A,0XAD,0X92,0X8C,0X92,
0X8B,0X92,0X8C,0X92,0XAC,0X92,0XED,0X92,0XED,0X8A,0X0D,0X8B,0XEE,0X8A,0X0E,0X8B,
0X2E,0X8B,0X0D,0X93,0X67,0X41,0X67,0X39,0X47,0X39,0X47,0X39,0X47,0X39,0X47,0X31,
0X47,0X31,0X47,0X31,0X47,0X39,0X67,0X39,0X68,0X41,0X88,0X39,0XA8,0X41,0X68,0X39,
0X47,0X39,0X47,0X39,0X88,0X41,0XC9,0X51,0X0B,0X62,0XCD,0X82,0X2E,0X9B,0X8E,0XB3,
0X70,0XCC,0X11,0XDD,0X93,0XE5,0XD4,0XE5,0XD3,0XE5,0XD3,0XED,0XF4,0XED,0X55,0XF6,
0X76,0XF6,0XB7,0XF6,0X96,0XF6,0X34,0XF6,0X92,0XED,0XCF,0XE4,0X4E,0XDC,0X2E,0XDC,
0XD0,0XE4,0XD3,0XED,0X95,0XF6,0XB6,0XF6,0XB7,0XF6,0XF8,0XFE,0XBA,0XFF,0XBB,0XFF,
0X9A,0XFF,0X9A,0XFF,0XBB,0XFF,0XFC,0XFF,0XFC,0XFF,0XFD,0XFF,0XFD,0XFF,0XFD,0XFF,
0XFD,0XFF,0XFC,0XFF,0XFD,0XFF,0X2C,0X73,0XF7,0XCD,0XFD,0XFF,0X9A,0XFF,0XBB,0XFF,
0X59,0XFF,0XD6,0XFE,0X71,0XED,0X0D,0XDC,0X2B,0XDB,0X4B,0XDB,0X0D,0XE4,0XAF,0XEC,
0X51,0XED,0X92,0XED,0X51,0XE5,0X35,0XEE,0XB7,0XF6,0X39,0XFF,0XBB,0XFF,0X9A,0XFF,
0XD7,0XF6,0X35,0XEE,0X35,0XEE,0X54,0XEE,0X92,0XE5,0X8A,0X8A,0X83,0X18,0X82,0X18,
0X82,0X10,0X28,0X82,0XA3,0X18,0X82,0X18,0X83,0X18,0XC3,0X20,0XC9,0XCA,0X2A,0XDB,
0XA6,0X51,0X83,0X18,0XA3,0X18,0X07,0X6A,0X4D,0XEC,0XAF,0XE4,0XCF,0XE4,0X50,0XED,
0X71,0XED,0X50,0XED,0X30,0XE5,0X30,0XE5,0XD0,0XDC,0XED,0XBB,0XC6,0X59,0X61,0X10,
0XE2,0X10,0X23,0X19,0X41,0X08,0X41,0X08,0X66,0X42,0XC7,0X39,0X4A,0X4A,0XC6,0X29,
0X64,0X21,0X05,0X63,0XE4,0X52,0XE5,0X73,0XE5,0X73,0XA4,0X52,0X63,0X21,0X81,0X08,
0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X00,0X61,0X10,0X27,0X52,0XD0,0XC4,0XB3,0XE5,
0X72,0XDD,0X51,0XDD,0XF1,0XD4,0XB0,0XD4,0X90,0XCC,0X2F,0XC4,0XAF,0XBB,0X6E,0XAB,
0X2D,0XA3,0XED,0X9A,0X0D,0X9B,0XAC,0X92,0XAC,0X9A,0XAC,0X9A,0XAC,0X9A,0X8B,0X92,
0XCC,0X8A,0XAC,0X8A,0XCC,0X92,0X0D,0X93,0X0D,0X93,0XED,0X8A,0XAC,0X8A,0X6C,0X82,
0X05,0X29,0X26,0X29,0X26,0X31,0X26,0X31,0X26,0X31,0X26,0X31,0X26,0X31,0X26,0X39,
0X47,0X39,0X67,0X39,0X68,0X39,0X47,0X39,0X67,0X41,0X47,0X39,0X67,0X41,0X89,0X49,
0XA9,0X49,0XA9,0X51,0X0A,0X5A,0X0B,0X6A,0X4B,0X72,0XAC,0X92,0X6D,0XB3,0XCE,0XC3,
0X2F,0XCC,0XD0,0XDC,0X51,0XE5,0X31,0XDD,0X31,0XE5,0XD2,0XED,0X13,0XEE,0X33,0XF6,
0X71,0XED,0X71,0XED,0X91,0XED,0X13,0XF6,0X54,0XF6,0X34,0XF6,0X34,0XF6,0XF3,0XED,
0X50,0XE5,0X71,0XED,0X94,0XF6,0X57,0XFF,0X99,0XFF,0XBA,0XFF,0XFC,0XFF,0XFD,0XFF,
0XFD,0XFF,0XFD,0XFF,0XFD,0XFF,0XFD,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,
0X5C,0XEF,0XAE,0X83,0XD5,0XC5,0X58,0XFF,0XB4,0XFE,0X11,0XF6,0XB0,0XED,0X0E,0XED,
0XCE,0XEC,0X71,0XF5,0XB5,0XFE,0X59,0XFF,0X78,0XFF,0XD5,0XFE,0X73,0XF6,0XF2,0XF5,
0X71,0XED,0XB2,0XED,0X92,0XE5,0XD1,0XD4,0X70,0XC4,0XAE,0XB3,0X2D,0XAB,0XCE,0XC3,
0X11,0XDD,0XD3,0XED,0XD3,0XE5,0XAD,0XB3,0XA3,0X18,0X83,0X18,0X82,0X10,0XC9,0X92,
0XE3,0X20,0X82,0X18,0X83,0X18,0XE4,0X28,0X8B,0XD3,0X4B,0XD3,0X25,0X41,0XA3,0X18,
0XA3,0X18,0X8B,0X83,0X30,0XC5,0X73,0XF6,0X53,0XF6,0X73,0XF6,0X53,0XF6,0X34,0XF6,
0XD2,0XED,0X30,0XDD,0XA9,0X6A,0X41,0X10,0X20,0X00,0X00,0X00,0XE4,0X39,0X66,0X42,
0X67,0X3A,0XC5,0X31,0X86,0X42,0X45,0X42,0X87,0X4A,0X49,0X4A,0X67,0X4A,0XC7,0X73,
0XE5,0X73,0X84,0X73,0XA4,0X73,0XE4,0X39,0XC3,0X29,0X43,0X21,0XC2,0X10,0X00,0X00,
0X61,0X08,0X20,0X00,0X00,0X00,0X00,0X00,0X41,0X08,0X88,0X6A,0X72,0XDD,0XD0,0XD4,
0XAD,0XC3,0XEE,0XC3,0XCE,0XC3,0XEE,0XBB,0X4F,0XC4,0X0F,0XC4,0X8E,0XBB,0X4D,0XAB,
0X2D,0XAB,0X0C,0XA3,0X6A,0X8A,0X4A,0X82,0X4A,0X82,0X6B,0X82,0X8B,0X82,0XAC,0X82,
0X6C,0X82,0XAC,0X82,0XCC,0X8A,0X4B,0X72,0XC9,0X59,0XC8,0X51,0X05,0X29,0X05,0X29,
0X05,0X39,0X25,0X41,0X25,0X41,0X26,0X39,0X26,0X31,0X46,0X31,0X47,0X39,0X47,0X39,
0X47,0X39,0X47,0X39,0X67,0X39,0X67,0X41,0X88,0X49,0XC9,0X59,0XEA,0X69,0X0A,0X6A,
0X0A,0X6A,0X2B,0X6A,0X2B,0X72,0XAC,0X92,0X0C,0XA3,0X8D,0XBB,0X2E,0XCC,0XCF,0XDC,
0XCF,0XDC,0X6F,0XDC,0X6F,0XDC,0XF0,0XDC,0X91,0XE5,0XD2,0XED,0XB1,0XED,0X91,0XE5,
0X8F,0XDC,0XEE,0XCB,0XCD,0XCB,0XCD,0XD3,0X8E,0XDC,0X10,0XE5,0XD2,0XED,0XF2,0XED,
0X74,0XF6,0X98,0XFF,0X98,0XFF,0X78,0XFF,0XB9,0XFF,0XBA,0XFF,0XDA,0XFF,0XFB,0XFF,
0XFC,0XFF,0XFD,0XFF,0XFD,0XFF,0XFD,0XFF,0XFD,0XFF,0XFE,0XFF,0X8A,0X62,0XAD,0X83,
0X71,0XBD,0X15,0XFF,0XF5,0XFE,0XF5,0XFE,0XF5,0XFE,0X57,0XFF,0X98,0XFF,0X98,0XFF,
0XB9,0XFF,0XBA,0XFF,0XBA,0XFF,0X98,0XFF,0X97,0XFF,0X15,0XFF,0XB4,0XF6,0X13,0XEE,
0XED,0XCB,0XEC,0XB2,0X6B,0X9A,0X2A,0X92,0X2A,0X9A,0X2A,0X9A,0X6B,0X9A,0X4A,0X92,
0X4A,0X92,0XE9,0X81,0XA3,0X18,0X82,0X18,0X82,0X10,0XCE,0X7B,0X04,0X29,0X82,0X18,
0X83,0X18,0X65,0X31,0XB4,0XFE,0X55,0XD6,0XE3,0X20,0XA3,0X18,0XA3,0X18,0X73,0XB5,
0X07,0X4A,0XB1,0XCD,0X32,0XF6,0X71,0XED,0X50,0XE5,0XEF,0XE4,0XCB,0XA3,0XA2,0X20,
0X00,0X00,0X00,0X00,0XC4,0X29,0X25,0X32,0XC5,0X42,0XE4,0X52,0X65,0X6B,0X65,0X42,
0X87,0X42,0X4A,0X42,0XE9,0X41,0XC9,0X52,0XA6,0X52,0X65,0X6B,0X26,0X84,0X85,0X63,
0X45,0X7C,0X44,0X32,0X43,0X19,0XA4,0X21,0X81,0X08,0X20,0X00,0X63,0X19,0X03,0X11,
0XE2,0X10,0X61,0X08,0X20,0X00,0X00,0X00,0X81,0X10,0X2A,0X93,0XCD,0XC3,0X2C,0XB3,
0XEC,0XA2,0XCB,0X9A,0X4C,0XAB,0X2C,0XAB,0XCB,0X92,0X6A,0X82,0X6B,0X82,0XAB,0X92,
0X4A,0X82,0XE9,0X69,0XE9,0X61,0XE9,0X59,0X0A,0X62,0X09,0X62,0XC9,0X51,0XE8,0X51,
0XC9,0X51,0XC9,0X51,0XA8,0X49,0X87,0X49,0X04,0X31,0X05,0X31,0X05,0X31,0X05,0X31,
0X25,0X29,0X25,0X29,0X25,0X31,0X26,0X31,0X26,0X31,0X26,0X31,0X46,0X31,0X47,0X31,
0X47,0X31,0X47,0X31,0X47,0X39,0X68,0X39,0X88,0X41,0X88,0X41,0XA8,0X49,0XC9,0X51,
0XEA,0X61,0X2A,0X72,0XAB,0X8A,0X2C,0XAB,0XCD,0XC3,0XEF,0XDC,0XD1,0XED,0XF1,0XE5,
0XD1,0XED,0XF1,0XED,0X91,0XE5,0X6E,0XD4,0X4C,0XB3,0X8B,0XA2,0XAA,0XB2,0X8C,0XCB,
0X0E,0XD4,0X6F,0XD4,0X4E,0XD4,0X6F,0XD4,0XEF,0XDC,0X10,0XDD,0XF0,0XDC,0X91,0XE5,
0X73,0XF6,0X93,0XF6,0X0C,0X9C,0X30,0XDD,0XAF,0XD4,0X31,0XDD,0X54,0XEE,0X78,0XFF,
0XFC,0XFF,0XFC,0XFF,0XFD,0XFF,0XFE,0XFF,0XAA,0X62,0XEB,0X6A,0X54,0XB5,0XFE,0XFF,
0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFD,0XFF,
0XFC,0XFF,0X76,0XFF,0X93,0XF6,0XF5,0XFE,0XD1,0XED,0XCF,0XE4,0X0D,0XD4,0X6A,0XAA,
0X0A,0X9A,0XE9,0X89,0XC9,0X79,0XC9,0X81,0XC9,0X89,0XC8,0X99,0XE8,0XB1,0XE7,0XB1,
0XC3,0X20,0X83,0X18,0X62,0X10,0X8A,0X5A,0XA3,0X18,0X82,0X18,0X82,0X18,0X25,0X31,
0X5B,0XF7,0X6D,0X73,0XA3,0X18,0X83,0X18,0XE3,0X28,0XB4,0XDE,0X06,0X42,0X50,0XC5,
0XD1,0XF5,0X0F,0XE5,0X4D,0XDC,0X06,0X62,0X20,0X00,0X00,0X00,0XE2,0X10,0XC2,0X08,
0X44,0X63,0X25,0X63,0XE7,0X52,0X26,0X3A,0XA7,0X42,0X85,0X3A,0XE6,0X52,0X09,0X63,
0X47,0X4A,0XCA,0X5A,0XA8,0X6B,0X46,0X7C,0XE6,0X94,0X65,0X6B,0X25,0X5B,0X44,0X3A,
0X23,0X19,0X64,0X19,0X23,0X11,0X03,0X11,0X63,0X19,0X43,0X19,0X63,0X19,0X43,0X11,
0X03,0X11,0XE2,0X08,0XA2,0X08,0X20,0X00,0X04,0X31,0X8A,0X8A,0X0C,0XAB,0XCB,0XA2,
0X8A,0X92,0X8A,0X8A,0X8B,0X92,0XAB,0X92,0XEC,0XA2,0XAB,0X9A,0X6A,0X8A,0X09,0X72,
0XC8,0X61,0XA8,0X51,0X66,0X41,0X26,0X39,0X25,0X31,0X25,0X29,0X05,0X21,0X04,0X21,
0XE4,0X20,0XE4,0X20,0XA3,0X18,0XC3,0X18,0XE4,0X20,0X05,0X29,0X05,0X29,0X05,0X29,
0X05,0X29,0X05,0X21,0XC4,0X18,0X82,0X10,0X62,0X10,0X41,0X08,0X82,0X20,0XA2,0X28,
0X41,0X08,0XA2,0X18,0XC4,0X20,0X05,0X29,0X46,0X39,0X67,0X39,0X67,0X39,0X67,0X39,
0X88,0X41,0XC9,0X51,0X29,0X62,0X6A,0X6A,0X6A,0X72,0X4A,0X72,0X8A,0X82,0X6A,0X92,
0X09,0X8A,0XE9,0X79,0XC9,0X71,0X0A,0X7A,0X6A,0X8A,0X8B,0X8A,0X4A,0X82,0X4A,0X7A,
0X6B,0X82,0X4B,0X7A,0X4B,0X7A,0X4B,0X7A,0X6B,0X8A,0X8B,0X9A,0XEB,0XAA,0XAD,0XC3,
0X89,0X7A,0XAD,0XC3,0X2C,0XAB,0XEC,0XA2,0X4C,0XBB,0XAE,0XDC,0X32,0XF6,0X57,0XFF,
0XB8,0XFF,0XD9,0XFF,0XA9,0X6A,0X28,0X52,0XF2,0XA4,0XFD,0XFF,0XFD,0XFF,0XFE,0XFF,
0XFE,0XFF,0XFF,0XFF,0XFE,0XFF,0XFE,0XFF,0XFE,0XFF,0XFD,0XFF,0XFB,0XFF,0XDA,0XFF,
0X56,0XFF,0X31,0XEE,0X4F,0XE5,0XCD,0XCB,0XCB,0XAA,0XE9,0X91,0XE9,0X99,0X08,0XB2,
0X08,0XC2,0X28,0XD2,0X28,0XDA,0X28,0XD2,0X28,0XCA,0X08,0XB2,0XE3,0X28,0X82,0X18,
0X82,0X10,0XC3,0X20,0X82,0X18,0X82,0X18,0X82,0X10,0XE4,0X20,0X54,0XAD,0XC3,0X20,
0X83,0X18,0XA3,0X18,0X48,0X52,0X97,0XF7,0X07,0X4A,0X13,0XCE,0X34,0XFF,0X6E,0XDD,
0X23,0X41,0X00,0X00,0X20,0X00,0X20,0X00,0XA6,0X42,0XA6,0X42,0XA6,0X32,0XE3,0X29,
0XE2,0X52,0XE4,0X5A,0X46,0X63,0XE8,0X52,0X48,0X42,0XED,0X52,0X6A,0X4A,0X48,0X63,
0X46,0X84,0XC6,0X73,0XE7,0X94,0X66,0X84,0X45,0X63,0X25,0X32,0X84,0X19,0X83,0X21,
0XA4,0X21,0X23,0X11,0X84,0X19,0X43,0X11,0X63,0X11,0X84,0X19,0X63,0X19,0X64,0X11,
0X64,0X09,0X41,0X00,0X00,0X00,0X61,0X18,0XE8,0X71,0X29,0X92,0X66,0X49,0X09,0X7A,
0X8A,0X9A,0XEB,0XAA,0X8A,0X9A,0X4A,0X8A,0X49,0X8A,0X09,0X7A,0XA8,0X69,0X87,0X59,
0X66,0X51,0X46,0X49,0X25,0X41,0X05,0X31,0X04,0X21,0XC3,0X18,0X04,0X21,0XE4,0X28,
0X82,0X18,0X42,0X10,0X62,0X10,0XA3,0X10,0XE4,0X20,0XE4,0X20,0XA3,0X10,0X21,0X00,
0X00,0X00,0X60,0X08,0XC2,0X30,0X45,0X79,0X86,0X99,0X85,0X71,0X26,0X3A,0X66,0X62,
0X23,0X41,0X22,0X29,0XA2,0X20,0XC4,0X20,0X46,0X31,0X47,0X39,0X47,0X39,0X47,0X39,
0X47,0X39,0X47,0X39,0X67,0X41,0X87,0X41,0X87,0X41,0X87,0X49,0X88,0X49,0X88,0X41,
0XA8,0X51,0X47,0X41,0XA8,0X59,0XA8,0X61,0XA8,0X69,0XA8,0X69,0XC9,0X61,0XC9,0X61,
0XC9,0X59,0XC9,0X61,0XC9,0X71,0XE9,0X71,0XC9,0X79,0XEA,0X81,0X87,0X59,0XCB,0XA2,
0XEC,0XAA,0XCB,0XA2,0XEB,0XB2,0X0B,0XBB,0X4B,0XCB,0X0C,0XDC,0XCF,0XF5,0X75,0XFF,
0X4B,0X73,0X27,0X52,0X90,0X94,0XFC,0XFF,0XFC,0XFF,0XFC,0XFF,0XFC,0XFF,0XFC,0XFF,
0XFD,0XFF,0XFD,0XFF,0XFE,0XFF,0XFE,0XFF,0XFD,0XFF,0X97,0XFF,0X11,0XF6,0XEC,0XDB,
0X09,0XDB,0X68,0XCA,0X27,0XC2,0X48,0XDA,0X48,0XD2,0X28,0XCA,0X08,0XC2,0X08,0XB2,
0XC9,0X99,0XE9,0X91,0X2A,0X92,0X6B,0XAB,0X24,0X31,0X83,0X18,0X82,0X18,0X82,0X18,
0X82,0X18,0X82,0X10,0X82,0X18,0X83,0X18,0XC3,0X20,0X83,0X18,0XA3,0X18,0XC3,0X20,
0X70,0XB5,0XB6,0XFF,0XE6,0X41,0X10,0XD6,0XEE,0XDD,0X22,0X31,0X00,0X00,0X20,0X00,
0X45,0X32,0XC4,0X29,0X65,0X53,0X25,0X53,0XA5,0X6B,0XE2,0X18,0X41,0X08,0X61,0X08,
0X42,0X21,0XC7,0X63,0X07,0X5B,0X68,0X6B,0X2B,0X63,0X69,0X6B,0X28,0X95,0X07,0X95,
0XA6,0X8C,0XE6,0X9C,0XE4,0X52,0X24,0X32,0XC4,0X19,0X43,0X11,0XA4,0X11,0X84,0X19,
0X63,0X19,0XA4,0X21,0X84,0X11,0X84,0X11,0X84,0X09,0X23,0X09,0X84,0X11,0X20,0X00,
0X00,0X00,0X00,0X00,0X41,0X08,0X87,0X61,0X83,0X18,0X87,0X69,0XA7,0X71,0XC8,0X79,
0XC7,0X79,0X87,0X61,0X66,0X59,0X87,0X51,0X66,0X49,0X46,0X49,0X25,0X41,0X25,0X39,
0X05,0X31,0X05,0X31,0XE4,0X28,0X41,0X08,0X04,0X29,0X04,0X29,0X82,0X10,0X41,0X08,
0X41,0X08,0X62,0X10,0X62,0X10,0X41,0X08,0X60,0X10,0X01,0X21,0X02,0X29,0XC1,0X28,
0X63,0X51,0X86,0X89,0XA6,0XA1,0XA6,0X91,0XC9,0X6A,0X09,0X63,0X46,0X72,0XE6,0X81,
0XC4,0X61,0XE2,0X28,0X41,0X08,0X04,0X39,0X46,0X41,0X47,0X39,0X47,0X39,0X47,0X39,
0X47,0X39,0X47,0X39,0X47,0X39,0X67,0X39,0X67,0X41,0X87,0X41,0X87,0X49,0XE5,0X28,
0X87,0X61,0X87,0X71,0XA7,0X79,0X87,0X79,0X88,0X79,0XA8,0X79,0XA8,0X71,0XA8,0X71,
0XA8,0X69,0XC9,0X69,0XEA,0X61,0X0A,0X6A,0X87,0X51,0X6A,0X92,0X4A,0X9A,0X4A,0X9A,
0X09,0X92,0X2A,0X8A,0XAB,0XA2,0X2C,0XC3,0X4D,0XD4,0XF0,0XF5,0X8A,0X7B,0XC6,0X49,
0X0D,0X8C,0XF9,0XFF,0XFA,0XFF,0XD9,0XFF,0X98,0XFF,0XB8,0XFF,0XD9,0XFF,0XD9,0XFF,
0XD8,0XFF,0X75,0XFF,0X13,0XFF,0XB0,0XFE,0XCF,0XF5,0X6F,0XF5,0X0F,0XED,0XEC,0XB3,
0XEB,0XAB,0X8D,0XE4,0X2B,0XCB,0XCB,0XBA,0XEB,0XBA,0XAB,0XC3,0X6D,0XD4,0X4F,0XE5,
0X92,0XF6,0X97,0XFF,0XE9,0X62,0X83,0X18,0X82,0X18,0X62,0X10,0X82,0X10,0X82,0X18,
0X82,0X18,0X82,0X18,0X83,0X18,0X83,0X18,0XA3,0X18,0X29,0X6B,0X96,0XFF,0X55,0XF7,
0X85,0X39,0XEC,0XAC,0X42,0X29,0X00,0X00,0XC2,0X18,0XC1,0X10,0XE5,0X73,0X05,0X53,
0X07,0X43,0X86,0X2A,0XC6,0X3A,0XA2,0X10,0X62,0X10,0XA2,0X10,0X05,0X32,0X0A,0X53,
0XAC,0X4A,0X6B,0X52,0XCB,0X73,0XC7,0X73,0XC7,0X94,0X48,0X9D,0X87,0X84,0X46,0X9D,
0XE5,0X4A,0X04,0X22,0X24,0X2A,0XE4,0X21,0XE4,0X21,0XC4,0X11,0X64,0X11,0XC4,0X11,
0XA4,0X11,0XA4,0X11,0XE5,0X11,0XC5,0X09,0X84,0X09,0XC5,0X09,0X03,0X09,0XC2,0X00,
0X41,0X00,0X20,0X00,0X61,0X10,0XC8,0X81,0XE8,0X91,0XE8,0X91,0XA7,0X81,0X87,0X69,
0X66,0X59,0X46,0X51,0X26,0X49,0X26,0X41,0X25,0X39,0X25,0X39,0X25,0X31,0X25,0X31,
0X05,0X29,0X41,0X08,0X04,0X21,0X04,0X31,0X62,0X10,0X61,0X08,0X41,0X08,0X21,0X08,
0X60,0X10,0XA2,0X41,0XE2,0X49,0XE3,0X51,0XA2,0X49,0XA2,0X49,0X83,0X51,0X85,0X81,
0X07,0XB2,0XE7,0XA9,0X09,0X7B,0X4B,0X5B,0XA7,0X7A,0X46,0X7A,0XA5,0X69,0X05,0X72,
0X43,0X49,0X20,0X00,0XA3,0X20,0X46,0X39,0X47,0X39,0X47,0X39,0X47,0X39,0X67,0X41,
0X66,0X59,0X87,0X69,0X87,0X61,0X66,0X61,0X87,0X71,0XE4,0X40,0X67,0X69,0X87,0X71,
0X87,0X61,0X67,0X51,0X67,0X41,0X88,0X49,0XA8,0X49,0XA8,0X51,0XC9,0X51,0XC9,0X59,
0XA8,0X61,0XC9,0X69,0X67,0X51,0X87,0X61,0XC9,0X71,0XC9,0X69,0XC9,0X71,0XC9,0X81,
0X29,0X92,0X49,0XAA,0X69,0XB2,0X8A,0XAA,0XA6,0X59,0XE3,0X20,0XC8,0X72,0XF0,0XED,
0XCF,0XED,0XCF,0XED,0X70,0XF6,0XF1,0XFE,0X12,0XFF,0X12,0XFF,0X13,0XFF,0X75,0XFF,
0X96,0XFF,0XB7,0XFF,0X96,0XFF,0X96,0XFF,0X75,0XFF,0XEB,0X8B,0X2D,0XCD,0X10,0XF6,
0X31,0XF6,0X10,0XF6,0X91,0XFE,0XF2,0XFE,0X50,0XFE,0X8E,0XF5,0X4C,0XEC,0X4C,0XEC,
0X49,0XB3,0XC3,0X20,0X82,0X10,0X82,0X18,0X82,0X18,0X82,0X18,0X82,0X18,0X83,0X18,
0XA3,0X18,0XA3,0X18,0XC9,0X62,0XF4,0XE6,0XB6,0XFF,0X36,0XF7,0X85,0X31,0X02,0X21,
0X00,0X00,0X81,0X10,0X67,0X5B,0X86,0X42,0X46,0X4B,0X45,0X5B,0XE5,0X6B,0XA5,0X63,
0XC5,0X4A,0X43,0X19,0XA2,0X10,0XA2,0X08,0XC6,0X42,0X48,0X63,0XAA,0X6B,0X09,0X4B,
0XED,0X7B,0X6A,0X84,0XE8,0X8C,0X08,0XB6,0XA6,0X84,0XC5,0X94,0X65,0X5B,0X04,0X2A,
0X45,0X1A,0XE5,0X11,0XC4,0X19,0X45,0X22,0X25,0X12,0X25,0X12,0X05,0X0A,0XC5,0X09,
0XE5,0X09,0XC5,0X09,0XC5,0X09,0X06,0X0A,0X06,0X02,0XC5,0X09,0XE6,0X09,0X41,0X00,
0X20,0X00,0X66,0X61,0XA7,0X79,0XA7,0X79,0X87,0X71,0X87,0X71,0X86,0X69,0X66,0X61,
0X66,0X59,0X66,0X59,0X46,0X59,0X46,0X51,0X45,0X49,0X25,0X41,0X05,0X39,0X41,0X08,
0XE4,0X20,0XE4,0X28,0X21,0X10,0X41,0X08,0X00,0X00,0X60,0X08,0X01,0X21,0X43,0X52,
0X43,0X5A,0X23,0X5A,0X03,0X52,0XE3,0X59,0X03,0X5A,0XA5,0X79,0X27,0XBA,0X68,0XC2,
0X2A,0X9B,0X8B,0X73,0X29,0X7B,0X02,0X31,0XA1,0X18,0X25,0X82,0X45,0X82,0XE4,0X71,
0X20,0X08,0XA3,0X20,0X46,0X41,0X46,0X49,0X66,0X49,0X66,0X59,0X66,0X61,0X67,0X51,
0X87,0X51,0X87,0X59,0X87,0X59,0XE4,0X28,0X66,0X41,0X87,0X49,0X67,0X41,0X46,0X39,
0X25,0X31,0X87,0X49,0X87,0X49,0XA8,0X49,0XA8,0X51,0X87,0X61,0X87,0X69,0X87,0X71,
0X67,0X51,0X46,0X49,0XA8,0X69,0XA8,0X61,0XA8,0X69,0XC8,0X69,0X46,0X51,0XC9,0X79,
0XC9,0X81,0X09,0X9A,0X66,0X59,0X82,0X18,0X86,0X59,0X6B,0XC3,0X4B,0XCB,0X8B,0XCB,
0XAB,0XD3,0X4A,0XCB,0X4A,0XCB,0X6A,0XD3,0XCB,0XD3,0XAB,0XE4,0XEE,0XF5,0X90,0XFE,
0XF1,0XFE,0XD0,0XFE,0XB0,0XFE,0X08,0X73,0X91,0XE5,0X91,0XED,0XEF,0XEC,0X2B,0XEC,
0X8A,0XDB,0X09,0XDB,0XA9,0XD2,0X68,0XCA,0X68,0XCA,0X49,0XC2,0X69,0XC2,0X28,0XAA,
0X87,0X71,0XA3,0X18,0X82,0X18,0X62,0X10,0X82,0X18,0X62,0X18,0X8B,0X7B,0X53,0XD6,
0X96,0XFF,0XD6,0XFF,0XD7,0XFF,0X38,0XEF,0XE3,0X20,0X81,0X10,0X04,0X32,0X23,0X3A,
0X64,0X84,0XE5,0X73,0XE6,0X6B,0X07,0X4B,0XC7,0X3A,0X07,0X43,0X44,0X3A,0XE2,0X10,
0X41,0X08,0XC2,0X10,0X6A,0X5B,0XAA,0X42,0X2F,0X63,0X0B,0X53,0XE9,0X73,0XC8,0X8C,
0X48,0X74,0XAA,0X9D,0XE8,0X8C,0X46,0X74,0XE5,0X3A,0X85,0X32,0XC4,0X3A,0XA5,0X32,
0X25,0X1A,0X45,0X12,0X05,0X0A,0XE5,0X11,0X46,0X0A,0X66,0X0A,0X67,0X02,0X46,0X0A,
0XE5,0X09,0X26,0X0A,0X26,0X02,0X26,0X0A,0X26,0X02,0X00,0X00,0X00,0X00,0X41,0X10,
0X46,0X69,0X87,0X71,0XA7,0X69,0X66,0X69,0X86,0X69,0X66,0X61,0X87,0X61,0X87,0X71,
0XA7,0X71,0XA7,0X71,0X87,0X71,0XA7,0X69,0X87,0X61,0X62,0X10,0X86,0X51,0XA6,0X69,
0X41,0X10,0X00,0X00,0X01,0X21,0X04,0X63,0X04,0X6B,0XC3,0X62,0XE3,0X6A,0XE3,0X6A,
0X83,0X6A,0X63,0X6A,0X43,0X72,0X05,0X7A,0X27,0XBA,0XC9,0XCA,0X4A,0XB3,0XED,0X7B,
0X8A,0X7B,0XC4,0X49,0XE2,0X20,0X45,0X72,0X26,0X92,0X45,0X92,0X26,0X8A,0X81,0X18,
0XE4,0X48,0XA7,0X99,0X87,0X91,0X66,0X89,0X86,0X89,0X66,0X81,0X67,0X79,0X66,0X71,
0X87,0X61,0XE4,0X30,0X66,0X49,0X87,0X51,0X87,0X49,0X67,0X41,0X25,0X39,0X87,0X51,
0X87,0X49,0X87,0X49,0X87,0X49,0XA8,0X51,0XA8,0X51,0X25,0X41,0X66,0X59,0X25,0X49,
0XA8,0X71,0X88,0X69,0XA8,0X79,0XA8,0X79,0X26,0X49,0XC9,0X79,0X09,0X8A,0XE9,0X99,
0X66,0X61,0X83,0X18,0X25,0X51,0X49,0XBA,0X49,0XBA,0X89,0XCA,0X89,0XCA,0X6A,0XDB,
0X8B,0XE4,0XCE,0XED,0X90,0XF6,0X32,0XFF,0XB0,0XFE,0XEE,0XF5,0X6B,0XEC,0X49,0XDB,
0XE9,0XD2,0X65,0X51,0X89,0XC2,0X49,0XB2,0X09,0XAA,0X08,0XA2,0X09,0XA2,0X09,0XAA,
0X09,0XB2,0X49,0XB2,0X89,0XBA,0X89,0XC2,0XA9,0XC2,0X09,0XCB,0X49,0XCB,0XE3,0X28,
0X82,0X18,0X82,0X18,0X82,0X18,0X82,0X18,0X6C,0X94,0X54,0XFF,0X75,0XFF,0X12,0XFF,
0XD8,0XFF,0X4E,0X8C,0X61,0X08,0XE2,0X39,0X46,0X7C,0X46,0X5B,0X67,0X53,0X06,0X53,
0X85,0X63,0X24,0X74,0XA5,0X6B,0XE6,0X73,0XE8,0X4A,0X29,0X53,0XE8,0X42,0X46,0X32,
0X08,0X6C,0X68,0X63,0XAA,0X6B,0XCD,0X6B,0X0C,0X74,0XCB,0XA5,0X28,0X95,0X28,0X95,
0X27,0X95,0XA6,0X53,0X25,0X4B,0X65,0X22,0X05,0X1A,0X65,0X22,0X65,0X22,0XE5,0X32,
0XA5,0X2A,0X86,0X12,0X46,0X0A,0X46,0X0A,0X05,0X0A,0X46,0X0A,0X67,0X02,0XA7,0X02,
0XA8,0X02,0X67,0X02,0X26,0X02,0XA5,0X01,0XC2,0X00,0X41,0X00,0X62,0X10,0X46,0X41,
0X46,0X49,0X46,0X49,0X46,0X49,0X66,0X49,0X66,0X49,0X26,0X41,0X05,0X31,0X66,0X51,
0X67,0X51,0X86,0X51,0X46,0X51,0X41,0X08,0X25,0X39,0X46,0X51,0X21,0X08,0XC1,0X18,
0X42,0X4A,0XC4,0X73,0XA4,0X73,0X83,0X7B,0X23,0X73,0X04,0X7B,0XA4,0X7A,0X84,0X72,
0X64,0X7A,0X45,0X7A,0X87,0XB2,0X08,0XC3,0X28,0XBB,0X0D,0X8C,0XCC,0X7B,0XA7,0X72,
0X86,0X62,0XC7,0X8A,0X66,0X92,0X46,0X9A,0X46,0X9A,0X26,0XA2,0XC2,0X38,0X86,0X79,
0XC7,0X99,0X87,0X81,0X87,0X71,0X87,0X69,0X87,0X59,0X87,0X59,0X87,0X59,0XC4,0X28,
0X66,0X49,0X87,0X51,0X87,0X51,0X66,0X51,0X05,0X41,0X87,0X69,0X87,0X71,0X87,0X71,
0X87,0X79,0X87,0X89,0X87,0X91,0X25,0X69,0X66,0X79,0XE5,0X38,0XA8,0X71,0XA8,0X71,
0XA7,0X69,0XA8,0X71,0X25,0X51,0XC8,0XA1,0XC8,0XA1,0XE8,0X99,0X25,0X59,0X83,0X18,
0XE4,0X40,0X29,0XB2,0X28,0XBA,0X29,0XBA,0X29,0XB2,0X49,0XB2,0XA6,0X69,0X28,0X92,
0XA9,0XBA,0X09,0XD3,0X88,0XCA,0XA5,0X69,0X28,0XBA,0X28,0XC2,0XE8,0XA1,0X45,0X51,
0XE8,0XB1,0X08,0XBA,0X28,0XC2,0X28,0XCA,0X48,0XD2,0X88,0XDA,0XE8,0XE2,0X09,0XE3,
0X29,0XE3,0X49,0XE3,0X89,0XDB,0X69,0XDB,0X49,0XCB,0XC3,0X20,0X82,0X18,0X82,0X18,
0X82,0X18,0X83,0X18,0X4E,0XB5,0X6F,0XFE,0X12,0XFF,0X95,0XFF,0X72,0XD6,0X44,0X42,
0X87,0X53,0X65,0X5B,0X84,0X84,0X84,0X84,0X45,0X7C,0XE6,0X73,0X46,0X5B,0X88,0X53,
0XE7,0X42,0X67,0X5B,0XC6,0X6B,0XC6,0X6B,0XE7,0X73,0XE7,0X4A,0X4C,0X63,0X2E,0X5B,
0XEC,0X52,0X4B,0X7C,0X07,0X74,0X68,0X9D,0X69,0X9D,0X09,0X8D,0XC9,0XA5,0X45,0X43,
0XC5,0X32,0XE4,0X42,0XC5,0X32,0XC4,0X3A,0X65,0X1A,0X05,0X12,0X86,0X12,0X45,0X1A,
0XC5,0X22,0XC5,0X22,0XA7,0X0A,0X87,0X0A,0X87,0X02,0X06,0X0A,0XA8,0X02,0XA8,0X02,
0XE9,0X02,0XE9,0X02,0XC8,0X02,0X23,0X01,0X00,0X00,0XA3,0X20,0X26,0X41,0X25,0X41,
0X25,0X39,0X25,0X39,0X25,0X39,0X25,0X31,0XA3,0X18,0X25,0X41,0X46,0X49,0X45,0X51,
0X25,0X49,0X41,0X08,0XC4,0X30,0X25,0X49,0X00,0X00,0X23,0X63,0X43,0X84,0XE3,0X7B,
0XC4,0X7B,0XA3,0X83,0X43,0X7B,0XE3,0X7A,0XC4,0X7A,0X64,0X72,0X44,0X7A,0X64,0X7A,
0X66,0XAA,0X09,0XCB,0X09,0XCB,0XEC,0X9B,0XED,0X7B,0X29,0X8B,0X08,0X9B,0X28,0X93,
0XA7,0X9A,0X27,0XA2,0X46,0XA2,0X86,0XAA,0X47,0XB2,0X45,0X71,0X66,0X69,0X87,0X61,
0X87,0X59,0X87,0X59,0X87,0X59,0X87,0X61,0X87,0X61,0XC4,0X28,0X66,0X51,0X87,0X71,
0X87,0X89,0X66,0X89,0X25,0X69,0X87,0XA1,0XA7,0XA9,0XA7,0XB1,0XA8,0XB9,0XA8,0XB9,
0XC8,0XB9,0X66,0X81,0XA7,0X89,0XE4,0X48,0XC8,0XB1,0XE8,0XA9,0XC8,0X99,0XC8,0X99,
0X05,0X49,0XC8,0X81,0XC8,0X81,0XC9,0X89,0X05,0X41,0X82,0X18,0XE4,0X38,0XE9,0X91,
0XE9,0X91,0XE8,0X99,0XE8,0X99,0X08,0XAA,0X45,0X51,0XA6,0X79,0X08,0XB2,0X28,0XBA,
0X48,0XC2,0XE3,0X28,0X07,0XBA,0X48,0XDA,0XE6,0X99,0X45,0X49,0X27,0XBA,0X48,0XDA,
0X48,0XDA,0X68,0XDA,0X68,0XDA,0X68,0XDA,0X48,0XDA,0X48,0XDA,0X48,0XD2,0X68,0XD2,
0X68,0XCA,0XA8,0XCA,0XE8,0XD2,0XE3,0X28,0X82,0X18,0X62,0X10,0X82,0X18,0X83,0X18,
0XEC,0XBC,0X6E,0XF6,0X6E,0XF6,0X6D,0XFE,0X84,0X52,0XE2,0X73,0X24,0X9D,0XC5,0X8C,
0XE6,0X6B,0X88,0X5B,0X06,0X53,0X26,0X74,0X04,0X74,0X65,0X84,0X06,0X74,0X87,0X63,
0XAA,0X5B,0XE8,0X4A,0XA8,0X5B,0X86,0X63,0XC7,0X73,0X2A,0X7C,0X2A,0X53,0X90,0X94,
0X0B,0X95,0X49,0X95,0X09,0XAE,0XE7,0X8C,0X26,0X8D,0X85,0X53,0X45,0X2A,0XA5,0X2A,
0X25,0X1A,0X85,0X2A,0XE4,0X3A,0XE5,0X32,0X06,0X2B,0X66,0X12,0X46,0X12,0X86,0X0A,
0X46,0X12,0X86,0X12,0XE6,0X12,0XE8,0X02,0XC8,0X02,0XC8,0X02,0X46,0X02,0X88,0X02,
0X09,0X03,0XE3,0X00,0X00,0X00,0X00,0X00,0XE4,0X30,0X05,0X39,0X05,0X39,0X05,0X39,
0X05,0X39,0X05,0X39,0X82,0X18,0X25,0X39,0X25,0X39,0X25,0X41,0X25,0X39,0X41,0X08,
0XE4,0X28,0X25,0X49,0X21,0X29,0X03,0X9D,0X63,0X94,0X23,0X8C,0XE3,0X83,0X83,0X83,
0X23,0X83,0XE4,0X82,0X84,0X7A,0X64,0X7A,0X24,0X72,0X44,0X7A,0X46,0X9A,0X29,0XD3,
0X2A,0XD3,0XAC,0XB3,0X4F,0X7C,0X8B,0X8B,0X88,0X9A,0X08,0XA3,0X07,0XA3,0XA7,0XAA,
0X27,0XAA,0X66,0XAA,0X46,0XB2,0X05,0X92,0XE4,0X50,0X87,0X91,0X86,0X89,0X87,0X81,
0X87,0X81,0X66,0X79,0X66,0X69,0XE4,0X30,0X86,0XA1,0XC8,0XC1,0X07,0XC2,0X27,0XB2,
0XE6,0X79,0X08,0XD3,0X08,0XD3,0X08,0XCB,0X48,0XCB,0X88,0XB2,0X68,0XB2,0X67,0X9A,
0XE7,0X89,0XE4,0X38,0XE8,0X99,0XE8,0XA1,0X08,0XAA,0X08,0XBA,0X25,0X61,0X28,0XC2,
0X28,0XC2,0X08,0XC2,0X45,0X61,0X62,0X10,0XE4,0X40,0X28,0XCA,0X28,0XC2,0X28,0XC2,
0X08,0XB2,0X28,0XB2,0X45,0X51,0X86,0X71,0X08,0XA2,0X48,0XBA,0X48,0XB2,0XE3,0X28,
0X27,0XAA,0XA8,0XDA,0XA5,0X69,0XE6,0X91,0XA8,0XD2,0XA8,0XD2,0XC8,0XDA,0XE8,0XDA,
0X28,0XE3,0X28,0XE3,0X28,0XE3,0X08,0XE3,0X28,0XE3,0X48,0XE3,0X48,0XE3,0X68,0XE3,
0X68,0XE3,0XE4,0X30,0X82,0X18,0X62,0X10,0X62,0X10,0X82,0X18,0X08,0XBC,0XEA,0XEC,
0XCA,0XEC,0X07,0XB4,0X61,0X10,0X86,0X5B,0XA6,0X5B,0X05,0X74,0X84,0X84,0XA5,0X8C,
0X45,0X7C,0X47,0X74,0XA7,0X5B,0X27,0X4B,0XA7,0X5B,0XC5,0X63,0X66,0X84,0XE6,0X6B,
0X87,0X63,0XAB,0X63,0XCC,0X52,0XB0,0X6B,0X6A,0X6B,0X88,0X8C,0XA7,0XAD,0XC8,0X8C,
0X2C,0XB6,0X6A,0X95,0XC6,0X84,0X05,0X43,0XC4,0X3A,0X44,0X53,0X05,0X43,0X45,0X2A,
0X86,0X22,0X25,0X12,0X25,0X1A,0XE5,0X32,0XE5,0X22,0XE6,0X2A,0XA6,0X12,0X46,0X0A,
0XA7,0X0A,0X66,0X0A,0XA7,0X0A,0X07,0X0B,0X29,0X03,0X09,0X03,0XE9,0X02,0XE9,0X02,
0X44,0X01,0X00,0X00,0X41,0X10,0X05,0X41,0X05,0X39,0X05,0X39,0X05,0X39,0X05,0X39,
0XA2,0X18,0X04,0X31,0X05,0X39,0X24,0X39,0X04,0X31,0X41,0X08,0XC3,0X20,0X05,0X41,
0XC2,0X5A,0X03,0X9D,0XA3,0X9C,0X23,0X94,0XC4,0X8B,0XA3,0X83,0X04,0X83,0XE4,0X82,
0X84,0X7A,0X64,0X7A,0X44,0X7A,0X24,0X82,0X25,0X8A,0X28,0XCB,0X8A,0XD3,0X4A,0XC3,
0X4E,0X9C,0X4D,0X94,0X09,0X9B,0XE8,0XA2,0X07,0XA3,0XC7,0XA2,0X67,0XB2,0X47,0XBA,
0X47,0XC2,0X27,0XAA,0XA2,0X28,0XA7,0XDB,0X87,0XE3,0X68,0XE3,0X48,0XE3,0X48,0XDB,
0X05,0X7A,0X43,0X39,0X47,0XDC,0XA7,0XEC,0X47,0XDC,0X67,0XD4,0X86,0X8A,0X28,0XE4,
0XC8,0XDB,0XA8,0XDB,0X48,0XCB,0X68,0XAA,0X48,0XA2,0XC7,0X81,0XA6,0X79,0XC4,0X38,
0X48,0XC2,0XC6,0X99,0X07,0XAA,0X27,0XB2,0X05,0X51,0XC8,0XC2,0XE9,0XCA,0XA6,0X71,
0X45,0X51,0X62,0X10,0XE3,0X30,0X49,0XC3,0XE9,0XBA,0XC9,0XB2,0XE8,0XC2,0XE8,0XCA,
0XA5,0X71,0XE5,0X69,0XA7,0XBA,0X28,0XDB,0X28,0XD3,0XC3,0X20,0X66,0XA2,0X28,0XDB,
0X64,0X51,0XC7,0XBA,0X48,0XE3,0X68,0XE3,0X68,0XE3,0X68,0XE3,0X48,0XE3,0X68,0XE3,
0X68,0XE3,0X48,0XDB,0X08,0XDB,0XE8,0XDA,0X08,0XDB,0X08,0XDB,0X08,0XDB,0XC3,0X28,
0X82,0X18,0X62,0X10,0X82,0X10,0X82,0X18,0X87,0XAA,0X08,0XD3,0X28,0XD3,0X42,0X41,
0X42,0X29,0X64,0X9D,0X04,0X95,0X66,0X7C,0X87,0X5B,0XA8,0X5B,0X86,0X5B,0X05,0X7C,
0X45,0X7C,0X45,0X7C,0X28,0X74,0X89,0X5B,0X8A,0X5B,0X69,0X5B,0XA6,0X63,0X48,0X7C,
0XA7,0X63,0XEB,0X73,0XEE,0X7B,0X8D,0X84,0X0B,0XAE,0X88,0XA5,0XE7,0XAD,0XC6,0XA5,
0X06,0X6C,0X05,0X4B,0X65,0X2A,0X65,0X2A,0X45,0X2A,0XA4,0X3A,0X24,0X4B,0X04,0X3B,
0XA5,0X22,0XC6,0X1A,0X66,0X0A,0X25,0X0A,0XC6,0X22,0XC6,0X1A,0X47,0X1B,0X07,0X13,
0XC8,0X02,0XE8,0X02,0XC8,0X02,0XA8,0X02,0X49,0X03,0X8A,0X03,0X44,0X01,0X00,0X00,
0X00,0X00,0XE4,0X30,0X25,0X49,0X25,0X49,0X25,0X49,0X25,0X51,0XA3,0X20,0XE4,0X30,
0X25,0X41,0X25,0X41,0X04,0X31,0X21,0X08,0X04,0X31,0X25,0X41,0X81,0X52,0XA2,0X7B,
0X03,0X84,0X43,0X94,0X03,0X94,0X83,0X83,0XC3,0X6A,0X83,0X6A,0XA4,0X72,0X64,0X7A,
0X24,0X7A,0X24,0X7A,0X25,0X82,0X87,0XAA,0X29,0XAB,0XC8,0XA2,0XCD,0X9B,0X2E,0X8C,
0X6A,0X9B,0XE9,0XAA,0XC7,0XB2,0XC7,0XB2,0X87,0XB2,0X47,0XB2,0X27,0XB2,0X27,0XB2,
0XA1,0X30,0X64,0X9B,0XE5,0XF5,0X65,0XFE,0XE5,0XFE,0XE5,0XFE,0XC6,0XF6,0XE4,0X51,
0XA6,0XF6,0XC6,0XFE,0XA5,0XBC,0XE5,0XDD,0X25,0X93,0XA6,0XD4,0X66,0XD4,0X67,0XEC,
0XA8,0XE3,0X88,0XE3,0X68,0XE3,0XA6,0XAA,0X26,0X8A,0XE3,0X38,0XA7,0XBA,0X06,0X8A,
0X67,0X9A,0X27,0X8A,0XA3,0X20,0XC8,0XAA,0XC6,0X79,0XA6,0X71,0X62,0X10,0X82,0X18,
0XA3,0X28,0X08,0XCB,0X08,0XCB,0X08,0XC3,0X08,0XC3,0X28,0XCB,0X26,0X82,0XA5,0X61,
0XE7,0XB2,0X48,0XD3,0X48,0XD3,0XC3,0X20,0XC7,0XAA,0X88,0XDB,0X24,0X39,0X48,0XCB,
0XA8,0XE3,0X08,0XC3,0X88,0XE3,0XA8,0XE3,0XE8,0XEB,0XC8,0XEB,0XC8,0XEB,0X08,0XEC,
0X28,0XEC,0X28,0XE4,0XE8,0XEB,0XC8,0XE3,0XA8,0XE3,0XC3,0X28,0X82,0X10,0X82,0X10,
0X82,0X10,0X82,0X10,0XA7,0XA2,0X49,0XCB,0XC7,0X9A,0X20,0X08,0XC3,0X31,0XE7,0X63,
0X65,0X74,0X04,0X95,0X04,0X95,0XE4,0X94,0X27,0X74,0XC7,0X63,0XA8,0X5B,0X47,0X53,
0XA6,0X63,0XE6,0X73,0XE6,0X7B,0X28,0X7C,0X68,0X5B,0XEE,0X6B,0X4E,0X6B,0X4C,0X6B,
0XAB,0X94,0XA7,0X84,0XC9,0XA5,0X8B,0XA5,0X4A,0X95,0XC9,0XA5,0X65,0X53,0X24,0X43,
0X24,0X4B,0X05,0X43,0X05,0X43,0X65,0X2A,0X45,0X22,0X65,0X1A,0X65,0X22,0X05,0X33,
0X25,0X3B,0XE6,0X1A,0XC6,0X1A,0XA7,0X0A,0X46,0X0A,0XE7,0X12,0XE7,0X12,0X49,0X0B,
0X48,0X0B,0X49,0X03,0XE8,0X02,0X2A,0X03,0X06,0X02,0XA2,0X00,0X00,0X00,0X61,0X18,
0X45,0X69,0X45,0X61,0X45,0X51,0X25,0X49,0XC3,0X28,0XC3,0X20,0X05,0X39,0X25,0X39,
0XE4,0X28,0X41,0X08,0XE4,0X28,0X05,0X31,0X21,0X10,0X21,0X08,0X41,0X08,0X41,0X10,
0XA1,0X18,0XA1,0X18,0X41,0X08,0X21,0X08,0X41,0X10,0X82,0X18,0X82,0X18,0X62,0X10,
0X62,0X18,0X42,0X10,0X21,0X08,0X41,0X08,0X82,0X10,0X04,0X31,0X85,0X59,0X47,0X72,
0XE6,0X71,0X46,0X9A,0X87,0XAA,0X87,0XAA,0X47,0XB2,0X07,0XB2,0X04,0X59,0X62,0X39,
0X84,0XB4,0XC5,0XE5,0X24,0XEE,0X44,0XF6,0X84,0XF6,0X64,0X41,0X65,0XF6,0XA5,0XFE,
0X85,0XB4,0XC5,0XDD,0X64,0X8B,0X44,0X83,0X26,0XDD,0X66,0XF5,0XC7,0XEC,0XA7,0XEC,
0X87,0XEC,0XA6,0XC3,0XE3,0X28,0X43,0X41,0XC7,0XCB,0XA6,0X92,0X87,0XCB,0X05,0X72,
0X82,0X18,0XE4,0X61,0X23,0X39,0X27,0XBB,0XC2,0X20,0XE3,0X38,0XA3,0X20,0X68,0XC3,
0XA9,0XCB,0X68,0XC3,0XA9,0XCB,0XA9,0XC3,0X87,0X8A,0X64,0X41,0XE7,0XAA,0XC8,0XD3,
0XC8,0XD3,0X03,0X29,0X47,0XB3,0X88,0XC3,0X44,0X39,0XE8,0XDB,0X08,0XDC,0XE7,0XA2,
0X08,0XDC,0XE8,0XDB,0XE8,0XDB,0XE8,0XDB,0XE8,0XDB,0XE8,0XE3,0XE8,0XE3,0X08,0XE4,
0XC8,0XDB,0XC8,0XDB,0X88,0XD3,0XE3,0X20,0X82,0X10,0X82,0X10,0X82,0X10,0X82,0X10,
0X07,0X9B,0X89,0XBB,0X83,0X49,0X21,0X08,0X83,0X6B,0X04,0X95,0X85,0X8C,0X86,0X5B,
0XC8,0X63,0XC7,0X6B,0XC6,0X84,0XC4,0X8C,0XE5,0X94,0X27,0X74,0XC7,0X63,0XCA,0X63,
0X49,0X53,0XE8,0X6B,0X86,0X6B,0XC8,0X7B,0X0C,0X7C,0X8B,0X63,0X71,0XAD,0X6B,0X9D,
0XC9,0XA5,0X29,0XB6,0X27,0X8D,0X67,0X95,0X85,0X53,0XA5,0X32,0XC5,0X32,0X45,0X2A,
0XE4,0X3A,0XC4,0X42,0X05,0X3B,0X04,0X43,0X85,0X22,0X65,0X1A,0XA6,0X12,0X65,0X1A,
0XE6,0X22,0X25,0X33,0X47,0X13,0X08,0X0B,0X08,0X03,0X67,0X0A,0X08,0X03,0X28,0X03,
0X8A,0X03,0XCA,0X03,0XAB,0X03,0X64,0X01,0X00,0X00,0X00,0X00,0X25,0X69,0X66,0X79,
0X45,0X79,0X65,0X79,0X04,0X51,0XC3,0X38,0X45,0X71,0X45,0X69,0X04,0X49,0X21,0X08,
0X04,0X49,0X45,0X59,0X21,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X82,0X10,
0XA3,0X30,0X04,0X51,0XC4,0X61,0X45,0X69,0XC3,0X30,0X82,0X18,0X62,0X10,0XC3,0X28,
0X23,0X39,0X43,0X39,0XC3,0X49,0XE3,0X30,0X25,0XAB,0X46,0XAB,0X44,0X7A,0X05,0X9B,
0X24,0X6A,0X64,0X49,0X05,0X9B,0X67,0XE4,0XA7,0XEC,0X87,0XE4,0X27,0XD4,0X86,0XB3,
0X82,0X20,0X23,0X39,0X86,0X8A,0X66,0X7A,0XC8,0XC3,0XE5,0X61,0XA3,0X20,0XC3,0X20,
0XE3,0X28,0XE7,0X9A,0XE5,0X59,0XC3,0X28,0X82,0X18,0X48,0XA3,0XE9,0XBB,0XE9,0XBB,
0XC9,0XBB,0XE9,0XBB,0X07,0X93,0XC3,0X18,0X47,0XA3,0XE9,0XCB,0XE9,0XCB,0XE3,0X20,
0XA8,0XB3,0XC7,0X92,0XC4,0X51,0X69,0XDC,0X88,0XE4,0X47,0XAB,0XA8,0XEC,0XA7,0XEC,
0XA8,0XEC,0X88,0XE4,0X68,0XE4,0X48,0XDC,0X48,0XDC,0X28,0XDC,0X28,0XDC,0X28,0XDC,
0XE8,0XD3,0XE3,0X28,0X82,0X10,0X62,0X10,0X82,0X10,0X82,0X18,0X68,0XAB,0X68,0XB3,
0X40,0X08,0X40,0X08,0X66,0X5B,0XA6,0X63,0X45,0X74,0X84,0X84,0XE4,0X94,0X65,0X84,
0X47,0X7C,0XE7,0X63,0XA7,0X5B,0X47,0X6C,0X45,0X7C,0XC5,0X8C,0XE6,0X73,0XC8,0X6B,
0XCB,0X6B,0X4D,0X5B,0XEF,0X7B,0XAA,0X73,0X0A,0X9D,0X89,0XA5,0X49,0X95,0X6C,0XBE,
0X8A,0X9D,0X07,0X8D,0X25,0X4B,0X05,0X43,0X64,0X53,0X05,0X3B,0XC5,0X32,0XA5,0X32,
0X25,0X22,0XA5,0X22,0XC5,0X2A,0X06,0X23,0X26,0X2B,0XE6,0X12,0XA6,0X0A,0XE7,0X0A,
0XA7,0X12,0XC7,0X12,0X27,0X1B,0X88,0X0B,0X89,0X0B,0X69,0X03,0X29,0X03,0XE8,0X02,
0XA7,0X02,0X03,0X01,0X00,0X00,0X00,0X00,0X82,0X20,0X45,0X61,0X45,0X59,0X45,0X59,
0X04,0X49,0X42,0X10,0XC3,0X30,0X45,0X61,0X04,0X49,0X21,0X08,0X04,0X41,0X25,0X51,
0X21,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X18,0X62,0X10,0X62,0X10,0X62,0X18,0X62,0X10,
0X62,0X18,0XE3,0X30,0XC4,0X61,0X03,0X39,0XA3,0X20,0X44,0X41,0X62,0X10,0XA3,0X20,
0XC5,0X51,0XA5,0X51,0X27,0X9B,0XA5,0X49,0X03,0X39,0X44,0X41,0X62,0X10,0X64,0X49,
0X64,0X49,0X62,0X10,0X82,0X10,0XE5,0X59,0XE5,0X51,0X05,0X5A,0X07,0X93,0X48,0X9B,
0X25,0X62,0X82,0X10,0X86,0X7A,0X08,0XBC,0X28,0XC4,0XC3,0X18,0XA7,0XB3,0X05,0X62,
0X65,0X72,0X67,0XAB,0X25,0X6A,0XA6,0X82,0XA7,0XB3,0XC7,0XBB,0XC7,0XEC,0XC7,0XEC,
0XC7,0XEC,0XC7,0XEC,0XA7,0XEC,0XC7,0XEC,0XC7,0XEC,0XA7,0XEC,0X27,0XC4,0XA3,0X18,
0X82,0X10,0X62,0X10,0X82,0X10,0X82,0X18,0XC8,0XB3,0X65,0X72,0X00,0X00,0X41,0X29,
0X64,0XA5,0XE4,0X8C,0X66,0X7C,0X66,0X5B,0X87,0X63,0XA6,0X63,0X45,0X74,0X84,0X84,
0X85,0X84,0X07,0X7C,0X88,0X5B,0XC9,0X6B,0XE8,0X6B,0XE6,0X6B,0X47,0X84,0XC7,0X6B,
0X0C,0X7C,0X6F,0X84,0XCD,0X94,0X0B,0XB6,0X89,0X9D,0XE9,0XAD,0X48,0X95,0X46,0X74,
0X65,0X53,0X65,0X32,0X65,0X32,0XE5,0X3A,0XE4,0X3A,0X24,0X43,0X25,0X33,0XE6,0X2A,
0XE6,0X22,0X66,0X12,0X86,0X12,0X06,0X1B,0X06,0X1B,0X67,0X1B,0X27,0X13,0XE8,0X0A,
0X28,0X03,0X29,0X0B,0X08,0X0B,0X68,0X0B,0XA9,0X0B,0X4A,0X03,0XA8,0X02,0XA8,0X02,
0X03,0X01,0X00,0X00,0X20,0X08,0X25,0X59,0X45,0X59,0X25,0X51,0X04,0X41,0X42,0X10,
0X62,0X10,0X04,0X39,0XC4,0X30,0X21,0X08,0XC3,0X20,0X04,0X31,0X21,0X10,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X62,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X42,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,
0X82,0X18,0X62,0X10,0X82,0X18,0X82,0X18,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X82,0X10,0X62,0X10,0X83,0X18,0X82,0X18,0X62,0X10,0X82,0X10,
0XA2,0X18,0X03,0X29,0X23,0X31,0X62,0X10,0X03,0X29,0XA2,0X18,0XA3,0X18,0X83,0X18,
0X83,0X10,0X83,0X18,0XA3,0X18,0XC3,0X20,0X25,0X62,0X06,0X9B,0XC7,0XBB,0X27,0XCC,
0X27,0XD4,0X47,0XD4,0XC7,0XBB,0XE6,0X82,0X43,0X39,0X62,0X10,0X82,0X10,0X82,0X10,
0X62,0X10,0X82,0X18,0X67,0X9B,0X02,0X31,0X00,0X00,0XE3,0X31,0XE7,0X6B,0XA5,0X63,
0XA4,0X84,0XE4,0X94,0XE5,0X94,0X47,0X74,0XA6,0X63,0XC8,0X5B,0X67,0X53,0X06,0X6C,
0X05,0X74,0X45,0X84,0X27,0X7C,0X68,0X5B,0X0D,0X74,0X8D,0X6B,0X69,0X6B,0XCA,0X94,
0XA6,0X6B,0X07,0X5B,0XE7,0X5A,0XCA,0X8C,0X48,0X95,0X25,0X53,0X63,0X29,0XC2,0X10,
0X82,0X10,0XE3,0X18,0X64,0X19,0X04,0X22,0X25,0X22,0X44,0X22,0X85,0X22,0X05,0X2B,
0X26,0X13,0X07,0X13,0XC7,0X0A,0X87,0X0A,0X28,0X13,0X07,0X0B,0X88,0X0B,0X88,0X0B,
0X8A,0X03,0X69,0X03,0X6A,0X03,0XA8,0X02,0XE9,0X02,0X29,0X03,0X03,0X01,0X00,0X00,
0X00,0X00,0X83,0X20,0XE4,0X30,0XE4,0X38,0XA3,0X28,0X41,0X08,0X42,0X10,0X62,0X18,
0X83,0X18,0X21,0X08,0X82,0X18,0XE4,0X28,0X41,0X10,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,
0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X82,0X10,0X62,0X10,0X62,0X10,
0X82,0X18,0X83,0X18,0X82,0X10,0X82,0X10,0XA3,0X18,0XA3,0X18,0XC3,0X20,0XC3,0X20,
0XA3,0X18,0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X82,0X10,
0XA2,0X18,0X62,0X10,0X83,0X18,0X83,0X31,0X64,0X4A,0X45,0X42,0X46,0X3A,0X45,0X3A,
0X44,0X42,0X43,0X6B,0X23,0X63,0X85,0X52,0X05,0X3A,0XC6,0X31,0XA5,0X29,0X24,0X29,
0X24,0X4A,0X83,0X52,0XA4,0X31,0XC3,0X18,0X82,0X10,0XA3,0X18,0X62,0X10,0X42,0X10,
0X42,0X10,0X62,0X10,0X82,0X18,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X03,0X11,0XC2,0X10,
0X03,0X11,0XA5,0X09,0X25,0X0A,0X46,0X0A,0X26,0X0A,0X66,0X0A,0X66,0X12,0XE5,0X09,
0X64,0X09,0X64,0X09,0XE3,0X08,0XA3,0X10,0X42,0X08,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X41,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X41,0X08,0X42,0X10,0X41,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X62,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,
0X62,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X82,0X10,0X82,0X10,0X62,0X10,0X82,0X10,0X82,0X10,0X82,0X10,
0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X82,0X10,0X82,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X21,0X08,0X42,0X10,0X42,0X10,0X41,0X08,0X42,0X10,0X42,0X10,0X21,0X08,0X42,0X10,
0X42,0X10,0X42,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X41,0X08,0X21,0X08,
0X42,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X41,0X10,
0X41,0X10,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X41,0X08,0X42,0X10,0X21,0X08,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X82,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X42,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X41,0X08,0X21,0X08,0X21,0X08,0X41,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X41,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X41,0X10,0X42,0X10,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,
0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X82,0X10,0X62,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X82,0X10,
0X82,0X10,0X62,0X10,0X82,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X62,0X10,0X62,0X10,
0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X62,0X10,0X42,0X10,0X62,0X10,0X42,0X10,
0X62,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X62,0X10,0X62,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,0X42,0X10,
0X42,0X10,0X42,0X10,0X42,0X10,0X21,0X08,0X41,0X08,0X21,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X41,0X08,0X21,0X08,0X21,0X08,0X41,0X08,0X21,0X08,0X21,0X08,
0X21,0X08,0X21,0X08,0X21,0X08,0X42,0X10,0X42,0X30,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X62,0X28,0X62,0X28,0X62,0X28,0X42,0X28,0X62,0X28,
0X62,0X28,0X62,0X28,0X62,0X28,0X62,0X28,0X62,0X28,0X62,0X30,0X42,0X28,0X62,0X30,
0X62,0X30,0X62,0X30,0X62,0X30,0X62,0X28,0X62,0X30,0X62,0X30,0X62,0X30,0X82,0X30,
0X62,0X30,0X62,0X30,0X62,0X30,0X62,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X82,0X30,
0X82,0X30,0X82,0X30,0X82,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X62,0X30,0X82,0X30,
0X62,0X30,0X62,0X30,0X82,0X30,0X82,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X62,0X30,
0X82,0X30,0X62,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0XA2,0X30,0X82,0X30,
0X62,0X30,0X82,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X62,0X30,0X82,0X30,0X82,0X30,
0X82,0X30,0X82,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0X62,0X30,
0X82,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0X62,0X30,0X82,0X30,
0X82,0X30,0X82,0X30,0X62,0X30,0X62,0X30,0X82,0X30,0X62,0X30,0X82,0X30,0X82,0X30,
0X62,0X30,0X62,0X30,0X62,0X30,0X82,0X30,0X62,0X30,0X62,0X30,0X62,0X30,0X42,0X28,
0X82,0X28,0X82,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,0X42,0X28,
0X42,0X28,0X42,0X30};


 //----------------------- Picture ett2 ---------------------
	

prog_char ett2[12800] = {
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XBE,0XFF,0X7D,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X5D,0XFF,0XFF,0XFF,0XFF,0XFF,0XBE,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X7D,0XFF,0XFF,0XFF,0XFF,0XFF,0X9E,0XFF,0X1C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,
0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X3C,0XFF,0X1C,0XFF,0X7E,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X7D,0XFF,
0X30,0XFC,0X87,0XF9,0XC4,0XF8,0X42,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,
0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,
0X21,0XF8,0XA7,0XF9,0XFF,0XFF,0XFF,0XFF,0X96,0XFD,0X42,0XF8,0X22,0XF8,0X22,0XF8,
0X22,0XF8,0X22,0XF8,0X21,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,0X22,0XF8,
0X22,0XF8,0X22,0XF8,0X22,0XF8,0X8B,0XFA,0XFF,0XFF,0XFF,0XFF,0XD0,0XFB,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X4D,0XFB,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,0X96,0XFD,0X45,0XF9,
0X21,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X66,0XF9,0XFF,0XFF,0XFF,0XFF,0X75,0XFD,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X4A,0XFA,0XFF,0XFF,0XFF,0XFF,0XAF,0XFB,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X2D,0XFB,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X92,0XFC,0X42,0XF8,0X01,0XF8,
0X01,0XF8,0X21,0XF8,0X04,0XF9,0X8A,0XFA,0XCB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,
0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,0XAB,0XFA,
0XAB,0XFA,0XCF,0XFB,0XFF,0XFF,0XFF,0XFF,0X59,0XFE,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,
0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,
0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,0XCB,0XFA,
0XCB,0XFA,0XCB,0XFA,0XAB,0XFA,0X71,0XFC,0XFF,0XFF,0XFF,0XFF,0X34,0XFD,0X4A,0XFA,
0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,
0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,
0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X6A,0XFA,0X4A,0XFA,0XB3,0XFC,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X75,0XFD,0X83,0XF8,0X01,0XF8,0X01,0XF8,
0XE8,0XF9,0X75,0XFD,0X5D,0XFF,0X7D,0XFF,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,
0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,0XFC,0XFE,
0XFC,0XFE,0X3C,0XFF,0XFF,0XFF,0XFF,0XFF,0XBE,0XFF,0X3C,0XFF,0X1C,0XFF,0X1C,0XFF,
0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X9E,0XFF,
0X9E,0XFF,0X3C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,
0X1C,0XFF,0X1C,0XFF,0X1C,0XFF,0X7D,0XFF,0XFF,0XFF,0XFF,0XFF,0X7D,0XFF,0XFB,0XFE,
0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,
0XFB,0XFE,0X7D,0XFF,0X7D,0XFF,0XFC,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,
0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0XFB,0XFE,0X7D,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X1C,0XFF,0X25,0XF9,0X01,0XF8,0X01,0XF8,0XAA,0XFA,
0X3C,0XFF,0X5D,0XFF,0X72,0XFC,0X29,0XFA,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,
0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,
0X66,0XF9,0X8B,0XFA,0XFF,0XFF,0XFF,0XFF,0XD7,0XFD,0X87,0XF9,0X87,0XF9,0X87,0XF9,
0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0XB6,0XFD,
0XFB,0XFE,0XC8,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,0X87,0XF9,
0X87,0XF9,0X87,0XF9,0X87,0XF9,0X6E,0XFB,0XFF,0XFF,0XFF,0XFF,0X72,0XFC,0X66,0XF9,
0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,
0X66,0XF9,0XD7,0XFD,0X38,0XFE,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,
0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0X46,0XF9,0XF0,0XFB,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XD2,0XFC,0X21,0XF8,0X01,0XF8,0X86,0XF9,0X9E,0XFF,
0X3C,0XFF,0XA7,0XF9,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X87,0XF9,0XFF,0XFF,0XFF,0XFF,0X55,0XFD,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X35,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X6A,0XFA,0XFF,0XFF,0XFF,0XFF,0XCF,0XFB,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X4D,0XFB,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X7D,0XFF,0X29,0XFA,0X01,0XF8,0X42,0XF8,0XB2,0XFC,0XFF,0XFF,
0XAB,0XFA,0X01,0XF8,0X01,0XF8,0X21,0XF8,0XC4,0XF8,0X25,0XF9,0X25,0XF9,0X25,0XF9,
0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,
0X05,0XF9,0X6A,0XFA,0XFF,0XFF,0XFF,0XFF,0XD7,0XFD,0X66,0XF9,0X66,0XF9,0X66,0XF9,
0X66,0XF9,0X66,0XF9,0X66,0XF9,0X66,0XF9,0XA3,0XF8,0X01,0XF8,0X01,0XF8,0X35,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X21,0XF8,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,
0X25,0XF9,0X25,0XF9,0X05,0XF9,0X4D,0XFB,0XFF,0XFF,0XFF,0XFF,0X72,0XFC,0X05,0XF9,
0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0X62,0XF8,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0X83,0XF8,0X25,0XF9,0X25,0XF9,
0X25,0XF9,0X25,0XF9,0X25,0XF9,0X25,0XF9,0XE5,0XF8,0XF0,0XFB,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X59,0XFE,0X25,0XF9,0X01,0XF8,0XE4,0XF8,0X3C,0XFF,0XF7,0XFD,
0X42,0XF8,0X01,0XF8,0XA3,0XF8,0XD3,0XFC,0X9E,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XDF,0XFF,0X10,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0X7D,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0X9E,0XFF,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XDF,0XFF,0XAB,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XCF,0XFB,0XBE,0XFF,0XBE,0XFF,
0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0XBE,0XFF,0X9E,0XFF,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XF8,0XFD,0X04,0XF9,0X01,0XF8,0X46,0XF9,0XFF,0XFF,0XCB,0XFA,
0X01,0XF8,0X21,0XF8,0X71,0XFC,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBA,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X59,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X22,0XF8,0X6A,0XFA,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,
0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,0X2D,0XFB,
0X2D,0XFB,0X31,0XFC,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X87,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X09,0XFA,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X87,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XFF,0XFF,0X9A,0XFE,
0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,
0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,0XF8,0XFD,
0XF8,0XFD,0X59,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XFF,0XFF,0X38,0XFE,
0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,
0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,0X55,0XFD,
0X55,0XFD,0XF7,0XFD,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X09,0XFA,
0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,
0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,0X42,0XF8,
0X42,0XF8,0XA7,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X49,0XFA,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,
0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,0X21,0XF8,
0X21,0XF8,0XA7,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X22,0XF8,0XCB,0XFA,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,
0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,0XAF,0XFB,
0X8F,0XFB,0X92,0XFC,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X39,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X18,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XDF,0XFF,0X29,0XFA,
0X01,0XF8,0X63,0XF8,0X39,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XF8,0XFD,0X05,0XF9,0X01,0XF8,0X46,0XF9,0XFF,0XFF,0X0C,0XFB,
0X01,0XF8,0X21,0XF8,0X10,0XFC,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X79,0XFE,0X45,0XF9,0X01,0XF8,0XE4,0XF8,0X3C,0XFF,0X59,0XFE,
0X42,0XF8,0X01,0XF8,0X62,0XF8,0X31,0XFC,0X1C,0XFF,0X5D,0XFF,0X3D,0XFF,0X3D,0XFF,
0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,0X3D,0XFF,
0X3C,0XFF,0X7D,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XBE,0XFF,0X6A,0XFA,0X01,0XF8,0X21,0XF8,0X30,0XFC,0XFF,0XFF,
0X4D,0XFB,0X01,0XF8,0X01,0XF8,0X21,0XF8,0XA3,0XF8,0XA4,0XF8,0XA3,0XF8,0XA3,0XF8,
0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,0XA3,0XF8,
0XA3,0XF8,0X08,0XFA,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X34,0XFD,0X21,0XF8,0X01,0XF8,0X25,0XF9,0XFB,0XFE,
0X3C,0XFF,0XEC,0XFA,0X62,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X87,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X5D,0XFF,0X45,0XF9,0X01,0XF8,0X01,0XF8,0XA7,0XF9,
0XFB,0XFE,0XFF,0XFF,0X96,0XFD,0X08,0XFA,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,
0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,0XE5,0XF8,
0XE4,0XF8,0X6A,0XFA,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XF7,0XFD,0XA3,0XF8,0X01,0XF8,0X01,0XF8,
0X25,0XF9,0X92,0XFC,0X7D,0XFF,0XDF,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,
0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,0X9E,0XFF,
0X9E,0XFF,0X9E,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XF3,0XFC,0XC3,0XF8,0X01,0XF8,
0X01,0XF8,0X42,0XF8,0XE4,0XF8,0XE8,0XF9,0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,
0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,0X29,0XFA,
0X29,0XFA,0X6E,0XFB,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X18,0XFE,0X87,0XF9,
0X21,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,0X01,0XF8,
0X01,0XF8,0X87,0XF9,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X31,0XFC,0X01,0XF8,0X01,0XF8,0X34,0XFD,
0XBB,0XFE,0X83,0XF8,0X01,0XF8,0X05,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XCC,0XFA,0X01,0XF8,
0X01,0XF8,0X96,0XFD,0XF7,0XFD,0X21,0XF8,0X01,0XF8,0XF0,0XFB,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X7D,0XFF,
0X75,0XFD,0XEC,0XFA,0XA7,0XF9,0X25,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,
0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,0X05,0XF9,
0X05,0XF9,0X4A,0XFA,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X72,0XFC,0XC4,0XF8,0XC4,0XF8,0X55,0XFD,
0XDB,0XFE,0X25,0XF9,0XC4,0XF8,0XA7,0XF9,0XBE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X4D,0XFB,0XE4,0XF8,
0XE4,0XF8,0XB7,0XFD,0X18,0XFE,0XE4,0XF8,0XE4,0XF8,0X51,0XFC,0XDF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X5D,0XFF,0XBB,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,
0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,0XBA,0XFE,
0X9A,0XFE,0XFC,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X3C,0XFF,0X39,0XFE,0X39,0XFE,0X7D,0XFF,
0XBE,0XFF,0X79,0XFE,0X59,0XFE,0X7A,0XFE,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X1C,0XFF,0X79,0XFE,
0X79,0XFE,0X9E,0XFF,0XBE,0XFF,0X9A,0XFE,0X79,0XFE,0X5D,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,
0XDF,0XBD,0XBF,0XD6,0X7F,0XEF,0X1F,0XA5,0X9F,0XF7,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,0X7F,0XEF,
0X3F,0X42,0XBF,0X94,0X7F,0XCE,0X7F,0X08,0XDF,0XDE,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X3F,0XC6,0X7F,0X6B,0X9F,0XF7,0X9F,0XD6,0X3F,0X84,0XFF,0XFF,
0XDF,0X9C,0X9F,0X94,0X9F,0X73,0X3F,0XE7,0X9F,0XD6,0XDF,0X9C,0XFF,0XFF,0XDF,0X7B,
0X5F,0X8C,0XDF,0X7B,0XFF,0XFF,0XDF,0XBD,0X9F,0X94,0XFF,0XFF,0X1F,0X84,0XDF,0XBD,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0X9C,0XBF,0X31,0X7F,0X4A,0X3F,0XE7,0XFF,0XBD,
0X9F,0X10,0X7F,0X29,0X7F,0X4A,0X1F,0X00,0XBF,0X73,0XBF,0XF7,0X5F,0X6B,0X9F,0X31,
0X9F,0X73,0XFF,0XFF,0XFF,0XFF,0XFF,0XBD,0XFF,0X39,0XDF,0X5A,0XDF,0X7B,0X1F,0XC6,
0XBF,0XB5,0X9F,0X73,0XBF,0X52,0XDF,0X5A,0X5F,0XCE,0X7F,0X4A,0XBF,0X52,0X3F,0XE7,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X9F,0XF7,0X9F,0X73,0X9F,0X31,0X1F,0X63,0XFF,0XDE,
0XFF,0XFF,0X9F,0XB5,0XDF,0X39,0X7F,0X4A,0XBF,0XD6,0XBF,0XF7,0XDF,0X7B,0XDF,0X7B,
0XDF,0X39,0X1F,0X84,0X3F,0XA5,0XDF,0X39,0X5F,0X8C,0X5F,0XEF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0X5F,0XEF,0XBF,0X31,0X5F,0XAD,0XDF,0X7B,0XDF,0X18,0X3F,0XE7,
0X7F,0X29,0XDF,0XBD,0X3F,0X21,0X3F,0XA5,0XDF,0X7B,0XBF,0X10,0X9F,0XD6,0X7F,0X08,
0XDF,0XBD,0XDF,0X39,0X7F,0XCE,0X9F,0X52,0X9F,0X31,0X7F,0XEF,0X5F,0X29,0X5F,0XCE,
0XFF,0XFF,0XFF,0XFF,0X7F,0XAD,0XFF,0X18,0X7F,0X8C,0X1F,0X63,0XFF,0X39,0X3F,0XC6,
0X3F,0X21,0XDF,0X5A,0X1F,0X84,0X3F,0X00,0X5F,0XAD,0XBF,0X73,0X9F,0X31,0XBF,0X94,
0X1F,0X42,0X7F,0X6B,0X9F,0XD6,0X5F,0X29,0X7F,0X6B,0XDF,0X7B,0X7F,0X08,0X1F,0XA5,
0X7F,0X8C,0X7F,0X08,0X1F,0X84,0X9F,0X31,0XFF,0X18,0X3F,0X84,0X3F,0X21,0XDF,0X7B,
0XDF,0XFF,0XFF,0XFF,0XDF,0XFF,0X9F,0X52,0XFF,0X39,0XBF,0X94,0XFF,0X5A,0XBF,0XB5,
0X3F,0XC6,0X3F,0X21,0XFF,0X7B,0X3F,0X63,0XBF,0X31,0X3F,0XE7,0X7F,0X29,0XDF,0X39,
0X1F,0X84,0X9F,0X10,0X3F,0X42,0X1F,0X84,0XBF,0X10,0X7F,0X8C,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XBF,0X94,0XBF,0X31,0X9F,0X31,0X7F,0X29,0X3F,0X63,
0XFF,0X39,0X9F,0XF7,0XFF,0X7B,0XDF,0X39,0X7F,0X29,0XDF,0X18,0X5F,0X4A,0X7F,0X29,
0X9F,0XF7,0X9F,0X73,0XFF,0X39,0X5F,0X29,0X7F,0X29,0X1F,0X63,0XDF,0X5A,0XDF,0XFF,
0XFF,0XFF,0XBF,0XF7,0X3F,0X63,0X3F,0X21,0X7F,0X4A,0X5F,0X4A,0XFF,0X18,0X5F,0XCE,
0X7F,0X4A,0XFF,0X9C,0X7F,0XCE,0X5F,0X08,0XFF,0XDE,0XDF,0X39,0XDF,0X39,0X9F,0X52,
0XFF,0X39,0XFF,0X18,0X9F,0X94,0X3F,0X21,0XBF,0XF7,0XFF,0XFF,0X7F,0X4A,0X1F,0XA5,
0X7F,0X8C,0X9F,0X31,0XBF,0XF7,0X9F,0X73,0XDF,0X39,0XFF,0XFF,0X7F,0X6B,0XFF,0X5A,
0XBF,0XF7,0XFF,0XFF,0XFF,0XDE,0XFF,0X18,0X9F,0XD6,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XBF,0X73,0X9F,0X52,0XFF,0XFF,0XFF,0XFF,0X1F,0X21,0XFF,0XBD,0X5F,0X29,0XDF,0X9C,
0X9F,0XF7,0X3F,0X21,0X3F,0XA5,0XBF,0XF7,0XFF,0X18,0XDF,0X7B,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XDF,0XDE,0X5F,0X08,0X3F,0X42,0X9F,0XB5,0X3F,0X00,
0X3F,0X84,0XFF,0XFF,0X7F,0XCE,0XDF,0X18,0X3F,0X42,0X5F,0X8C,0X1F,0X00,0X1F,0XA5,
0XFF,0XFF,0X1F,0XC6,0X1F,0X00,0X7F,0X6B,0X9F,0X94,0X1F,0X00,0XFF,0X9C,0XFF,0XFF,
0XFF,0XDE,0X7F,0XEF,0X3F,0X84,0X9F,0X52,0X3F,0XE7,0XBF,0XD6,0XDF,0XDE,0X7F,0XEF,
0X1F,0X42,0X9F,0X94,0X5F,0XCE,0X5F,0X08,0XFF,0XDE,0X5F,0X4A,0X3F,0X84,0X7F,0XEF,
0X7F,0XCE,0X7F,0XEF,0XBF,0XB5,0XDF,0X18,0X7F,0XCE,0X7F,0XCE,0XFF,0X18,0X1F,0XA5,
0X5F,0X8C,0XDF,0X39,0X9F,0XF7,0X9F,0X73,0XDF,0X39,0XFF,0XFF,0X7F,0X6B,0XFF,0X5A,
0X7F,0XEF,0XFF,0XDE,0X7F,0XEF,0X5F,0X29,0X7F,0X8C,0X3F,0XE7,0X3F,0XC6,0X3F,0XE7,
0XFF,0X9C,0X7F,0X29,0X1F,0XE7,0X1F,0XC6,0XFF,0X18,0X9F,0XD6,0X3F,0X21,0XDF,0X9C,
0X7F,0XEF,0X3F,0X21,0X3F,0XA5,0XBF,0XF7,0XFF,0X18,0XDF,0X7B,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0X9F,0XF7,0X9F,0X52,0XDF,0X9C,0XFF,0XDE,0X7F,0X29,
0X5F,0XCE,0XFF,0XFF,0XDF,0XFF,0XDF,0X5A,0XFF,0X9C,0X1F,0XE7,0XDF,0X39,0X3F,0XE7,
0XFF,0XFF,0X5F,0XEF,0XBF,0X31,0XFF,0XBD,0X7F,0XCE,0X1F,0X21,0XFF,0XDE,0X7F,0XEF,
0X1F,0X42,0X3F,0XA5,0X3F,0XE7,0X5F,0X4A,0X7F,0X29,0X3F,0X21,0XDF,0X9C,0X9F,0XF7,
0XDF,0X5A,0XFF,0X9C,0X9F,0XD6,0X1F,0X21,0XFF,0XDE,0X3F,0XC6,0X7F,0X29,0X7F,0X29,
0XBF,0X31,0X7F,0XCE,0XBF,0XF7,0XFF,0X7B,0XBF,0X10,0X9F,0X31,0X9F,0X31,0X7F,0XAD,
0XDF,0X9C,0X9F,0X52,0XBF,0XF7,0X1F,0X84,0X9F,0X52,0XFF,0XFF,0XFF,0X7B,0X9F,0X73,
0XFF,0XBD,0X9F,0X31,0XFF,0XFF,0X7F,0XAD,0X7F,0X29,0X1F,0X21,0X5F,0X29,0XDF,0XBD,
0XBF,0XF7,0X3F,0X63,0XFF,0X18,0XFF,0X18,0X9F,0X94,0X9F,0XF7,0XFF,0X39,0X3F,0XA5,
0X7F,0XEF,0XFF,0X39,0X7F,0XAD,0XBF,0XF7,0XBF,0X31,0X5F,0X8C,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,0XFF,
};






/******************************************************
 *			 Font 7x11(English)				          *
 *											          *
 * -ASCII fonts from 0x20 ~ 0x7F(DEC 32 ~ 126)     	  *
 *  composed of following characters 			      *
 *                                                    * 
 * -XSize : 7  pixcels							      *
 * -YSize : 11 pixcels						       	  *
 *											       	  *
 *   ....... <- b0, Blank, Not used				      *
 *   ....... <- b1, Blank, Not used				      *
 *   ....... <- b2, Blank, Not used				      *
 *   ....... <- b3, Blank, Not used				      *
 *   ....... <- b4, Blank, Not used				      *
 *   ..End.. <- b5(2nd byte)					      *
 *   ..OO... <- b6(2nd byte)					      *
 *   ..OO... <- b7(2nd byte)					      *
 *   ..OO... <- b0(1st byte)					      *
 *   ..OO... <- b1(1st byte)					      *
 *   ..OO... <- b2(1st byte)					      *
 *   ..OO... <- b3(1st byte)					      *
 *   ..OO... <- b4(1st byte)				 	      *
 *   ..OO... <- b5(1st byte)					      *
 *   ..OO... <- b6(1st byte)					      *
 *   .Start..<- b7(1st byte)					      *
 *												      *
 ******************************************************/

//7x11 ascii font, each character takes 14 bytes, total 1330 bytes, draw from left bottom to left top

  prog_char ascii_7x11[95][14] = {                                                       
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x20 space
  {0x00, 0x00, 0x07, 0xc0, 0xdf, 0xe0, 0xdf, 0xe0, 0x07, 0xc0, 0x00, 0x00, 0x00, 0x1f}, //0x21 !
  {0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x1f}, //0x22 "
  {0x00, 0x00, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x00, 0x1f}, //0x23 #
  {0x13, 0x00, 0x27, 0x80, 0x24, 0x80, 0x7f, 0xc0, 0x24, 0x80, 0x38, 0x80, 0x19, 0x1f}, //0x24 $
  {0x30, 0x80, 0x19, 0x40, 0x2d, 0x40, 0x56, 0x80, 0x53, 0x00, 0x21, 0x80, 0x00, 0x1f}, //0x25 %
  {0x38, 0x00, 0x7d, 0x80, 0x47, 0xc0, 0x5e, 0x40, 0x7b, 0xc0, 0x21, 0x80, 0x58, 0x1f}, //0x26 &
  {0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x27 '
  {0x00, 0x00, 0x00, 0x00, 0x3f, 0x80, 0x7f, 0xc0, 0xc0, 0x60, 0x80, 0x20, 0x00, 0x1f}, //0x28 (
  {0x00, 0x00, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x29 )
  {0x00, 0x00, 0x24, 0x80, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0x24, 0x80, 0x00, 0x1f}, //0x2a *
  {0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2b +
  {0x00, 0x00, 0x80, 0x00, 0xe0, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2c ,
  {0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2d -
  {0x00, 0x00, 0x00, 0x00, 0xc0, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2e .
  {0xc0, 0x00, 0xf0, 0x00, 0x3c, 0x00, 0x0f, 0x00, 0x03, 0xc0, 0x00, 0xe0, 0x00, 0x3f}, //0x2f /
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x30 0
  {0x00, 0x00, 0x80, 0x80, 0x80, 0xc0, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x00, 0x1f}, //0x31 1
  {0xe0, 0xc0, 0xf0, 0xe0, 0x98, 0x20, 0x8c, 0x20, 0x84, 0x20, 0x87, 0xe0, 0x83, 0xdf}, //0x32 2
  {0x60, 0xc0, 0xe0, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x33 3
  {0x38, 0x00, 0x3e, 0x00, 0x27, 0x80, 0x21, 0xe0, 0xff, 0xe0, 0xff, 0xe0, 0x20, 0x1f}, //0x34 4
  {0x83, 0xe0, 0x83, 0xe0, 0x82, 0x20, 0x82, 0x20, 0xc2, 0x20, 0x7e, 0x20, 0x3c, 0x3f}, //0x35 5
  {0x7f, 0x00, 0xff, 0xc0, 0x82, 0x60, 0x82, 0x20, 0x82, 0x20, 0xfe, 0x20, 0x7c, 0x3f}, //0x36 6
  {0x00, 0x20, 0x00, 0x20, 0xe0, 0x20, 0xfc, 0x20, 0x1f, 0x20, 0x03, 0xe0, 0x00, 0xff}, //0x37 7
  {0x7b, 0xc0, 0xff, 0xe0, 0x86, 0x20, 0x86, 0x20, 0x8c, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x38 8
  {0x03, 0xc0, 0x87, 0xe0, 0x84, 0x20, 0xc4, 0x20, 0x64, 0x20, 0x3f, 0xe0, 0x1f, 0xdf}, //0x39 9
  {0x00, 0x00, 0x00, 0x00, 0x31, 0x80, 0x31, 0x80, 0x31, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3a :
  {0x00, 0x00, 0x00, 0x00, 0xb1, 0x80, 0xf1, 0x80, 0x71, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3b ;
  {0x04, 0x00, 0x0e, 0x00, 0x1b, 0x00, 0x31, 0x80, 0x60, 0xc0, 0xc0, 0x60, 0x80, 0x3f}, //0x3c <
  {0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x1f}, //0x3d =
  {0x80, 0x20, 0xc0, 0x60, 0x60, 0xc0, 0x31, 0x80, 0x1b, 0x00, 0x0e, 0x00, 0x04, 0x1f}, //0x3e >
  {0x01, 0xc0, 0x01, 0xe0, 0x00, 0x20, 0xd8, 0x20, 0xdc, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x3f ?
  {0x7f, 0x40, 0x83, 0x20, 0x83, 0x20, 0xff, 0x20, 0x80, 0x60, 0xff, 0xe0, 0x7f, 0xdf}, //0x40 @
  {0xff, 0x00, 0xff, 0x80, 0x08, 0xc0, 0x08, 0x60, 0x08, 0xc0, 0xff, 0x80, 0xff, 0x1f}, //0x41 A
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x42 B
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xe0, 0xe0, 0x60, 0xdf}, //0x43 C
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x9f}, //0x44 D
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x80, 0x3f}, //0x45 E
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x3f}, //0x46 F
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x88, 0x20, 0xf8, 0xe0, 0xf8, 0xdf}, //0x47 G
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x48 H
  {0x00, 0x00, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x00, 0x00, 0x00, 0x1f}, //0x49 I
  {0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x4a J
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x1f, 0x00, 0x7b, 0xc0, 0xe0, 0xe0, 0x80, 0x3f}, //0x4b K
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x4c L
  {0xff, 0xe0, 0xff, 0xe0, 0x00, 0x80, 0x07, 0x00, 0x00, 0x80, 0xff, 0xe0, 0xff, 0xff}, //0x4d M
  {0xff, 0xe0, 0xff, 0xe0, 0x03, 0x80, 0x0e, 0x00, 0x38, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x4e N
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x4f O
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x50 P
  {0x7f, 0xc0, 0xff, 0xe0, 0x90, 0x20, 0xb0, 0x20, 0xe0, 0x20, 0x7f, 0xe0, 0xff, 0xdf}, //0x51 Q
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x0c, 0x20, 0xff, 0xe0, 0xf3, 0xdf}, //0x52 R
  {0x43, 0xc0, 0xc7, 0xe0, 0x86, 0x20, 0x84, 0x20, 0x8c, 0x20, 0xfc, 0x60, 0x78, 0x5f}, //0x53 S
  {0x00, 0x20, 0x00, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x20, 0x00, 0x20, 0x00, 0x1f}, //0x54 T
  {0x7f, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x55 U
  {0x1f, 0xe0, 0x3f, 0xe0, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0xe0, 0x1f, 0xff}, //0x56 V
  {0x1f, 0xe0, 0xff, 0xe0, 0xe0, 0x00, 0x1c, 0x00, 0xe0, 0x00, 0xff, 0xe0, 0x1f, 0xff}, //0x57 W
  {0xc0, 0x60, 0xf1, 0xe0, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0xf1, 0xe0, 0xc0, 0x7f}, //0x58 X
  {0x03, 0xe0, 0x07, 0xe0, 0xfc, 0x00, 0xfc, 0x00, 0x07, 0xe0, 0x03, 0xe0, 0x00, 0x1f}, //0x59 Y
  {0xc0, 0x20, 0xf0, 0x20, 0xbc, 0x20, 0x8f, 0x20, 0x83, 0xa0, 0x80, 0xe0, 0x80, 0x7f}, //0x5a Z
  {0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0x00, 0x1f}, //0x5b [
  {0x00, 0x20, 0x00, 0xe0, 0x03, 0xc0, 0x0f, 0x00, 0x3c, 0x00, 0xf0, 0x00, 0xc0, 0x1f}, /*0x5c \*/
  {0x00, 0x00, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x1f}, //0x5d ]
  {0x00, 0x80, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0x1f}, //0x5e ^
  {0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x5f _
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x1f}, //0x60 `
  {0x70, 0x00, 0xf9, 0x00, 0x89, 0x00, 0x89, 0x00, 0x89, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x61 a
  {0xff, 0xe0, 0xff, 0xe0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x62 b
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xc3, 0x00, 0x42, 0x1f}, //0x63 c
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x64 d
  {0x7e, 0x00, 0xff, 0x00, 0x91, 0x00, 0x91, 0x00, 0x91, 0x00, 0x9f, 0x00, 0x5e, 0x1f}, //0x65 e
  {0x04, 0x00, 0xff, 0xc0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x1f}, //0x66 f
  {0x9e, 0x00, 0xbf, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x67 g
  {0xff, 0xe0, 0xff, 0xe0, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x68 h
  {0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0xff, 0x60, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x69 i
  {0x80, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0x7f, 0x60, 0x00, 0x1f}, //0x6a j
  {0xff, 0xe0, 0xff, 0xe0, 0x18, 0x00, 0x3c, 0x00, 0x66, 0x00, 0xc3, 0x00, 0xc1, 0x1f}, //0x6b k
  {0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x6c l
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x3f, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6d m
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6e n
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x6f o
  {0xff, 0x00, 0xff, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0x3f, 0x00, 0x1e, 0x1f}, //0x70 p
  {0x1e, 0x00, 0x3f, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x71 q
  {0x01, 0x00, 0xff, 0x00, 0xff, 0x00, 0x04, 0x00, 0x02, 0x00, 0x03, 0x00, 0x03, 0x1f}, //0x72 r
  {0x8e, 0x00, 0x9f, 0x00, 0x99, 0x00, 0x99, 0x00, 0x99, 0x00, 0xf9, 0x00, 0x71, 0x1f}, //0x73 s
  {0x01, 0x00, 0x01, 0x00, 0x7f, 0xc0, 0xff, 0xc0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x1f}, //0x74 t
  {0x7f, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0x00, 0xff, 0x1f}, //0x75 u
  {0x1f, 0x00, 0x3f, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0x00, 0x1f, 0x1f}, //0x76 v
  {0x3f, 0x00, 0xff, 0x00, 0xc0, 0x00, 0x3c, 0x00, 0xc0, 0x00, 0xff, 0x00, 0x3f, 0x1f}, //0x77 w
  {0xe3, 0x00, 0xf7, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0xf7, 0x00, 0xe3, 0x1f}, //0x78 x
  {0x8f, 0x00, 0x9f, 0x00, 0x90, 0x00, 0x90, 0x00, 0x90, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x79 y
  {0xc1, 0x00, 0xe1, 0x00, 0xb1, 0x00, 0x99, 0x00, 0x8d, 0x00, 0x87, 0x00, 0x83, 0x1f}, //0x7a z
  {0x00, 0x00, 0x04, 0x00, 0x7f, 0xc0, 0xe0, 0xe0, 0x80, 0x20, 0x80, 0x00, 0x00, 0x1f}, //0x7b {
  {0x00, 0x00, 0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x7c |
  {0x00, 0x00, 0x80, 0x20, 0xe0, 0xe0, 0x7f, 0xc0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x5f}, //0x7d }
  {0x00, 0x60, 0x00, 0x20, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0xc0, 0x00, 0x5f}  //0x7e ~
};



/*########***** Define Color *****########
  #####  Color = RRRRR GGGGGG BBBBB ######
  ########************************######## */

#define no_bg               0x0001			//No Color Back Ground
#define BLACK               0x0000
#define WHITE               0xFFFF
#define RED                 0x8000
#define GREEN               0x0400
#define BLUE                0x0010
#define YELLOW           	0xFFF0
#define CYAN                0x0410
#define MAGENTA             0x8010
#define BROWN               0xFC00
#define OLIVE	            0x8400

#define BRIGHT_RED          0xF800
#define BRIGHT_GREEN        0x07E0
#define BRIGHT_BLUE         0x001F
#define BRIGHT_YELLOW       0xFFE0
#define BRIGHT_CYAN         0x07FF
#define BRIGHT_MAGENTA    	0xF81F

#define LIGHT_GRAY          0x8410
#define DARK_GRAY         	0x4208
#define LIGHT_BLUE          0x841F
#define LIGHT_GREEN         0x87F0
#define LIGHT_CYAN          0x87FF
#define LIGHT_RED         	0xFC10
#define LIGHT_MAGENTA      	0xFC1F

#define GRAY0       	    0xE71C
#define GRAY1         	    0xC618
#define GRAY2             	0xA514
#define GRAY3             	0x630C
#define GRAY4            	0x4208
#define GRAY5	            0x2104
#define	GRAY6	          	0x3186	

#define	BLUE0	            0x1086	
#define	BLUE1       		0x3188 
#define	BLUE2          	    0x4314
#define BLUE3         	    0x861C	

#define	CYAN0	            0x3D34
#define	CYAN1               0x1DF7		

#define	GREEN0              0x0200	
#define	GREEN1              0x0208	


//------------ Color For Build Button -------------

#define BT_RED              1
#define BT_GREEN            2
#define BT_BLUE             3
#define	BT_YELLOW          	4
#define BT_WHITE            5
#define BT_CYAN             6
#define BT_MAGENTA      	7
#define BT_BROWN            8

#define BT_LRED             9
#define BT_LGREEN          10
#define BT_LBLUE           11
#define	BT_LYELLOW         12
#define BT_LWHITE          13
#define BT_LCYAN           14
#define BT_LMAGENTA        15
#define BT_LBROWN          16





//********** Valiable for Touch Screen Function GLCD ************

long dif_adc_X,dif_adc_Y              ;
long buf_adc_X[100],buf_adc_Y[100]    ;
long ave_adc_X,ave_adc_Y              ;


long tcs_ave_X[3]				       ;	// Keep Value adc Touch Point X
long tcs_ave_Y[3]                      ;    // Keep Value adc Touch Point Y
long dis_XD[3] = {24,215,120}          ;    // Value refer Point X at 10% of Display(X=240:0-239) 3 Position
long dis_YD[3] = {32,160,287}          ;    // Value refer Point Y at 10% of Display(Y=320:0-319) 3 Position
long divider,An,Bn,Cn,Dn,En,Fn,X,Y     ;	// Valiable for keep coefficient Calibrat and position touch Screen
unsigned char num1,num2,num3           ;	// Variable for keep data Dec to Ascii



//************ 1.Function Control Graphic LCD ET-TFT240320TP-3.2 REV.B (Driver SPFD5408A) ***********


void gp_cs_hi(void)                      ;
void gp_cs_lo(void)                      ;
void gp_rs_hi(void)					     ;
void gp_rs_lo(void)                      ;
void gp_wr_hi(void)					     ;
void gp_wr_lo(void)                      ;
void gp_rd_hi(void)					     ;
void gp_rd_lo(void)					     ;
void gp_res_hi(void)				     ;
void gp_res_lo(void)				     ;
void gp_bl_on(void)					     ;
void gp_bl_off(void)				     ;
void gp_wr_cmm(unsigned char cmm)        ;
void gp_wr_data(unsigned int dw)         ;
void init_glcd(void)                     ;


//************** 2.Function Control SPI Touch-Screen (ADS7846) ****************


void tcs_sck_hi(void)                             ;
void tcs_sck_lo(void)                             ;
void tcs_cs_hi(void)					          ;
void tcs_cs_lo(void)					          ;
void tcs_sdo_hi(void)					          ;
void tcs_sdo_lo(void)				  	          ;
void tcs_busy(void)                               ;
char status_touch(void)                           ;
unsigned char tcs_wr(unsigned char DataByte)      ;
void tcs_read(void)                               ;
void ave_x_y(unsigned char num)	                  ;

void touch_calibrate(void)                        ;
void set_matrix(void)                             ;
void get_point(unsigned char num)                 ;


//********************** 3.Function Application of GLCD **********************

void bg_color(unsigned int bg_color)                                                                 ;
void plot_mark(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2,unsigned int color)   ;
void text_7x11(unsigned char row,int adx,int ady,unsigned int fg_clr,unsigned int bg_clr)            ;
void lcd_printStr(char *str,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)         ;
void lcd_print3Cha(char ch1,char ch2,char ch3,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color);

void plot_picture(char *bmp,unsigned int xad,unsigned int yad,unsigned int width,unsigned int high)       ;  
void button_build(unsigned int px,unsigned int py,unsigned int width,unsigned int high,unsigned int c_bt,char *tx,unsigned char num_tx,unsigned int c_tx) ;       
void plot_dot(unsigned int Xadd,unsigned int Yadd,unsigned int color)                                     ;
void Line(int x_1,int y_1,int x_2,int y_2,unsigned char line_type,unsigned int color)                     ;
void Circle(int CX,int CY,unsigned char Radius,unsigned tick,unsigned int color,unsigned char fill)       ;
void Ellipse(unsigned int h,unsigned int k,long rx,long ry,unsigned char tick,unsigned int color,unsigned char fill) ;
void Rectan(int x1,int y1,int x2,int y2,unsigned char tick,unsigned int color,unsigned char fill)                    ;   




/***************************************************************
 **                 Function use For Printf(UART0)            **
 ***************************************************************/

/*
//--------------------- Prototype function ---------------------

static int uart_putchar(char c,FILE *stream) ;
static FILE uart_str = FDEV_SETUP_STREAM(uart_putchar,NULL,_FDEV_SETUP_WRITE);
void Init_Serial(unsigned int baudrate);


//----------------------- UART0 Putchar -------------------------

static int uart_putchar(char c,FILE *stream)
 {
   if(c=='\a')
     {
	  fputs("*ring*\n",stderr) ;
	  return 0 ;
     }
   
   if(c=='\n')
     uart_putchar('\r',stream) ;

   loop_until_bit_is_set(UCSR0A,UDRE);
   UDR0 = c ;

   return 0 ;
 }

//------------------- Initial UART0 -----------------------

static void UART0_Init(void)
 {

   UCSR0B = 0x00      ;     //disable while setting baud rate
   UBRR0H = 0         ;
   UBRR0L = 103       ;     //Set Baud Rate 9600
  
   UCSR0C = 0x06      ;     //Set Frame format : 8data,NoneParity,1stop bit
   UCSR0B = 0x98      ;     //Uart0 Control Enable Reciver and transmitter
   
   stdout = &uart_str ;     //Set address uart_str to stdout

 }
*/


/*****************************************
 **         Function Delay ms           **
 *****************************************/

 void delay_ms(unsigned int cnt)
  {
    for(;cnt>0;cnt--)
       _delay_ms(1);
  }


/*****************************************
 **        Function Initial Port        **
 *****************************************/

 void init_port(void)
  {

   DDRF  = 0xFF     ;        //Set PortF(0..7) = Output for data GLCD Byte Hig

   DDRE |= 0xFC     ;        //Set PortE(2..7) = Output for Control GLCD(CS,RS,WR,RD,Reset,BL)  
   
   DDRD  = 0x98     ;        //Set PortD = PD7,PD4,PD3:Output ;PD6,PD5,PD2:Input for Touch Screen
   
   //------ Set Value Default Port ------ 

   PORTF = 0        ;
   PORTE = 0x7C     ;
   PORTD = 0x80     ;

  }


//*******************************************************************
//*********** Convert Dec to Ascii  3 digit(000-999) ****************
//*******************************************************************        

void dec_to_ascii(unsigned int num)
 {
	 num1  = (num/100) + 0x30  ;//colum 100
	 num   =  num%100          ;
	 num2  = (num/10)  + 0x30  ;//colum 10
	 num3  =  (num%10) + 0x30  ;//colum 1	 

 }




 


/*####################################################
  ##                  MAIN PROGRAM                  ##
  ####################################################*/

int main(void)
 {
   
   int cr_x,cr_y,x1,y1,x2,y2,s_color =BRIGHT_YELLOW  ;
   unsigned char cl=0,k                              ;
   int x3,h1,h2                                      ;
   char str_buf1[25]                                 ;
   char str_buf2[50] = {"www.etteam.com ==> ET-TFT240320TP-3.2 Rev.B DEMO  "}   ;




 //------------------- Start GLCD and Cribrate Touch Screen ---------------
  
   init_port()			 ;
   init_glcd()           ;
   touch_calibrate()	 ;
  // UART0_Init()          ;

  
 Line(2,1,239,1,2,BRIGHT_BLUE)      ;		//Line Hor. top
 Line(2,318,239,318,2,BRIGHT_BLUE)  ;	    //Line Hor. botton
 Line(2,1,2,318,2,BRIGHT_BLUE)      ;		//Line Ver. left
 Line(238,1,238,318,2,BRIGHT_BLUE)  ;	    //Line Ver. right

 Line(8,6,234,6,5,BRIGHT_GREEN)     ;		//Line Hor. top
 Line(8,190,234,190,5,BRIGHT_GREEN) ;	    //Line Hor. botton
 Line(8,6,8,190,5,BRIGHT_GREEN)     ;		//Line Ver. left
 Line(230,6,230,190,5,BRIGHT_GREEN) ;	    //Line Ver. right

 Line(8,160,234,160,5,BRIGHT_GREEN) ;	    //Line Hor. botton
 Rectan(13,165,230,189,1,WHITE,1)   ;


 //---------------------- Button --------------------

 button_build(26,310,45,22,BT_RED,"*",1,WHITE)     ;
 button_build(97,310,45,22,BT_RED,"0",1,WHITE)     ;
 button_build(168,310,45,22,BT_RED,"#",1,WHITE)    ;

 button_build(26,283,45,22,BT_GREEN,"7",1,WHITE)   ;
 button_build(97,283,45,22,BT_GREEN,"8",1,WHITE)   ;
 button_build(168,283,45,22,BT_GREEN,"9",1,WHITE)  ;

 button_build(26,256,45,22,BT_MAGENTA,"4",1,WHITE) ;
 button_build(97,256,45,22,BT_MAGENTA,"5",1,WHITE) ;
 button_build(168,256,45,22,BT_MAGENTA,"6",1,WHITE);

 button_build(26,229,45,22,BT_CYAN,"1",1,WHITE)    ;
 button_build(97,229,45,22,BT_CYAN,"2",1,WHITE)    ;
 button_build(168,229,45,22,BT_CYAN,"3",1,WHITE)   ;

 Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture
 plot_picture(ett2,81,45,80,80)                    ;  
  

  cr_x = 20    ;	      //Set Positon Start text when touch button
  cr_y = 182   ;
  
  while(1)
	 {	  		
	  
		if(status_touch())               	    //return (1) if tap touch screen is detected.
		{
	      get_point(30)	 ;

         //-------------------------- Check Touch Button Line1 ('1','2','3') --------------------  

		  if((Y>=208)&&(Y<=229))               	//Check Position Y-Touch 
			{

			  if((X>=26)&&(X<=70))			    //Check Position X-Touch 
			    {
			      button_build(26,229,45,22,BT_LCYAN,"1",1,BRIGHT_RED) ; //Plot Button status Touch

				 //--------- Show Text "1" -----------
				  lcd_printStr("1",cr_x,cr_y,BRIGHT_RED,no_bg)    ; // Plot text "1"
				  cr_x = cr_x+10			                      ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											    //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)             ;	//Clear Screen Text
				     cr_x = 20		                              ; //Set Position Start Text New 
					 lcd_printStr("1",cr_x,cr_y,BRIGHT_RED,no_bg) ; //Plot text "1" Positio start
					 cr_x = cr_x+10                               ; //Position Next Text

				   }

				   while(status_touch())                             ; //Protect Press Button Double	 
				   X=0 ;
				 //---------- Show Positon Touch ------------

				   Rectan(14,12,228,158,1,BLACK,1)               ; //Clear Screen Positin Show Picture
				 
				   lcd_printStr("Touch X: ",70,130,WHITE,no_bg)              ; //Plot String Color White
			       lcd_printStr("Touch Y: ",70,150,WHITE,no_bg)              ;
				  
                   x1= 114;
				   x2= 85 ; 
				   y1= 85 ;
				   y2= 78 ;				   

				   do
				   {
				     if(status_touch())               	  //return (1) if tap touch screen is detected.
		              {
			            get_point(20)                 ; //Calculate Position Address Display Keep result in valiable X,Y
   
					    if((X>=22)&&(X<=220)&&(Y>=20)&&(Y<=150))	 //Check Area for Touch 
					     {
			              plot_mark(x1,y1,x2,y2,BLACK) ; //Del Cursot '+' in Position Touch old

			              x1 = X-7 ;
			              y1 = Y   ;
			              x2 = X   ;
			              y2 = Y-7 ;
			              plot_mark(x1,y1,x2,y2,LIGHT_RED)			                ; //Plot Cursot '+' in Position Touch Screen

			              lcd_printStr("Touch X: ",70,130,WHITE,no_bg)              ; //Plot String Color White
			              lcd_printStr("Touch Y: ",70,150,WHITE,no_bg)              ;

		  	              dec_to_ascii(X)	                                        ; //Convert Position X-Addres to ASCII
			              lcd_print3Cha(num1,num2,num3,145,130,BRIGHT_YELLOW,BLACK) ; //Plot Position X-Address on GLCD
			              dec_to_ascii(Y)	                                        ; //Convert Position Y-Addres to ASCII
		   	              lcd_print3Cha(num1,num2,num3,145,150,BRIGHT_YELLOW,BLACK) ; //Plot Position Y-Address on GLCD	
		                }
					  }   
					}while (!((X>=26)&&(X<=70)&&(Y>=208)&&(Y<=229)));   //Touch button "1" Exit loop


				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch
				  
			      button_build(26,229,45,22,BT_CYAN,"1",1,WHITE)    ; //Plot Button status Normal

			    }


			  if((X>=97)&&(X<=141))			   //Check Position X-Touch 
			    {
			       button_build(97,229,45,22,BT_LCYAN,"2",1,BRIGHT_RED) ; //Plot Button status Touch

				  	//--------- Show Text "2" -----------
				  
				  lcd_printStr("2",cr_x,cr_y,BRIGHT_RED,no_bg)    ; // Plot text "2"
				  cr_x = cr_x+10			                      ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											    //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)             ;	//Clear Screen Text
				     cr_x = 20		                              ; //Set Position Start Text New 
					 lcd_printStr("2",cr_x,cr_y,BRIGHT_RED,no_bg) ; //Plot text "2" Positio start
					 cr_x = cr_x+10                               ; //Position Next Text

				   }

				    while(status_touch())                             ; //Protect Press Button Double	 
				    X=0 ;

				 //---------- Show Write Touch ------------

				   Rectan(14,12,228,158,1,BLACK,1)                          ; //Clear Screen Positin Show Picture	 
				   lcd_printStr("'Give Touch Display'",45,90,WHITE,no_bg)   ; //Plot String Color White
			   	   Rectan(105,199,133,204,1,s_color,1)                      ; //Plot Tab Selec color
				   do
				   {
				     if(status_touch())               	  //return (1) if tap touch screen is detected.
		              {
			            get_point(20)                   ; //Calculate Position Address Display Keep result in valiable X,Y
   													 
					    if((X>=17)&&(X<=225)&&(Y>=15)&&(Y<=155))	 //Check Area for Touch 
					     {
			              plot_dot(X,Y,s_color)                             ; //Plot dot1
						  plot_dot(X+1,Y+1,s_color)                         ; //Plot dot2 
		                 }
                      
						 if((X>=105)&&(X<=133)&&(Y>=195)&&(Y<=200))	 //Check Area for Touch
						 {
						   switch(cl)								          //Selec Color for Write
						     {
							     case 0 :
								        s_color = LIGHT_RED	                  ;
										cl = cl+1                             ;
								        Rectan(105,199,133,204,1,s_color,1)   ;	 //Plot Tab color
								  break;

								 case 1 :
								        s_color = LIGHT_GREEN	              ;
										cl = cl+1                             ;
								        Rectan(105,199,133,204,1,s_color,1)   ;
								  break;
	    

								 case 2 :
								        s_color = LIGHT_BLUE	              ;
										cl = cl+1                             ;
								        Rectan(105,199,133,204,1,s_color,1)   ;
								  break;

								 case 3 :
								        s_color = WHITE	                      ;
										cl = cl+1                             ;
								        Rectan(105,199,133,204,1,s_color,1)   ;
								  break;



								 case 4 :
								        s_color = BRIGHT_YELLOW	              ;
										cl = 0                                ;
								        Rectan(105,199,133,204,1,s_color,1)   ;
								  break;

							 
							 }
						   while(status_touch())                                  ; //Protect Press Button Double	                
			               X=0                                                    ; //Clear Position X-Touch 	   
						}


					   if((X>=22)&&(X<=43)&&(Y>=330)&&(Y<=342))			         //Check Press Icon Home
		                {
		                  Rectan(14,12,228,158,1,BLACK,1)                        ; //Clear Screen Positin Show Picture
			              while(status_touch())                                  ; //Protect Press Button Double	                
			              X=0                                                    ; //Clear Position X-Touch 
			            }

					  }   
				   }while (!((X>=97)&&(X<=141)&&(Y>=208)&&(Y<=229)));   //Touch button "2" Exit loop


				  while(status_touch())                             ;   //Protect Press Button Double	                
			      X=0                                               ;	//Clear Position X-Touch
			      
				  Rectan(105,199,133,204,1,BLACK,1)                 ;   //Clear Tab Color
				  button_build(97,229,45,22,BT_CYAN,"2",1,WHITE)    ;	//Plot Button status Normal
			    }



			  if((X>=168)&&(X<=212))		  //Check Position X-Touch 
			    {
			      button_build(168,229,45,22,BT_LCYAN,"3",1,BRIGHT_RED) ; //Plot Button status Press

				  	//--------- Show Text "3" -----------
				  
				  lcd_printStr("3",cr_x,cr_y,BRIGHT_RED,no_bg)    ; // Plot text "3"
				  cr_x = cr_x+10			                      ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											    //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)             ;	//Clear Screen Text
				     cr_x = 20		                              ; //Set Position Start Text New 
					 lcd_printStr("3",cr_x,cr_y,BRIGHT_RED,no_bg) ; //Plot text "3" Positio start
					 cr_x = cr_x+10                               ; //Position Next Text

				   }

				   while(status_touch())                          ; //Protect Press Button Double	 
				   X =0 ;

				   //---------- Show Picture engine  ------------

				   Rectan(14,12,228,158,1,BLACK,1)                ; //Clear Screen Positin Show Picture	 
				   Rectan(40,87,201,145,1,WHITE,1)                ; //BlackGround Picture											    
				   plot_picture(intro,42,89,158,55)               ;	
				   			 
				   do
				   {
				    plot_picture(engine1,84,20,73,55)             ;
					delay_ms(10)                                  ; 
					plot_picture(engine2,84,20,73,55)             ; 
					delay_ms(10)                                  ;  
					plot_picture(engine3,84,20,73,55)             ; 
					delay_ms(10)                                  ; 
					plot_picture(engine4,84,20,73,55)             ; 
					delay_ms(10)                                  ;
				
					
					if(status_touch())               	             //return (1) if tap touch screen is detected.
			          {
					   get_point(20)                              ; //Calculate Position Address Display Keep result in valiable X,Y
					  }
				   while(status_touch())                          ; //Protect Press Button Double
					     
				   }while (!((X>=168)&&(X<=212)&&(Y>=208)&&(Y<=229)));   //Touch button "3" Exit loop             


				  while(status_touch())                           ; //Protect Press Button Double	                
			      X=0                                             ;	//Clear Position X-Touch		      
		         
			      button_build(168,229,45,22,BT_CYAN,"3",1,WHITE) ; //Plot Button status Normal
			    }
		   }

		 //-------------------------- Check Touch Button Line2 ('4','5','6')--------------------

	      if((Y>=235)&&(Y<=256))              //Check Position Y-Touch 
			{

			  if((X>=26)&&(X<=70))	          //Check Position X-Touch   
			    {
			       button_build(26,256,45,22,BT_LMAGENTA,"4",1,BRIGHT_RED) ; //Plot Button Status Touch

		           	//--------- Show Text "4" -----------
				  
				  lcd_printStr("4",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "4"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("4",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "4" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				   while(status_touch())                            ; //Protect Press Button Double
                   X=0 ;
				  	 	 
				 //---------- Show Circle1 ------------

				   Rectan(14,12,228,158,1,BLACK,1)                  ; //Clear Screen Positin Show Picture
				 
				 do
				  {
				     delay_ms(80)                                   ;

					 Circle(67,55,30,1,BRIGHT_BLUE,1)	            ;
					 Circle(174,55,30,1,BRIGHT_BLUE,1)	            ;
					 Circle(121,120,20,1,BRIGHT_RED,1)		        ; 
	      	  
					 Circle(67,55,20,1,BLACK,1)	                    ;
					 Circle(174,55,20,1,BLACK,1)	                ;
					 Circle(121,120,20,1,BLACK,0)		            ; 
					 
					
					  	
				 	 if(status_touch())               	               //return (1) if tap touch screen is detected.
			            get_point(20)                               ;  //Calculate Position Address Display Keep result in valiable X,Y
        
					}while (!((X>=26)&&(X<=70)&&(Y>=235)&&(Y<=256)));  //Touch button "1" Exit loop


				   while(status_touch())                            ;  //Protect Press Button Double	                
			       X=0                                              ;  //Clear Position X-Touch
			

			       button_build(26,256,45,22,BT_MAGENTA,"4",1,WHITE); //Plot Button Status Normal
			    }


			  if((X>=97)&&(X<=141))		                           	  //Check Position X-Touch 
			    {
			      button_build(97,256,45,22,BT_LMAGENTA,"5",1,BRIGHT_RED) ;  //Plot Button Status Press

		           	//--------- Show Text "5" -----------
				  
				  lcd_printStr("5",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "5"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("5",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "5" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				   while(status_touch())                            ; //Protect Press Button Double
                   X=0 ;
				  	 	 
				 //---------- Show Move ------------

				   Rectan(14,12,228,158,1,BLACK,1)                  ; //Clear Screen Positin Show Picture
				  
				   x1=25 ;
				   x2=207;
				   y1=145;

				   do
				    {
					  delay_ms(20);
     
					 //------------ RED -----------
					  if(x1<=218)
					   {  
                         Circle(x1,44,10,1,BRIGHT_RED,1)	     ;  //Plot Circle color RED
						 Line(x1-11,34,x1-11,55,1,BLACK)         ;	//Line   
			             x1++                                    ;											 
		               }

					  if(x1>218)     						        //check end for move Red
					   { 
                         for(y2=0;y2<=20;y2++)					 
						  {
						   Line(x1-11,34,x1-11,55,1,BLACK)       ;		
						   x1++ ;
						   delay_ms(20);
						  }
					      x1 = 25 ;	  
					   }

				    //---------- Magenta-----------

					   if(x2>=30)
					   {   
						 Circle(x2,125,10,1,BRIGHT_MAGENTA,1)	  ;  //Plot Circle color Magenta
						 Line(x2+11,115,x2+11,136,1,BLACK)        ;  //Line                           
			             x2--                                     ;											 
		               }

					  if(x2<30)     						         //check end for move magenta
					   { 
                         for(y2=0;y2<=20;y2++)					 
						  {
						   Line(x2+11,115,x2+11,136,1,BLACK)      ;		
						   x2-- ;
						   delay_ms(20);
						  }
					      x2 = 207 ;	  
					  }


					 //---------- YELLOW-----------

					   if(y1>=25)
					   {   
						 Circle(121,y1,10,1,BRIGHT_YELLOW,1)	  ;  //Plot Circle color Magenta
						 Line(111,y1+11,132,y1+11,1,BLACK)        ;  //Line                           
			             y1--                                     ;											 
		               }

					  if(y1<25)     						         //check end for move magenta
					   { 
                         for(y2=0;y2<=20;y2++)					 
						  {
						   Line(111,y1+11,132,y1+11,1,BLACK)       ;		
						   y1-- ;
						   delay_ms(20);
						  }
					      y1 = 145 ;	  
					  }


					  if(status_touch())               	             //return (1) if tap touch screen is detected.
			            get_point(20)                                  ; //Calculate Position Address Display Keep result in valiable X,Y
        
                 	   

					}while (!((X>=97)&&(X<=141)&&(Y>=235)&&(Y<=256)))  ; //Touch button "1" Exit loop


				    while(status_touch())                              ; //Protect Press Button Double	                
			        X=0                                                ; //Clear Position X-Touch

			        button_build(97,256,45,22,BT_MAGENTA,"5",1,WHITE)  ; //Plot Button Status Normal
			    }


			  if((X>=168)&&(X<=212))		 //Check Position X-Touch 
			    {
			      button_build(168,256,45,22,BT_LMAGENTA,"6",1,BRIGHT_RED) ;//Plot Button Status Touch
				  
				   	//--------- Show Text "6" -----------
				  
				  lcd_printStr("6",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "6"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("6",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "6" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				   while(status_touch())                            ; //Protect Press Button Double
                   X=0 ;

				   //---------- Show Touch Move Shooting ------------

				   Rectan(14,12,228,158,1,BLACK,1)                  ; //Clear Screen Positin Show Picture
				  
				   x1=110   ;
				   x2=129   ; 
				
				   x3 = x1  ;
			

				   h1 = 132 ;
				   h2 = 136 ;

				   Rectan(28,62,215,102,1,BRIGHT_YELLOW,1)                 ; //Plot Rectan
			       Rectan(x1,138,x2,148,1,BRIGHT_CYAN,1)                   ; //Plot Rectan Base
				   lcd_printStr("Touch '*'= Move Left",20,24,WHITE,no_bg)  ; 	
				   lcd_printStr("Touch '0'= Shoot",20,39,WHITE,no_bg)      ;
				   lcd_printStr("Touch '#'= Move Right",20,54,WHITE,no_bg) ;  		
				 
				   do
				    {
					  if(status_touch())               	                     //return (1) if tap touch screen is detected.
			           { 
						get_point(20)                                     ;  //Calculate Position Address Display Keep result in valiable X,Y
        
                 	    if((Y>=289)&&(Y<=310))	                             //Check Line 4
						 {
						   if((X>=26)&&(X<=70))						         //Check Touch button '*' Move Left
						    {
							  h1 = 132 ;
				              h2 = 136 ;
							
							  Rectan(x1,138,x2,148,1,BRIGHT_CYAN,1)      ;	//Rectan Base
							  Line(x2+1,138,x2+1,149,1,BLACK)            ;  //Line Del

							  x3=x1 ;
							  x1--  ;
							  x2--  ;	


							  if(x1<18)
							   {
								x1++ ;
								x2++ ;
							   }
							}
						 	    
						   if((X>=168)&&(X<=212))				         //Check Touch button '#' Move Right
						    {
					 	     h1 = 132 ;
				             h2 = 136 ;
							
							 Rectan(x1,138,x2,148,1,BRIGHT_CYAN,1)     ;  //Rectan Base
							 Line(x1-1,138,x1-1,149,1,BLACK)           ;  //Line Del
						   
						     x3=x1     ;
							 x1++      ;
							 x2++      ;	
							 if(x2>225)
							  {
								x1--   ;
								x2--   ;
							  }
						    }
							
							if((X>=97)&&(X<=141))			                   //Check Touch button '0'Shoot 
							 {   
							  do
							   {
							     Rectan(x3+8,h1,x3+11,h2,1,LIGHT_RED,1)       ; //Rectan Shoot
								 Line(x3+8,h2,x3+11,h2,1,BLACK)               ;
								 h1--    ;										//Position Shoot Move
								 h2--    ;
								 k = h1  ;
								 
								 if(k<60)
								   {
									 Rectan(x3+8,h1,x3+11,h2,1,BLACK,1)       ; //Clear Shoot
									 h1 = 132 ;								    //Position start Shoot Move
				                     h2 = 136 ;									    
								   }

                                 delay_ms(10);
							   }while(status_touch()) ;	
							 }
							  Rectan(x3+8,h1,x3+11,h2,1,BLACK,1)       ;
							  h1 = 132 ;
				              h2 = 136 ;
						  }										
					   }

					}while(!((X>=168)&&(X<=212)&&(Y>=235)&&(Y<=256)))  ; //Touch button "1" Exit loop


				  while(status_touch())                                ; //Protect Press Button Double	                
			      X=0                                                  ; //Clear Position X-Touch
     
			     button_build(168,256,45,22,BT_MAGENTA,"6",1,WHITE)    ; //Plot Button Status Normal
			   }

			}

		  //-------------------------- Check Touch Button Line3 ('7','8','9')--------------------

	      if((Y>=262)&&(Y<=283))              //Check Position Y-Touch 
			{

			  if((X>=26)&&(X<=70))	          //Check Position X-Touch   
			    {
			      button_build(26,283,45,22,BT_LGREEN,"7",1,BRIGHT_RED)   ;	//Plot Button Status Touch

		          	//--------- Show Text "7" -----------
				  
				  lcd_printStr("7",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "7"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("7",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "7" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch
     

			      //------------ Show Text RUN Right to Left -------------
                  
				  
				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture
				
				  plot_picture(ett2,81,50,80,80)                    ; 

				  for(k=0;k<25;k++)
                      str_buf1[k] = 0x20                            ;  // Clear Buffer
				 
			      do
			       {
                      h1 = 0                                        ;  //Pointer data start=0
                      for(k=0;k<50;k++)                                //Loop Shift data Right to Left
                       {
					   x1 = 22;
                        str_buf1[0] = str_buf2[h1]                  ;  //Shift Next Charecter 

                        for(x3=24;x3>=0;x3--)                                //Loop Print data 24 Byte(Full Line)Start Byte 24
                        {
                          text_7x11(str_buf1[x3],x1,40,LIGHT_BLUE,WHITE)  ;  //Print charecter
						  x1=x1+8                                         ;  //Increment X-position for Next charecter
                        

						  if(status_touch())               	                 //return (1) if tap touch screen is detected.
			               {
						    get_point(20)                                 ; //Calculate Position Address Display Keep result in valiable X,Y
                            if((X>=26)&&(X<=70)&&(Y>=262)&&(Y<=283))  
							 {      
						      k  = 55                                     ; //exit loop for if  touch "7"                     
						      x3 = 1-2                                    ; 
							 }
						   }
                          
						}

	                    delay_ms(140)                                     ;
      
                        h1++                                              ;   //Increment Pointer data 
	                  
	                    h2 = 24                                           ;   //Pointer Charecter /Line (24 Charecter/Line)
	                    for(x3=23;x3>=0;x3--)                                 //Loop Shift Data in Buffer
	                     {
                          str_buf1[h2] = str_buf1[x3]                     ;	  //Shift data buf[22] to buf[23],buf[21] to buf[22],...
                          h2-- ;
	                     }
						
                      } //for k			  
					  	
				 
				   }while (!((X>=26)&&(X<=70)&&(Y>=262)&&(Y<=283)))  ; //Touch button "1" Exit loop

				   while(status_touch())                             ; //Protect Press Button Double	                
			       X=0  ;                              

				  button_build(26,283,45,22,BT_GREEN,"7",1,WHITE)    ; //Plot Button Status Normal

			    }




			  if((X>=97)&&(X<=141))			                               //Check Position X-Touch 
			    {
			      button_build(97,283,45,22,BT_LGREEN,"8",1,BRIGHT_RED) ;  //Plot Button Status Touch

		           	//--------- Show Text "8" -----------
				  
				  lcd_printStr("8",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "8"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("8",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "8" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture
				  plot_picture(ett2,81,45,80,80)                    ; 

				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch
          
				  button_build(97,283,45,22,BT_GREEN,"8",1,WHITE)   ; //Plot Button Status Normal
				 
			    }


			  if((X>=168)&&(X<=212))		 //Check Position X-Touch 
			    {
			      button_build(168,283,45,22,BT_LGREEN,"9",1,BRIGHT_RED) ;//Plot Button Status Touch

		         	//--------- Show Text "9" -----------
				  
				  lcd_printStr("9",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "9"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("9",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "9" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }

				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture	
				  plot_picture(ett2,81,45,80,80)                    ; 
				 
				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch   

			      button_build(168,283,45,22,BT_GREEN,"9",1,WHITE)  ; //Plot Button Status Normal
			    }

			}

        //-------------------------- Check Touch Button Line4 ('*','0','#')--------------------

	      if((Y>=289)&&(Y<=310))              //Check Position Y-Touch 
			{

			  if((X>=26)&&(X<=70))	          //Check Position X-Touch   
			    {
			       button_build(26,310,45,22,BT_LRED,"*",1,BRIGHT_RED)      ;  //Plot Button Status Touch

		           	//--------- Show Text "*" -----------
				  
				  lcd_printStr("*",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "*"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("*",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "*" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }


				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture	
				  plot_picture(ett2,81,45,80,80)                    ; 

				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch   
 
				  button_build(26,310,45,22,BT_RED,"*",1,WHITE)     ; //Plot Button Status Normal
			    }


			  if((X>=97)&&(X<=141))			                          //Check Position X-Touch 
			    {
			       button_build(97,310,45,22,BT_LRED,"0",1,BRIGHT_RED)  ;  //Plot Button Status Touch
				   	
					//--------- Show Text "0" -----------
				  
				  lcd_printStr("0",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "9"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("0",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "0" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }


				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture
				  plot_picture(ett2,81,45,80,80)                    ; 
				 
				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch   


			       button_build(97,310,45,22,BT_RED,"0",1,WHITE)     ; //Plot Button Status Normal
			    }


			  if((X>=168)&&(X<=212))		 //Check Position X-Touch 
			    {
			      button_build(168,310,45,22,BT_LRED,"#",1,BRIGHT_RED)      ;  //Plot Button Status Touch

		       	  	//--------- Show Text "#" -----------
				  
				  lcd_printStr("#",cr_x,cr_y,BRIGHT_RED,no_bg)      ; //Plot text "#"
				  cr_x = cr_x+10			                        ; //Position Next Text
				                                 
                  if(cr_x>220)
				   {											      //Check Text End Line
				     Rectan(13,165,230,189,1,WHITE,1)               ; //Clear Screen Text
				     cr_x = 20		                                ; //Set Position Start Text New 
					 lcd_printStr("#",cr_x,cr_y,BRIGHT_RED,no_bg)   ; //Plot text "#" Positio start
					 cr_x = cr_x+10                                 ; //Position Next Text

				   }


				  Rectan(14,12,228,158,1,WHITE,1)                   ; //Clear Screen Positin Show Picture
				  plot_picture(ett2,81,45,80,80)                    ; 

				  while(status_touch())                             ; //Protect Press Button Double	                
			      X=0                                               ; //Clear Position X-Touch   
		      
				  button_build(168,310,45,22,BT_RED,"#",1,WHITE)    ; //Plot Button Status Normal
			    }
			}

		   if((Y>=330)&&(Y<=342))
		    {
		     if((X>=115)&&(X<=138))			                            //Check Touch Icon book
		      {
			    
		        Rectan(13,165,230,189,1,WHITE,1)                      ; //Clear Screen Text
				cr_x = 20                                             ; //Set Position Start Text
                cr_y = 182                                            ;
			    while(status_touch())                                 ; //Protect Press Button Double	                
			    X=0                                                   ; //Clear Position X-Touch 
			  }

		     if((X>=22)&&(X<=43))			                             //Check Touch Icon Home
		      {
		       Rectan(14,12,228,158,1,BLACK,1)                        ; //Clear Screen Positin Show Picture
			   while(status_touch())                                  ; //Protect Press Button Double	                
			   X=0                                                    ; //Clear Position X-Touch 
			  }
		    }

   	    } // if status
	 }	  // while 



   return 0 ;

}



/* ############################################################################################
   ##                                                                                        ##
   ##                                        SUB-PROGRAM                                     ##
   ##                                                                                        ##
   ############################################################################################ */





/*##########**********************************************************************************#########
  ##########**                                                                              **#########
  ##########** 1.Function Control of Graphic LCD EL-TFT240320TP-3.2 REV.B(Driver SPFD5408S) **#########
  ##########**                                                                              **#########                 
  ##########**********************************************************************************######### */


//******** Control Pin cs(PE2=CS) ********

void gp_cs_hi(void)
 {
   sbi(PORTE,2) ;	   //PE2 = Hi
 }

void gp_cs_lo(void)
{
  cbi(PORTE,2) ;	   //PE2 = Lo
}

//******* Control Pin rs (PE3=RS) *******

void gp_rs_hi(void)
 {
   sbi(PORTE,3);	   //PE3 = Hi
 }


void gp_rs_lo(void)
{
  cbi(PORTE,3) ;	   //PE3 = Lo
}

//******* Control Pin wr(PE4=WR) *********

void gp_wr_hi(void)
 {
   sbi(PORTE,4);	   //PE4 = Hi
 }

void gp_wr_lo(void)
{
   cbi(PORTE,4) ;      //PE4 = Lo
}

//****** Control Pin rd(PE5=RD) ******
void gp_rd_hi(void)
 {
    sbi(PORTE,5) ;	   //PE5 = Hi
 }

void gp_rd_lo(void)
{
    cbi(PORTE,5) ;	   //PE5 = Lo
}
 
//******** Control Pin res(PE6=RES) *********
void gp_res_hi(void)
 {
   sbi(PORTE,6)  ;	   //PE6 = Hi
 }


void gp_res_lo(void)
{
   cbi(PORTE,6) ;	   //PE6 = Lo
}

//********* Control Back Light(PE7=Back Light)**********

void gp_bl_on(void)
{
  sbi(PORTE,7)    ;	   //PE7 = Hi(ON)
}

void gp_bl_off(void)
{
  cbi(PORTE,7)    ;	   //PE7 = Lo(OFF)
}




//*********************************************************
//************* Write Address Command to LCD **************
//*********************************************************

void gp_wr_cmm(unsigned char cmm)
 {

    //**********-- Sent Command 8 bit High --**********

     gp_cs_lo()                        ;

   //-----Write the command to the data bus.----

	 PORTF  = 0x00                     ;

   //----Set the RS signal low, indicating a command.-----
     gp_rs_lo()                        ;  //RS=0
    
   //-----Assert the write enable signal.-------
	 gp_wr_lo()                        ;  //wr=0 

   //------Deassert the write enable signal.-------
   	 gp_wr_hi()	                       ;  //wr=1


    //**********-- Sent Command 8 bit Low --*********

	 
   //-----Write the command to the data bus.----

	 PORTF  = cmm                      ;

    
   //-----Assert the write enable signal.-------
	 gp_wr_lo()                        ;  //wr=0 

   //------Deassert the write enable signal.-------
   	 gp_wr_hi()	                       ;  //wr=1

  
   //------Set the RS signal high, indicating that following writes are data.------
     gp_rs_hi()                        ;  //RS=1

 }
 



//***********************************************
//************* Write data to LCD ***************
//***********************************************

void gp_wr_data(unsigned int dw)
 {	

  //**********-- Sent Command 8 bit High --*********

      gp_cs_lo()                           ;

   //----- Write data to data bus. 8 bit PF[8..15] ------
	  

      PORTF    = dw>>8                     ;
	  	  
   //----- Assert the write enable signal.--------

	  gp_wr_lo()                           ;  //wr = 0
    
   //------ Deassert the write enable signal.-------- 

 	  gp_wr_hi()                           ;  //wr = 1
	 

 //**********-- Sent Command 8 bit Low --*********
    

   //----- Write data to data bus. 16 bit PF[8..15] ------
	  
	
      PORTF    = dw                        ;
	  	  
   //----- Assert the write enable signal.--------

	  gp_wr_lo()                           ;  //wr = 0
    
   //------ Deassert the write enable signal.-------- 

 	  gp_wr_hi()                           ;  //wr = 1
	 
	  gp_cs_hi()                           ;

 }
 
 
//***************************************************** 
//***************** Initial GLCD **********************
//*****************************************************

void init_glcd(void)
 {
    long cnt ;
    
    //-------------- Start GLCD --------------

	  gp_rd_hi()            ; 

  	  gp_res_lo()           ;    // Reset GLCD 
      delay_ms(1)           ;    // Delay for 1ms.	 
      gp_res_hi()           ;    // Stop Signal Reset GLCD    
      delay_ms(1)           ;	 // Delay for 1ms while the LCD comes out of reset.
   
  
      gp_wr_cmm(0x00)       ;	 // Command Start Oscillater
      gp_wr_data(0x0001)    ;	 // Enable the oscillator.
   
      delay_ms(10)          ;    // Delay for 10ms while the oscillator stabilizes.
   

	 //----- Display Setting -----
	 	
	  gp_wr_cmm(0x01)       ;	 // Configure the output drivers.
      gp_wr_data(0x0100)    ;
      gp_wr_cmm(0x02)       ;	 // Configure the LCD A/C drive waveform.
      gp_wr_data(0x0700)    ;	 // Line inversion 0x0700
      gp_wr_cmm(0x03)       ;    // Configure Entry Mode
      gp_wr_data(0x1230)    ; 	 // Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
   

   //------ Configure and enable the LCD power supply.-------

	 //---- Power Control -----

	  gp_wr_cmm(0x07)       ;    // Enable internal operation of the LCD controller.
      gp_wr_data(0x0101)    ;	 // 0011
	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x0000)    ;  //0x00B0
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0007)    ;   //0x0037
	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x0000)    ;  //0x011E
	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x0000)    ;  //0x1A00
	  delay_ms(20)          ;

	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x16B0)    ;
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0037)    ;
	  delay_ms(20)          ; //50

	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x013E)    ;
	  delay_ms(20)          ;  //50

	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x1A00)    ;
      gp_wr_cmm(0x29)       ;
      gp_wr_data(0x000F)    ;      
      delay_ms(20)          ;    //50- Delay for 40ms while the power supply stabilizes.
    
   
	 
	  //------- Clear the contents of the display buffer. --------
   
      gp_wr_cmm(0x22);


      for(cnt = 0;cnt<((long)240*320);cnt++)
         gp_wr_data(BLACK)   ;    // Back-Ground Color Black
      
   

	  gp_wr_cmm(0x20)       ;    //Command Horizontal GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 
	  gp_wr_cmm(0x21)       ;    //Command Vertical GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 

	  gp_wr_cmm(0x50)       ;  	 //Command Horizontal Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x51)       ;  	 //Command Horizontal Address End Position(239)
	  gp_wr_data(0x00EF)    ;  

	  
	  gp_wr_cmm(0x52)       ;  	 //Command Vertical Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x53)       ;  	 //Command Verticall Address End Position(239)
	  gp_wr_data(0x013F)    ;  
  

      gp_wr_cmm(0x60)       ;    // Set the number of lines to scan.
      gp_wr_data(0x2700)    ;

      gp_wr_cmm(0x61)       ;    // Enable grayscale inversion of the source outputs.
      gp_wr_data(0x0001)    ;

    
   //------- GAMMA Control -------

	  gp_wr_cmm(0x30)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x31)       ; 
	  gp_wr_data(0x0403)    ;
  	  gp_wr_cmm(0x32)       ; 
	  gp_wr_data(0x0404)    ;
      gp_wr_cmm(0x35)       ; 
	  gp_wr_data(0x0002)    ;
      gp_wr_cmm(0x36)       ; 
	  gp_wr_data(0x0707)    ;
	  gp_wr_cmm(0x37)       ; 
	  gp_wr_data(0x0606)    ;
      gp_wr_cmm(0x38)       ; 
	  gp_wr_data(0x0106)    ; 
	  gp_wr_cmm(0x39)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x3C)       ; 
	  gp_wr_data(0x0700)    ;
  	  gp_wr_cmm(0x3D)       ; 
	  gp_wr_data(0x0707)    ;

    
      gp_wr_cmm(0x07)       ;	 //Enable the image display.
      gp_wr_data(0x0173)    ;	 
	
      gp_bl_on()            ;	 //Back Light ON
	  
	  delay_ms(20)          ;	 //Delay for 20ms, which is equivalent to two frames.

 }





/* #########*********************************************************#############
   #########**                                                     **############# 
   #########**   2.Function Touch Screen (ADS7846:SPI-Interface)   **#############
   #########**                                                     **#############            
   #########*********************************************************#############	*/


//*********** CS [PD7] *************
 void tcs_cs_hi(void)
 {
   sbi(PORTD,7)  ;   //PD7 =1
 }

void tcs_cs_lo(void)
 {
   cbi(PORTD,7)  ;   //PD7 =0
 }

//*********** SCK [PD4] **************

void tcs_sck_hi(void)
 {
   sbi(PORTD,4) ;   //PD4 =1
 }

void tcs_sck_lo(void) 
{
   cbi(PORTD,4) ;  //PD4 =0
}


 //*********** Data MOSI [PD3]************

void tcs_sdo_hi(void)
 {
   sbi(PORTD,3) ;   //PD3 =1
 }

void tcs_sdo_lo(void)
 {
  cbi(PORTD,3) ;	//PD3 =0
 }



///************************************************************************
//*******          Function Check Status Press Touch Screen       ********
//************************************************************************
//******** Return Value : 											   ***
//********               Function return value '0'= Not Touched screen ***
//********               Function return value '1'= Touched screen     ***
//************************************************************************


char status_touch(void)
 {
  
  if((PIND & 0x40)== 0x40)	//Read PD6
	return 0 ;			    //No Touch
  else
    return 1 ;			    //Press Touch screen
 }


 //****************************************************************************************
 //******      Function SPI Write and Read Data 1 Byte from Touch Screen ADS7846     ******
 //****************************************************************************************
 //******       Parameter : 												         ******
 //******                  DataByte = data or command control ADS7846 (1Byte)        ****** 
 //******    Return Value : 														 ******
 //******                  Return value adc from touched times 1 byte Pass Function  ******         
 //****************************************************************************************

unsigned char tcs_wr(unsigned char DataByte)			  	
{
  unsigned char Bit,result=0 ;				 	   

     for(Bit=0;Bit<8;Bit++)                  // 8 Bit Write
      {		 
	   	tcs_sck_hi()                     ;	 // Start Clock  High

		//--------- write Data(PD3) ---------

		if((DataByte & 0x80)== 0x80)
		   tcs_sdo_hi()                  ;   // Set bit data(PD3) = 1
		else
		   tcs_sdo_lo()                  ;	 // Clear bit data(PD3) = 0
		   	
		tcs_sck_lo()                     ;   // Clock low

		DataByte <<= 1                   ;	 // Shift Next Bit Data
      
        
	  	//---------- Read Data(PD2) -------

	    result <<= 1                   ;	
		 										  
	     if ((PIND & 0x04)== 0x04)                // If SDI = "1"						
	      {
	       result |= 0x01                ;       // Keep Result of Read-data  									
	      }
			  
	  }

	  return (result)                    ;
}



//******************************************************************************** 
//******    Function Read X-Y-Position ADC Touch Screen-12 bit adc          ******
//********************************************************************************
//******        Parameter :	No                                              ******
//******    Return Value  :													******
//******                    dif_adc_X = Keep Result ADC X-Position(12 bit)  ******
//******                    dif_adc_Y = Keep result ADC Y-Position(12 bit)  ******                      
//********************************************************************************

void tcs_read(void)
 {
    unsigned int tcs_adc_X ,tcs_adc_Y       ;
	unsigned char buf_data[4]               ;

	if(status_touch())
	{
	//--- Set default Pin control Touch Screen ----
	  tcs_sdo_lo() ;
	  tcs_sck_lo() ;
	  tcs_cs_lo () ;


      buf_data[0] = tcs_wr(0xD0)            ;	//Write Command Measure X-Position 
	  buf_data[0] = tcs_wr(0x00)            ;	//Read ADC data X-Position (7-bit byte High) data: 0ddddddd	(bit) 
	  buf_data[1] = tcs_wr(0x90)            ;	//Write Command Measure Y-Position ; Read ADC data X-Position (5-bit byte Low)  data:ddddd000(bit)
	  buf_data[2] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(7-bit byte High) data: 0ddddddd (bit)
	  buf_data[3] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(5-bit byte Low)  data: ddddd000 (bit)

	  tcs_adc_X  = buf_data[0]              ;	// Mark ADC Data X-Position 12 bit
	  tcs_adc_X  = tcs_adc_X << 5           ;
	  tcs_adc_X |= buf_data[1] >> 3         ;
	  tcs_adc_X  = tcs_adc_X & 0x0FFF       ;
	 
	  tcs_adc_Y  = buf_data[2]              ;   // Mark ADC Data Y-Position 12 bit
	  tcs_adc_Y  = tcs_adc_Y << 5           ;   // Shift 7 bit High
	  tcs_adc_Y |= buf_data[3] >> 3         ;   // Shift 5 bit low
	  tcs_adc_Y  = tcs_adc_Y & 0x0FFF       ;   // total ADC data 12 bit
	

	  tcs_cs_hi()                           ;
	 
	 
	  dif_adc_X = tcs_adc_X                 ;	//ADC 12 bit :LCD Start Portrait
	  dif_adc_Y = 4095-tcs_adc_Y	        ;

   }
 }


//******************************************************************
//**              Function Calibrate Touch Green                  **
//**  Touch at Point + on Screen	3 Point(3 step)	for Caribrate **
//******************************************************************
//**      Parameter : No                                          **
//**   Return value : No                                          **
//**              												  **
//******************************************************************

void touch_calibrate(void)	
{									
	unsigned char n,touch_mark_point=0;
	

	plot_mark(17,32,24,25,BRIGHT_RED)                                    ; //Plot Signe + for Position Calibrate Touch Point1
	lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ; //Plot Text	Color Bright blue
	lcd_printStr("        STEP 1 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
    lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK)       ; //Plot Text	Color Black
	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK)       ;


	while(touch_mark_point != 3)	//Number Mark Point Calibrate = 3 Point  
	{
		if(status_touch())        	//return (1) if touch screen is detected.
		{		
			for(n=0;n<50;n++)
			 {
				tcs_read()                ;	//Read portrait(ADC X,Y) by touch value from ADS7846 and update dif_adc_X & dif_adc_Y
				buf_adc_X[n] = dif_adc_X  ;	//General buffer for averaging and filter touch screen adc read outs.
				buf_adc_Y[n] = dif_adc_Y  ;	//General buffer for averaging and filter touch screen adc read outs.
			 }
		
			ave_x_y(50)                   ;	//Take average out of 50 touch measurements, excluding max and min measurement.

			tcs_ave_X[touch_mark_point]=  ave_adc_X ;
			tcs_ave_Y[touch_mark_point]=  ave_adc_Y ;
		
			 
			touch_mark_point++            ;	//touch_cal_status;	//0=xy1 cal, 1=xy2 cal, 2=xy3 cal, 3=calibration success. 0xff=calibration not performed.
		
			while(status_touch()){;}        //Waite until not Touch screen ,

			switch(touch_mark_point)		//Select Caribrate Next Step 2 and 3 
			 {
					case 1:
					
						bg_color(BLACK)              ;	// Clear Screen
						plot_mark(208,160,215,153,BRIGHT_RED);	// Plot Signe + for Position Calibrate Touch Point2

                       	lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ; //Plot Text	Color Bright blue
                      	lcd_printStr("        STEP 2 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
                        lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK) ;	   //Plot Text	Color white
                     	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK) ;
 


						break;
					case 2:
					
						bg_color(BLACK)              ; // // Clear Screen
						plot_mark(113,287,120,280,BRIGHT_RED);	// Plot Signe + for Position Calibrate Touch Point3
						
						lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK) ;	//Plot Text	Color Bright blue
                      	lcd_printStr("        STEP 3 of 3         ",8,130,BRIGHT_BLUE,BLACK) ;
                        lcd_printStr(" Click on center of '+' icon",8,200,WHITE,BLACK) ;	   //Plot Text	Color white
                     	lcd_printStr(" for Touch screen calibrate ",8,220,WHITE,BLACK) ;
                          

						break;
					case 3:
					
						bg_color(BLACK)              ; //Clear Screen

						lcd_printStr("<Touch Screen Calibratetion>",8,110,BRIGHT_BLUE,BLACK)  ;	//Plot Text	Color Bright blue
                      	lcd_printStr("          Succeded!         ",8,150,BRIGHT_GREEN,BLACK) ;
                        lcd_printStr("   -Tap Screen for exit-    ",8,190,WHITE,BLACK)        ;

                       
					    while(!status_touch()){;}  
						delay_ms(100);
						bg_color(BLACK)              ; // Clear Screen

						break;
			}
			 
		}
	 delay_ms(100) ;	
	}
																			 
	set_matrix()   ;  // Calculate Coefficient for use find Position Address real on Display
	

	//You can Keep Value data in valiabie devider,An,Bn,Cn,Dn,En,Fn to E2Prom  so that not 
	//come back	 Caribrate.


//  printf("\n\r divider = %d",divider) ;
//	printf("\n\r      An = %d",An) ;
//	printf("\n\r      Bn = %d",Bn) ;
//	printf("\n\r      Cn = %d",Cn) ;
//  printf("\n\r      Dn = %d",Dn) ;
//	printf("\n\r      En = %d",En) ;
//	printf("\n\r      Fn = %d",Fn) ;
  
 
  }




//*********************************************************************************************
//******  Function Calculate average of measurements,by cut value max and min measurement. ****
//*********************************************************************************************
//******      Parameter :					                                          *********
//******			     num = number times of measurement adc from touch screen      *********
//******    Return value:														      *********
//******                 ave_adc_X = Keep result average adc X-Position			      *********
//******                 ave_adc_Y = Keep result average adc Y-Position               *********     
//*********************************************************************************************

void ave_x_y(unsigned char num)				
{
	unsigned char i;
	long sum_x, sum_y;
	long min_x, min_y, max_x, max_y;

	sum_x=0;sum_y=0;min_x=0xffff;min_y=0xffff,max_x=0;max_y=0;

	for(i=0;i<num;i++)
	{
		sum_x += buf_adc_X[i]         ;
		sum_y += buf_adc_Y[i]         ;

		if(buf_adc_X[i]< min_x)		                         //search for minimum X value
		{
			min_x = buf_adc_X[i]      ;
		}
		else if (buf_adc_X[i]> max_x)	                    //search for maximum X value
		{
			max_x = buf_adc_X[i]      ;
		}

		if(buf_adc_Y[i]< min_y)	  	                        //search for minimum Y value
		{
			min_y = buf_adc_Y[i]      ;
		}
		else if(buf_adc_Y[i]> max_y)	  	                //search for maximum Y value
		{
			max_y = buf_adc_Y[i]      ;
		}
	}
	ave_adc_X = (sum_x - min_x - max_x ) / (num-2) ;  	   //update x average value
	ave_adc_Y = (sum_y - min_y - max_y ) / (num-2) ;	   //update y average value
}




//****************************************************************************
//********* Function Calculate Matrix Find value Coeffcient            *******
//****************************************************************************
//*********     Parameter  : No                                        *******
//*********   Return Value : Keep value Coeffcient for use calculate   *******                                      ***********
//*********                  X-Y-Position in variable divider,An,Bn,Cn *******                   
//*********                  Dn,En,Fn                                  *******
//****************************************************************************

 void set_matrix()
  {
	divider =  	((tcs_ave_X[0]-tcs_ave_X[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
	            ((tcs_ave_X[1]-tcs_ave_X[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))      ;


         An = 	((dis_XD[0]-dis_XD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_XD[1]-dis_XD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

	 	 Bn = 	((tcs_ave_X[0]-tcs_ave_X[2])*(dis_XD[1]-dis_XD[2]))-
				((dis_XD[0]-dis_XD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Cn =  	(tcs_ave_X[2]*dis_XD[1]-tcs_ave_X[1]*dis_XD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_XD[2]-tcs_ave_X[2]*dis_XD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_XD[0]-tcs_ave_X[0]*dis_XD[1])*tcs_ave_Y[2]   ;

		 Dn =   ((dis_YD[0]-dis_YD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_YD[1]-dis_YD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

		 En =   ((tcs_ave_X[0]-tcs_ave_X[2])*(dis_YD[1]-dis_YD[2]))-
				((dis_YD[0]-dis_YD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Fn =  	(tcs_ave_X[2]*dis_YD[1]-tcs_ave_X[1]*dis_YD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_YD[2]-tcs_ave_X[2]*dis_YD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_YD[0]-tcs_ave_X[0]*dis_YD[1])*tcs_ave_Y[2]   ;

  }


//****************************************************************************
//*********** Function Get Position Address real of Display ******************
//****************************************************************************
//***         Parameter  :   										      ****
//***                     num = Number times for Measure adc from Touched **** 
//***       Return Value :											      ****
//***                       X = Keep value address X-Position		      ****
//***                       Y = Keep value address Y-Position             **** 
//****************************************************************************

void get_point(unsigned char num)
 {
    unsigned char nm  ; 
	
		  nm=0                              ;
		  while(status_touch() && nm<num)	  //Measure touch x,y 10 times if the Touch kept pressed,
		   {
			  tcs_read()                    ; //Read value ADC Touch X-Y 
			  buf_adc_X[nm]=	dif_adc_X   ; //keep value ADC Touch-X
		      buf_adc_Y[nm]=	dif_adc_Y   ; //keep value ADC Touch-Y

 			  nm++                          ; 
		   }

		  if(nm==num)	                      //if the touch xy successfuly collected,
		   {	
				ave_x_y(num)                ; //take average out of 10 touch measurements, excluding max and min measurement.
			    
				dif_adc_X = ave_adc_X       ; //keep average ADC Touch-X
				dif_adc_Y = ave_adc_Y       ; //keep average ADC Touch-Y
               
			    X = ((An*dif_adc_X)+(Bn*dif_adc_Y)+Cn)/divider  ; 
	            Y = ((Dn*dif_adc_X)+(En*dif_adc_Y)+Fn)/divider  ;
		   }

 }



/* #########*********************************************************#############
   #########**                                                     **############# 
   #########**           3.Function Application of GLCD            **#############
   #########**                                                     **#############            
   #########*********************************************************#############	*/


//***********************************************************************
//**         Function Set  Background color or Clear Screen            **
//***********************************************************************
//**                 Parameter :							           **
//**                            bg_color =  BackGround color of Screen **
//***********************************************************************

void bg_color(unsigned int bg_color)
 {	
    long  cnt  ;
 
    gp_wr_cmm(0x20)           ;    //Command Horizontal GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000 
    gp_wr_cmm(0x21)           ;    //Command Vertical GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000  
    
	gp_wr_cmm(0x22);
	
    for(cnt = 0;cnt<((long)240*320);cnt++)
       gp_wr_data(bg_color)                ;    // Back-Ground Color Black
   
 }

 

//**********************************************************
//*****             Function Plot Signe+               *****
//**********************************************************
//***** Paramiter:                                     *****
//*****           x1,y1 = Position start Line '-'(Hor) *****
//*****                   plot from Left to Right      *****
//*****           x2,y2 = Position start Line '|'(Ver) *****
//*****                   plot from botton to top      *****
//**********************************************************

void plot_mark(unsigned int x1,unsigned int y1,unsigned int x2,unsigned int y2,unsigned int color)
 {
    unsigned char i          ;

  	//---- Line- ----
     gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(x1)          ;  //Sent X_address  CGRAM
	 		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(y1)          ;  //Sent Y_address  CGRAM
	 
	 gp_wr_cmm(0x22)         ;  //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;

	//------Line | --------
      gp_wr_cmm(0x03)        ;   //Command Configure Entry Mode
      gp_wr_data(0x1238)     ; 	 //Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.

	 gp_wr_cmm(0x20)         ;   //Command Set Adddress Hor(X)
     gp_wr_data(x2)          ;   //Sent X_Address CGRAM 
	 		
     gp_wr_cmm(0x21)         ;   //Command Set Address Ver(Y)
     gp_wr_data(y2)          ;	 //Sent Y_Address CGRAM 
	 
	 gp_wr_cmm(0x22)         ;   //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;
 
	 gp_wr_cmm(0x03)         ;   //Command Configure Entry Mode
     gp_wr_data(0x1230)      ; 	 //Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.

 	
 }

    

//**********************************************************************
//**         Function Plot Picture(BMP) to Display                    **
//**  Start Plot data of picture from  top left to top Right          **
//**********************************************************************
//**	 Parameter : 											      **
//**                *bmp = Name reg. array keep data of picture       **
//**             xad,yad = Position X,Y for begin plot data picture   **
//**          width,high = Size Picture unit Pixel                    **
//**********************************************************************

 void plot_picture(char *bmp,unsigned int xad,unsigned int yad,unsigned int width,unsigned int high)        
 {
   unsigned long pt=0,picture            ;
   unsigned int hi,wid                   ;
   unsigned int xad1,yad1                ;

	   xad1 = xad ;
	   yad1 = yad ;

	   for(hi=0;hi<high;hi++)		                   //Number Y-Pixel(High) of Picture
	    {      		
          gp_wr_cmm(0x21)                           ;  //Command Set Address Ver(Y)
          gp_wr_data(yad1)                          ;  //Sent Y_Address CGRAM		 		  

		  for(wid=0;wid<width;wid++)	               //number X-pixel(Width) of Picture  
		   {
			  gp_wr_cmm(0x20)                       ;  //Command Set Adddress Hor(X)
              gp_wr_data(xad1)                      ;  //Sent X_Address CGRAM	
		   				 
			  gp_wr_cmm(0x22)                       ;  //Command Write data 	
			 	   
			  picture = pgm_read_byte(&(bmp[pt+1])) ;  //Read data picture byte High
			 
			  picture =	picture <<8                 ;
			  
			  picture|= pgm_read_byte(&(bmp[pt]))   ;  //read data picture Byte low

              gp_wr_data(picture)                   ;
			  pt = pt+2                             ;  //Increment pointer of data picture 2 byte
			    
			  xad1++                                ;
			  
		  }

		 yad1++                                     ;
		 xad1 = xad                                 ;
	   }
 }




//*************************************************************************
//**                 Function build Button (Draw-rectang)                **
//**    Start Plot button from  botton left to botton Right of button    **  
//*************************************************************************
//**																	 **
//** - Parameter:														 **
//** 		     px,py = Position X,Y  begin build  button               **
//**        width,high = width and high of button(unit Pixel)            **
//**                     for width not exceed 50 pixel will good         **
//**              c_bt = Color of button                                 **
//**               *tx = ASCII Charecter on Button                       **
//**                     1 Charecter width= 9pixel. Number pixel of		 **
//**                     charecter total must less pixel width of button **  
//**                     												 **
//**            num_tx = Number Charecter at display on button           **
//**              c_tx = Color charecter on button                       **
//**       																 **
//**             |<--- width --->|								    	 **
//**              -------------->   ---									 **
//**             |           end |	 ^									 **
//**             |               |	 |									 **
//**             |               |	high								 **
//**             |start          |	 |									 **
//**      (px,py) -------------->	---									 **
//**																	 **
//*************************************************************************



void button_build(unsigned int px,unsigned int py,unsigned int width,unsigned int high,unsigned int c_bt,char *tx,unsigned char num_tx,unsigned int c_tx)      
 {
   unsigned int x1,y1,x2,y2,ay,tpx  ;
   unsigned char tab,n              ;
   unsigned int cbt1=0,cbt2=0,sub=0 ;

    switch(c_bt)
	 {
	    case BT_RED   :
		                c_bt = 0xF800    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x9800       ; //color last  Min (cbt1-(sub*12))
					    sub  = 0x0800       ; //decrement color
		        break;

		case BT_GREEN :
		                c_bt = 0x07C0    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x04D0       ; //color last  Min (cbt1-(sub*12))
					    sub  = 0x0040       ; //decrement color		         
			    break;

	    case BT_BLUE   :
		                c_bt = 0x001F    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x0014       ; //color last  Min (cbt1-(sub*12))
					    sub  = 0x0001       ; //decrement color
		        break;

       	case BT_YELLOW :
		               	c_bt = 0xFFC0    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x9CC0       ; //color last  Min(cbt1-(sub*12))
					    sub  = 0x0840       ; //decrement color
		         break;

		case BT_WHITE  :
		                c_bt = 0xFFFF    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x9CF3       ; //color last  Min (cbt1-(sub*12))
					    sub  = 0x0841       ; //decrement color
		        break;

	   	case BT_CYAN  :
		                c_bt = 0x87FF    	; //Color 
					    cbt1 = c_bt         ; //color start Max
					    cbt2 = 0x24F3       ; //color last  Min(cbt1-(sub*12))
					    sub  = 0x0841       ; //decrement color
		         break;


	  	case BT_MAGENTA :
		                 c_bt = 0xFC1F     	 ; //Color 
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0x9913       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0841       ; //decrement color
		         break;

	   	case BT_BROWN :
		                 c_bt = 0xEC00    	; //Color 
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0x8900       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0840       ; //decrement color
		         break;
	
//-----------------------------------------------------------------------------------

	    case BT_LRED  :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0xFCF3       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0041       ; //decrement color
		         break;	
				 
	    case BT_LGREEN :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0x9FF3       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0801       ; //decrement color
		         break;	
				 
	     case BT_LBLUE :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0x9CFF       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0840       ; //decrement color
		         break;

	  
		 case BT_LYELLOW :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0xFFC3       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0001       ; //decrement color
		         break;
         
		 case BT_LWHITE :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0xFFDE      ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0021       ; //decrement color
		         break;


		 case BT_LCYAN :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0x9FFF       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0800       ; //decrement color
		         break;
						
		case BT_LMAGENTA :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0xFCFF       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0040       ; //decrement color
		         break;	
				 
	     case BT_LBROWN :
		                 c_bt = 0xFFFF       ;
					     cbt1 = c_bt         ; //color start Max
					     cbt2 = 0xFE73       ; //color last  Min(cbt1-(sub*12))
					     sub  = 0x0021       ; //decrement color
		         break;												
	 }


    x1   = px           ;    
	y1   = py           ;
    x2   = px+width     ; 
    y2   = py-high+1    ;
  

    high  = high/2      ;
    ay    = py-high     ;
    tab   = high/12     ;
	
	if(tab==0)
	   tab=1            ;
	else
	 {
	  if((high%12)>3)
	    tab = tab+1     ;
	 }

 //---------Start Plot halft button from botton to top ---------


   while(y1>=ay)
    {
     for(n=0;n<tab;n++)
      {
       Line(x1,y1,x2,y1,1,cbt1) ;
       y1--                     ;
	   
	   if(y1 <= ay)
	    n=n+20                  ;
      }
      cbt1 = cbt1-sub           ;
     
	 if(cbt1<cbt2)
        cbt1=cbt2               ;
    }

  //---------Plot halft button from top to botton---------
 
  cbt1 = c_bt                   ; 

   while(y2<=ay)
    {
     for(n=0;n<tab;n++)
      {
       Line(x1,y2,x2,y2,1,cbt1) ;
       y2++                     ;
	 
	   if(y2 >= ay)
	     n=n+20                 ;
      }
     cbt1 = cbt1-sub            ;
     
	 if(cbt1<cbt2)
       cbt1=cbt2                ;
    }

   //----------write Text ---------
	
   	tpx = (width-(num_tx*8))/2  ;
	tpx = tpx+px                ;
		
	py = (py-high)+6            ;
	

	for(n=0; tx[n] != '\0'; n++)
	 {					 
	  text_7x11(tx[n],tpx,py,c_tx,no_bg) ;
	  tpx += 9                           ; 		
	 }

 }



//********************************************************************************
//**                Function Print Text 1 Charecter size 7x11                   **
//********************************************************************************
//**    Parameter :  															**
//**               row      = Ascii Code (Position buffer keep text)		    **
//**               adx,ady  = Position X,Y for begin plot text by will 			**
//**                          begin plot from bottom left to top left   		**
//**               fg_clr   = Color of text										**
//**               bg_clr   = Color background of text(if bg_clr = no_bg or 1=	**
//**                          non color background)								**
//********************************************************************************


 void text_7x11(unsigned char row,int adx,int ady,unsigned int fg_clr,unsigned int bg_clr)
  {
   int ax,ay                  ;
   unsigned char m,n,tx       ;
   
     
	 ax = adx         ;
	 ay = ady         ; 
	 row = row-0x20   ;

  //------- Print Text 1 Charecter(data 14 Byte) -------------
	  
	  for(m=0;m<14;m++)
	   {
	    //---Sent data byte1=8 bit----

        tx = pgm_read_byte(&(ascii_7x11[row][m]))     ; //Read data Ascii
		 
	 
		for(n=0;n<8;n++)		       //Loop Sent data  1 byte(8bit)
		 {
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ax)        ;  //Sent X_Address CGRAM	 		
              gp_wr_cmm(0x21)       ;  //Command Set Address Ver(Y)
              gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data 
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ax)        ;  //Sent X_Address CGRAM	
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;  //Sent Data
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay   = ay-1              ;  //Decrement Y-address
		 } 

		 m++                        ;  //Increment Next pointter byte Data 


		  //------- Sent data byte2=3 bit -----

		 tx =  pgm_read_byte(&(ascii_7x11[row][m]))     ;  //Read data byte2
	
		for(n=0;n<3;n++)			   //Loop sent data byte2 = 3 bit
		 {						    ;
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ax)        ;  //Sent X_Address CGRAM		
              gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
              gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ax)        ;  //Sent X_Address CGRAM	 		
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ay)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay = ay-1                ;  //Decrement Y-address
		 } 

		ax = ax+1                   ; //Complet sent data 2 byte(11bit) Increment X-Address
		ay = ady                    ; //Set Position Y-address old
 
	  }	


	 //----Fill Back ground Color Position space between Charecter 1 Colum -------
 
     if(bg_clr != 1)     
     {
	  for(n=0;n<11;n++)
	  {
	    gp_wr_cmm(0x20)            ;  //Command Set Adddress Hor(X)
        gp_wr_data(ax)             ;  //Sent X_Address CGRAM	 		
        gp_wr_cmm(0x21)            ;  //Command Set Adddress Ver(Y)
        gp_wr_data(ay)             ;  //Sent Y_Address CGRAM
	    gp_wr_cmm(0x22)            ;  //Command Write data
        gp_wr_data(bg_clr)         ;
		ay = ay-1                  ;  //Incre ment Y-Address
	  }
     }
  }


 //************************************************************************
 //**                         Function Print String                      **
 //************************************************************************
 //**      Parameter :													 **
 //**                 *str = Charecter ASCII (String)					 **
 //**          cur_x,cur_y = Position X,Y for begin plot text by will 	 **
 //**                        begin plot from bottom left to top left     **
 //**             fg_color = color of Text								 **
 //**             bg_color = color Background of text                    **											
 //************************************************************************

void lcd_printStr(char *str,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)
{
  unsigned char i;

	for (i=0; str[i] != '\0'; i++)
	{					 
	 text_7x11(str[i],cur_x,cur_y,fg_color,bg_color) ;
	 cur_x += 8                                      ; 
		
	}
 }
 
//*********************************************************
//************ Function Print Text 3 Charecter ************
//*********************************************************

void lcd_print3Cha(char ch1,char ch2,char ch3,int cur_x,int cur_y,unsigned int fg_color,unsigned int bg_color)
{	 
 			 
  text_7x11(ch1,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ; 
  text_7x11(ch2,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ; 
  text_7x11(ch3,cur_x,cur_y,fg_color,bg_color);
  cur_x += 8 ;  	
 }

 
//*************************************************************
//**                Function Plot Dot 1 Pixel                **
//*************************************************************
//**      Parameter :										 **
//**                 Xadd,Yadd = Position X,Y for Plot Dot   **
//**                     color = Color of dot                **
//*************************************************************

 void plot_dot(unsigned int Xadd,unsigned int Yadd,unsigned int color)
  {
	
	 gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(Xadd)        ;  //Sent X_address for CGRAM 
		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(Yadd)        ;  //Sent Y_address for CGRAM 
	  			
     gp_wr_cmm(0x22)         ;  //Command Write Data RGB

     gp_wr_data(color)       ;  //write data
 }


 //*******************************************************************
 //**                   Function Draw Line 							**
 //*******************************************************************
 //** Function : Line(x_1,y_1,x_2,y_2,line_type);			        **
 //**																**
 //**   Input  : x_1,y_1    = Start Coordinate Line					**
 //**            x_2,y_2    = End Coordinate Line					**
 //**            line_type  = Value Thick of Line (1-100)           **     
 //**	         color      = Color of line			                **
 //**      															**
 //*******************************************************************

  void Line(int x_1,int y_1,int x_2,int y_2,unsigned char line_type,unsigned int color)
   {
    unsigned char tick           ;
    int  x1=x_1,xx1=x_1          ;
    int y1=y_1                   ;
    int x2=x_2,xx2=x_2           ;
    int y2=y_2                   ;
	int dx,dy,inc_dec,two_dx,two_dy,two_dy_dx,two_dx_dy,x,y,p  ;

    if(x_1>x_2)
	 {
	   x1=x_2 ;
	   y1=y_2 ;

	   x2=x_1 ;
	   y2=y_1 ;
	 }

       dx  = abs(x2-x1) ;
       dy  = abs(y2-y1) ;

	   if(y2>=y1)
	     inc_dec = 1    ;
	   else
		 inc_dec = -1   ;

       if(dx>dy)
	    {
	      two_dy=(2*dy)         ;
	      two_dy_dx=(2*(dy-dx)) ;
	      p=((2*dy)-dx)         ;

	      x=x1 ;
	      y=y1 ;

	      while(x<=x2)
		   {

		    for(tick=0;tick<line_type;tick++)
			  {	   
			    if(xx1==xx2)
			      plot_dot(x+tick,y,color)   ;
			    else
			      plot_dot(x,y+tick,color)   ;

			  //if(xx1==xx2)
			  //  plot_dot(x+tick,y+tick,color)   ;
			  // else
			  //  plot_dot(x+tick,y+tick,color)   ;
			  
			  }

		    x++              ;

		    if(p<0)
		       p += two_dy   ;
		    else
		     {
			  y += inc_dec   ;
			  p += two_dy_dx ;
		     }
		  }  //while
	    }	 //if

       else
	    {
	      two_dx=(2*dx)            ;
	      two_dx_dy=(2*(dx-dy))    ;
	      p=((2*dx)-dy)            ;

	      x=x1                     ;
	      y=y1                     ;

	     while(y!=y2)
		  {
		   for(tick=0;tick<line_type;tick++)
		    {		     
			 if(xx1==xx2)					    
			   plot_dot(x+tick,y,color)       ;
			 else
			   plot_dot(x,y+tick,color)       ;
			
			// if(xx1==xx2)
			//    plot_dot(x+tick,y+tick,color)   ;
			// else
			//    plot_dot(x+tick,y+tick,color)   ;
			  
		    }
		   y+=inc_dec         ;

		   if(p<0)
		      p+=two_dx       ;

		   else
		    {
			 x++              ;
			 p += two_dx_dy   ;
		    }
		 }  //while
	   }	//else

    }	    //end.




//********************************************************************************
//**                         Function Draw Circle 					            **
//********************************************************************************   
//**  Function : Circle(CX,CY,Radius,tick,color,fill)		                    **
//**																		    **
//**   Input   :  CX,CY    = Center Point of Circle							    **
//**              Radius   = radius of Circle					                **
//**            tick(1-100)= bold of line (have effect when fill=0)	            **
//**              color    = color of Circle						   	        **
//**              fill     = 0: No fill color in Circle,1:Fill color in Circle  **
//********************************************************************************
 
 void Circle(int CX,int CY,unsigned char Radius,unsigned tick,unsigned int color,unsigned char fill)
  {
    int tswitch=0   ;
    unsigned char d,x=0,y=0,bd   ;


    d = CY-CX ;
	y = Radius ;
	tswitch = 3-(2*Radius);

	while(x<=y)	
	 {
	   if(fill==1)
	    {
		 Line(CX-x,CY+y,CX+x,CY+y,1,color) ;
		 Line(CX-x,CY-y,CX+x,CY-y,1,color) ;
		 Line(CX-y,CY+x,CX+y,CY+x,1,color) ;
		 Line(CX-y,CY-x,CX+y,CY-x,1,color) ;

		}
	   else
		{
	
		 for(bd=0;bd<tick;bd++)
		  {
		   plot_dot((CX+x)+bd,(CY+y)+bd,color)     ;
		   plot_dot((CX+x)+bd,(CY-y)+bd,color)     ;

		   plot_dot((CX-x)+bd,(CY+y)+bd,color)     ;
		   plot_dot((CX-x)+bd,(CY-y)+bd,color)     ;

		   plot_dot((CY+y-d)+bd,(CY+x)+bd,color)   ;
		   plot_dot((CY+y-d)+bd,(CY-x)+bd,color)   ;

		   plot_dot((CY-y-d)+bd,(CY+x)+bd,color)   ;
		   plot_dot((CY-y-d)+bd,(CY-x)+bd,color)   ;
		  }
	   	}

	   if(tswitch<0)
	     tswitch += (4*x)+6     ;
	   else
	    {
         tswitch += 4*(x-y)+10  ;
		 y--;
	    }
       x++;
     }              //while
	
  }



//************************************************************************************
//**                           Function Draw Ellipse 							    **
//************************************************************************************   
//**  Function : Ellipse(h,k,rx,ry,tick,color,fill)								    **
//**																			    **
//**   Input   :  h,k        = Center Point of Ellipes  				            **
//**              rx,ry      = radius of Ellipse in X,Y axis			            **
//**              tick(1-100)= bold of line (have effect when fill=0)	            **
//**              color      = color of Ellipse						   	            **
//**              fill       = 0: No fill color in Ellipes,1:Fill color in Ellipes  **
//************************************************************************************ 

 void Ellipse(unsigned int h,unsigned int k,long rx,long ry,unsigned char tick,unsigned int color,unsigned char fill)
    {
	  unsigned char bd       ;
      long  ax  =(rx*rx)     ;
      long   by  =(ry*ry)    ;
      float  ax2 =(ax*2)     ;
      float  by2 =(by*2)     ;

      long   x   = 0         ;
      long   y   = ry        ;

      float  fx  = 0         ;
      float  fy  = (ax2*ry)  ;

      float     p=(int)(by-(ax*ry)+(0.25*ax)+0.5);


	   if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color) ;
		 Line(h-x,k-y,h+x,k-y,1,color) ;
	    }
	   else
	    {
		 for(bd=0;bd<tick;bd++)
		  {
           plot_dot((h+x)+bd,(k+y)+bd,color)      ;
		   plot_dot((h+x)+bd,(k-y)+bd,color)      ;
		   plot_dot((h-x)+bd,(k-y)+bd,color)      ;
		   plot_dot((h-x)+bd,(k+y)+bd,color)      ;
		  }
		}
			   
       while(fx<fy)
	   {
	     x++;
	     fx+=by2;

	     if(p<0)
		p+=(fx+by);

	     else
		{
		   y--;
		   fy-=ax2;
		   p+=(fx+by-fy);
		}

		if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color) ;
		 Line(h-x,k-y,h+x,k-y,1,color) ;
	    }
	   else
	    {
		  for(bd=0;bd<tick;bd++)
		   {		  
		    plot_dot((h+x)+bd,(k+y)+bd,color)      ;
		    plot_dot((h+x)+bd,(k-y)+bd,color)      ;
		    plot_dot((h-x)+bd,(k-y)+bd,color)      ;
		    plot_dot((h-x)+bd,(k+y)+bd,color)      ;
		   }
		}


	  }

       p=(int)((by*(x+0.5)*(x+0.5))+(ax*(y-1)*(y-1))-(ax*by)+0.5);

       while(y>0)
	  {
	     y--                           ;
	     fy-=ax2                       ;

	     if(p>=0)
		p+=(ax-fy)                     ;

	     else
		{
		   x++                         ;
		   fx+=by2                     ;
		   p+=(fx+ax-fy)               ;
		}

		if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color) ;
		 Line(h-x,k-y,h+x,k-y,1,color) ;
	    }
	   else
	    {
		  for(bd=0;bd<tick;bd++)
		   {
		    plot_dot((h+x)+bd,(k+y)+bd,color)      ;
		    plot_dot((h+x)+bd,(k-y)+bd,color)      ;
		    plot_dot((h-x)+bd,(k-y)+bd,color)      ;
		    plot_dot((h-x)+bd,(k+y)+bd,color)      ;
		   }
		}   //else
	    
	  }
	 
    }




//******************************************************************************************
//**                 Function Draw a rectangle 			                             	  **
//******************************************************************************************
//**    Function : Rectan(x1,y1,x2,y2,tick,color,fill)									  **
//**																					  **
//**    Inputs   : (x1, y1)    = Start Point coordinate Rectan							  **
//**               (x2, y2)    = End Point coordinate  Rectan							  **
//**               tick(1-100) = bold of line(have effect when fill=0					  **
//**                color      =  color of rectangle									  **
//**               fill        = 0: No fill color in rectangle,1:Fill color in rectangle  **
//**       																				  **
//**                  (x1,y1) -------------												  **
//**                         |             |											  **
//**                         |             |											  **
//**                         |             |											  **
//**                         |             |											  **
//**                          -------------	(x2,y2)										  **
//**																					  **
//******************************************************************************************
																						  
void Rectan(int x1,int y1,int x2,int y2,unsigned char tick,unsigned int color,unsigned char fill)
{
   int xmin,xmax,ymin,ymax,i ;
   unsigned char tk = tick-1 ;

     if(fill==1)
	  {
	       if(x1 < x2)                            //  Find x min and max
              {
               xmin = x1;
               xmax = x2;
			  }
			 else
			  {
			    xmin = x2;
			    xmax = x1;
			   }
			
			 if(y1 < y2)                            // Find the y min and max
			   {
			     ymin = y1 ;
			     ymax = y2 ;
			   }
			 else
			   {
			    ymin = y2 ; 
			    ymax = y1 ;
			   }
			
			 for(; xmin<=xmax;xmin++)
			   {
			     for(i=ymin; i<=ymax; i++)
				   plot_dot(xmin,i,color)  ;
			   }


	  }
	 else	// No-Fill
	  {
	    Line(x1,y1,x2,y1,tick,color)       ; 
	    Line(x1,y2-tk,x2,y2-tk,tick,color) ;
	    Line(x1,y1,x1,y2,tick,color)       ;
	    Line(x2-tk,y1,x2-tk,y2,tick,color) ;
	  }      
		 
 }






